function [stats] = cycProp2(varargin)

printTrend = 0;
printDev = 0;

data = xlsread('Master_for_Import');

year = data(:,1);
popDef = data(:,2); 
popDef13 = data(:,3); 
pop = data(:,4)/1d9; % # of households, convert to billions
Hours = data(:,5); 
KY = data(:,6);
disch = data(:,7);
rGDP = data(:,8); % Measured in billions
rCsn = data(:,9);
rInv = data(:,10);
rDebt = data(:,11)/1000; % Measured in millions, convert to billions
rIntRate = data(:,12);
rDisDebt = disch.*rDebt; % Discharge rate * rDebt

plot(year,rDebt./rGDP)
plot(year,disch.*rDebt./rGDP)
plot(year,disch)
plot(year,rIntRate)
plot(year,Hours)
plot(year,popDef)

% Plot the default rate for all available years
figure(1)
plot(year,100*popDef,'b-o','LineWidth',2)
xlabel('Year','FontSize',12)
ylabel('% of Households Filing','FontSize',12)
title('Percent of Households Filing','FontSize',12)
axis([year(1) year(end) -Inf Inf])
if (printTrend); 
    print('pics/hhsFiling','-depsc'); 
end

% Plot deviations in default rate and gdp for 1960-2004
figure(2)
tmp = year>=1960 & year<=2004;
[~,devDef] = logHP(popDef(tmp)); 
[~,devRGDP] = logHP(rGDP(tmp)); 
plot(year(tmp),devDef,'b-',year(tmp),devRGDP,'r--','LineWidth',2)
xlabel('Year','FontSize',12)
ylabel('Deviation from Trend','FontSize',12)
title('Deviations from Trend','FontSize',12)
tmp = legend('Pct of Filings','Real GDP');
set(tmp,'Location','SouthEast','FontSize',12);
if (printDev)
    print('pics/devTrend','-depsc'); 
end

% Averages over certain years
if (nargin==0)
    tmp = (year>=1999 & year<=2003);
    fprintf(1,'Avg default rate * 100, 1999-2003   %5.2f\n',100*mean(popDef(tmp)));
    tmp = (year>=1995 & year<=2004);
    fprintf(1,'Avg default rate * 100, 1995-2004   %5.2f\n',100*mean(popDef(tmp)));
    fprintf(1,'Avg debt/gdp ratio * 100, 1995-2004 %5.2f\n',100*mean(rDebt(tmp)./rGDP(tmp)));
    fprintf(1,'Avg discharge debt/gdp ratio * 100, 1995-2004 %5.2f\n',100*mean(rDisDebt(tmp)./rGDP(tmp)));
    fprintf(1,'\n');
end

% Compute statistics over various time periods
gs = @(lbyear,ubyear) genstats(lbyear,ubyear,year,popDef,Hours,KY,disch,...
         rGDP,rCsn,rInv,rDebt,rIntRate,rDisDebt,nargin);
     
% gs(1960,1984); % Pre 'structural break' in data
% gs(1960,2012); % All years (some data series not available)
% gs(2006,2012); % Post BAPCPA
% gs(1985,2012); % Full sample (all data series available for all years)

if (nargin==0)
    gs(1985,2004); % Full sample preBAPCPA (note: interest rate is personal loans, which are no longer dischargeable since BAPCPA)
    gs(1960,2004); % All years preBAPCPA
else
    stats = gs(varargin{1},varargin{2}); 
end

end

function [stats] = genstats(lbyear,ubyear,year,popDef,Hours,KY,disch,...
                       rGDP,rCsn,rInv,rDebt,rIntRate,rDisDebt,nvar)
   

fprintf(1,'\n\n~~~~~~~~~~%i-%i~~~~~~~~~~~\n\n',lbyear,ubyear);                   
                   
range = lbyear<=year & year<=ubyear;


year = year(range);
popDef = popDef(range);
Hours = Hours(range); 
KY = KY(range);
disch = disch(range);
rGDP = rGDP(range);
rCsn = rCsn(range);
rInv = rInv(range);
rDebt = rDebt(range); 
rIntRate = rIntRate(range);
rDisDebt = rDisDebt(range);

if (nvar==0)
    disp('Means (not deviation)')
    fprintf(1,'Pop Def*100:       %5.2f \n',100*mean(popDef));
    fprintf(1,'Hours:             %5.2f \n',mean(Hours));
    fprintf(1,'K/Y:               %5.2f \n',mean(KY));
    fprintf(1,'disch*100:         %5.2f \n',100*mean(disch));
    fprintf(1,'rDebt/rGDP*100:    %5.2f \n',100*mean(rDebt./rGDP));
    fprintf(1,'rDisDebt/rGDP*100: %5.2f \n',100*mean(rDisDebt./rGDP));
    fprintf(1,'rIntRate*100:      %5.2f \n',100*mean(rIntRate));
    disp(' ')
end

% Log and HP filter
[~,devDef] = logHP(popDef); 
[~,devRGDP] = logHP(rGDP); 
[~,devRCSN] = logHP(rCsn); 
[~,devRINV] = logHP(rInv); 
[~,devHOURS] = logHP(Hours); 
[~,devRDEBT] = logHP(rDebt); 
[~,devRDISDEBT] = logHP(rDisDebt); 
[~,devRINT] = logHP(rIntRate); 

r = (3:numel(devRGDP)-2)'; %range

% Get lead and lag variables
[Ym2,Ym1,Y,Yp1,Yp2] = leadlag(devRGDP,r);
[Im2,Im1,I,Ip1,Ip2] = leadlag(devRINV,r);
[Cm2,Cm1,C,Cp1,Cp2] = leadlag(devRCSN,r);
[mu01m2,mu01m1,mu01,mu01p1,mu01p2] = leadlag(devDef,r);
[Hm2,Hm1,H,Hp1,Hp2] = leadlag(devHOURS,r);
[Dm2,Dm1,D,Dp1,Dp2] = leadlag(devRDEBT,r);
[DDm2,DDm1,DD,DDp1,DDp2] = leadlag(devRDISDEBT,r);
[Rm2,Rm1,R,Rp1,Rp2] = leadlag(devRINT,r);

absstd = std([Y,C,I,H,mu01,D,DD,R])'*100;
if (nvar==0)
    disp('Deviation, std*100')
    fprintf(1,'Y    %5.2f  \n',absstd(1));
    fprintf(1,'C    %5.2f  \n',absstd(2));
    fprintf(1,'I    %5.2f  \n',absstd(3));
    fprintf(1,'H    %5.2f  \n',absstd(4));
    fprintf(1,'mu01 %5.2f  \n',absstd(5));    
    fprintf(1,'D    %5.2f  \n',absstd(6));    
    fprintf(1,'DD   %5.2f  \n',absstd(7));    
    fprintf(1,'R    %5.2f  \n',absstd(8));    
    disp(' ')
end

% disp('Deviations, std/std(Y)')
% relstd = std([Y,C,I,H,mu01,D,DD,R])'./std(Y);
% fprintf(1,'Y    %5.2f  \n',relstd(1));
% fprintf(1,'C    %5.2f  \n',relstd(2));
% fprintf(1,'I    %5.2f  \n',relstd(3));
% fprintf(1,'H    %5.2f  \n',relstd(4));
% fprintf(1,'mu01 %5.2f  \n',relstd(5));
% fprintf(1,'D    %5.2f  \n',relstd(6));
% fprintf(1,'DD   %5.2f  \n',relstd(7));
% fprintf(1,'R    %5.2f  \n',relstd(8));
% disp(' ')

disp('                        std*100,  Correlation of lagged variable with output')
rho = corrcoef([Y,Ym2,Ym1,Y,Yp1,Yp2]);    
fprintf(1,'Output (y)             & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',absstd(1),rho(1,2:end));
rho = corrcoef([Y,Cm2,Cm1,C,Cp1,Cp2]);
fprintf(1,'Consumption            & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',absstd(2),rho(1,2:end));   
rho = corrcoef([Y,Im2,Im1,I,Ip1,Ip2]);
fprintf(1,'Investment             & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',absstd(3),rho(1,2:end));
rho = corrcoef([Y,Hm2,Hm1,H,Hp1,Hp2]);
fprintf(1,'Hours Worked           & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',absstd(4),rho(1,2:end));
rho = corrcoef([Y,mu01m2,mu01m1,mu01,mu01p1,mu01p2]);
fprintf(1,'Population Filing      & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',absstd(5),rho(1,2:end));
rho = corrcoef([Y,Dm2,Dm1,D,Dp1,Dp2]);
fprintf(1,'Debt                   & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',absstd(6),rho(1,2:end));
rho = corrcoef([Y,DDm2,DDm1,DD,DDp1,DDp2]);
fprintf(1,'Discharged Debt        & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',absstd(7),rho(1,2:end));
rho = corrcoef([Y,Rm2,Rm1,R,Rp1,Rp2]);
fprintf(1,'Interest Rates         & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',absstd(8),rho(1,2:end));

tmp = corrcoef([Y,mu01]);
stats.corrDefY = tmp(2);
stats.stdDef = absstd(5);
stats.stdY = absstd(1);
stats.meanDef = 100*mean(popDef);

                   
end

function [smlogx devlogx] = logHP(x)

    [smlogx]= hpfilter(log(x),100);
    devlogx = log(x) - smlogx;

end

function [smx devx] = HP(x)

    [smx]= hpfilter(x,100);
    devx = x - smx;

end

function [xm2,xm1,x,xp1,xp2] = leadlag(x,r)
    xm1 = x(r-1);
    xm2 = x(r-2);
    xp1 = x(r+1);
    xp2 = x(r+2);  
    x = x(r);    
end
