#include <iostream>
#include <iomanip>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>

#include "H5cpp.h"

#define CLIBGREYMAXRANK 6

//template <class T>
//__host__ save(std::string name, array<T> x){
//}

void STOP(std::string msg, std::string file, int line){
    std::cerr<<"ERROR: "<<file<<"("<<line<<"): "<<msg<<std::endl;
    exit(EXIT_FAILURE);
}

template <class T>
class array{
public:
    T *data;
    int n[CLIBGREYMAXRANK]; // dimensions
    int rank; // number of dimensions
    int size; // number of elements
    bool isAllocated; // whether or not allocated
    
    // constructor (this calls init but separate in case want to free or reallocate later)
    array(int n1, int n2, int n3, int n4, int n5, int n6){isAllocated=false;init(n1,n2,n3,n4,n5,n6);isAllocated=true;};
    array(int n1, int n2, int n3, int n4, int n5){isAllocated=false;init(n1,n2,n3,n4,n5);isAllocated=true;};
    array(int n1, int n2, int n3, int n4){isAllocated=false;init(n1,n2,n3,n4);isAllocated=true;};
    array(int n1, int n2, int n3){isAllocated=false;init(n1,n2,n3);isAllocated=true;};
    array(int n1, int n2){isAllocated=false;init(n1,n2);isAllocated=true;};
    array(int n1){isAllocated=false;init(n1);isAllocated=true;};
    array(){isAllocated=false; size=-1; rank=-1;}; // uninitialized

    // destructor 
    ~array(){if (isAllocated) free();};

    // initialize
    void init(int n1, int n2, int n3, int n4, int n5, int n6){
        if (!isAllocated) {
            for (int i=0; i<CLIBGREYMAXRANK; i++) {
                n[i] = 1;
            }
            rank = 6;
            n[0] = n1; n[1] = n2; n[2] = n3; n[3] = n4; n[4] = n5; n[5] = n6;
            size = n1*n2*n3*n4*n5*n6;
            data = new T [size];
            isAllocated = true;
        } else {
            std::cerr<<"Error: "<<__FILE__<<" line "<<__LINE__<<" array already initialized"<<std::endl;
//            std::exit(EXIT_FAILURE);
        }
    }
    void init(int n1, int n2, int n3, int n4, int n5){init(n1,n2,n3,n4,n5,1); rank = 5;}
    void init(int n1, int n2, int n3, int n4){init(n1,n2,n3,n4,1,1);rank = 4;}
    void init(int n1, int n2, int n3){init(n1,n2,n3,1,1,1);rank = 3;}
    void init(int n1, int n2){init(n1,n2,1,1,1,1);rank = 2;}
    void init(int n1){init(n1,1,1,1,1,1);rank = 1;}

    // free
    void free(){
        if (isAllocated) {delete[] (data);};
        isAllocated = false;
    }

    // overload brackets for easy access to data
    T& operator[](int i){return data[i];}; //setter

    // overload paraentheses for easy access to multidimensional arrays
    T& operator()(int i, int j){return data[i + j*n[0]];};
    T& operator()(int i, int j, int k){return data[i + j*n[0] + k*n[0]*n[1]];};

    // overloaded values
    array<T>& operator=(const array<T> &source); // this = source
    array<T>& operator=(const T x); // this = x
    array<T> operator-(const array<T> &rhs); // return = this - rhs
    array<T> operator+(const array<T> &rhs); // return = this + rhs

    // display some properties
    void stats(){
        std::cout<<"allocated: "<<isAllocated<<std::endl;
        std::cout<<"rank:      "<<rank<<std::endl;
        std::cout<<"shape:     ";
        for (int i=0; i<rank; i++) {std::cout<<n[i]<<" ";};
        std::cout<<std::endl;
        std::cout<<"size:      "<<size<<std::endl;
    };

    // fill the array with any argument of the same type
    array<T>& fill(T val){for (int i=0;i<size;i++) data[i] = val;}

    // reshape
    void reshape(int n1, int n2, int n3, int n4, int n5, int n6);
    void reshape(int n1, int n2, int n3, int n4, int n5){reshape(n1,n2,n3,n4,n5,1);rank=5;};
    void reshape(int n1, int n2, int n3, int n4){reshape(n1,n2,n3,n4,1,1);rank=4;};
    void reshape(int n1, int n2, int n3){reshape(n1,n2,n3,1,1,1);rank=3;};
    void reshape(int n1, int n2){reshape(n1,n2,1,1,1,1);rank=2;};
    void reshape(int n1){reshape(n1,1,1,1,1,1);rank=1;};

};
template <class T>
void array<T>::reshape(int n1, int n2, int n3, int n4, int n5, int n6){
    if (n1*n2*n3*n4*n5*n6 == size){
        n[0] = n1;
        n[1] = n2;
        n[2] = n3;
        n[3] = n4;
        n[4] = n5;
        n[5] = n6;
        rank = 6;
    } else {
        STOP("cannot reshape array, number of elements is not the same.",__FILE__,__LINE__);
    }
}

// function to check whether dims are the same
template <class T>
bool sameshape(const array<T> &a,const array<T> &b){
    if (!a.isAllocated || !b.isAllocated){
        return false; 
    } 
    if (a.rank!=b.rank) {
        return false;
    }
    for (int i = 0; i<a.rank; i++){
        if (a.n[i]!=b.n[i]) {return false;}
    }
    return true;
};

template <class T>
int init_from_source(array<T> &lhs,const array<T> &rhs){
    // now target is free and can reinit
    if (rhs.rank>6) STOP("init_from_source: Have not programmed this many dims yet",__FILE__,__LINE__);
    lhs.init(rhs.n[0],rhs.n[1],rhs.n[2],rhs.n[3],rhs.n[4],rhs.n[5]);
    lhs.rank = rhs.rank;
    return 0;
}


// assignment array operator (a = b is like a.=(b). this refers to a and is well-defined.)
template <class T>
array<T>& array<T>::operator=(const array<T> &source){

    if (sameshape(*this,source)){
        // in this case, no allocation or reallocation is necessary, just copy all elements over
    } else if (!isAllocated) {
        // now target is free and can reinit
        init_from_source(*this,source);
    } else {
        // here, they do not conform and the lhs is allocated. I do not allow this case.
        STOP("LHS is allocated but does have same size as RHS.",__FILE__,__LINE__);
    }

    // copy over the data
    for (int i = 0; i<source.size; i++){data[i] = source.data[i];}

    return *this;
}

template <class T>
void minus(array<T> &c, const array<T>&a, const array<T>&b){

    if (!c.isAllocated) init_from_source(c,a);           

    if (!sameshape(c,a) || !sameshape(a,b)) STOP("Minus: Dimensions do not conform. Please allocate LHS first.",__FILE__,__LINE__);

    for (int i = 0; i<a.size; i++){c.data[i] = a.data[i] - b.data[i];}
    
}

template <class T>
void plus(array<T> &c, const array<T>&a, const array<T>&b){

    if (!c.isAllocated) init_from_source(c,a);           

    if (!sameshape(c,a) || !sameshape(a,b)) STOP("Minus: Dimensions do not conform. Please allocate LHS first.",__FILE__,__LINE__);

    for (int i = 0; i<a.size; i++){c.data[i] = a.data[i] + b.data[i];}
    
}

template <class T>
void abs(array<T> &x){

    // perform the minus operation
    for (int i = 0; i<x.size; i++){x.data[i] = fabs(x.data[i]);};
    
}

// minus array operator, works as lhs = this - rhs
template <class T>
array<T> array<T>::operator-(const array<T> &rhs){
    array<T> lhs;
    init_from_source(lhs,rhs);
    minus(lhs,*this,rhs);
    return lhs;
}

// plus array operator, works as lhs = this + rhs
template <class T>
array<T> array<T>::operator+(const array<T> &rhs){
    array<T> lhs;
    init_from_source(lhs,rhs);
    plus(lhs,*this,rhs);
    return lhs;
}



template <class T>
__host__ __device__ T max(T x, T y){return ( x > y ? x : y );}

template <class T>
__host__ __device__ T min(T x, T y){return ( x < y ? x : y );}

template <class T>
T maxval(array<T> &x){
    int i;
    T sup;
    
    sup = fabs(x.data[0]);
    for (i=1;i<x.size;i++){
        sup = max<T>(sup,fabs(x.data[i]));
    }

    return sup;
}


template <class T>
__host__ __device__ T supnorm(T *x, T *y, int n){
    int i;
    T sup;
    
    sup = fabs(y[0]-x[0]);
    for (i=1;i<n;i++){
        sup = max<T>(sup,fabs(y[i]-x[i]));
    }

    return sup;
}

template <class T>
__host__ void disp(T* x,int n1=1, int n2=1){
    int i,j;
    for (i=0; i<n1; i++) {
        for (j=0; j<n2; j++) {
            // can use std::fixed to print zeros everywhere like matlab
            std::cout<<std::setw(15)<<std::setprecision(6)<<x[i+j*n1];
        }
        std::cout<<std::endl;
    }
}
template <class T> __host__ void disp(std::string s){std::cout<<s<<std::endl;}
template <class T> __host__ void disp(std::string str, T* x, int n1=1, int n2=1){std::cout<<str<<":"<<std::endl; disp(x,n1,n2);}
template <class T> __host__ void disp(std::string str, array<T> &x){std::cout<<str<<":"<<std::endl; disp(x);}
template <class T> __host__ void disp(array<T> &x){disp(x.data,x.n[0],x.n[1]);}

template <class T>
__host__ __device__ T normcdf(T x,T mean,T std){
    T one = 1, two = 2;
    return one/two*(one+erf((x-mean)/(std*sqrt(two))));
}

template <class T>
__host__ __device__ void linspace(T* x,T a,T b, int n){
    T m;
    int i;
    const T one = 1;

    x[0] = a;
    for (i=1; i<n-1; i++){
        m = T(i) / (n-1); // use a double, but case x into T
        x[i] = a*(one-m) + b*m;
    }
    x[n-1] = b;

    return;

}

template <class T>
__host__ __device__ void linspace(array<T> &x, T a, T b){ linspace(x.data,a,b,x.size); }

template <class T>
__host__ __device__ void tauchen(T *z, T *Fzz, T cover, T ar1, T cdnalstd, int n){

    T unstd,cdnalmean,mid,mid2;
    const T one=1,two=2,half=one/two;
    int i,j;

    // unconditional stdev
    unstd = cdnalstd/sqrt(one - ar1*ar1);

    // set grid
    linspace(z,-cover*unstd,cover*unstd,n);

    // construct transition matrix
    for (i=0; i<n; i++){
        // get conditional mean
        cdnalmean = ar1*z[i];
    
        // at left boundary, use weight F(m(0))
        j = 0;
        mid = half*z[j] + half*z[j+1];
        Fzz[i + j*n] = normcdf(mid,cdnalmean,cdnalstd);

        for (j=1; j<n-1; j++){
            // use weight F(m(j+1)) - F(m(j))
            mid = half*z[j-1] + half*z[j];
            mid2 = half*z[j] + half*z[j+1];
            Fzz[i + j*n] = normcdf(mid2,cdnalmean,cdnalstd)-normcdf(mid,cdnalmean,cdnalstd);
        }

        // at right boundary, use weight F(m(n-1))
        j = n-1;
        mid = half*z[j-1] + half*z[j];
        Fzz[i + j*n] = one - normcdf(mid,cdnalmean,cdnalstd);

    }

    // convert grid out of logs
    for (i=0; i<n; i++){
        z[i] = exp(z[i]);
    }

}

template <class T>
__host__ __device__ void tauchen(array<T> &z, array<T> &Fzz, T cover, T ar1, T cdnalstd){tauchen(z.data,Fzz.data,cover,ar1,cdnalstd,z.size);}


// bisection method
template <class T>
class optsettings {
public:
    T xtoler;
    int maxiter, pmod;

    // constructor
    optsettings(){xtoler = 1e-6f; maxiter= 10000; pmod = 1;};

    // destructor
    ~optsettings(){};

//    optsettings optimset(std::string s, void * x);
};

template <class T>
T bisect(T (*f)(T x),T a,T b, const optsettings<T> &o){
    T a_ = a, b_ = b, fa, fb;
    T half = (T) 1 / (T) 2;
    int iter = 1;
    fa = f(a);
    fb = f(b);
    return bisectcore(f,a_,b_,o,fa,fb,iter,half);
}
template <class T>
T bisectcore(T (*f)(T x),T& a,T& b, const optsettings<T> &o, T& fa, T& fb, int& iter, const T& half){
    
    // display info
    if (iter%o.pmod==0) std::cout<<"bisect: iter "<<iter<<" a "<<a<<" b "<<b<<" fa "<<fa<<" fb "<<fb<<std::endl;

    // check that the function is bracketed
    if ((fa>0 && fb > 0) || (fa<0 && fb<0)) STOP("Root is not bracketed",__FILE__,__LINE__);
    
    // check termination criterion
    if (fabs(a-b)<o.xtoler || iter>o.maxiter) return (fabs(fa)<fabs(fb) ? a : b );

    // construct midpoint and evalaute
    T m = half*a + half*b;
    T fm = f(m); 
    
    // call bisect again
    if (fm*fa>0){
        return bisectcore(f,m,b,o,fm,fb,++iter,half);
    } else {
        return bisectcore(f,a,m,o,fa,fm,++iter,half);
    }

}

double testfunctions_squaredmin2(double x){return x*x - 2.;};
int testbisect(){

    // compute the sqrt of 2 via f(x) = 0 where f x*x = 2 
    optsettings<double> o;
    o.pmod=5;
    double x = bisect(testfunctions_squaredmin2,0.,3.33333,o);
    std::cout<<"sqrt of 2 is "<<x<<std::endl;

}



//void bellman(double *V, double *c, int *api, double *W, double (*budget)(int iap, int ia, int ie), int nAp, int nA, int nE){

using namespace H5;
class greyDataSet{

    H5File* fileptr;
    H5std_string filename;
    std::string fname;
    bool fileIsOpened;

    const PredType getTypeName(array<double> &x){return PredType::NATIVE_DOUBLE;}
    const PredType getTypeName(array<float> &x){return PredType::NATIVE_FLOAT;}
    const PredType getTypeName(array<int> &x){return PredType::NATIVE_INT;}

    bool fileExists(){
        std::ifstream filecheck(fname.c_str(),std::ios::binary);
        if (filecheck.is_open()){
            filecheck.close();
            return true;
        } else { 
            return false;
        }
    }

public:

    greyDataSet(const std::string fname_){filename = fname_; fname = fname_; fileIsOpened=false;};
    ~greyDataSet(){};

    // open read write, deletes existing contents as soon as a file is saved
    void create(){
        fileptr = new H5File(filename, H5F_ACC_TRUNC); // read write access replacing, new data overwrites existing
        fileIsOpened=true;
    };

    // open read only
    void read(){
        fileptr = new H5File(filename, H5F_ACC_RDONLY); // read only access
        fileIsOpened=true;
    };

    void close(){
        delete fileptr;
        fileIsOpened=false;
    };

    // write to file
    template <class T>
    void save(std::string xname, array<T> &x){

        if (fileIsOpened){

            // copy x dimensions to hsize_t which HDF5 uses
            hsize_t xdims[CLIBGREYMAXRANK];
            for (int i = 0; i<CLIBGREYMAXRANK; i++){
                xdims[i] = x.n[i];
            }

            // create a variable for the dimensions
            DataSpace xspace(x.rank,xdims);

            // create the dataset and write to file 
            DataSet* xdataset = new DataSet(fileptr->createDataSet(xname,getTypeName(x),xspace));
            xdataset->write(x.data,getTypeName(x),xspace);
            delete xdataset;
        
        } else {
            
            std::cerr<<"greyDataSet.save: WARNING: could not save "<<xname<<" because no file is opened. use create or read first."<<std::endl;

        }

    };

    // read from file
    template <class T>
    void load(std::string xname, array<T> &x){
        
        if (fileIsOpened){

            // create the data set and write to file (I do not actually know what some of this means)
            DataSet xdataset = fileptr->openDataSet(xname);
            xdataset.read(x.data,getTypeName(x));
        
        } else {
        
            std::cerr<<"greyDataSet.load: WARNING: could not load "<<xname<<" because no file is opened. use create or read first."<<std::endl;

        }
    };


};

