! A stubs version mod_mympi
module mod_mympi
    implicit none
    
contains

integer function mympi_iproc()
    implicit none
    mympi_iproc = 0
end function 
integer function mympi_nproc()
    implicit none
    mympi_nproc = 1
end function 

subroutine mympi_barrier()
    implicit none

end subroutine 

! Aborts all mpi routines and also displays a message
subroutine mympi_stop(msg)
    implicit none
    character(len=*), intent(in) :: msg

    ! Display the error message
    print*,msg
    STOP 

end subroutine 

subroutine mympi_assign_range(a,b,n)
    implicit none
    integer, intent(in) :: n
    integer, intent(out) :: a,b
    
    a = 1
    b = n

end subroutine 
! Helper functions
integer function mympi_a(n)
    implicit none
    integer, intent(in) :: n
    ! local
    integer :: b
    call mympi_assign_range(mympi_a,b,n)
end function mympi_a
integer function mympi_b(n)
    implicit none
    integer, intent(in) :: n
    ! local
    integer :: a
    call mympi_assign_range(a,mympi_b,n)
end function mympi_b
! Gives the combined list
subroutine mympi_assign_range_list(a,b,n)
    implicit none
    integer, intent(in) :: n
    integer, intent(out) :: a(1),b(1)

    a(1) = 1
    b(1) = n
end subroutine 

! This gathers data from all tasks and delivers the combined data to all tasks.
! NOTE: it assumes the work was distributed across the trailing dimension according
!       to mympi_assign_range_list.
subroutine mympi_allgatherv_mat(A)
    use mod_matlab
    implicit none
    real(8), intent(inout) :: A(:,:)
end subroutine 
subroutine mympi_allgatherv_core(A,nrow,ncol)
    use mod_matlab
    implicit none
    integer, intent(in) :: nrow,ncol
    real(8), intent(inout) :: A(nrow,*)

end subroutine

subroutine mympi_init()
    implicit none
end subroutine
subroutine mympi_finalize()
    implicit none
end subroutine 

subroutine mympi_hello_world()
    implicit none

    ! Display
    write(*,*) 'Hello, running stubs mpi so I am process 0 of 1'

end subroutine 
    

end module mod_mympi

module mod_mympi_test
    use mod_matlab
    use mod_mympi
    implicit none
contains
    subroutine test_mympi
        implicit none
        integer, parameter :: n = 9
        integer :: a,b,i
        real(8) :: x(17,n)
        integer :: avec(100),bvec(100)
        
        call mympi_init()
        call mympi_hello_world()
        call mympi_assign_range(a,b,n)
        if (mympi_a(n)/=a) then
            call mympi_stop('ERROR: in mympi_a')
        elseif (mympi_b(n)/=b) then
            call mympi_stop('ERROR: in mympi_b')
        end if

        call mympi_barrier() ! prettier i/o
        print*,'Id ',mympi_iproc(),'has range',a,b,'#elts',b-a+1

        call mympi_barrier() ! prettier i/o


        do i = a,b
            x(:,i) = sqrt(linspace(1d0,real(size(x,1),8),size(x,1))) + mympi_iproc()
        end do

        call mympi_allgatherv_mat(x)

        avec = -1
        bvec = -1
        call mympi_assign_range_list(avec(1:mympi_nproc()),bvec(1:mympi_nproc()),n)
        
        if (mympi_iproc()==0) then
            print*,'avec',avec(1:mympi_nproc())
            print*,'bvec',bvec(1:mympi_nproc())
            print*,'list:'
            do i = 1,mympi_nproc()
                write(*,'(3(i0,A))') i,' ',avec(i),' ',bvec(i),' '
            end do

            call disp('x',x)
        end if

        call mympi_finalize()

    end subroutine

end module mod_mympi_test
