module mod_gini

implicit none

integer, parameter :: nLorenzPts = 10001

contains 

! Based on description in wikipedia

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! y_in is the value and fy_in is the associated 
! density function.  e.g., a grid on wealth and 
! the invariant distribution of wealth
function fn_gini(y_in,fy_in) result(G)
    use mod_matlab
    implicit none
    
    real(8), dimension(:), intent(in) :: y_in,fy_in
    real(8), dimension(1:size(y_in)):: y,fy
    integer, dimension(1:size(y_in)) :: srt_ind
    real(8), dimension(1:size(y_in)+1) :: S
    real(8), dimension(1:size(y_in)) :: Siminus1_Si
    real(8) :: G
    integer :: n, i
    
    n=size(y)
    y=y_in(1:n)
    fy=fy_in(1:n)
    
    do i=1,n
        srt_ind(i) = i
    end do
    
    call sort2(y,srt_ind,y)
    fy=fy(srt_ind)
    
    S(1)=0.d0
    call sub_cumsum(S(2:n+1),y*fy)
    
    Siminus1_Si=S(2:n+1)+S(1:n)
    
    G=1-dot_product(fy,Siminus1_Si)/S(n+1)

end function fn_gini

subroutine sub_gini(gini,Lorenz,Lorenz_grid,dsn,grid)
    use mod_myprint
    use mod_matlab, only: interp1, linspace, cumsum, issorted, unique
    implicit none
    real(8), intent(out) :: gini
    real(8), dimension(nLorenzPts), intent(out)  :: Lorenz,Lorenz_grid
    real(8), dimension(1:), intent(in) :: dsn,grid
    !local
    real(8), dimension(size(dsn)+2) :: pct_of_stuff, pct_of_pop
    integer, dimension(size(dsn)+2) :: undo
    real(8), allocatable :: pctpop(:), pctstuff(:)
    integer, allocatable :: redo(:)

    !Check inputs
    if (size(dsn)/=size(grid)) STOP 'sub_gini: dsn and grid size don''t agree'
    if (.not. issorted(grid,'ascend')) STOP 'sub_gini: grid is not sorted'
    
    Lorenz_grid = linspace(0.d0,1.d0,nLorenzPts)
    
    pct_of_stuff(1) = 0.d0
    pct_of_stuff(2:size(dsn)+1) = cumsum(grid*dsn)/dot_product(grid,dsn)
    pct_of_stuff(size(dsn)+2) = 1.d0
    
    pct_of_pop(1) = 0.d0
    pct_of_pop(2:size(dsn)+1) = cumsum(dsn)/sum(dsn)
    pct_of_pop(size(dsn)+2) = 1.d0

    call unique(pctpop,redo,undo,pct_of_pop)
    allocate(pctstuff(size(pctpop)))
    pctstuff = pct_of_stuff(redo)

    Lorenz = interp1(pctpop,pctstuff,Lorenz_grid)
    gini = sum(Lorenz_grid-Lorenz)/(sum(Lorenz_grid-Lorenz)+sum(Lorenz))  

end subroutine sub_gini


end module mod_gini
