! Author: Grey Gordon
! Year: 2011
!
! A Fortran implementation of the matlab Smolyak routines on my website.
! 
! All rights reserved. Please do not redistribute without permission.
!
! Soli Deo Gloria
!
! step1: get collocation points and compute many things
! step2: get coefficients of polynomial
! step3: evaluate polynomial at points x
! 
! Optionally, step3 can be broken into two parts
! step3A: get a work array of size that depends on x
! step3B: use the work array to evaluate the polynomial
module mod_smolyak

    private
    public :: smolyakFcn, smolyakapprox_step1, smolyakapprox_step2, smolyakapprox_step3, &
              smolyakapprox_step3A, smolyakApprox_step3B, test_smolyak, equals, &
              smolyakGetWorkRows, smolyakGetWorkCols


    real(8), parameter, private :: pi = 3.141592653589793238462643383279d0
    
    type smolyakFcn
        integer :: d, mu, q, leni, napprox, work_array_cols
        integer, allocatable :: ibar(:)
        integer, allocatable :: i(:,:), j(:,:), constVec(:), redo(:), undo(:)
!         integer, allocatable :: j_eq_2(:,:), j_eq_3(:,:), j_eq_4(:,:), j_eq_5(:,:), j_gt_5(:,:)
        integer, allocatable :: Lb(:), Ub(:) 
        real(8), allocatable :: a(:), b(:), Tovercjprod(:,:), cjprod(:), const(:)
        real(8), allocatable :: constTimesTheta(:,:) ! Polynomial coefficients
        integer :: laststep = -1
    end type smolyakFcn

    interface assignment (=)
        module procedure equals
    end interface

    interface smolyakapprox_step2
        module procedure smolyakapprox_step2_vec, smolyakapprox_step2_mat 
    end interface smolyakapprox_step2
    interface smolyakapprox_step3
        module procedure smolyakapprox_step3_vec, smolyakapprox_step3_mat, &
                         smolyakapprox_step3_vec_evalpt, smolyakapprox_step3_mat_evalpt
    end interface smolyakapprox_step3
    interface smolyakapprox_step3A
        module procedure smolyakapprox_step3A_vec, smolyakapprox_step3A_mat
    end interface smolyakapprox_step3A
!     interface smolyakapprox_step4
!         module procedure smolyakapprox_step4_core
!     end interface smolyakapprox_step4

contains

! Provides the size
function smolyakGetWorkRows(s,nx) result(nrows)
    implicit none
    type(smolyakFcn), intent(in) :: s
    integer, intent(in) :: nx
    integer :: nrows
    nrows = nx
end function 
function smolyakGetWorkCols(s,nx) result(ncols)
    implicit none
    type(smolyakFcn), intent(in) :: s
    integer, intent(in) :: nx
    integer :: ncols
    ncols = s%work_array_cols
end function 

subroutine equals(b,a)
    implicit none
    type(smolyakFcn), intent(out) :: b
    type(smolyakFcn), intent(in) :: a
    integer :: statmsg

    if (a%laststep<=0) then
        print*,'WARNING: COPYING AN UNINITIALIZED SMOLYAKFCN DOES NOTHING'
        return
    end if
    
    ! Copy over scalars
    b%laststep = a%laststep
    b%d = a%d
    b%mu = a%mu
    b%q = a%q
    b%leni = a%leni
    b%work_array_cols = a%work_array_cols

    ! Allocate space
    allocate(b%ibar(size(a%ibar)),STAT=statmsg)
    allocate(b%i(size(a%i,1),size(a%i,2)),STAT=statmsg)
    allocate(b%j(size(a%j,1),size(a%j,2)),STAT=statmsg)
!     allocate(b%j_eq_2(size(a%j_eq_2,1),size(a%j_eq_2,2)),STAT=statmsg)
!     allocate(b%j_eq_3(size(a%j_eq_3,1),size(a%j_eq_3,2)),STAT=statmsg)
!     allocate(b%j_eq_4(size(a%j_eq_4,1),size(a%j_eq_4,2)),STAT=statmsg)
!     allocate(b%j_eq_5(size(a%j_eq_5,1),size(a%j_eq_5,2)),STAT=statmsg)
!     allocate(b%j_gt_5(size(a%j_gt_5,1),size(a%j_gt_5,2)),STAT=statmsg)
    allocate(b%constVec(size(a%constVec)),STAT=statmsg)
    allocate(b%redo(size(a%redo)),STAT=statmsg)
    allocate(b%undo(size(a%undo)),STAT=statmsg)
    allocate(b%Lb(size(a%Lb)),STAT=statmsg)
    allocate(b%Ub(size(a%Ub)),STAT=statmsg)
    allocate(b%Tovercjprod(size(a%Tovercjprod,1),size(a%Tovercjprod,2)),STAT=statmsg)
    allocate(b%cjprod(size(a%cjprod)),STAT=statmsg)
    allocate(b%const(size(a%const)),STAT=statmsg)
    allocate(b%a(size(a%a)),STAT=statmsg)
    allocate(b%b(size(a%b)),STAT=statmsg)

    ! Copy over allocatable arrays that are always allocated
    b%ibar = a%ibar
    b%i = a%i
    b%j = a%j
!     b%j_eq_2 = a%j_eq_2
!     b%j_eq_3 = a%j_eq_3
!     b%j_eq_4 = a%j_eq_4
!     b%j_eq_5 = a%j_eq_5
!     b%j_gt_5 = a%j_gt_5
    b%constVec = a%constVec
    b%redo = a%redo
    b%undo = a%undo
    b%Lb = a%Lb
    b%Ub = a%Ub    
    b%Tovercjprod = a%Tovercjprod
    b%cjprod = a%cjprod
    b%const = a%const
    b%a = a%a
    b%b = a%b

    if (allocated(a%constTimesTheta)) then
        allocate(b%constTimesTheta(size(a%constTimesTheta,1),size(a%constTimesTheta,2)))
        b%constTimesTheta = a%constTimesTheta
    end if

!     if (allocated(a%Tprod)) then
!         allocate(b%Tprod(size(a%Tprod,1),size(a%Tprod,2)),STAT=statmsg)
!         b%Tprod = a%Tprod
!     end if

end subroutine

! subroutine uniqueRowsInt(y,i,j,x) 
!     implicit none
!     integer, intent(in) :: x(1:,1:)
!     integer, allocatable, intent(out) :: y(:,:)
!     integer, allocatable, intent(out) :: i(:) !Redo
!     integer, intent(out) :: j(size(x,1)) !Undo
!     !local
!     integer :: tmp(size(x,1),size(x,2))
!     integer :: tmpLoc(size(x,1))
!     integer :: ind, nUnique, ind2
!     
!     !Find which rows are unique (current method is brute force)
!     !In future, should iteratively sort by columns
!     nUnique = 1
!     tmp(1,:) = x(1,:)
!     tmpLoc(1) = 1
!     do ind = 2,size(x,1)        
!         if (all(sum(abs(tmp(1:nUnique,:)-spread(x(ind,:),1,nUnique)),2)/=0)) then
!             nUnique = nUnique + 1
!             tmp(nUnique,:) = x(ind,:)
!             tmpLoc(nUnique) = ind
!         end if
!     end do
!     allocate(y(nUnique,size(x,2)), i(nUnique))
! !    write(*,*) nUnique
!     y = tmp(1:nUnique,:)    
!     i = tmpLoc(1:nUnique)
! 
!     !Find indices that will undo this operation -- again this seems terribly inefficient 
!     j = -1
!     do ind = 1,size(x,1)        
!         do ind2 = 1,nUnique
!             if (sum(abs(y(ind2,:)-x(ind,:)))==0) then
!                 j(ind) = ind2
!             end if
!         end do
!     end do
! 
! end subroutine
! subroutine uniqueRowsDouble(y,i,j,x) 
!     implicit none
!     real(8), intent(in) :: x(1:,1:)
!     real(8), allocatable, intent(out) :: y(:,:)
!     integer, allocatable, intent(out) :: i(:) !Redo
!     integer, intent(out) :: j(size(x,1)) !Undo
!     !local
!     real(8) :: tmp(size(x,1),size(x,2))
!     integer :: tmpLoc(size(x,1))
!     integer :: ind, nUnique, ind2
!     
!     !Find which rows are unique (current method is brute force)
!     !In future, should iteratively sort by columns
!     nUnique = 1
!     tmp(1,:) = x(1,:)
!     tmpLoc(1) = 1
!     do ind = 2,size(x,1)
!         if (all(sum(abs(tmp(1:nUnique,:)-spread(x(ind,:),1,nUnique)),2)>1d-13)) then
!             nUnique = nUnique + 1
!             tmp(nUnique,:) = x(ind,:)
!             tmpLoc(nUnique) = ind
!         end if
!     end do
!     allocate(y(nUnique,size(x,2)), i(nUnique))
!     y = tmp(1:nUnique,:)    
!     i = tmpLoc(1:nUnique)
! 
!     !Find indices that will undo this operation -- again this seems terribly inefficient 
!     j = -1
!     do ind = 1,size(x,1)        
!         do ind2 = 1,nUnique
!             if (sum(abs(y(ind2,:)-x(ind,:)))<=1d-13) then
!                 j(ind) = ind2
!             end if
!         end do
!     end do
!    
! end subroutine

! This function is by Andreas Klimke, adapted to Fortran
elemental function spdimcc(n,d) result(sp)
    implicit none
    integer, intent(in) :: n,d
    integer :: sp
    ! % SPDIMCC   Compute number of sparse grid points, Clenshaw-Curtis grid
    ! %    SP = SPDIMCC(N,D) Computes the number of sparse grid points
    ! %      of the Clenshaw-Gurtis grid of dimension D and level N.
    ! %    (Internal function)

    ! % The function uses the formulas for N <= 7 given in A. Schreiber:
    ! % Die Methode von Smoliak bei der multivariaten Interpolation,
    ! % (2000), p.35/36. 
    !     
    !     % Author : Andreas Klimke, Universitaet Stuttgart
    !     % Version: 1.0
    !     % Date   : August 5, 2003

    !     % ------------------------------------------------------------
    !     % Sparse Grid Interpolation Toolbox
    !     % Copyright (c) 2006 W. Andreas Klimke, Universitaet Stuttgart 
    !     % Copyright (c) 2007-2008 W. A. Klimke. All Rights Reserved.
    !     % See LICENSE.txt for license. 
    !     % email: klimkeas@ians.uni-stuttgart.de
    !     % web  : http://www.ians.uni-stuttgart.de/spinterp
    !     % ------------------------------------------------------------

    select case (n)
        case (0)
            sp = 1
        case (1)
            sp = 2*d + 1
        case (2)
            sp = 2*d**2 + 2*d + 1
        case (3)
            sp = nint((4*d**3 + 6*d**2 + 14*d)/3d0) + 1
        case (4)
            sp = nint((2*d**4 + 4*d**3 + 22*d**2 + 20*d)/3d0) + 1
        case (5)
            sp = nint((4*d**5 + 10*d**4 + 100*d**3 + 170*d**2 + 196*d)/15d0) + 1
        case (6)
            sp = nint((4*d**6 + 12*d**5 + 190*d**4 + 480*d**3 + 1246*d**2 + 948*d)/45d0) + 1
        case (7)
            sp = nint((8*d**7 + 28*d**6 + 644*d**5 + 2170*d**4 + 9632*d**3 + 15442*d**2 + 12396*d)/315d0) + 1
        case default 
            sp = -1 ! should crash the program
    !         STOP 'dimensionality case if too high, have not setup the routine for this'
    !         sp = nint((8*d^7 + 28*d^6 + 644*d^5 + 2170*d^4 + 9632*d^3 + 15442*d^2 + 12396*d)/315) + 1
    !         do m = 8:n
    !             seq = spgetseq(m,d);
    !             do k = 1:size(seq,1)
    !                 temp = 1;
    !                 do l = 1:d
    !                     lval = seq(k,l);
    !                     if (lval == 0) then 
    !                     elseif (lval < 3) then
    !                         temp = temp * 2;
    !                     else
    !                         temp = temp * 2^double(lval-1);
    !                     endif 
    !                 end do
    !                 sp = sp + temp;
    !             end do
    !         end do
    end select

end function 

! smolyakEnumerate2
! Given d and some ibar, finds all multi-indices i>=1 s.t. |i|=ibar.
! Just a wrapper for spgetseq
subroutine smolyakEnumerate2(i,d,ibar)
    use mod_matlab, only: pure_choose
    implicit none
    integer, intent(in) :: d,ibar
    integer, intent(out) :: i(pure_choose(ibar-1,d-1),d)

    if (ibar<d) STOP 'ERROR: smolyakEnumerate2: no such combination exists'

    call spgetseq(i,ibar-d,d)
    i = i+1

end subroutine smolyakEnumerate2

! An adaptation of spgetseq by A. Klimke
subroutine spgetseq(seq,n,d)
    use mod_matlab, only: pure_choose, disp
    implicit none
    integer, intent(in) :: n,d
    integer :: seq(pure_choose(n+d-1,d-1),d)
    integer :: nlevels,sum,max,k,l,m,temp
    ! % SPGETSEQ  Compute the sets of indices 
    ! %    (internal function)
    ! 
    ! % Author : Andreas Klimke, Universitaet Stuttgart
    ! % Version: 1.1
    ! % Date   : November 18, 2007
    ! 
    ! % Change log:
    ! % V1.0   : August 5, 2003
    ! %          Initial release
    ! % V1.1   : January 24, 2006
    ! %          Changed algorithm to operate on a uint8 rather than
    ! %          a double array.
    ! % V1.2   : November 18, 2007
    ! %          Added new grid type : Gauss-Patterson
    ! 
    ! % ------------------------------------------------------------
    ! % Sparse Grid Interpolation Toolbox
    ! % Copyright (c) 2006 W. Andreas Klimke, Universitaet Stuttgart 
    ! % Copyright (c) 2007-2008 W. A. Klimke. All Rights Reserved.
    ! % See LICENSE.txt for license. 
    ! % email: klimkeas@ians.uni-stuttgart.de
    ! % web  : http://www.ians.uni-stuttgart.de/spinterp
    ! % ------------------------------------------------------------
    ! 	
    ! % Examples:
    ! % For dimension 2, level n = 3, the sequence is
    ! %     3     0
    ! %     2     1
    ! %     1     2
    ! %     0     3
    ! %
    ! % For dimension 3, level n = 3, the sequence is
    ! %     3     0     0
    ! %     2     1     0
    ! %     1     2     0
    ! %     0     3     0
    ! %     2     0     1
    ! %     1     1     1
    ! %     0     2     1
    ! %     1     0     2
    ! %     0     1     2
    ! %     0     0     3

    nlevels = pure_choose(n+d-1,d-1)

    seq = 0

    seq(1,1) = n 
    max = n 
    do k = 2,nlevels
        if (seq(k-1,1) > 0) then
            seq(k,1) = seq(k-1,1) - 1 
            do l = 2,d
                if (seq(k-1,l) < max) then
                    seq(k,l) = seq(k-1,l) + 1 
                    do m = l+1,d
                        seq(k,m) = seq(k-1,m) 
                    end do
                    exit
                end if
            end do
        else
            sum = 0
            do l = 2,d
                if (seq(k-1,l) < max) then
                    seq(k,l) = seq(k-1,l) + 1 
                    sum = sum + seq(k,l) 
                    do m = l+1,d
                        seq(k,m) = seq(k-1,m) 
                        sum = sum + seq(k,m) 
                    end do
                    exit
                else
                    temp = 0 
                    do m = l+2,d
                        temp = temp + seq(k-1,m) 
                    end do
                    max = n-temp 
                    seq(k,l) = 0 
                end if 
            end do
            seq(k,1) = n - sum 
            max = n - sum 
        end if
    end do

end subroutine spgetseq



!smolyakapprox_step1 smolyak grid construction 
!   [x s] = smolyakapprox_step1(d,mu,a,b) constructs the smolyak
!   grid points for a hypercube in r^d defined by bounds a,b with level of 
!   approximation mu.  it outputs two elementts, x and s.  x is
!   the collocation points that your function should be evaluated.
!   s is a structure containing information used by
!   smolyakapprox_step2 and smolyakapprox_step3.
!   copyright grey gordon
!   date: 1/18/11
!   last revised: 7/22/13
!function [x s] = smolyakapprox_step1(d,mu,a,b)
subroutine smolyakapprox_step1(x,s,d,mu,a,b)
    use mod_matlab
    implicit none
    integer, intent(in) :: d,mu
    real(8), intent(in) :: a(1:),b(1:)
    real(8), intent(out), allocatable :: x(:,:) !Note b/c of intent(out) this is deallocated upon entry
    type(smolyakFcn), intent(out) :: s
    !local
    integer, allocatable :: k(:,:)
    real(8), allocatable :: tmpMat(:,:),z(:,:)
    integer :: q, ibar, tmpi, tmpn, iind, dind, lind, jind
    real(8) :: tmpProd, tmpSum

    ! Check inputs 
    if (size(a)/=size(b) .or. size(a)/=d) then
        STOP 'ERROR: smolyakapprox_step1: a and b must have length d'
    end if
    if (any(a>=b)) then
        STOP 'ERROR: smolyakapprox_step1: a must be strictly less than b'
    end if
    if (mu<1) STOP 'ERROR: smolyakapprox_step1: mu should be >= 1'
    if (d<1) STOP 'ERROR: smolyakapprox_step1: d should be >= 1' 

    s%laststep = 1

    q = max(d,mu+1)

    s%q = q
    s%d = d
    s%mu = mu
    
    allocate(s%a(d),s%b(d))
    s%a = a
    s%b = b    

    ! Construct necessary coefficients using fx values.
    ! first, enumerate all necessary theta
    ! -for all i satisfying q<=ibar<=d+mu, 
    ! -need { l | l(1) is in 1...m(i1), l(2) is in 1...m(i1)}
    
    !enumerate all such unique i, then all such m(i), then all such l
    ! determine all i that fit the criterion q<=|i|<=d+mu
    !!! Determine the number of such i
    tmpi = 0
    do ibar = q,d+mu
        tmpi = tmpi + choose(ibar-1,d-1)
    end do

    !!! Allocate space and get all such i
    allocate(s%i(tmpi,d))
    tmpi = 0
    do ibar = q,d+mu 
        tmpn = choose(ibar-1,d-1)
        call smolyakEnumerate2(s%i(tmpi+1:tmpi+tmpn,:),d,ibar)
        tmpi = tmpi + tmpn
    end do
    
    s%leni = size(s%i,1)
    
    allocate(k(s%leni,d))
    allocate(s%ibar(s%leni),s%Lb(s%leni),s%Ub(s%leni))

    s%ibar = sum(s%i,2)
    k = smolyakM(s%i)

    ! Lb(i) and Ub(i) give the starting and ending range for 
    ! all the cartesian products of G(m(i1))x...xG(m(id))
    tmpi = 0
    do iind = 1,s%leni        
        s%Lb(iind) = tmpi + 1
        tmpi = tmpi + product(k(iind,:))
        s%Ub(iind) = tmpi
    end do    

    allocate(x(s%Ub(s%leni),d),z(s%Ub(s%leni),d),s%j(s%Ub(s%leni),d))
    allocate(s%undo(s%Ub(s%leni)))

    ! Compute the cartesian products of the points
    !!! First, get their indices
    do iind = 1,s%leni
        call cartProdInt(s%j(s%Lb(iind):s%Ub(iind),:),k(iind,:))
    end do

    !!! Second, get their numeric values
    allocate(tmpMat(maxval(k),maxval(k))) ! Not sure the bounds on this are correct
    do tmpi = 1,maxval(k)
        call smolyakG(tmpMat(1:tmpi,tmpi),tmpi)
    end do
    do iind = 1,s%leni
        do dind = 1,d
            do tmpi = s%Lb(iind),s%Ub(iind) 
                z(tmpi,dind) = tmpMat(s%j(tmpi,dind),k(iind,dind))
            end do
        end do
    end do

    !!! Third, get the unique values (b/c of their nested nature, only return unique values)
    call unique(x,s%redo,s%undo,z,'rows')

    !!! Last, scale to [a,b], this gives the Sparse grid points
    do dind = 1,d
        x(:,dind) = (x(:,dind)+1.d0)*(b(dind)-a(dind))/2.d0 + a(dind)
    end do


    !!! PRECOMPUTE SOME THINGS FOR SUBSEQUENT ROUTINES

    !To ease computation, we compute the values of t which are evaluated at roots in formula (25)
    allocate(s%Tovercjprod(s%Ub(s%leni),maxval(s%Ub-s%Lb)+1))
    allocate(s%cjprod(size(s%undo)))
    allocate(s%const(s%leni))

    ! For each l/j, there is a unique cjprod combination c_j1*c_j2*...*c_jd where c_1=c_kd=2 and c_i=1 o/w
    s%cjprod = -100.d0
    do iind = 1,s%leni
        do lind = s%Lb(iind),s%Ub(iind)
            s%cjprod(lind) = 1.d0
            do dind = 1,d
                ! Dimensions with only one dimensions are "dropped" according to Krueger and Kubler
                if (k(iind,dind)>1) then
                    ! The product is 2**(l==1 or l==k)
                    if ((s%j(lind,dind)==1) .or. (s%j(lind,dind)==k(iind,dind)))  then
                        s%cjprod(lind) = s%cjprod(lind)*2.d0
                    end if
                end if
            end do
        end do
    end do

    ! To construct the tensor product coefficients theta_l, we need T_l1(zeta_j1)*...T_ld(zeta_j2) for each l,j combination    
    ! That is why T is quite large. However, we don't actually need to save T, but only T / cjprod
    do iind = 1,s%leni
        do lind = s%Lb(iind),s%Ub(iind)
            do jind = s%Lb(iind),s%Ub(iind)
                tmpProd = 1.d0
                do dind = 1,d
                    if (k(iind,dind)>1) then
                        if (mod(s%j(lind,dind),2)==1) then
                            tmpProd = tmpProd*cos(pi*real((s%j(lind,dind)-1)*(s%j(jind,dind)-1),8)/real(k(iind,dind)-1,8))
                        else
                            tmpProd = -tmpProd*cos(pi*real((s%j(lind,dind)-1)*(s%j(jind,dind)-1),8)/real(k(iind,dind)-1,8))
                        end if
                    end if
                end do
                s%Tovercjprod(jind,lind-s%Lb(iind)+1) = tmpProd
            end do
        end do        
    end do
    do tmpi = 1,size(s%Tovercjprod,2)
        s%Tovercjprod(:,tmpi) = s%Tovercjprod(:,tmpi)/s%cjprod(:)
    end do

    ! For each i, there is a unique constant 2^dhat/((k_1-1)*...(k_dhat-1))/(c_l1*...*c_ldhat) 
    ! The first part, before "cjprod" (c_l1*...*c_ldhat), is "const".
    ! where dhat is the number of dimensions that has k=m(i)>1 and 1,...,dhat is a reordering of
    ! dimensions 1,...,d that drops dimensions with k=m(i)=1. 
    do iind = 1,s%leni
        tmpProd = 1d0 ! (k_1-1)*...*(k_dhat-1)
        tmpSum = 0d0 ! dhat
        do dind = 1,d
            if (k(iind,dind)>1) then
                tmpSum = tmpSum + 1d0
                tmpProd = tmpProd*real(k(iind,dind)-1,8)
            end if
        end do
        s%const(iind) = (2d0**tmpSum)/tmpProd
    end do

    ! Store the constant values associated with the Smolyak hierarchy
    ! These are (-1)^(d+mu-|i|)*choose(d-1,d+\mu-|i|)
    allocate(s%constVec(d+mu+1-q))
    do ibar = q,d+mu
        s%constVec(ibar-q+1) = (-1)**(d+mu-ibar)*choose(d-1,d+mu-ibar)
    end do

    ! Set sizes for work array
    s%work_array_cols = size(s%undo)

    ! Precompute, for each dind, the indices such that s.j(:,dind)==x or >x. This speeds computation
    ! in step3.
!     allocate(s%j_eq_2(count(s%j(:,1)==2),d))
!     allocate(s%j_eq_3(count(s%j(:,1)==3),d))
!     allocate(s%j_eq_4(count(s%j(:,1)==4),d)) ! may be zero sized
!     allocate(s%j_eq_5(count(s%j(:,1)==5),d)) ! may be zero sized
!     allocate(s%j_gt_5(count(s%j(:,1)>5),d)) ! may be zero sized
!     do dind = 1,d
!         s%j_eq_2(:,dind) = find(s%j(:,dind)==2)
!         s%j_eq_3(:,dind) = find(s%j(:,dind)==3)
!         if (size(s%j_eq_4,1)>0) s%j_eq_4(:,dind) = find(s%j(:,dind)==4)
!         if (size(s%j_eq_5,1)>0) s%j_eq_5(:,dind) = find(s%j(:,dind)==5)
!         if (size(s%j_gt_5,1)>0) s%j_gt_5(:,dind) = find(s%j(:,dind)>5)
!     end do

    ! What we need going forward is the terms
    ! phi(l,i) := (-1)^(d+mu-|i|)*choose(d-1,d+\mu-|i|)*2^dhat/((k_1-1)*...(k_dhat-1))/(c_l1*...*c_ldhat) for each i
    ! The smolyak formula can be written as 
    ! \sum_{q<=|i|<=d+mu} \sum_l\in L(i) phi(l,i) 

end subroutine smolyakapprox_step1


! This routine takes as given the function values from the collocation points and constructs a polynomial approximation.
! Several sets of function values at collocation points are passed (organized so that each column contains a set of 
! collocation point values) and so several polynomial approximation are constructed, one for each column.
subroutine smolyakapprox_step2_vec(fx,s,tF)
    implicit none
    real(8), dimension(1:), intent(in) :: fx
    type(smolyakFcn), intent(inout) :: s
    character(len=1), intent(in), optional :: tF

    if (present(tF)) print*,'WARNING: smolyakapprox_step2: when only approximating one function, tF does nothing'

    if (size(fx,1)/=size(s%redo)) STOP 'function values at collocation points must be along first dimension'
    s%napprox = 1

    call smolyakapprox_step2_core(fx,s,size(s%redo),'N') ! Since one dimensional , 

end subroutine smolyakapprox_step2_vec

subroutine smolyakapprox_step2_mat(fx,s,tF)
    implicit none
    real(8), dimension(1:,1:), intent(in) :: fx
    type(smolyakFcn), intent(inout) :: s
    character(len=1), intent(in), optional :: tF
    ! local
    character(len=1) :: tFselect

    tFselect = 'N'; if (present(tF)) tFselect = tF

    if (tFselect=='N') then
        if (size(fx,1)/=size(s%redo)) STOP 'function values at colloc points must be along first dimension'
        s%napprox = size(fx,2)
    else
        if (size(fx,2)/=size(s%redo)) STOP 'function values at colloc points must be along second dimension'
        s%napprox = size(fx,1)
    end if

    call smolyakapprox_step2_core(fx,s,size(s%redo),tFselect)

end subroutine smolyakapprox_step2_mat

! This is the core routine. Normally, t='N' and then the approximated functions are the columns of fx. 
! However, if t='T', then the approximated functions are the rows of fx.
subroutine smolyakapprox_step2_core(fx,s,ncolloc,tFselect)
    use mod_matlab
    implicit none
    integer, intent(in) :: ncolloc
    type(smolyakFcn), intent(inout) :: s
    real(8), dimension(*), intent(in), target :: fx 
    character(len=1), intent(in) :: tFselect
    ! local
    integer :: iind,lind,tmpi !,iapprox
    real(8) :: tmp
    real(8), pointer :: fxmat(:,:)
    real(8), allocatable :: funpackedtrans(:,:) ! ,funpacked(:,:)

    if (allocated(s%constTimesTheta)) deallocate(s%constTimesTheta)
    allocate(s%constTimesTheta(s%napprox,size(s%undo)))
!     allocate(funpacked(size(s%undo),s%napprox))
    allocate(funpackedtrans(s%napprox,size(s%undo)))

    if (s%laststep<=0) STOP 'smolyakapprox_step3: step1 should be called before step2' 
    s%laststep = 2

    if (tFselect=='N') then 

        ! For convenience, make a fxmat out of fx
        fxmat(1:ncolloc,1:s%napprox) => fx(1:ncolloc*s%napprox)

        ! Unpack the function values to the sparse grid that has redundant values
    !     funpacked = fxmat(s%undo,:)
    !     funpackedtrans = transpose(funpacked)
        do tmpi = 1,size(s%undo)
            funpackedtrans(:,tmpi) = fxmat(s%undo(tmpi),:)
        end do
!         funpackedtrans = fxmat(s%undo,:)
    
    else
        
        fxmat(1:s%napprox,1:ncolloc) => fx(1:ncolloc*s%napprox)
        funpackedtrans = fxmat(:,s%undo)

    end if

!     print*,'size funpacked trans',shape(funpackedtrans)

    !Construct the polynomial coefficients theta_l and multiply by the constant terms (-1)^(d+mu-|i|)*choose(d-1,d+\mu-|i|)
!     do iapprox = 1,s%napprox
!         do iind = 1,s%leni
!             do lind = s%Lb(iind),s%Ub(iind)
!                 s%constTimesTheta(iapprox,lind) = (s%constVec(s%ibar(iind)-s%q+1)*s%const(iind)/s%cjprod(lind))*   &
!                                        dot_product(funpacked(s%Lb(iind):s%Ub(iind),iapprox),&
!                                                    s%Tovercjprod(s%Lb(iind):s%Ub(iind),lind-s%Lb(iind)+1))
!                                                  ! s%T(s%Lb(iind):s%Ub(iind),lind-s%Lb(iind)+1)/s%cjprod(s%Lb(iind):s%Ub(iind))) 
!             end do
!         end do
!     end do
    do iind = 1,s%leni
        associate (a=>s%Lb(iind),b=>s%Ub(iind))
        do lind = a,b
            tmp = s%constVec(s%ibar(iind)-s%q+1)*s%const(iind)/s%cjprod(lind) ! just a scalar
!             s%constTimesTheta(:,lind) = tmp*matmul(s%Tovercjprod(a:b,lind-a+1),funpacked(a:b,:))
!             s%constTimesTheta(:,lind) = tmp*matmul(s%Tovercjprod(a:b,lind-a+1),transpose(funpackedtrans(:,a:b)))
!             s%constTimesTheta(:,lind) = tmp*matmul(s%Tovercjprod(a:b,lind-a+1),transpose(funpackedtrans(:,a:b)))
            call mydgemm(s%constTimesTheta(:,lind),s%Tovercjprod(a:b,lind-a+1),funpackedtrans(:,a:b),&
                         1,b-a+1,s%napprox,b-a+1,tB='T',alpha=tmp)
        end do
        end associate
    end do

end subroutine smolyakapprox_step2_core

! smolyakapprox_step3
! 
! Evaluates the approximations made in step2.  This particular routine applies if multiple approximations were made.
! If s was created with multiple functions, then it will be evaluated with them. 
! x should be #x1 by d  where #x1 is up to the user
! feval should be #x1 by napprox where napprox is the number of functions that were to be approximated
subroutine smolyakapprox_step3_vec_evalpt(feval,s,x) 
    implicit none
    type(smolyakFcn), intent(in) :: s
    real(8), dimension(1:), intent(in) :: x
    real(8), intent(out) :: feval
    ! local
    real(8) :: tmp(1)

    if (size(x)/=s%d) STOP 'smolyakapprox_step3: x has the wrong size. Since it is a vector, it should have length s%d'

    call smolyakapprox_step3_core(tmp,s,x,1,1)
    feval = tmp(1)

end subroutine smolyakapprox_step3_vec_evalpt

subroutine smolyakapprox_step3_vec(feval,s,x) 
    implicit none
    type(smolyakFcn), intent(in) :: s
    real(8), dimension(1:,1:), intent(in) :: x
    real(8), dimension(1:), intent(inout) :: feval

    if (size(feval)/=size(x,1)) STOP 'smolyakapprox_step3: feval dim 2 is incorrect.  Wrong # of x points being evaluated'
    if (size(x,2)/=s%d) STOP 'smolyakapprox_step3: x dim 2 is incorrect, it should be s%d'

    call smolyakapprox_step3_core(feval,s,x,size(x,1),1)

end subroutine smolyakapprox_step3_vec

subroutine smolyakapprox_step3_mat_evalpt(feval,s,x) 
    implicit none
    type(smolyakFcn), intent(in) :: s
    real(8), dimension(1:), intent(in) :: x
    real(8), dimension(1:), intent(inout) :: feval

    if (size(feval)/=s%napprox) STOP 'smolyakapprox_step3: size of feval is incorrect.  Inconsistent # of fcns being approximated'
    if (size(x)/=s%d) STOP 'smolyakapprox_step3: x size is incorrect, it should be s%d'

    call smolyakapprox_step3_core(feval,s,x,1,size(feval))

end subroutine smolyakapprox_step3_mat_evalpt

subroutine smolyakapprox_step3_mat(feval,s,x) 
    implicit none
    type(smolyakFcn), intent(in) :: s
    real(8), dimension(1:,1:), intent(in) :: x
    real(8), dimension(1:,1:), intent(inout) :: feval

    if (size(feval,1)/=size(x,1)) STOP 'smolyakapprox_step3: feval dim 2 is incorrect.  Wrong # of x points being evaluated'
    if (size(feval,2)/=s%napprox) STOP 'smolyakapprox_step3: feval dim 1 is incorrect.  Inconsistent # of fcns being approximated'
    if (size(x,2)/=s%d) STOP 'smolyakapprox_step3: x dim 2 is incorrect, it should be s%d'

    call smolyakapprox_step3_core(feval,s,x,size(x,1),size(feval,2))

end subroutine smolyakapprox_step3_mat


! step3A: just get Tprod
subroutine smolyakapprox_step3A_vec(work,s,x) 
    implicit none
    real(8), intent(in) :: x(1:)
    type(smolyakFcn), intent(in) :: s
    real(8), intent(out) :: work(1,s%work_array_cols)

    if (size(x)/=s%d) STOP 'smolyakapprox_step3A: x is wrongly size'

    call smolyakapprox_step3A_core(work,s,x,1)

end subroutine smolyakapprox_step3A_vec
subroutine smolyakapprox_step3A_mat(work,s,x) 
    implicit none
    real(8), intent(in) :: x(1:,1:)
    type(smolyakFcn), intent(in) :: s
    real(8), intent(out) :: work(size(x,1),s%work_array_cols)

    if (size(x,2)/=s%d) STOP 'smolyakapprox_step3A: x is wrongly size'

    call smolyakapprox_step3A_core(work,s,x,size(x,1))

end subroutine smolyakapprox_step3A_mat

! NOTE: this routine only access components of s that are defined in 
! step1. Consequently, it can be called before step2 is called.
subroutine smolyakapprox_step3A_core(Tprod,s,xin,nx) 
    implicit none
    integer, intent(in) :: nx
    real(8), dimension(*), intent(in), target :: xin
    type(smolyakFcn), intent(in) :: s
    real(8), intent(out) :: Tprod(nx,s%work_array_cols)
    !
    real(8), pointer :: x(:,:)
    real(8), allocatable :: twoz(:),T(:,:)
    integer :: dind, oind, iind, lind
    
    ! Create a matrix x out of the assumed size array xin
    x(1:nx,1:s%d) => xin(1:nx*s%d)

    allocate(twoz(nx))
    allocate(T(nx,2:smolyakM(s%mu+1))) ! Note indexing in 2nd dim begins at 2

    ! The smolyak approximation can be written as 
    ! \sum_{q\le |i| \le d+mu} (-1)^{d+mu-|i|}choose(d-1,d+\mu-|i|)p^i(x)
    ! p^i(x) itself is composed of a summation across l-values (in the notation of MKK)
    ! of theta_l*T_l1(x_1)*T_l2(x_2)*...*T_ld(x_d). So, get the product across T and save
    ! as Tprod for each l.
    ! So, the summation can be written as 
    ! \sum_{q\le |i| \le d+mu} [(-1)^{d+mu-|i|}choose(d-1,d+\mu-|i|)theta_l ]*
    !                          [T_l1(x_1)*T_l2(x_2)*...*T_ld(x_d)]
    ! The first bracketed object has already been computed and saved as constTimesTheta(iapprox,l)
    ! The second has not been computed since it depends on the (transformed) x values
    ! 
    Tprod = 1d0
    do dind = 1,s%d

        !Convert the x points to z via linear transformation
        twoz = 4d0/(s%b(dind)-s%a(dind))*(x(:,dind)-s%a(dind)) - 2d0 ! two*z 
        
        ! Get the values of the Chebyshev basis at 
!         T(:,1) = 1d0 ! the first index is never accessed
        T(:,2) = .5d0*twoz
        T(:,3) = twoz*T(:,2) - 1d0 ! don't access T(:,1)==1
        ! q<=ibar<=d+mu.  So ibar=d+mu is largest.  
        !Largest any order Cheby can be is the largest any component of m(i) can be.  
        !This occurs when i = 1 except for one component. 
        ! => icomp = d+mu-(d-1) = mu+1 => i = mu+1 => m(i) = m(mu+1);
        do oind = 4,smolyakM(s%mu+1) 
            T(:,oind) = twoz*T(:,oind-1) - T(:,oind-2)
        end do
        
!         do lind = 1,size(s%j,1)
!             if (s%j(lind,dind)>1) then
!                 Tprod(:,lind) = Tprod(:,lind)*T(:,s%j(lind,dind))
!             end if
!         end do

        do iind = 1,s%leni
            ! Test whether i>1. If not, then j = 1,...,M(i) has M(i)==1 so j<=1 implying T(:,j)==1
            if (s%i(iind,dind)>1) then
                do lind = s%Lb(iind),s%Ub(iind)
                    ! Again, test whether j>1 since can save possibly many multiplications if j==1
                    if (s%j(lind,dind)>1) then
                        Tprod(:,lind) = Tprod(:,lind)*T(:,s%j(lind,dind))
                    end if
                end do
            end if 
        end do

    end do

end subroutine smolyakapprox_step3A_core

! I've extended this to allow for a tranposed feval where, instead of having 
! nx # of rows and napprox # of columns it is switched. 
subroutine smolyakapprox_step3B(feval,s,work,tF) 
    implicit none
    type(smolyakFcn), intent(in) :: s
    real(8), dimension(:,:), intent(inout) :: feval ! so can handle either vector or matrix
    real(8), intent(in) :: work(1:,1:)
    character(len=1), intent(in), optional :: tF
    ! local
    character(len=1) :: tFselect

    tFselect = 'N'; if (present(tF)) tFselect = tF

    if (tFselect=='N') then
        if (size(feval,1)/=size(work,1)) STOP 'smolyakapprox_step3B: feval and work do not agree'
        if (size(feval,2)/=size(s%constTimesTheta,1)) STOP 'smolyakapprox_step3B: feval does not have rows equal to napprox'
    else
        if (size(feval,1)/=size(s%constTimesTheta,1)) STOP 'smolyakapprox_step3B: feval does not have rows equal to napprox'
        if (size(feval,2)/=size(work,1)) STOP 'smolyakapprox_step3B: feval and work do not agree'
    end if

    call smolyakapprox_step3B_core(feval,s,work,tFselect)

end subroutine smolyakapprox_step3B

subroutine smolyakapprox_step3B_core(feval,s,work,tFselect) 
    use mod_matlab
    implicit none
    type(smolyakFcn), intent(in) :: s
    real(8), dimension(*), intent(inout) :: feval ! so can handle either vector or matrix
    real(8), intent(in) :: work(1:,1:)
    character(len=1), intent(in) :: tFselect
    !local
    integer :: nx,napprox

    nx = size(work,1)
    napprox = size(s%constTimesTheta,1)

    if (size(work,2)/=s%work_array_cols) STOP 'smolyakapprox_step3B: work array is wrongly sized'

    !!! The rows of Tprod (work) represent different x points so nrows is size(x,1)
    !!! The columns of Tprod give T_l1(x_1)*T_l2(x_2)*....*T_ld(x_d) for a given l=(l1,l2,...,ld)
    !!! The columns of s%constTimesTheta give [(-1)^{d+mu-|i|}choose(d-1,d+\mu-|i|)theta_l ] for a given l
    !!! The rows of s%constTimesTheta represent different functions that have been approximated so # rows is s%napprox
    !!! Feval should have size(x,1) rows and s%napprox columns

!     ! SUBROUTINE DGEMM(TRANSA, TRANSB, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
!     ! C := alpha*op(A)*op(B) + beta*C,
!     ! NOTE: M is rows of C, N is columns of C, K is # columns of B / # rows of A
!     ! Want: feval_{nx,napprox} = Tprod_{nx,size(s%undo)}*(s%constTimesTheta_{napprox,size(s%undo)}))'
!     ! If feval is tranposed, then 
!     ! Want: feval_{napprox,nx} = s%constTimesTheta_{napprox,size(s%undo)}*Tprod'
    if (tFselect=='N') then 
!         call dgemm('N', 'T', nx, napprox, size(s%undo), 1d0, work, nx, s%constTimesTheta, napprox, 0d0, feval, nx)
        call mydgemm(feval,work,s%constTimesTheta,nx,size(s%undo),napprox,size(s%undo),tA='N',tB='T')
    else
        call mydgemm(feval,s%constTimesTheta,work,napprox,size(s%undo),nx,size(s%undo),tA='N',tB='T')
    end if

end subroutine smolyakapprox_step3B_core


subroutine smolyakapprox_step3_core(feval,s,xin,nx,napprox) 
    implicit none
    integer, intent(in) :: nx,napprox
    type(smolyakFcn), intent(in) :: s
    real(8), dimension(*), intent(in), target :: xin
    real(8), dimension(*), intent(inout) :: feval ! so can handle either vector or matrix
    !local
    real(8), allocatable :: Tprod(:,:) 

    if (smolyakM(s%mu+1)<3) then
        STOP 'smolyakapprox_step3: I was not expecting this case, change T(:,3)'
    end if

    ! Get Tprod
    allocate(Tprod(nx,s%work_array_cols))
    call smolyakapprox_step3A_core(Tprod,s,xin,nx) 

    ! Evaluate the approximation
    call smolyakapprox_step3B_core(feval,s,Tprod,'N') 

!     ! Create a matrix x out of the assumed size array xin
!     x(1:nx,1:s%d) => xin(1:nx*s%d)
!     !!! NOTE: no need to do the same thing with feval since only used by DGEMM (F77 routine)
! 
!     
!     allocate(z(nx))
!     allocate(T(nx,smolyakM(s%mu+1)))
! 
!     if (allocated(Tprod)) deallocate(Tprod)
!     allocate(Tprod(nx,size(s%undo)))
! 
!     ! The smolyak approximation can be written as 
!     ! \sum_{q\le |i| \le d+mu} (-1)^{d+mu-|i|}choose(d-1,d+\mu-|i|)p^i(x)
!     ! p^i(x) itself is composed of a summation across l-values (in the notation of MKK)
!     ! of theta_l*T_l1(x_1)*T_l2(x_2)*...*T_ld(x_d). So, get the product across T and save
!     ! as Tprod for each l.
!     ! So, the summation can be written as 
!     ! \sum_{q\le |i| \le d+mu} [(-1)^{d+mu-|i|}choose(d-1,d+\mu-|i|)theta_l ]*
!     !                          [T_l1(x_1)*T_l2(x_2)*...*T_ld(x_d)]
!     ! The first bracketed object has already been computed and saved as constTimesTheta(iapprox,l)
!     ! The second has not been computed since it depends on the (transformed) x values
!     ! 
!     Tprod = 1d0
!     do dind = 1,s%d
! 
!         !Convert the x points to z via linear transformation
!         z = 2d0/(s%b(dind)-s%a(dind))*(x(:,dind)-s%a(dind)) - 1d0
!         
!         ! Get the values of the Chebyshev basis at 
!         T(:,1) = 1d0
!         T(:,2) = z
!         T(:,3) = 2d0*z**2 - 1d0 ! instead of 2d0*z*z - 1 which has three multiplications, do only two
!         ! q<=ibar<=d+mu.  So ibar=d+mu is largest.  
!         !Largest any order Cheby can be is the largest any component of m(i) can be.  
!         !This occurs when i = 1 except for one component. 
!         ! => icomp = d+mu-(d-1) = mu+1 => i = mu+1 => m(i) = m(mu+1);
!         do oind = 4,smolyakM(s%mu+1) 
!             T(:,oind) = 2d0*z*T(:,oind-1) - T(:,oind-2)
!         end do
!         
! !         do lind = 1,size(s%j,1)
! !             if (s%j(lind,dind)>1) then
! !                 Tprod(:,lind) = Tprod(:,lind)*T(:,s%j(lind,dind))
! !             end if
! !         end do
! 
!         do iind = 1,s%leni
!             ! Test whether i>1. If not, then j = 1,...,M(i) has M(i)==1 so j<=1 implying T(:,j)==1
!             if (s%i(iind,dind)>1) then
!                 do lind = s%Lb(iind),s%Ub(iind)
!                     ! Again, test whether j>1 since can save possibly many multiplications if j==1
!                     if (s%j(lind,dind)>1) then
!                         Tprod(:,lind) = Tprod(:,lind)*T(:,s%j(lind,dind))
!                     end if
!                 end do
!             end if 
!         end do
! 
!     end do

    ! NOTE: Could easily make the above parallel by distributing the ix values

    ! Now, sum the values. For greater accuracy, can sum positive values into one accumulation and negative into another
    ! NOTE: I checked this and it turns out to be less precise

    ! This should be much faster
    !!! The rows of Tprod represent different x points so nrows is size(x,1)
    !!! The columns of Tprod give T_l1(x_1)*T_l2(x_2)*....*T_ld(x_d) for a given l=(l1,l2,...,ld)
    !!! The rows of s%constTimesTheta give [(-1)^{d+mu-|i|}choose(d-1,d+\mu-|i|)theta_l ] for a given l
    !!! The columns of s%constTimesTheta represent different functions that have been approximated so # rows is s%napprox
    !!! Feval should have size(x,1) rows and s%napprox columns

!     ! SUBROUTINE DGEMM(TRANSA, TRANSB, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC)
!     ! C := alpha*op(A)*op(B) + beta*C,
!     ! NOTE: M is rows of C, N is columns of C, K is # columns of B / # rows of A
!     ! Want: feval_{nx,napprox} = Tprod_{nx,size(s%undo)}*s%constTimesTheta_{size(s%undo),napprox})
!     call dgemm('N', 'T', nx, napprox, size(s%undo), 1d0, Tprod, nx, s%constTimesTheta, napprox, 0d0, feval, nx)

end subroutine smolyakapprox_step3_core


! Given an integer vector x>=1, returns all the different vectors
! y s.t. 1<=y<=x elt by elt. This gives the cartesian product of the 
! vectors (1:x(1)) cross (1:x(2)) cross ... (1:x(end)) 
! x will be treated as a row vector
! y is returned as a matrix with columns equal to the length of x and rows
! equal to (prod(x)) (which gives all the enumerations).
subroutine cartProdInt(y,x)
    use mod_matlab
    implicit none
    integer, intent(in) :: x(1:)
    integer, intent(out) :: y(1:product(x),1:size(x))
    ! local
    integer :: nr,nx,ix
    integer :: base(size(x)),xrev(size(x)),i(size(y,1)),modvals(size(y,1),size(y,2))

    ! The technique here is to convert the 1:prod(x) base 10 numbers into 
    ! a custom base system given by x. 
    ! 
    ! Given x = [x(1)...x(n)], 
    ! we want for any i in 1:prod(x) a unique d = [d(1)...d(n)] s.t.
    ! i = d(1)*(x(2)*x(3)...*x(n)) + d(2)*(x(3)*...*x(n)) + ... +
    !    d(n-1)*x(n) + d(n)
    ! e.g., say
    ! x = [3,1,2];
    ! we convert i=0 to d=[0,0,0], 
    !            i=1 to d=[0,0,1], 
    !            i=2 to d=[1,0,0],
    !            i=3 to d=[1,0,1], ...
    ! This will give all enumerates once we add 1 to each d.
    ! 
    ! We see mod(i,x(n)) = d(n), mod(i,x(n-1)*x(n)) = d(n-1)*x(n) + d(n),
    ! ... and so we can find all the d values recursively.


    nr = product(x)
    nx = size(x)

    do ix = 1,nx
        xrev(ix) = x(nx-ix+1)
    end do

    base = cumprod(xrev)        

    i = colon(0,nr-1)

    !!! NOTE: mod in MATLAB is the same thing as modulo in Fortran (NOT mod)

    ! Convert from base10 to new base system
    modvals(:,nx) = modulo(i,base(1))
    do ix = nx-1,1,-1
        modvals(:,ix) = modulo(i,base(nx-ix+1))
    end do

    y(:,nx) = modvals(:,nx)
    do ix = nx-1,1,-1
        y(:,ix) = (modvals(:,ix) - modvals(:,ix+1))/base(nx-ix)
    end do

    ! Convert to one-based indexing
    y = y + 1

end subroutine cartProdInt

! Given an integer n, construct the set of n extrema of the Chebyshev 
! polynomials zeta_j = -cos(pi*(j-1)/(n-1)), j = 1,n
subroutine smolyakG(set,n)
    implicit none
    integer, intent(in) :: n
    real(8), intent(out) :: set(n)
    !local
    integer :: ind
    
    if (n<1) then
        STOP 'smolyakG: n must be >=1'
    elseif (n==1) then
        set = 0d0
    else       
        set(1) = -1d0
        do ind = 2,n-1
            set(ind) = -cos(pi*(real(ind,8)-1.d0)/(real(n,8)-1.d0))
        end do
        set(n) = 1d0
        if (mod(ind,2)==1) then
            set((ind+1)/2) = 0d0
        end if 
    end if

end subroutine smolyakG

! Determines number of points in the set of smolyakG(set,i) I think
elemental function smolyakM(i) result(o)
    implicit none
    integer, intent(in) :: i
    integer :: o
    if (i==1) then
        o = 1
    elseif (i>1) then
        o = 2**(i-1) + 1
    elseif (i<-1) then
        o = -100000000
    end if
end function smolyakM

subroutine test_smolyak
    use mod_matlab, only: rand, disp, tic, toc
    implicit none
    integer, parameter :: d=200,mu=1
    integer, parameter :: nrand = 100
    real(8), allocatable :: x(:,:)
    type(smolyakFcn) :: s,s_mult
    real(8) :: a(d),b(d)
    real(8) :: frand(nrand), frand_mult(nrand,3), xrand(nrand,d)
    real(8), allocatable :: apa(:,:), work(:,:)
    
    a = 0.d0
    b = 1.d0

    write(*,*) 'Smolyak'    
    call smolyakapprox_step1(x,s,d,mu,a,b)

    print*, 'Copying over to s_mult'
    s_mult = s

!     write(*,*) 's%i'
!     call disp(s%i)
!     
!     write(*,*) 's%ibar'
!     call disp(s%ibar)
! 
!     write(*,*) 's%j'
!     call disp(s%j)
!         
!     write(*,*) 's%cjprod'
!     call disp(s%cjprod)
! 
!     write(*,*) 's%const'
!     call disp(s%const)
!     
!     write(*,*) 's%constVec'
!     call disp(s%constVec)
! 
!     write(*,*) 'x (collocation points)'
!     call disp(x)
! 
!     write(*,*) 's%redo'
!     call disp(s%redo)
! 
!     write(*,*) 's%undo'
!     call disp(s%undo)       

print*,'made past step1'

    ! Check the step2 routine
    call smolyakapprox_step2(f1(x),s)

    allocate(apa(size(x,1),2))
    apa(:,1) = f1(x)
    apa(:,2) = f2(x)
    call smolyakapprox_step2(apa,s_mult)

print*,'made past step2'

    ! Check the step3 routines
    xrand = rand(100,d)
    call smolyakapprox_step3(frand,s,xrand)
    write(*,*) 'These values should be zero to numerical precision'
    write(*,*) maxval(f1(xrand) - frand),minval(f1(xrand) - frand)

    call smolyakapprox_step3(frand_mult(:,1:2),s_mult,xrand)
    write(*,*) 'These values should be zero to numerical precision'
    write(*,*) maxval(f1(xrand) - frand_mult(:,1)),minval(f1(xrand) - frand_mult(:,1))
    write(*,*) maxval(f2(xrand) - frand_mult(:,2)),minval(f2(xrand) - frand_mult(:,2))

    ! Check the step3A and step3B routines
    !!! Change x without changing f
    xrand = rand(100,d)
    allocate(work(smolyakGetWorkRows(s,size(xrand,1)),smolyakGetWorkCols(s,size(xrand,1))))
    call smolyakapprox_step3A(work,s_mult,xrand)
    call smolyakapprox_step3B(frand_mult(:,1:2),s_mult,work)
    write(*,*) 'These values should be zero to numerical precision'
    write(*,*) maxval(f1(xrand) - frand_mult(:,1)),minval(f1(xrand) - frand_mult(:,1))
    write(*,*) maxval(f2(xrand) - frand_mult(:,2)),minval(f2(xrand) - frand_mult(:,2))

    !!! Change f without changing x
    apa(:,1) = f2(x)
    apa(:,2) = f1(x)
    call smolyakapprox_step2(apa,s_mult) 
    call smolyakapprox_step3B(frand_mult(:,1:2),s_mult,work)

    write(*,*) 'These values should be zero to numerical precision'
    write(*,*) maxval(f2(xrand) - frand_mult(:,1)),minval(f2(xrand) - frand_mult(:,1))
    write(*,*) maxval(f1(xrand) - frand_mult(:,2)),minval(f1(xrand) - frand_mult(:,2))

    ! Check the step4 routine
!     allocate(apa(size(x,1),3))
!     apa(:,1) = f2(x)
!     apa(:,2) = f3(x)
!     apa(:,3) = f1(x)
!     call smolyakapprox_step2(apa,s_mult)
!     call smolyakapprox_step4_core(frand_mult(:,1:3),s_mult)
! 
!     write(*,*) 'These values should be zero to numerical precision'
!     write(*,*) maxval(f2(xrand) - frand_mult(:,1)),minval(f2(xrand) - frand_mult(:,1))
!     write(*,*) maxval(f3(xrand) - frand_mult(:,2)),minval(f3(xrand) - frand_mult(:,2))
!     write(*,*) maxval(f1(xrand) - frand_mult(:,3)),minval(f1(xrand) - frand_mult(:,3))
! 



!     write(*,*) 'Speed testing step2 by running 10000 times'
!     call tic()
!     do ind = 1,10000
!         call smolyakapprox_step2(sum(x**2,dim=2),s)
!     end do
!     call toc()
! ! pause 'smolyak pause'
!     write(*,*) 'Speed testing step3 by running 10000 times'
!     call tic()
!     do ind = 1,10000
!         call smolyakapprox_step3(frand,s,xrand)
!         xrand = rand(100,d)
!     end do
!     call toc()
! 
!     STOP 'test_smolyak: here'

contains
    function f1(x) 
        real(8) :: x(1:,1:)
        real(8) :: f1(size(x,1))
        f1 = sum(x**2,dim=2)
    end function
    function f2(x) 
        real(8) :: x(1:,1:)
        real(8) :: f2(size(x,1))
        f2 = sum(12.d0*x**2-3.d0*x,dim=2)
    end function
    function f3(x) 
        real(8) :: x(1:,1:)
        real(8) :: f3(size(x,1))
        f3 = sum(-5d0*x**2-x+.2d0,dim=2)
    end function
    
    

end subroutine test_smolyak

end module mod_smolyak
