module mod_spline
    implicit none

    interface ppval
        module procedure :: ppval_scal,ppval_vec
    end interface
    interface ppval2
        module procedure :: ppval2_scal,ppval2_vec
    end interface
    interface pchip
        module procedure :: pchip_getpp,pchip_interp
    end interface
    interface schumaker_interp
        module procedure :: schumaker_interp_vec, schumaker_interp_scal
    end interface

    ! One-dimensional spline
    type spline
        integer :: n = -1 ! Number of knots 
        integer :: d = -1 ! d-1 is the order of the polynomial in each interval
        real(8), allocatable :: k(:) ! Knots 
        real(8), allocatable :: co(:,:) ! Coefficients 
        character(len=20) :: method ! Method of spline
    !!! I removed this final procedure on 4/5/13 since gfortran doesn't support it yet
!     contains
!         final :: freeSpline
    end type

    ! Two dimensional spline
    type spline2 
        integer :: n1 = -1, n2=-1 ! Number of knots 
        integer :: d = -1 ! d-1 is the order of polynomial in each interval
        real(8), allocatable :: k1(:),k2(:) ! Knots 
        real(8), allocatable :: co1(:,:,:) ! Coefficients for each 2nd dim knot
!         real(8), allocatable :: co2(:,:) ! Coefficients: these are not used currently
        character(len=20) :: method ! Method of spline (for now, assume same each dimension)
    end type

!     interface vardim_eval
!         module procedure :: vardim_eval_vec, vardim_eval_scal
!     end interface

    type, public :: vardimspline
        real(8), allocatable :: xdata(:)
        real(8), allocatable, private :: knots(:),taustar(:),f_taustar(:) ! basically work arrays
        real(8) :: lb,ub
        integer :: p,nx
    contains 
        procedure, pass :: free => vardim_free
        procedure, pass :: config => vardim_config
        procedure, pass :: neval => vardim_getnumberofevalpts
        procedure, pass :: getieval => vardim_getithevalpt
        procedure, pass :: setieval => vardim_setithevalpt
        procedure, pass :: plot => vardim_plot
        procedure, pass :: test => vardim_test
        procedure, pass :: eval => vardim_eval_vec
        procedure, pass :: eval1 => vardim_eval_scal
        procedure, pass :: disp => vardim_disp
    end type

contains
    
    subroutine vardim_free(this)
        implicit none
        class(vardimspline), intent(inout) :: this
        
        if (allocated(this%knots)) then
            deallocate(this%xdata)
            deallocate(this%knots)
            deallocate(this%taustar)
            deallocate(this%f_taustar)
            this%p = -1
            this%nx = -1
            this%lb = -1d0
            this%ub = -1d0
        end if

    end subroutine 

    subroutine vardim_config(this,order,xdata)
        implicit none
        class(vardimspline), intent(inout) :: this
        integer, intent(in) :: order
        real(8), intent(in) :: xdata(:)
        ! local
        integer :: p,nx,j

        if (allocated(this%xdata)) call this%free()

        this%p = order
        this%nx = size(xdata)

        p = this%p
        nx = this%nx

        allocate(this%xdata(nx))
        this%xdata = xdata

        this%lb = xdata(1)
        this%ub = xdata(nx)

        allocate(this%knots(2*p + nx))
        this%knots(1:p) = xdata(1)
        this%knots(p+1:p+nx) = xdata
        this%knots(p+nx+1:2*p+nx) = xdata(nx)

        allocate(this%taustar(nx+p-1))
        do j = 1,nx+p-1
            this%taustar(j) = sum(this%knots(j+1:j+p))/p
        end do

        allocate(this%f_taustar(nx+p-1))

    end subroutine 
    function vardim_getnumberofevalpts(this) result(n)
        implicit none
        class(vardimspline), intent(in) :: this
        integer :: n

        n = size(this%taustar)

    end function 
    function vardim_getithevalpt(this,i) result(x)
        implicit none
        class(vardimspline), intent(in) :: this
        integer, intent(in) :: i
        real(8) :: x

        x = this%taustar(i)

    end function
    subroutine vardim_setithevalpt(this,i,fx) 
        implicit none
        class(vardimspline), intent(inout) :: this
        integer, intent(in) :: i
        real(8), intent(in) :: fx

        this%f_taustar(i) = fx


    end subroutine
    function vardim_eval_vec(this,x) result(y) 
        implicit none
        class(vardimspline), intent(in) :: this
        real(8), intent(in) :: x(:)
        real(8) :: y(size(x))
        ! local
        integer :: ix
        ! local
!         real(8), allocatable :: work(:)
!         integer :: j
! 
!         allocate(work(size(x)))
! 
!         work = 0d0
! 
!         do j = 1,this%nx + this%p - 1
!             work = work + this%f_taustar(j)*bspline(x,j,this%p+1,this%knots)
!         end do
! 
!         y = work
        
        do ix = 1,size(x)
            y(ix) = vardim_eval_scal(this,x(ix))
        end do

    end function 
    function vardim_eval_scal(this,x) result(y) 
        implicit none
        class(vardimspline), intent(in) :: this
        real(8), intent(in) :: x
        real(8) :: y
        ! local
        real(8) :: B(this%p+1)
        integer :: ilo,ihi
!         real(8) :: work(1),xvec(1)
!         integer :: j

!         xvec(1) = x
 !        work(1) = 0d0

        call bsplinefast(B,ilo,ihi,x,this%p+1,this%knots) 
        y = sum(this%f_taustar(ilo:ihi)*B)

!         do j = 1,this%nx + this%p - 1
!             work = work + this%f_taustar(j)*bspline(xvec,j,this%p+1,this%knots)
! 
!         end do
! 
!         y = work(1)

    end function 
        
    subroutine vardim_disp(this)
        implicit none
        class(vardimspline), intent(in) :: this

        print*,'*vardim ************************************************************************'
        if (allocated(this%xdata)) then
            print*,'*vardim xdata',this%xdata
        else
            print*,'*vardim xdata not allocated'
        end if
        if (allocated(this%knots)) then
            print*,'*vardim knots',this%knots
        else
            print*,'*vardim knots not allocated'
        end if
        if (allocated(this%taustar)) then
            print*,'*vardim taustar',this%taustar
        else
            print*,'*vardim taustar not allocated'
        end if
        if (allocated(this%f_taustar)) then
            print*,'*vardim f_taustar',this%f_taustar
        else
            print*,'*vardim f_taustar not allocated'
        end if
        print*,'*vardim p',this%p
        print*,'*vardim nx',this%nx
        print*,'*vardim ************************************************************************'

    end subroutine

    subroutine vardim_plot(this,xopt) 
        use mod_matlab
        implicit none
        class(vardimspline), intent(in) :: this
        real(8), intent(in), optional :: xopt(:)
        ! local
        real(8), allocatable :: x(:),y(:)
        integer :: nx

        nx = 1000
        if (present(xopt)) nx = size(xopt)

        allocate(x(nx),y(nx))

        x = linspace(this%lb,this%ub,nx)
        if (present(xopt)) x = xopt

        y = this%eval(x)

!         call this%disp()
!         print*,'x',x
!         print*,'y',y

        call plot(x,y,'vardimspline')

    end subroutine

    subroutine vardim_test(this,fcn,xdata,order) 
        use mod_matlab, only: linspace,plot
        implicit none
        class(vardimspline), intent(inout) :: this
        real(8), intent(in) :: xdata(:)
        integer, intent(in) :: order
        interface 
            function fcn(x) 
                implicit none
                real(8), intent(in) :: x
                real(8) :: fcn
            end function
        end interface
        ! local
        integer :: i
        real(8) :: tmp
!         real(8), allocatable :: x(:),y(:)

        call this%config(order,xdata)

        do i = 1,this%neval()
            tmp = fcn(this%getieval(i)) ! evaluate the function at the ith requested point
            call this%setieval(i,tmp) ! set the ith point
        end do

        call this%disp()

        call this%plot()

!         allocate(x(1000),y(1000))
!         
!         x = linspace(xdata(1),xdata(size(xdata)),1000)
!         y = this%eval(x)
! 
!         call plot(x,y,'test vardimspline')
! 

    end subroutine

    ! For a given value of x and knots t, finds the i such that x is in 
    ! [t(i),t(i+1)) and produces the non-zero B-splines from i-k+1,...,i or order k
    ! at x. These are the only non-zero basis functions for this x.
    ! 
    ! The routine, for convenience, returns ilo:=i-k+1 and ihi:=i which
    ! implicitly give the non-zero B-splines in the vector B(ilo:ihi). B itself
    ! may be much larger. 
    !
    ! The B-splines are given recursively by 
    ! 
    !  B_{i,1}(x) =  1[ x in [x_i,x_{i+1})] 
    !  B_{i,k}(x) = \frac{x-x_i}{x_{i+k-1}-x_i} B_{i,k-1}(x) + \frac{x_{i+k} - x}{x_{i+k} - x_{i+1}} B_{i+1,k-1}(x).
    ! 
    ! To use the 
    subroutine bsplinefast(Bout,ilo,ihi,x,k,t) 
        use mod_matlab
        implicit none
        real(8), intent(in) :: x,t(:)
        integer, intent(in) :: k
        integer, intent(out) :: ilo,ihi
        real(8), intent(out), target :: Bout(k)
        ! local
        ! real(8), target :: work(k,2)
        real(8), pointer ::  Bnew(:)=>null(), B(:)=>null()
        integer :: i,j,p

        call bsearch(i,x,t)

        if (.not. (x>=t(i) .and. x<t(i+1))) then
            Bout = 0d0
            return
        end if

        if (k==1) then
            Bout = 1d0
            return
        end if

!         B(i-k+1:i) => work(:,2) 
!         Bnew(i-k+1:i) => work(:,1)

        do p = 2,k
            
            B(i-p+2:i) => Bout(k-(p-2):k)
            Bnew(i-p+1:i) => Bout(k-(p-1):k)

            if (p==2) then
                B(i) = 1d0
            end if

            ! Keep order
            do j = i-p+1,i 
!             do j = i,i-p+1,-1
                ! Compute B_{j,p} using B_{j,p-1} and B_{j+1,p-1}
                if (j==i) then
                    ! Combine B_{j,p-1}~=0 with the zero term B_{j+1,p-1}
                    Bnew(j) = (x-t(j))/(t(j+p-1)-t(j))*B(j)
                elseif (j==i-p+1) then
                    ! Combine B_{j+1,p-1}~=0 with the zero term B_{j,p-1}
                    Bnew(j) = (1-(x-t(j+1))/(t(j+1+p-1)-t(j+1)))*B(j+1)
                else
                    ! Combine B_{j,p-1}~=0 with B_{j+1,p-1}~=0
                    Bnew(j) = (x-t(j))/(t(j+p-1)-t(j))*B(j) + &
                              (1d0-(x-t(j+1))/(t(j+1+p-1)-t(j+1)))*B(j+1)
                end if
            end do

            ! Do an unnecessary (can be improved on using pointers) copy 
!             B = Bnew

        end do

!         Bout = B

        ilo = i-k+1
        ihi = i


    end subroutine 


    ! Evaluates the ith B-spline of a given order k at data x given knots t.
    ! The B-splines are given recursively by 
    ! 
    !  B_{i,1}(x) =  1[ x in [x_i,x_{i+1})] 
    !  B_{i,k}(x) = \frac{x-x_i}{x_{i+k-1}-x_i} B_{i,k-1}(x) + \frac{x_{i+k} - x}{x_{i+k} - x_{i+1}} B_{i+1,k-1}(x).
    ! 
    ! x should be sorted, as I plan to exploit this in the future.
    !
    recursive function bspline(x,i,k,t) result(B)
        implicit none
        real(8), intent(in) :: x(:),t(:)
        integer, intent(in) :: i,k
        real(8) :: B(size(x))
        ! local
        integer :: n,ix,nx

        if (k<=0) STOP 'ERROR: bspline: bad input value for k. Must be >= 1'

        n = size(t) 
        nx = size(x)
        
        if (i < 1 .or. i > n-k) then
            B = 0d0
            return
        end if

        if (k == 1) then

            do ix = 1,nx
                if (x(ix)>=t(i) .and. x(ix)<t(i+1)) then
                    B(ix) = 1d0
                else
                    B(ix) = 0d0
                end if
            end do

        else
            
            if (t(i+k-1)==t(i) .and. t(i+k)==t(i+1)) then
                B = 0d0
            elseif (t(i+k-1)==t(i)) then
                B = bspline(x,i+1,k-1,t)*(t(i+k)-x)/(t(i+k)-t(i+1))
            elseif (t(i+k)==t(i+1)) then
                B = bspline(x,i,k-1,t)*(x - t(i))/(t(i+k-1)-t(i)) 
            else
                B = bspline(x,i,k-1,t)*(x - t(i))/(t(i+k-1)-t(i)) + &
                    bspline(x,i+1,k-1,t)*(t(i+k)-x)/(t(i+k)-t(i+1))
            end if

        end if

    end function


    ! Constructs the coefficients for tensor product pchip routine.
    ! Note: this precomputes the coeficients (which are slopes) for only one dimension. 
    subroutine pchip2(pp,X1,X2,Y)
        implicit none
        real(8), dimension(1:), intent(in) :: X1,X2
        real(8), dimension(1:,1:), intent(in) :: Y
        type(spline2), intent(out) :: pp
        ! real(8), dimension(size(X1)-1,4,size(X2)), intent(out) :: coef
        ! local
        integer :: i
        type(spline) :: pp_tmp

        pp%n1 = size(X1)
        pp%n2 = size(X2)
        pp%method = 'pchip'
        pp%d = 4

        ! Check sizes
        if (size(Y,1)/=pp%n1) STOP 'size(Y,1) doesn''t agree with size(X1)'
        if (size(Y,2)/=pp%n2) STOP 'size(Y,2) doesn''t agree with size(X2)'

        ! Precompute the coefficients for 1 dimension
        allocate(pp%k1(pp%n1),pp%co1(pp%n1-1,4,pp%n2),pp%k2(pp%n2))
        pp%k1 = x1
        pp%k2 = x2
        do i = 1,pp%n2
            call pchip(pp_tmp,X1,Y(:,i))
            pp%co1(:,:,i)= pp_tmp%co(:,:)
        end do

    end subroutine pchip2 

    ! Evaluates a 2-dimensional piecewise polynomial
    subroutine ppval2_scal(y,x1,x2,pp)
        use mod_interp, only: search
        implicit none
        type(spline2), intent(in) :: pp 
        real(8), intent(in) :: x1,x2
        real(8), intent(out) :: y
        !local
        real(8) :: del
        real(8), allocatable :: yint1(:)
        integer :: i2,ilo1,id,d !,ilo2
        type(spline) :: pp_tmp

!         if (checkSpline(pp)/=0) STOP 'ppval: ERROR: input spline has an error.'     
        
        d = pp%d

        allocate(yint1(pp%n2))
            
        ! Find the location of x1 and x2 in their grids
        call search(ilo1,x1,pp%k1)
!         call search(ilo2,x2,pp%k2)

        ! A spline has been computed for y(.,x2) for each x2 on the grid.
        ! So evaluate the spline at (xi1,x2) for each xi1, x2, to get 
        ! yhat(xi1,x2) for each xi1 and x2 on the grid.

        ! Evaluate the polynomial using Horner's method
        del = x1 - pp%k1(ilo1)
        do i2 = 1,pp%n2
            yint1(i2) = pp%co1(ilo1,d-1,i2) + pp%co1(ilo1,d,i2)*del
            do id = d-2,1,-1
                yint1(i2) = pp%co1(ilo1,id,i2) + yint1(i2)*del
            end do
        end do

        ! Now have yhat(xi1,x2). Construct one last spline for (.,yhat(xi1,.)) and 
        ! evaluate it. 
        if (pp%method=='pchip') then
            ! pchip is mostly a local method, so this could be made more efficient
            ! Would be better to construct a "pchipint" that would be of use also
            ! in the one dimensional case, constructing just the local info.
            call pchip(pp_tmp,pp%k2,yint1)
            call ppval(y,x2,pp_tmp)
        else
            STOP 'ppval2: ERROR: currently, only pchip is an option'
        end if

    end subroutine ppval2_scal

    ! Evaluates a 2-dimensional piecewise polynomial
    ! This is a "vectorized" version where x1 and x2 are vectors, the pp
    ! is evaluted at the tensor product of x1 and x2. There is computational savings
    ! in the tensor approach because, for #x1=#x2=n, there are only n spline 
    ! constructions rather than n**2 evaluations.
    subroutine ppval2_vec(y,x1,x2,pp)
        use mod_interp, only: search
        implicit none
        type(spline2), intent(in) :: pp 
        real(8), dimension(1:), intent(in) :: x1,x2
        real(8), dimension(1:size(x1),1:size(x2)), intent(out) :: y
        !local
        real(8) :: del
        real(8), allocatable :: yint1(:)
        integer :: i2,ilo1,id,d, nx1,nx2, i1 
        type(spline) :: pp_tmp

!         if (checkSpline(pp)/=0) STOP 'ppval: ERROR: input spline has an error.'     
        
        nx1 = size(x1)
        nx2 = size(x2)

        d = pp%d

        allocate(yint1(pp%n2))
            
        ! Loop over the x1 values
        do i1 = 1,nx1

            ! Find the location of x1 in the 1st dim grid
            call search(ilo1,x1(i1),pp%k1)

            ! A spline has been computed for y(.,x2) for each x2 on the grid.
            ! So evaluate the spline at (xi1,x2) for each xi1, x2, to get 
            ! yhat(xi1,x2) for each xi1 and x2 on the grid.

            ! Evaluate the polynomial using Horner's method
            del = x1(i1) - pp%k1(ilo1)
            do i2 = 1,pp%n2
                yint1(i2) = pp%co1(ilo1,d-1,i2) + pp%co1(ilo1,d,i2)*del
                do id = d-2,1,-1
                    yint1(i2) = pp%co1(ilo1,id,i2) + yint1(i2)*del
                end do
            end do

            ! Now have yhat(xi1,x2). Construct one last spline for (.,yhat(xi1,.)) and 
            ! evaluate it. 
            if (pp%method=='pchip') then
                ! pchip is mostly a local method, so this could be made more efficient
                ! Would be better to construct a "pchipint" that would be of use also
                ! in the one dimensional case, constructing just the local info.
                call pchip(pp_tmp,pp%k2,yint1)
                call ppval_vec(y(i1,:),x2,pp_tmp) ! This is the vectorized version
            else
                STOP 'ppval2: ERROR: currently, only pchip is an option'
            end if

        end do

    end subroutine ppval2_vec


    ! freeSpline
    ! Free the spline memory
    subroutine freeSpline(pp) 
        implicit none
        type(spline) :: pp ! this deallocates the information
        if (allocated(pp%k)) deallocate(pp%k)
        if (allocated(pp%co)) deallocate(pp%co)
        pp%n = -1 ! this initializes to the default state 
        pp%d = -1 ! this initializes to the default state  
        pp%method = '' ! clears the method
    end subroutine

    ! checkSpline
    ! 
    !  0 == allocated and dimensions agree
    ! -1 == not allocated
    ! -2 == allocated but dimensions don't agree
    function checkSpline(pp) result(flag)
        implicit none
        type(spline), intent(in) :: pp
        integer :: flag

        if (allocated(pp%k) .and. allocated(pp%co)) then
            if (pp%n==size(pp%k) .and. pp%n-1==size(pp%co,1) &
                .and. pp%d==size(pp%co,2) .and. pp%d>0 .and. pp%n>1) then
                flag = 0
                return
            else
                flag = -2
                return
            end if
        else
            flag = -1
            return
        end if

    end function

    ! Given a piecewise polynomial pp, evaluates to recover values y.
    ! y(x) = c(i,1) + c(i,2)(x-xi) + c(i,3)(x-xi)**2 + ... if x \in [ki,ki+1]
    subroutine ppval_vec(y,x,pp) 
        use mod_interp, only: search
        implicit none
        type(spline), intent(in) :: pp 
        real(8), intent(in) :: x(1:)
        real(8), intent(out) :: y(1:size(x))
        !local
        real(8) :: del
        integer :: ilo,ix,id,nx,d

        if (checkSpline(pp)/=0) STOP 'ppval: ERROR: input spline has an error.'     
        
        d = pp%d
        nx = size(x)
        
        ! Loop over each value of x
        do ix = 1,nx
            
            ! Find the location of x in grid k
            call search(ilo,x(ix),pp%k)
            del = x(ix)-pp%k(ilo)

            ! Evaluate the polynomial using Horner's method
            y(ix) = pp%co(ilo,d-1) + pp%co(ilo,d)*del
            do id = d-2,1,-1
                y(ix) = pp%co(ilo,id) + y(ix)*del
            end do

        end do

    end subroutine ppval_vec

    ! Given input x,k,c where k is knots, c is coefficients, and x is data,
    ! evaluate the spline defined by k,co at x to recover values y.
    ! y(x) = c(i,1) + c(i,2)(x-xi) + c(i,3)(x-xi)**2 + ... if x \in [ki,ki+1]
    subroutine ppval_scal(y,x,pp,ilo_in,ilo_out) 
        use mod_interp, only: search
        implicit none
        type(spline), intent(in) :: pp 
        real(8), intent(in) :: x
        real(8), intent(out) :: y
        integer, intent(in), optional :: ilo_in
        integer, intent(out), optional :: ilo_out
        !local
        real(8) :: del
        integer :: ilo,id,d

        if (checkSpline(pp)/=0) STOP 'ppval: ERROR: input spline has an error.'     
        
        d = pp%d
            
        ! Find the location of x in grid k
        if (present(ilo_in)) then
            ilo = ilo_in
            ! Verify that x is in ilo,ilo+1
            if (x<pp%k(ilo) .or. x>pp%k(ilo+1)) then
                STOP 'ppval: ERROR: x is not in x(ilo,ilo+1)'
            end if
        else
            call search(ilo,x,pp%k)
        end if
        if (present(ilo_out)) then
            ilo_out = ilo
        end if

        ! Evaluate the polynomial using Horner's method
        del = x - pp%k(ilo)
        y = pp%co(ilo,d-1) + pp%co(ilo,d)*del
        do id = d-2,1,-1
            y = pp%co(ilo,id) + y*del
        end do

    end subroutine ppval_scal

    ! Given input x,k,c where k is knots, c is coefficients, and x is data,
    ! evaluate the spline defined by k,co at x to recover values y.
    ! y(x) = c(i,1) + c(i,2)(x-xi) + c(i,3)(x-xi)**2 + ... if x \in [ki,ki+1]
    subroutine ppderiv(yp,ypp,x,pp) 
        use mod_interp, only: search
        implicit none
        real(8), intent(in) :: x(1:)
        type(spline), intent(in) :: pp
        real(8), intent(out) :: yp(1:size(x)),ypp(1:size(x))
        !local
        real(8) :: del
        integer :: ilo,ix,id,nx,d

        if (checkSpline(pp)/=0) STOP 'ppval: ERROR: input spline has an error.'     
        
        d = pp%d
        nx = size(x)
        
        ! Loop over each value of x
        do ix = 1,nx
            
            ! Find the location of x in grid k
            call search(ilo,x(ix),pp%k)
            del = x(ix)-pp%k(ilo)

            ! Evaluate the first derivative using Horner's method
            yp(ix) = real(d-1,8)*pp%co(ilo,d)
            do id = d-1,2,-1
                yp(ix) = real(id-1,8)*pp%co(ilo,id) + yp(ix)*del
            end do

            ! Evaluate the second derivative using Horner's method
            ypp(ix) = real(d-2,8)*real(d-1,8)*pp%co(ilo,d) ! This evaluates to zero if d is 1 or 2
            do id = d-1,3,-1
                ypp(ix) = real(id-2,8)*real(id-1,8)*pp%co(ilo,id) + ypp(ix)*del
            end do

        end do

    end subroutine ppderiv

    function schumaker_interp_vec(x,y,xp) result(yp)
        implicit none
        real(8), dimension(:), intent(in) :: x,y,xp
        real(8), dimension(size(xp)) :: yp
        type(spline) :: pp

        call schumaker(pp,x,y)
        call ppval(yp,xp,pp)

    end function
    function schumaker_interp_scal(x,y,xp) result(yp)
        implicit none
        real(8), dimension(:), intent(in) :: x,y
        real(8) :: xp
        real(8) :: yp
        type(spline) :: pp

        call schumaker(pp,x,y)
        call ppval(yp,xp,pp)

    end function

    ! Produce a Schumaker shape-preserving spline 
    ! Input data is either Lagrange data x,y or Hermite data x,y,s where s is slope
    subroutine schumaker(pp,x,y,s)
        use mod_core
        implicit none
        real(8), dimension(1:), intent(in) :: x,y
        real(8), dimension(1:), intent(in), optional, target :: s
        type(spline), intent(out) :: pp
        !local
        real(8), dimension(:), allocatable :: yp,dx,dy,m,L,Lm,dyp
        logical, dimension(:), allocatable :: specialCase
        real(8) :: xsi,alpha,beta,ypbar,yplo_minus_m,yphi_minus_m,theta
        real(8), parameter :: eps = epsilon(0d0), sqrteps = sqrt(eps)
        integer :: i,n,j,nk

        pp%method = 'schumaker'
        pp%d = 3
        
        n = size(x)

        ! Check inputs
        if (size(y)/=n) STOP 'error: y and x do not have the same size'
!         if (any(isNaN(y))) then
!             print*,'y',y
!             STOP 'y is NaN'
!         end if

        ! Allocate space for local variables
        allocate(yp(n),dx(n-1),dy(n-1),m(n-1),L(n-1),Lm(n-1),dyp(n-1),specialCase(n-1))

        dx = x(2:n)-x(1:n-1)
        dy = y(2:n)-y(1:n-1)
        m = dy/dx

        ! Construct or copy the spline 
        if (present(s)) then
            ! Copy by assigning pointer
            if (size(s)/=n) STOP 'error: s and x do not have the same size'
            yp = s
        else

            ! Construct the slopes
            ! L = sqrt(dx**2 + dy**2)
            do i = 1,n-1
                L(i) = norm((/dx(i),dy(i)/))
            end do

            Lm = L*m
            do i = 2,n-1
                if ((m(i-1)>0d0 .and. m(i)>0d0) .or. (m(i-1)<0d0 .and. m(i)<0d0)) then
                    yp(i) = (Lm(i-1)+Lm(i))/(L(i-1)+L(i))
                else
                    yp(i) = 0d0
                end if
            !    print*,(Lm(i-1)+Lm(i))/(L(i-1)+L(i))
            end do
            yp(1) = (3d0*m(1) - yp(2))/2d0
            yp(n) = (3d0*m(n-1) - yp(n-1))/2d0

        end if

        dyp = yp(2:n)-yp(1:n-1)
        specialCase = abs((yp(1:n-1)+yp(2:n))/2d0 - m)<4d0*eps

        ! Get the number of knots - 1
        nk = 2*(n-1) - count(specialCase)

        pp%n = nk + 1
        
        ! Allocate space for output arguments
        allocate(pp%k(nk+1),pp%co(nk,3))

        ! Loop over intervals and compute coeffs for each
        j = 1
        do i = 1,n-1
        
            ! Check whether the special case where a quadratic polynomial works is satisfied
!             if (specialCase(i)) then
            if (specialCase(i)) then
                ! If so, store the coeffs and knots
                pp%co(j,1) = y(i)
                pp%co(j,2) = yp(i)
                pp%co(j,3) = dyp(i)/dx(i)/2d0
                pp%k(j) = x(i)
                j = j+1

            else

                yplo_minus_m = yp(i)-m(i)
                yphi_minus_m = yp(i+1)-m(i)

                ! Note yplo_minus_m = yphi_minus_

                ! Test whether we can ensure xsi will be in (x(i),x(i+1))
                ! For this, ened |y'_j-m| \in (0, (|y'_{i+1}-y'_i|)^-1) for j = i,i+1
                

                ! Otherwise in the typical case where a quad poly doesn't work, insert a knot
                ! Three cases which will give the knot
                if (abs(yp(i+1)-yp(i))<epsilon(0d0) .or. &
                    (yplo_minus_m<epsilon(0d0) .and. yphi_minus_m<epsilon(0d0)) .or. &
                    (yplo_minus_m>-epsilon(0d0) .and. yphi_minus_m>-epsilon(0d0))) then
!                 if ((yplo_minus_m<1d2*sqrteps .and. yphi_minus_m<=1d2*sqrteps) .or. (yplo_minus_m>=-1d2*sqrteps .and. yphi_minus_m>=-1d2*sqrteps)) then
                    ! Cannot shape preserve
                    xsi = (x(i)+x(i+1))/2d0
                elseif (abs(yplo_minus_m)>abs(yphi_minus_m)) then
!                     xsibar = x(i) + 2d0*dx(i)*yphi_minus_m(i)/dyp(i) 
!                     xsi = (x(i) + xsibar)/2d0
!                     xsi = x(i) + dx(i)*yphi_minus_m/dyp(i) 

                    ! theta*dx will be the step size. we need to ensure this is not too close to 0 or 1
                    theta = yphi_minus_m/dyp(i)  ! < 0
                    theta = min(max(theta,sqrteps),1d0-sqrteps)
                    xsi = x(i) + dx(i)*theta
                else 
!                     xsibar = x(i+1) + 2d0*dx(i)*yplo_minus_m(i)/dyp(i)
!                     xsi = (x(i+1) + xsibar)/2d0
!                     xsi = x(i+1) + dx(i)*yplo_minus_m/dyp(i)
                    theta = yphi_minus_m/dyp(i)  ! < 0
                    theta = min(max(theta,sqrteps),1d0-sqrteps) 
                    theta = theta-1d0
                    xsi = x(i+1) + dx(i)*theta
                end if

if (xsi<=x(i) .or. xsi>=x(i+1)) then
    print*,'spec case criterion',abs( (yp(i)+yp(i+1))/2d0 - m(i) )
    print*,'xsi',xsi
    print*,'x(i:i+1)',x(i:i+1)
    print*,'dyp(i)',dyp(i)
    print*,'dx(i)',dx(i)
    print*,'yphi_minus_m',yphi_minus_m
    print*,'yplo_minus_m',yplo_minus_m
    STOP 'xsi is not between x(i) and x(i+1)'
end if

                ! With the knot, there is a unique quadratic polynomial
                alpha = xsi - x(i)
                beta = x(i+1) - xsi
                ypbar = 2d0*m(i) - (alpha*yp(i) + beta*yp(i+1))/dx(i)
                pp%co(j,1) = y(i)
                pp%co(j,2) = yp(i)

                pp%co(j,3) = (ypbar - yp(i))/(2d0*alpha)
                pp%k(j) = x(i)
                pp%co(j+1,1) = pp%co(j,1) + alpha*(pp%co(j,2) + alpha*pp%co(j,3))
                pp%co(j+1,2) = ypbar
                pp%co(j+1,3) = (yp(i+1)-ypbar)/(2d0*beta)
                pp%k(j+1) = xsi

!                 if (any(pp%co(j+1,:)>huge(0d0) .or. pp%co(j,:)>huge(0d0)) &
!                     .or. any(isNaN(pp%co(j+1,:)) .or. isNaN(pp%co(j,:))) &
!                     .or. max(pp%co(j+1,3),pp%co(j,3))>.5d0*huge(0d0)) then
                if (any(pp%co(j+1,:)>huge(0d0) .or. pp%co(j,:)>huge(0d0)) &
                    .or. max(pp%co(j+1,3),pp%co(j,3))>.5d0*huge(0d0)) then
                    print*,'xsi',xsi
                    print*,'x(i)',x(i),'x(i+1)',x(i+1)
                    print*,'y(i)',y(i),'y(i+1)',y(i+1)
                    print*,'dx(i)',dx(i)
                    print*,'dy(i)',dy(i)
                    print*,'m(i)',m(i)
                    print*,'L(i)',L(i)
                    print*,'c(j,1)',pp%co(j,1),'c(j+1,1)',pp%co(j+1,1)
                    print*,'c(j,2)',pp%co(j,2),'c(j+1,2)',pp%co(j+1,2)
                    print*,'c(j,3)',pp%co(j,3),'c(j+1,3)',pp%co(j+1,3)
                    print*,'yp(i)',yp(i),'yp(i+1)',yp(i+1)
                    print*,'yplo_minus_m',yplo_minus_m,'yphi_minus_m',yphi_minus_m
                    print*,'dx',dx(i)
                    print*,'ypbar',ypbar
                    print*,'alpha',alpha
                    print*,'beta',beta
                    print*,'c(j,1)+alpha(c(j,2)+alpha*c(j,3))',pp%co(j,1) + alpha*(pp%co(j,2) + alpha*pp%co(j,3))
                    print*,'c(j,1),alpha*c(j,2),alpha**2*c(j,3)',pp%co(j,1) , alpha*pp%co(j,2) , alpha**2*pp%co(j,3)
                    print*,'c(j,1)+alpha*c(j,2),alpha**2*c(j,3)',pp%co(j,1) + alpha*pp%co(j,2) , alpha**2*pp%co(j,3)
                    print*,'c(j,1),alpha*c(j,2)+alpha**2*c(j,3)',pp%co(j,1) , alpha*pp%co(j,2) + alpha**2*pp%co(j,3)
                    print*,'c(j,1)+alpha*c(j,2)+alpha**2*c(j,3)',pp%co(j,1) + alpha*pp%co(j,2) + alpha**2*pp%co(j,3)
                    print*,'j+1',j+1,'size(c,1)',size(pp%co,1)
                    STOP 'c is inf'
                end if

                j = j + 2

            end if
        end do

        ! Finalize k by adding the last point
        pp%k(j) = x(n)
       
        ! Deallocate space for local variables
        deallocate(dx,dy,m,L,Lm,dyp,specialCase)

        ! Deallocate space for yp if it was allocated
        if (.not. present(s)) deallocate(yp)

    end subroutine schumaker


    ! Piecewise cubic hermite interpolating polynomial
    ! Preserves montonicity 
    subroutine pchip_interp(Yi,X,Y,Xi)
        implicit none
        real(8), dimension(1:), intent(in) :: X,Y,Xi
        real(8), dimension(1:size(Xi)), intent(out) :: Yi
        ! local
        type(spline) :: pp 

        ! Get pp form of pchip
        call pchip_getpp(pp,X,Y)

        ! Eval at Xi
        call ppval(Yi,Xi,pp)

    end subroutine 

    subroutine pchip_getpp(pp,X,Y)
        implicit none
        type(spline), intent(out) :: pp ! intent(out) should deallocate all the components
        real(8), dimension(1:), intent(in) :: X,Y
        ! local
        real(8), allocatable :: s(:)
        real(8) :: h
        integer :: n,ierr,i

        n = size(X)

        pp%method = 'pchip'
        pp%d = 4

        if (size(Y)/=n) STOP 'pchip: ERROR: X and Y dims don''t agree'

        pp%n = n
        allocate(pp%k(n),pp%co(n-1,4))
        allocate(s(n))

        ! Knots are just the X values
        pp%k = X

        ! Coefficients: 
        pp%co(:,1) = Y(1:n-1)
        call dpchim(n,X,Y,s,1,ierr) ! Gets the slopes at each end point
        pp%co(:,2) = s(1:n-1)
        do i = 1,n-1
            h = x(i+1) - x(i)
            if (h<=0d0) STOP 'pchip: ERROR: X is not increasing' 
            pp%co(i,3) = - (s(i+1) + 2d0*pp%co(i,2))/h + 3d0*(Y(i+1)-pp%co(i,1))/(h**2)
            pp%co(i,4) = (s(i+1) - 2d0*pp%co(i,3)*h - pp%co(i,2))/(3d0*h**2)
        end do

        deallocate(s)
    end subroutine

end module mod_spline

module mod_spline_test 
    implicit none

contains

    real(8) function f1(x)
        implicit none
        real(8), intent(in) :: x
        f1 = sin(x)
    end function 
    real(8) function f2(x)
        implicit none
        real(8), intent(in) :: x
        f2 = sqrt(x)
    end function 

    subroutine test_vardim()
        use mod_spline
        implicit none
        type(vardimspline) :: a

        call a%test(f1,[-2d0,-1.5d0,-1d0,-.5d0,1d0,3d0],1)
        call a%test(f1,[-2d0,-1.5d0,-1d0,-.5d0,1d0,3d0],2)
        call a%test(f1,[-2d0,-1.5d0,-1d0,-.5d0,1d0,3d0],3)
        call a%test(f1,[-2d0,-1.5d0,-1d0,-.5d0,1d0,3d0],4)
        call a%test(f2,[0d0,.5d0,1d0,1.5d0,2d0,3d0],1)
        call a%test(f2,[0d0,.5d0,1d0,1.5d0,2d0,3d0],2)
        call a%test(f2,[0d0,.5d0,1d0,1.5d0,2d0,3d0],3)
        call a%test(f2,[0d0,.5d0,1d0,1.5d0,2d0,3d0],4)

    end subroutine

    subroutine test_spline()
        use mod_matlab, only: plot,tic,toc,cumsum,linspace
        use mod_spline
        implicit none
        type(spline) :: pp
        type(spline2) :: pp2
        real(8) :: X(6),Y(6),X2(8),V(6,8),Vcheck(6,8)
        real(8) :: Xi(100),Yi(100),Ypi(100),Yppi(100),Xi2(100),Vi(100),Vitrue(100)
        real(8), allocatable :: tmp2(:,:)
        integer :: i,j

        ! Test the interpolation properties of pchip
        call random_number(Xi) 
        Xi = cumsum(Xi+1d-6) ! xi must be strictly increasing
        call random_number(Yi)
        Yi = 1000d0*Yi + log(Xi)

        ! Test the interpolation properties of pchip
        call pchip(pp,Xi,Yi)
        call ppval(Ypi,Xi,pp)
        if (maxval(abs(Yi-Ypi))>1d3*epsilon(0d0)) then
            print*,'maxval(abs(Yi-Ypi))',maxval(abs(Yi-Ypi))
            STOP 'test_spline: failed test 1'
        end if

        ! Same for schumaker
        call schumaker(pp,Xi,Yi)
        call ppval(Ypi,Xi,pp)
        if (maxval(abs(Yi-Ypi))>1d3*epsilon(0d0)) STOP 'test_spline: failed test 2'

        ! Now test that linear functions can be recovered
        Yi = -100d0 + .05d0*Xi
        call pchip(pp,Xi,Yi)
        call ppval(Ypi,Xi,pp)
        if (maxval(abs(Yi-Ypi))>epsilon(0d0)) STOP 'test_spline: failed test 3'

        call schumaker(pp,Xi,Yi)
        call ppval(Ypi,Xi,pp)
        if (maxval(abs(Yi-Ypi))>epsilon(0d0)) STOP 'test_spline: failed test 4'

        ! Now test that quadratic functions can be recovered
        Yi = -100d0 + .05d0*Xi - 6d0*Xi**2
        call pchip(pp,Xi,Yi)
        call ppval(Ypi,Xi,pp)
        if (maxval(abs(Yi-Ypi))>epsilon(0d0)) STOP 'test_spline: failed test 5'

        call schumaker(pp,Xi,Yi)
        call ppval(Ypi,Xi,pp)
        if (maxval(abs(Yi-Ypi))>epsilon(0d0)) STOP 'test_spline: failed test 6'

        ! Now test additional

        X = (/0d0,.1d0,.5d0,1d0,2d0,10d0/)
        X2 =(/.0001d0,1d0,2d0,3d0,4d0,7d0,15d0,20d0/)
        Y = sqrt(X)


        call pchip(pp,X,Y)

        Xi = linspace(X(1),X(size(X)),size(Xi))
        Xi2 = linspace(X2(1),X2(size(X2)),size(Xi))

        ! Test one dimensional
!         call sub_pchip_eval(Yi,Xi,co,X,Y)
!         call plot(Xi,Yi)

!         call sub_pchip_deriv(Ypi,Yi,Xi,co,X,Y)
!         call plot(Xi,Yi)
!         call plot(Xi,Ypi)

!         call myprint('X',X)
!         call myprint('Y',Y)
!         call myprint('Yi',Yi)
!         call sub_pchip_int(yi,xi,X,Y)

        call pchip(pp,x,y)
        call ppval(yi,xi,pp)
! call plot(x,y)        
! call plot(xi,yi,title='pchip')
        call ppderiv(ypi,yppi,xi,pp)
! call plot(xi,ypi,title='pchip, deriv')
! call plot(xi,yppi,title='pchip, second deriv')
        call schumaker(pp,x,y)
        call ppval(yi,xi,pp)
! call plot(xi,yi,title='schumaker')
        call ppderiv(ypi,yppi,xi,pp)
! call plot(xi,ypi,title='schumaker, deriv')
! call plot(xi,yppi,title='schumaker, second deriv')
! STOP 'plotting'

        ! Test two dimensional
        do j = 1,8
            do i = 1,6
                V(i,j) = sqrt(X(i)) - 10d0*sqrt(X2(j))  - X(i)*X2(j)
            end do
        end do
        Vitrue = sqrt(Xi) - 10d0*sqrt(Xi2) - Xi*Xi2


        call pchip2(pp2,X,X2,V)

        ! Check the vectorized version
        call ppval2(Vcheck,X,X2,pp2)

        if (maxval(abs(Vcheck-V))>1000d0*epsilon(0d0)) then
            print*,'maxval(abs(Vcheck-V))',maxval(abs(Vcheck-V))
            STOP 'test_pchip: ERROR: failed test: ppval2 doesn''t interpolate'
        end if

        do i = 1,size(xi2)
            call ppval2(Vi(i),Xi(i),Xi2(i),pp2)
        end do

        ! Can compare this with the matlab "test_pchip2.m" file. Good on 1/6/2013
!         do i = 1,size(xi2)
!             print*, Vi(i),Vitrue(i)
!         end do
!     STOP 'here'


        if (maxval(abs(Vi-Vitrue)/abs(Vitrue))>.46d0) then
            print*,'Max relative error Vi,Vitrue',maxval(abs(Vi-Vitrue)/abs(Vitrue))
            STOP 'test_pchip: ERROR: failed test1: spline error larger than expected'
        end if
        
! call plot(Vi)
! call plot(Vitrue)

!         STOP 'checking ppval2'


! call tic
! do i = 1,10000
!         call sub_pchip2_coef(co2,X,X2,V)
! end do
! call toc
! do i = 1,10000
!         call sub_pchip2_eval(Vi,xi,xi2,co2,X,X2,V)
! end do
! call toc
!end do 
!         call myprint('V',V)
!         call myprint('co2',co2)
!         call myprint('X2',X2)
!         call myprint('Xi2',xi2)
!         call myprint('Xi',xi)
!         call myprint('Vi',Vi)
!         call myprint('Vitrue',Vitrue)

        ! Test extreme values, 1 dim routine
        call pchip(pp,[-2d0,-1d0,0d0,1d0,2d0],[-1d150,-1d0,0d0,1d0,1d150])
        Xi = linspace(-2d0,2d0,size(Yi))
        call ppval(Yi,Xi,pp)
        call plot(Xi,Yi)

        call pchip2(pp2,[-2d0,-1d0,0d0],[-2d0,-1d0,0d0],reshape(&
        (/ -2d0, -10d0, -1d200, &
           -1d0,  -5d0, -200d0, &
            0d0,  -1d0,  -50d0/)&
           ,(/3,3/)))
        Xi = linspace(-2d0,0d0,size(Xi))
        Yi = linspace(-2d0,0d0,size(Yi))
        allocate(tmp2(size(Xi),size(Yi)))
        call ppval2(tmp2,Xi,Yi,pp2)
        print*,'maxval(tmp2)',maxval(tmp2)
        
        call plot(Xi,tmp2)

        print*,'Yi:',Yi
        STOP 'testing spline extreme'

    end subroutine
end module 
