 
module mod_supnorm
implicit none

interface supnorm
    module procedure supnorm_4d, supnorm_3d,supnorm_mat, supnorm_vec
end interface supnorm

contains 


function supnorm_4d(A1,A0) result(supnorm)
    implicit none    
    real(8), dimension(1:,1:,1:,1:), intent(in) :: A1, A0
    real(8) :: supnorm
    !local
    real(8) :: diff, biggest    

    if (any(shape(A1)/=shape(A0))) then
        write(*,*) 'WARNING: supnorm: shapes do not conform' 
        write(*,*) 'A1: ', shape(A1)
        write(*,*) 'A0: ', shape(A0)
    end if

    if (any(abs(A0)<huge(0.d0)/3.d0)) then
    
        diff = maxval(abs(A1-A0),abs(A0)<huge(0.d0)/3.d0)
        biggest = maxval(abs(A0),abs(A0)<huge(0.d0)/3.d0)    
        
        if (biggest>0.d0) then
            supnorm = diff/biggest
        else
            supnorm = huge(0.d0)
        end if
        
    else !all are greater than huge(0.d0)/3.d0
        supnorm = maxval(abs(A1-A0))
    end if        
        
end function supnorm_4d

function supnorm_3d(A1,A0) result(supnorm)
    implicit none    
    real(8), dimension(1:,1:,1:), intent(in) :: A1, A0
    real(8) :: supnorm
    !local
    real(8) :: diff, biggest    

    if (any(shape(A1)/=shape(A0))) then
        write(*,*) 'WARNING: supnorm: shapes do not conform' 
        write(*,*) 'A1: ', shape(A1)
        write(*,*) 'A0: ', shape(A0)
    end if

    if (any(abs(A0)<huge(0.d0)/3.d0)) then
    
        diff = maxval(abs(A1-A0),abs(A0)<huge(0.d0)/3.d0)
        biggest = maxval(abs(A0),abs(A0)<huge(0.d0)/3.d0)    
        
        if (biggest>0.d0) then
            supnorm = diff/biggest
        else
            supnorm = huge(0.d0)
        end if
        
    else !all are greater than huge(0.d0)/3.d0
        supnorm = maxval(abs(A1-A0))
    end if        
        
end function supnorm_3d

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! option values
! not present, default
! 1, debug
! 2. ignore values larger than in abs val than 1/epsilon
! 3. switch the roles of A1 and A0
! 4. don't normalize by dividing by biggest
function supnorm_mat(A1,A0,option) result(supnorm)
    implicit none
    real(8), dimension(1:,1:), intent(in) :: A1, A0
    real(8) :: supnorm
    integer, intent(in), optional :: option
    !local
    real(8), dimension(1:size(A0,1),1:size(A0,2)) :: B1, B0
    real(8) :: diff, biggest

    if (any(shape(A1)/=shape(A0))) then
        write(*,*) 'WARNING: supnorm: shapes do not conform' 
        write(*,*) 'A1: ', shape(A1)
        write(*,*) 'A0: ', shape(A0)
    end if

    B1 = A1
    B0 = A0

    if (present(option)) then
        if (option==2) then
            B0 = max(min(B0,1.d0/epsilon(0.d0)),-1.d0/epsilon(0.d0))
            B1 = max(min(B1,1.d0/epsilon(0.d0)),-1.d0/epsilon(0.d0))
        end if
        if (option==3) then
            B1 = A0
            B0 = A1
        end if
    end if        

    

    if (any(abs(B0)<huge(0.d0)/3.d0)) then
       
        diff = maxval(abs(B1-B0),abs(B0)<huge(0.d0)/3.d0)
        biggest = maxval(abs(B0),abs(B0)<huge(0.d0)/3.d0)    
        
        if (biggest>0.d0) then
            supnorm = diff/biggest
            if (present(option)) then
                if (option==4) supnorm = diff
            end if
        else
            if (present(option)) then
                if (option==1) write(*,*) 'supnorm: every value of A0 is 0.d0'
            end if
            supnorm = huge(0.d0)
        end if

    else !all are greater than huge(0.d0)/3.d0
        supnorm = maxval(abs(A1-A0))
        if (present(option)) then
            if (option==1) write(*,*) 'supnorm: every value of A0 is greater than ', huge(0.d0)/3.d0
        end if
    end if        

        
end function supnorm_mat

function supnorm_vec(A1,A0) result(supnorm)
    implicit none    
    real(8), dimension(1:), intent(in) :: A1, A0
    real(8) :: supnorm
    !local
    real(8) :: diff, biggest    

    if (any(shape(A1)/=shape(A0))) then
        write(*,*) 'WARNING: supnorm: shapes do not conform' 
        write(*,*) 'A1: ', shape(A1)
        write(*,*) 'A0: ', shape(A0)
    end if

    if (any(abs(A0)<huge(0.d0)/3.d0)) then    
    
        diff = maxval(abs(A1-A0),abs(A0)<huge(0.d0)/3.d0)
        biggest = maxval(abs(A0),abs(A0)<huge(0.d0)/3.d0)    
        
        if (biggest>0.d0) then
            supnorm = diff/biggest
        else
            supnorm = huge(0.d0)
        end if
    
    else !all are greater than huge(0.d0)/3.d0
        supnorm = maxval(abs(A1-A0))
    end if        
        
end function supnorm_vec

end module mod_supnorm
