! The main program for "Evaluating Default Policy: The Business Cycle Matters"
! Grey Gordon
! First code was 2009
! This version is from 2014
!
program main
    use mod_main
    use mod_transition
    use mod_SS
    use mod_data
    use mod_matlab, only: tic
    use mod_mympi
    use mod_minNdim
    implicit none
    ! modelOption determines what is run
    character(len=10) :: modelOptionString
    character(len=30) :: machineTypeString
    integer :: modelOption = 1
    real(8) :: param(7)
    integer :: i,unitno
    integer, allocatable :: inttmparray(:)
    logical :: doTransition,tmptf

    call mympi_init()
    call tic

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! Set the data directory
    call get_environment_variable('HOME',rootdatadir)
    rootdatadir = trim(rootdatadir)//'/Data/bankKS/'

    call get_environment_variable('MACHTYPE',machineTypeString)
    if (trim(machineTypeString)=='x86_64-suse-linux') then
        ! Big Red 2, write to tmp directory
        rootdatadir = '/N/dc2/scratch/rggordon/Data/bankKS/'
    end if

    ! Place your desired data directory here
    ! rootdatadir = '/fill/in/the/absolute/path/with/a/slash/at/the/end/'

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    ! Check that the iZ sequence exists
    inquire(file=trim(rootdatadir)//'iZs.dat',exist=tmptf); if (.not. tmptf) STOP 'ERROR: iZs.dat is not in rootdatadir. Please add it.'


    if (command_argument_count()>=1) then

        ! Models were selected at the command line
        call get_command_argument(1,modelOptionString)
        read(modelOptionString,'(I10)') modelOption

    else
        print*,'**Please selected a model option**'
        print*,'1) benchmark (default)'
        print*,'2) no default'
        print*,'3) no default PE'
        print*,'4) BAPCPA '
        print*,'5) BAPCPA PE'
        print*,'6) CDR'
        print*,'7) CDR PE'
        print*,'8) CDE '
        print*,'9) CDE PE'
        print*,'21) benchmark (default) w/ expenditure shocks'
        print*,'22) no default w/ expenditure shocks'
        print*,'23) no default PE w/ expenditure shocks'
        print*,'24) BAPCPA w/ expenditure shocks'
        print*,'25) BAPCPA PE w/ expenditure shocks'
        print*,'26) CDR w/ expenditure shocks'
        print*,'27) CDR PE w/ expenditure shocks'
        print*,'28) CDE w/ expenditure shocks'
        print*,'29) CDE PE w/ expenditure shocks'
        print*,'31) benchmark (default) w/ expenditure shocks and large debt calibration'
        print*,'32) no default w/ expenditure shocks and large debt calibration'
        print*,'Since expenditure shocks are now adopted as the benchmark, all the below have expenditure shocks'
        print*,'10xx) Robustness, chi1 = 0d0, chi0 = xx'
        print*,'11xx) Robustness, chi1 = 0d0, chi0 = 1d0-1d{-xx}'
        print*,'2xy) Robustness, threshold level for low e case x, defBnkAllowed=y'
        print*,'3xy) Robustness, retirement process case x, defBnkAllowed=y'
        print*,'40y) No mortality profile, defBnkAllowed=y'
        print*,'41y) No consumption equivalence profile, defBnkAllowed=y'
        print*,'42y) No earnings profile, defBnkAllowed=y'
        print*,'43y) STY estimates (no variance / persistence profile), defBnkAllowed=y'
        print*,'50y) Flexible portfolios, defBnkAllowed=y '
        print*,'60y) No CEV (countercyclical earnings variance), defBnkAllowed=y'
        print*,'61y) No CEV and no LSS (labor supply shifter), defBnkAllowed=y'
        print*,'62y) No CEV and no LSS and no TFP (productivity), defBnkAllowed=y'
        print*,'9999) Calibrate'
        print*,'1000000xy) Transition from model option 2x to option 2y'
        print*
        print*,'**Enter model option**'
        read(*,*) modelOption

    end if

    ! If two command line arguments are present, only save the steady state and law of motion (but not the large business cycle variables)
    if (command_argument_count()>=2) then
        print*,'Economizing on data storage' 
        onlySaveLaw = .true.
    end if

    ! Solve for it
    ! nExpenditure = merge(3,1,(modelOption>=21 .and. modelOption<=39) .or. modelOption==9999)
    nExpenditure = merge(3,1,modelOption>=21) ! adopted expenditure shocks as the benchmark
    nE = nExpenditure*(npers*ntran+nCCNRaugment)

    ! CALIBRATED PARAMETER VALUES
    ! 1) discount
    ! 2) eloccnraugment
    ! 3) phiccnraugment
    ! 4) ehiccnraugment
    ! 5) pi11 
    ! 6) chi0
    ! 7) chi1
    param = (/ 0.957167D+00, log(0.106439D+01),-0.183570D+01, log(0.387278D+01), -log(1d0/0.853272D+00-1d0), 0.420639D+00, -0.729726D+00/) ! 1st submission updated
    param = [     0.9559395484d0,     1.6639017584d0,     0.0535165848d0,    66.0773949929d0,     0.2702081376d0,     0.9177455042d0,    -8.6012592946d0]

    ! For a robustness check, I also want to have a high debt calibration where the targets are mostly based on LMT. See CalibrationDetails.xlsx for details. 
    ! The debt target is an order of magnitue larger, but the filing rate is the same. The below parameters are for this case. THIS IS A ROBUSTNESS CHECK.
    if ((modelOption>=31 .and. modelOption<=39) .or. modelOption==9999) then
        param = [     0.9558925674d0,     0.4543710864d0,     0.1833567329d0,    27.9540666622d0,     0.7873697353d0,     0.9737337791d0,    -0.5627147946d0]
    end if

    ! Set the parameters (may be overwritten for various experiments below)
    call set_param(param)


    ! Set other values or overwrite param according to the model option selected
    doTransition = modelOption>100000000
    if (.not. doTransition) then
        call set_case(modelOption,.true.)
    else
        ! Set a write directory
        print*,'setting write directory to ',trim(rootdatadir)//'transition/'
        call usedir(trim(rootdatadir)//'transition/')
    end if

    ! Allocate data
    call allocate_main()

    ! Set shocks
    call sub_setshocks()

    ! Set grid
    call set_gA()

    ! Solve the SS and BC
    if (.not. doTransition .and. .not. doCalibrate) then
        call solveSSandBC(.true.,.true.)
        ! print*,'warning just loading ss'
        ! call solveSSandBC(.false.,.true.)
    end if

    ! If doing transition, need to set the case twice and solve for two steady states
    if (doTransition) then

        print*,'Computing the transition'

        ! First, make sure there is enough memory
        call allocate_transition()

        if (modelOption==100000014) then
            
            print*,'Solving for the first steady state'
            ! Solve for the first steady state

            !!! Set the case for "x" --- the second to last digit in the model option
            call set_case(21,.false.)

            !!! Solve the steady state
            call solveSSandBC(.true.,.false.)
            
            !!! Copy in to the first transition period
            call copyin_SS_to_transition(1)

            print*,'Solving for the second steady state'
            ! Solve for the second steady state

            !!! Now, for "y" --- the last digit in the model option
            call set_case(24,.false.)

            !!! Solve the steady state
            call solveSSandBC(.true.,.false.)
            
            !!! Copy in to the last transition period
            call copyin_SS_to_transition(2)

            ! NOTE: do not free the SS data the last time. The grids and shocks are used in sub_transition.

            ! Now solve for the transition (the rules in place regarding default/parameter values are those associated with the second SS).
            ! The "6" means the old law is in effect for 6 years (from i=1,6). BAPCPA went into effect October 17, 2005, so the best approx
            ! in years is "2000 to 2005" is the old law, 2006 and on is the new
            call sub_transition(21,24,6) 
            ! call sub_transition(21,24,2) 

        else
            STOP 'ERROR: haven''t programmed this yet'
        end if
        
        STOP 'Finished transition'

    end if

    !Calibrate SS
    if (doCalibrate) then
        write(*,*) 'Calibrating'
        if (mympi_nproc()>1) then
            ! call sub_parrandom(param,fn_SSRS,e_in=[.01d0,.1d0,.001d0,5d0,.01d0,.1d0,.5d0],maxiter_in=10000,xtoler_in=1d-7)
            ! call sub_parnoisyLM(param,fn_resvec,nf=7,stdev=1d-1*[.01d0,.1d0,.001d0,5d0,.01d0,.1d0,.5d0],ndraws=max(100,mympi_nproc()-1))
            call sub_parnoisyLM(param,fn_resvec,nf=7,stdev=1d-3*param,ndraws=max(300,3*mympi_nproc()-1)) ! 1d-3 seems okay, 1d-4 also
        else
            ! call sub_mynlopt(param,fn_SSRS,NLOPT_GN_CRS2_LM,&
            !                  a_in = [.91d0 ,1d-4,1d-4,1d0,.0001d0,0d0,-10d0],&
            !                  b_in = [.965d0,10d0,1d0,1d3,.9999d0,1d0,0d0])
            do 
                call sub_simplex(param,fn_SSRS,e_in=[.01d0,.1d0,.001d0,5d0,.01d0,.1d0,.5d0],maxiter_in=10000,xtoler_in=1d-8,randomize_in=.true.)
            end do
            ! call sub_noisyLM(param,fn_resvec,nf=7,stdev=1d-1*[.01d0,.1d0,.001d0,5d0,.01d0,.1d0,.5d0],ndraws=100)
        end if

        print*,'Calibrated param is ',param
        STOP 'Finished Calibrating'

    end if

    call deallocate_main()

    call mympi_barrier() ! this must go at the end so the first node to finish does not terminate the program
    call mympi_finalize()

end program main
