
module mod_SS
    implicit none

contains

    subroutine sub_SS(print_results,statlist_out,s_out)
        use mod_gini
        use mod_matlab
        use mod_myprint
        use mod_param, only: alpha_k, delta, nA, nH, nAmm, nE, Tend, &
                                  discount, chi, Tret, isPE, qbarPE, print_common_var, nCCNRaugment, doCalibrate
        use mod_root1dim
        use mod_stats
        use mod_data, only: invar_dsn_SS,EEE_SS,v_SS,c_SS,ap_SS,d_SS,q_SS,gE,invar_dsn_e_SS
        implicit none
        real(8), intent(out), optional :: statlist_out(100)
        type(stat_type), intent(out), optional :: s_out
        real(8) :: qbar, w !, K, Y, N, A, K_msr_1, K_msr_2 
        logical, intent(in) :: print_results
        integer :: iter_res
        real(8) :: statlist(100)
        type(stat_type) :: s

        !Begin program
        ! write(*,"('Beginning steady state computation',/)")
        ! write(*,"('Parameter  discount     = ',f14.3)") discount

        iter_res = 1
        
        ! Solve, either the partial or general eq vesion
        if (.not. isPE) then 
            !Find eq qbar
            if (doCalibrate) then
                qbar = .98339719d0 ! This is the risk free price associated with K/Y=3.08, use it when calibrating
                ! [.95,.99]
                ! [.97,.99]
                ! [.98,.99]
                ! [.98,.985]
                ! [.9825,.985]
                ! call sub_bisect(qbar,res,.9825d0,.985d0,maxiter_in=40,printmod_in=1)
                ! call sub_bisect(qbar,res,0.983125d0,0.9834375d0,maxiter_in=40,printmod_in=1)
                ! call sub_bisect(qbar,res,0.98337890625d0,0.9833984375d0,maxiter_in=40,printmod_in=1)
            else
                ! call sub_brent(qbar,res,.95d0,.99d0,maxiter_in=40,printmod_in=1)
                call sub_bisect(qbar,res,.95d0,.99d0,maxiter_in=40,printmod_in=1)
            end if
        else
            qbar = qbarPE
        end if

        ! Given either the equilibrium qbar or the qbarPE, solve for the equilibrium
        w = wfn(qbar,1.d0) ! Get implied w from qbar
        
        ! Solve for PE 
        ! call subVFI(v_SS,c_SS,ap_SS,d_SS,q_SS,qbar,w,gA,gE,trans_e_SS,EEE_SS,.true.)
        call subVFI(qbar,w,.true.)

        !Calculate the invariant dsn
        call sub_dsn() !(invar_dsn_SS,d_SS,ap_SS,trans_e_SS,init_dsn_e_SS,gA)
        
        ! Compute statistics
        call computeStats(statlist,s,0,invar_dsn_SS,1d0,1d0,v_SS,c_SS,ap_SS,d_SS,q_SS,gE)

        if (.not. doCalibrate) then
            print*,'Average EEE_SS', sum(EEE_SS(nAmm+1:nA,:,1:Tend-1)*invar_dsn_SS(nAmm+1:nA,:,1:Tend-1,1)) &
                                     /sum(invar_dsn_SS(nAmm+1:nA,:,1:Tend-1,1))
            ! print *,'Two measures of K: ',s%K_msr_1,s%K_msr_2
        end if

        ! Display them
        if (.not. doCalibrate) call print_statlist(statlist)
         
        ! Save them
        call myprint('statlist',statlist) ! write to file for easy access
        
        if (present(statlist_out)) statlist_out = statlist
        if (present(s_out)) s_out = s

        !Print results
        if (print_results) call print_all


    contains

        !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function res(qbar_guess) 
            implicit none
            real(8), intent(in) :: qbar_guess
            real(8) :: res
            !local
            real(8) :: qbar_implied
            call induced_qbar(qbar_implied,qbar_guess)
            res = qbar_implied-qbar_guess
        end function res

        subroutine induced_qbar(qbar_implied,qbar_guess)
!             use mod_SS_data
            use mod_param
            implicit none    
            real(8), intent(in) :: qbar_guess
            real(8),intent(out) :: qbar_implied
            real(8) :: w_guess
            !local
            real(8) :: r, K_msr_1, K_msr_2, K_msr_avg

            w_guess = wfn(qbar_guess,1.d0)        

            ! Solve for PE 
            ! call subVFI(v_SS,c_SS,ap_SS,d_SS,q_SS,qbar_guess,w_guess,gA,gE,trans_e_SS,EEE_SS,.false.)
            call subVFI(qbar_guess,w_guess,.false.)

            ! Calculate the invariant dsn
            call sub_dsn()
            ! call sub_dsn(invar_dsn_SS,d_SS,ap_SS,trans_e_SS,init_dsn_e_SS,gA)
            if (abs(sum(invar_dsn_SS)-1.d0)>1d-10) write(*,*) 'Result: Dsn doesnt sum to 1 but to ',sum(invar_dsn_SS)

            ! Calculate the implied capital stock
            K_msr_1 = computeKp(invar_dsn_SS,q_SS,ap_SS) !,gA,nA,nAmm,nE,Tend,nH)
            K_msr_1 = max(K_msr_1,1d-6)
            K_msr_2 = computeK(invar_dsn_SS,1d0,d_SS,gE)
            K_msr_2 = max(K_msr_2,1d-6)
            K_msr_avg = .5d0*(K_msr_1 + K_msr_2) ! In steady state, K_msr_1==K_msr_2
            
            r = alpha_k*(K_msr_avg/sum(invar_dsn_e_SS*gE))**(alpha_k-1.d0)
            qbar_implied = 1.d0/(1.d0+r-delta)
            
            iter_res=iter_res+1
            
        end subroutine induced_qbar

        subroutine print_all
            use mod_data, only: print_data_SS
            use mod_myprint
            implicit none
            
            call myprint('w_SS',w)
            call myprint('qbar_SS',qbar)
            call myprint('q_SS',q_SS)
            call myprint('K_SS',s%K)
            call myprint('N_SS',s%N)
            call myprint('A_SS',s%A)

            call print_data_SS() ! prints SS values in mod_data
            call print_common_var() ! prints parameters

        end subroutine print_all

    end subroutine sub_SS


    !Solves forward the invariant distribution by computing the distribution conditional on each age.
    subroutine sub_dsn() !(dsn, d_SS, ap_SS, trans_e_SS, init_dsn_e_SS, gA)
        use mod_param, only: nA,nE,Tend,nH
        use mod_nonstochsimul, only: sub_dsn_aux
        use mod_data, only: invar_dsn_SS, d_SS, ap_SS, trans_e_SS, init_dsn_e_SS, gA
        implicit none
        !local
        real(8), allocatable :: dreal(:,:,:,:)
        real(8), allocatable :: work(:,:)
        integer :: age

        allocate(dreal(nA,nE,Tend,nH)) 
        allocate(work(nA,nE))

        dreal = merge(1d0,0d0,d_SS)

        do age = 1,Tend

            ! call sub_dsn_aux(age,invar_dsn_SS,dreal,ap_SS,trans_e_SS,init_dsn_e_SS)
            call sub_dsn_aux(invar_dsn_SS,invar_dsn_SS,age,dreal,ap_SS,trans_e_SS,init_dsn_e_SS,work)

        end do

    end subroutine sub_dsn


    !!! NOTE: this is contingent upon one age
    subroutine sub_noPi(q_SS, qbar, w, d_SS, gA, gE, trans_e_SS)
        use mod_param, only: isUsuryLaw, usuryLimit, chiGarnish, lambda, nA, nAmm, nE, nH
        ! use mod_supnorm
        use mod_matlab, only: mydgemm
        implicit none    
        real(8), intent(in) :: qbar, w
        real(8), dimension(nA), intent(in)  :: gA
        logical, dimension(nA,nE,nH), intent(in)  :: d_SS
        real(8), dimension(nE), intent(in)  :: gE
        real(8), dimension(nE,nE), intent(in)  :: trans_e_SS
        real(8), dimension(nA,nE,nH), intent(out) :: q_SS
        !local
        integer :: iEp, iAp, iHp
        real(8),  dimension(:,:,:), allocatable :: recoveryrate 
        real(8) :: chiPaid

        allocate(recoveryrate(nA,nE,nH))

        !Use d_SS to compute q_SS
        do iHp = 1,nH
            do iEp = 1,nE
                do iAp = 1,nA
                    if (iAp==nAmm+1) then 
                        recoveryrate(iAp,iEp,iHp) = recoveryrate(iAp-1,iEp,iHp) ! recoveryrate can be arbitrary, so, for pretty graphs, set to iAp-1
                        cycle
                    end if
                    chiPaid = chiGarnish(gE(iEp),1d0)
                    if (d_SS(iAp,iEp,iHp)) then
                        recoveryrate(iAp,iEp,iHp) = chiPaid*w*gE(iEp)/(-gA(iAp))
                    else
                        recoveryrate(iAp,iEp,iHp) = 1d0
                    end if
                end do
            end do
        end do
        

!         q_SS(:,:,1) = qbar*matmul(recoveryrate(:,:,1),transpose(trans_e_SS))
!         q_SS(:,:,2) = qbar*matmul((1d0-lambda)*recoveryrate(:,:,1) + lambda*recoveryrate(:,:,2),transpose(trans_e_SS))

        ! This dgemm code should do the same as above
        recoveryrate(:,:,2) = (1d0-lambda)*recoveryrate(:,:,1) + lambda*recoveryrate(:,:,2)
        call mydgemm(q_SS(:,:,1),recoveryrate(:,:,1),trans_e_SS,nA,nE,nE,nE,tB='T',alpha=qbar)
        call mydgemm(q_SS(:,:,2),recoveryrate(:,:,2),trans_e_SS,nA,nE,nE,nE,tB='T',alpha=qbar)

        ! If there is a usury law in place, change the discounted price to 0 where necessary
        if (isUsuryLaw) where (q_SS<usuryLimit) q_SS = 0.d0

    end subroutine sub_noPi


    ! subValue
    ! Given an age, value function for age+1, a default decision for age+1, and prices w and qbar, solves
    ! for the the value and policy functions for a household with age "age" as well as the price schedule.
    subroutine subValue(age,v_SS,c_SS,ap_SS,d_SS,q_SS,qbar,w,gA,gE,trans_e_SS,EEE_SS)
        use mod_param
        use mod_shocks, only: state
        use mod_matlab
        use mod_compute_vR
        implicit none
        real(8), intent(in) :: qbar,w,gA(:),gE(:,:),trans_e_SS(:,:,:)
        real(8), dimension(:,:,:,:), intent(inout) :: v_SS,c_SS,ap_SS,q_SS
        logical, dimension(:,:,:,:), intent(inout) :: d_SS
        real(8), dimension(:,:,:), intent(inout), optional :: EEE_SS
        
        ! local
        logical :: collectEEE
        real(8), dimension(nA,nE,nH) :: CV, qap
        real(8), dimension(nA,nE) :: dCV ! Used in the collection of euler equation errors

        integer :: iE, iA, iH, age

        real(8), dimension(nE) :: vDefBnk,cDefBnk
        real(8) :: tmp

        real(8), dimension(nA,nE,nH) :: vR,cR,apR
        real(8), dimension(nE,nH) :: endow
        integer, dimension(nE) :: option
        

        collectEEE = present(EEE_SS)

        if (age<Tend) then 
        
            ! Compute the price schedule taking into account next period's default decision
            call sub_noPi(q_SS(:,:,age,:), probLive(age)*qbar, w, d_SS(:,:,age+1,:), gA, gE(:,age+1), trans_e_SS(:,:,age))

            ! Compute expectations over e' for all v_SS. Can do this because a',h' trans is independent of e'
            do iH = 1,nH            
!                 CV(:,:,iH) = matmul(v_SS(:,:,age+1,iH),transpose(probLive(age)*discount*trans_e_SS(:,:,age))) !Note effective discount factor is pulled into trans_e_SS                
                !!! This should do the same as the above
                call mydgemm(CV(:,:,iH),v_SS(:,:,age+1,iH),trans_e_SS(:,:,age),nA,nE,nE,nE,tB='T',alpha=probLive(age)*discount)
            end do

        else
            
            ! Compute the ``price schedule'' faced in the last period of life (assume hhs default for sure next period)
            q_SS(1:nAmm,:,age,:) = -.01234d0
            q_SS(nAmm+1:nA,:,age,:) = 1d0 ! put a q_SS of 1 in so that they choose a'=0 

            ! Set continuation utility
            CV = 0d0 ! 0 continuation utility

        end if

        ! Compute q_SS(a')*a', the expenditure schedule
        do iH = 1,nH
            do iE = 1,nE
                qap(:,iE,iH) = q_SS(:,iE,age,iH)*gA 
            end do
        end do

        ! Check that the continuation utilities look alright
        if (nExpenditure==1 .and. maxval(CV(nAmm+1:nA,:,2)-CV(nAmm+1:nA,:,1))>1000d0*epsilon(0d0)) then
            print*,'WARNING: CV(2) larger than CV(1)',maxval(CV(nAmm+1:nA,:,2)-CV(nAmm+1:nA,:,1))
        end if


        if (any(CV(nAmm+1:nA,:,1)<-1d5)) then
            print*,'WARNING: CV(1) is really negative'
        end if

        ! Compute the value of defaulting into bankruptcy
        cDefBnk = w*gE(:,age)*(1d0-chi(gE(:,age),1d0))
        where (cDefBnk>0d0)
            vDefBnk = -csnEquiv(age)/cDefBnk + CV(nAmm+1,:,2)
        elsewhere
            vDefBnk = death
        endwhere

        CV(:,:,2) = (1d0-lambda)*CV(:,:,1) + lambda*CV(:,:,2) 
!         where (abs(CV(:,:,2)-CV(:,:,1))<10d0*epsilon(0d0)*abs(CV(:,:,1))) CV(:,:,2) = CV(:,:,1) ! operations on CV induce noise, so undo this in a special case

        endow(:,1) = w*gE(:,age) - state(:)%expend
        endow(:,2) = w*gE(:,age)*(1d0-chiBnk(gE(:,age),1d0)) - state(:)%expend

        option = 1 ! here unrestricted and restricted are the same

        ! Compute the value of repayment and being in bankruptcy
        call compute_vR_SS(vR,cR,apR,CV,qap,endow,gA,csnEquiv(age),option)

        ! if (age==1) then
        !     call plot(gA,vR(:,:,1),'vR')
        !     call plot(gA,cR(:,:,1),'cR')
        !     call plot(gA,apR(:,:,1),'apR')
        !     call plot(gA(nAmm+1:nA),vR(nAmm+1:nA,:,2),'vB')
        !     call plot(gA(nAmm+1:nA),cR(nAmm+1:nA,:,2),'cB')
        !     call plot(gA(nAmm+1:nA),apR(nAmm+1:nA,:,2),'apB')
        ! end if


        ! Check some theoretical properties
        if (nExpenditure==1) then
            if (any(vR(nAmm+1:nA,:,1)<=.1d0*death)) print*,'WARNING: vRepay(a>=0) is really negative' 
            if (any(diff(vR(:,:,1))<-1d-10)) print*,'WARNING: vRepay not increasing in a'
            if (any(diff(vR(nAmm+1:nA,:,2))<-1d-10)) print*,'WARNING: vBnk not increasing in a'
!             if (any(vR(nAmm+1:nA,:,1)<=vR(nAmm+1:nA,:,2) - 2d0*epsilon(0d0)*abs(vR(nAmm+1:nA,:,2)))) then
!                 call plot(gA(nAmm+1:nA),vR(nAmm+1:nA,:,2)-vR(nAmm+1:nA,:,1))
!                 print*,'maxval(vBnk-vRepay(nAmm+1:nA,:))',maxval(vR(nAmm+1:nA,:,2)-vR(nAmm+1:nA,:,1))
!                 print*,'WARNING: vRepay(a>=0) is worse than vBnk(a>=0)' 
!             end if
!             if (any(vR(nAmm+1:nA,:,1)<spread(vDefBnk,1,nA-nAmm))) then
!                 print*,'maxval(spread(vDefBnk,1,nA-nAmm)-vRepay(nAmm+1:nA,:))',maxval(spread(vDefBnk,1,nA-nAmm)-vR(nAmm+1:nA,:,1))
!                 print*,'WARNING: vRepay(a>=0) is less than vDefBnk'
!             end if
        end if
            
        
        ! With calculated value and policy functions in hand, update the combined value function v_SS(a,e,h) and policies
        ! based on what default options are available to households

        !!! Determine whether to default or not
        do iH = 1,nH
            do iE = 1,nE
                do iA = 1,nA

                    ! Get default decision
                    if (defBnkAllowed) then 
                        d_SS(iA,iE,age,iH) = vR(iA,iE,iH)<vDefBnk(iE) - abs(vDefBnk(iE))*2d0*epsilon(0d0)
                    else
                        ! If no default allowed, and there are expenditure shocks, permit default only when x>0 and ...
                        d_SS(iA,iE,age,iH) = vR(iA,iE,iH)<vDefBnk(iE) - abs(vDefBnk(iE))*2d0*epsilon(0d0) .and. &
                                          nExpenditure>1 .and. state(iE)%expend > 1d-10  ! was this
                                          ! nExpenditure>1 .and. cR(iA,iE,iH)<.1d0*w*gE(iE,age) !gA(iA) - state(iE)%expend<0d0
                    end if

                    ! Fill in from there
                    if (d_SS(iA,iE,age,iH)) then
                        v_SS(iA,iE,age,iH) = vDefBnk(iE)
                        c_SS(iA,iE,age,iH) = cDefBnk(iE)
                        ap_SS(iA,iE,age,iH) = 0d0
                    else
                        v_SS(iA,iE,age,iH) = vR(iA,iE,iH)
                        c_SS(iA,iE,age,iH) = cR(iA,iE,iH)
                        ap_SS(iA,iE,age,iH) = apR(iA,iE,iH)
                    end if

                end do
            end do
        end do

        ! Check V(a>=0) is not vastly negative
        if (any(v_SS(nAmm+1:nA,:,age,1)<-1d5)) then
            print*,'age',age,'Tend',Tend
            print*,'WARNING: V(a>0) is really negative'
        end if


        ! If computing the Euler Equation Error, do that here
        !!! Assume everyone pays risk-free price and then only count those who save (where this is true)
        if (collectEEE) then
            EEE_SS(:,:,age) = 0d0
            if (age<Tend) then

                dCV = probLive(age)*discount*matmul(csnEquiv(age+1)/c_SS(:,:,age+1,1)**2,transpose(trans_e_SS(:,:,age))) 

                ! u' is csnEquiv/c_SS^2 so u'^{-1} is sqrt(csnEquiv/u')
                ! risk-free price is qbar*probLive(age)
                ! u'(c_SS)*qbar*probLive(age) = dCV(a')
                ! EEE_SS = 1d0-u'^{-1}(dCV(a')/(qbar*probLive(age)))/c_SS
                ! EEE_SS = 1d0 - sqrt(csnEquiv(age)*qbar*probLive(age)/dCV(a'))/c_SS


                do iE = 1,nE
                    ! Only collect for positive iA
                    do iA = nAmm+1,nA
                        tmp = interp1(gA,dCV(:,iE),ap_SS(iA,iE,age,1))
                        !tmp = dCV(apInd(iA,iE,age,1),iE)
                        EEE_SS(iA,iE,age) = log10(abs(1d0-sqrt(csnEquiv(age)*qbar*probLive(age)/tmp)/c_SS(iA,iE,age,1)))
                    end do
                end do

            end if

!             call plot(EEE_SS(nAmm+1:nA,:,age),'EEE_SS')

        end if


    end subroutine

    ! subVFI
    ! Computes the value function using prices
    ! subroutine subVFI(v_SS,c_SS,ap_SS,d_SS,q_SS,qbar,w,gA,gE,trans_e_SS,EEE_SS,collectEEE)
    subroutine subVFI(qbar,w,collectEEE)
        use mod_param, only: defBnkAllowed, isBAPCPA, nA, nE, nH, Tend
        use mod_myprint
        use mod_matlab, only: colon, plot, interp1, disp
        use mod_data, only: v_SS, c_SS, ap_SS, d_SS, q_SS, gA, gE, trans_e_SS, EEE_SS
        implicit none
!         real(8), dimension(nA,nE,Tend,nH), intent(out) :: v_SS,c_SS
!         real(8), dimension(nA,nE,Tend,nH), intent(out) :: ap_SS
!         logical, dimension(nA,nE,Tend,nH), intent(out) :: d_SS
!         real(8), dimension(nA,nE,Tend), intent(out) :: EEE_SS
        
        real(8), intent(in) :: qbar, w !, gA(nA), gE(nE,Tend), trans_e_SS(nE,nE,Tend)
!         real(8), intent(out) :: q_SS(nA,nE,Tend,nH)
        logical, intent(in) :: collectEEE

        !local
        integer :: age

        if (.not. defBnkAllowed .and. isBAPCPA) STOP 'subVFI: inconsistent choices'

        do age = Tend,1,-1

            ! Solve for the value function of a household 
            if (collectEEE) then
                call subValue(age,v_SS,c_SS,ap_SS,d_SS,q_SS,qbar,w,gA,gE,trans_e_SS,EEE_SS)
            else
                call subValue(age,v_SS,c_SS,ap_SS,d_SS,q_SS,qbar,w,gA,gE,trans_e_SS)
            end if

        end do

    end subroutine subVFI

    !Compute w from qbar and z
    elemental function wfn(qbar,z) result(w)
        use mod_param, only: alpha_k, delta
        implicit none
        real(8), intent(in) :: z, qbar
        real(8) :: w
        w = z*(1d0-alpha_K)*(((1d0/qbar - 1d0 + delta)/(z*alpha_k))**(1d0/(alpha_k-1d0)))**alpha_k        
    end function wfn


end module mod_SS
