module mod_compute_vR_work
    implicit none
    type workdef
        real(8), pointer :: gA(:)=>null(), qap(:,:,:,:,:)=>null(), CV(:,:,:,:,:)=>null(), endow(:,:,:)=>null()
        real(8) :: aplbsearch,apubsearch,csnEquiv
        integer :: iA,iE,iH,iM,nZ
    end type
end module

module mod_compute_vR
    use mod_compute_vR_work
    implicit none

    private 
    public :: compute_vR_BC, compute_vR_SS

    real(8), allocatable :: gY(:),vL(:),vLnew(:),apL(:,:,:),CVsumz(:),qapsumz(:),vM(:),apM(:,:)
    real(8) :: aplb, apub, csnEquiv
    integer :: iY,iE,iH,iZ,iM
    integer :: nZ

    real(8), pointer :: gA(:)=>null(), CV(:,:,:,:,:)=>null(), qap(:,:,:,:,:)=>null()
    ! real(8), allocatable :: gA(:), CV(:,:,:,:,:), qap(:,:,:,:,:)


    real(8), private :: csnEquivPrivate
    !$OMP THREADPRIVATE(gY,vL,vLnew,apL,CVsumz,qapsumz,vM,apM,aplb,apub,iY,iE,iH,iZ,iM)


    ! real(8), allocatable, private, target :: sub_simplex_core_workarray(:,:)
    real(8), target :: workarray(2,4,32),workarray2(2*3,32),workarray3(3,32)
    ! !$OMP THREADPRIVATE(sub_simplex_core_workarray)

contains

    subroutine sub_simplex_core(x,f,work,e,xtoler,maxiter,printmod)
        use mod_orderpack
        use mod_sort
        use omp_lib
        implicit none
        real(8), dimension(1:), intent(inout) :: x
        type(workdef), intent(in) :: work
        interface 
            function f(x,work) 
                use mod_compute_vR_work
                implicit none
                real(8), dimension(:), intent(in) :: x
                type(workdef), intent(in) :: work
                real(8) :: f
            end function f
        end interface
        real(8), dimension(1:size(x)), intent(in) :: e
        real(8), intent(in) :: xtoler
        integer, intent(in) :: maxiter,printmod
        !local
        !controls
        real(8), parameter :: alpha = 1.d0, beta = 1.d0, gamma = .5d0, eta = .5d0
        !tools
        real(8), dimension(:,:), pointer :: xs
        real(8), dimension(:), pointer :: fs
        real(8), dimension(:), pointer :: x_gravity,x_reflect,x_expand,x_contract
        ! real(8), dimension(1:size(x),1:size(x)+1) :: xs
        ! real(8), dimension(1:size(x)+1) :: fs
        ! real(8), dimension(1:size(x)) :: x_gravity,x_reflect,x_expand,x_contract
        real(8) :: f_reflect,f_expand,f_contract
        integer, dimension(1:size(x)+1) :: ranking
        integer :: ib,iw,ip ! best, worst, and penultimate (second worst)
        real(8) :: dist
        integer :: n, i, iter
        logical :: doNotTryReflect

        n = size(x)        

        if (n>2) STOP 'ERROR: sub_simplex_core: for efficiency, assumed n<=2'
        if (omp_get_num_threads()>32) STOP 'ERROR: sub_simplex_core: for efficiency, I assumed omp thread nums<=32'
!         if (.not. allocated(sub_simplex_core_workarray)) then
!             allocate(sub_simplex_core_workarray(2,4))
!         end if
        x_gravity => workarray(1:n,1,1+omp_get_thread_num())
        x_reflect => workarray(1:n,2,1+omp_get_thread_num())
        x_expand => workarray(1:n,3,1+omp_get_thread_num())
        x_contract => workarray(1:n,4,1+omp_get_thread_num())

        xs(1:n,1:n+1) => workarray2(1:n*(n+1),1+omp_get_thread_num())
        fs => workarray3(1:n+1,1+omp_get_thread_num())

        !Create initial simplex and evaluate at simplex corners
        xs(:,1) = x 
        do i = 2,n+1
            xs(:,i) = x
            xs(i-1,i) = x(i-1) + e(i-1)            
        end do

        !Evaluate simplex at corners
        do i = 1,n+1
            fs(i) = f(xs(:,i),work)
        end do

        if (printmod>0) write(*,'(A)') 'NM:iter     min(f)        max(f)        simplex size'

        !Main routine
        doNotTryReflect = .false.
        do iter = 1,maxiter

            ! Reorder the simplex from smallest to largest (ascending sort)
            !!! Use insertion sort because almost surely this will be a very small list
            ! call isort(fs,ranking) 

            select case (n)
                case (1)
                    if (fs(1)<=fs(2)) then
                        ib = 1
                        ip = 1
                        iw = 2
                    else
                        ib = 2
                        ip = 2
                        iw = 1
                    end if
                case default
                    call mrgrnk(fs,ranking) 
                    ib = ranking(1)
                    ip = ranking(n)
                    iw = ranking(n+1)
            end select

 
            ! Output
            ! if (printmod>0 .AND. mod(iter,printmod)==0) write(*,'(A,i6,A,e12.5,A,e12.5,A,e12.5)') 'NM:',iter,'  ', fs(ib),'  ', fs(iw),'  ', dist
            if (printmod>0 .AND. mod(iter,printmod)==0) write(*,'(A,i6,A,e12.5,A,e12.5,A,e12.5)') 'NM:',iter,'  ', fs(ib),'  ', fs(iw),'  ', dist

            !Check whether length of simplex sides from best point are less than xtoler
            dist = 0.d0
            ! do i = 2,n+1
            !     dist = max(dist,sqrt(sum((xs(:,i)-xs(:,ib))**2)))
            ! end do
            do i = 1,n+1
                if (i==ib) cycle
                dist = max(dist,sqrt(sum((xs(:,i)-xs(:,ib))**2)))                
            end do

            ! Check for convergence or end of loop
            if (dist<xtoler .or. iter==maxiter) then
                if (printmod>0) write(*,'(A,i6,A,e12.5,A,e12.5,A,e12.5)') 'NM:',iter,'  ', fs(ib),'  ', fs(iw),'  ', dist
                ! Finished
                x = xs(:,ib)
                return
            end if

            !Try to replace the worst point, n+1, by another better point

            !Calc center of gravity of n other pts
            ! x_gravity = sum(xs(:,1:n),2)/real(n,8) 
            x_gravity = 0d0
            do i = 1,n+1
                if (i==iw) cycle ! exclude the worst
                x_gravity = x_gravity + xs(:,i) ! sum(xs(:,1:n),2)
            end do
            x_gravity = x_gravity/n

            !Reflect xr = xg + alpha*(xg-x(n+1)) and calculate value
            x_reflect = x_gravity + alpha*(x_gravity-xs(:,iw))
            f_reflect = f(x_reflect,work)
            if (.not. doNotTryReflect) then
                f_reflect = f(x_reflect,work)
            else
                f_reflect = huge(0d0)
                doNotTryReflect = .false.
            end if
            
            !Determine what to do with reflected point
            !if (fs(ib)<f_reflect .and. f_reflect<fs(ip)) then ! The old algorithm had this
            if (fs(ib)<f_reflect .and. f_reflect<fs(iw)) then 
                ! NOTE: next time, reflect will not satisfy this criterion---could optimize by avoiding it.
                ! If this is true, then next time reflect is tried, it will be the old point.
                ! We already know this is worse -- so don't waste an feval!
                if (f_reflect>=fs(ip)) then
                    doNotTryReflect = .true.
                end if 

                !Reflect: If f(x1)>f(xr)>f(xn) then replace and continue
                xs(:,iw) = x_reflect
                fs(iw) = f_reflect

            elseif (f_reflect>=fs(ip)) then

                ! If f_reflect is worst point, generate a new point by contraction
                x_contract = x_gravity + gamma*(xs(:,iw)-x_gravity)
                f_contract = f(x_contract,work)

                if (f_contract<fs(iw)) then

                    ! Contract: found a better point, so use it
                    xs(:,iw) = x_contract
                    fs(iw) = f_contract
                
                else
                
                    ! Contract all: no better point found, contract the entire simplex toward the best point
!                     do i = 2,n+1                        
!                         xs(:,i) = xs(:,ib) + eta*(xs(:,i)-xs(:,ib))
!                         fs(i) = f(xs(:,i),work)
!                     end do
                    do i = 1,n+1                        
                        if (i==ib) cycle
                        xs(:,i) = xs(:,ib) + eta*(xs(:,i)-xs(:,ib))
                        fs(i) = f(xs(:,i),work)
                    end do

                end if 

            elseif (f_reflect<=fs(ib)) then
                
                ! If f_reflect is best point, then try expanding in this direction for further improve
                x_expand = x_reflect + beta*(x_reflect-x_gravity)
                f_expand = f(x_expand,work)

                if (f_expand<f_reflect) then
                    ! Expand
                    xs(:,iw) = x_expand
                    fs(iw) = f_expand
                else
                    ! Reflect
                    xs(:,iw) = x_reflect
                    fs(iw) = f_reflect
                end if
            else 
                !Case not found
                print*,'WARNING: simplex: case not found! returning initial point.'
                print*,'xs values',xs
                print*,'fs values',fs
                return
            end if

        end do

    end subroutine 

    ! Solves the following bellman routines
    ! 
    ! If option==0, then solves a'_g a'_b unrestricted
    ! If option==1, then solves a'_g=a'_b (bond only)
    ! If option==2, then solves a'_g=a'_b whenever negative
    ! 
    subroutine compute_vR_core_newest(vR,cR,apR,CV,qap,endow,gA,csnEquiv,nM,nZ,option)
        use mod_matlab
        use mod_param, only: nA,nE,nH,death,nAmm
        implicit none
        integer, intent(in) :: nM,nZ
        real(8), intent(out), target :: vR(nA,nE,nH,*), cR(nA,nE,nH,*) ! (a,e,h,m)
        real(8), intent(out), target :: apR(nA,nE,nH,nM,*) ! (a,e,h,m,z)
        real(8), intent(in), target :: CV(nA,nE,nH,nM,*), qap(nA,nE,nH,nM,*) ! (a,e,h,m,z)
        real(8), intent(in), target :: endow(nE,nH,*) ! (e,h,m)
        real(8), intent(in), target :: gA(nA)
        real(8), intent(in) :: csnEquiv
        integer, intent(in) :: option(nE)
        ! local
!         integer, parameter :: factor = 1
!         integer, parameter :: nAP = 1 + factor*(nA-1) 
        integer, allocatable :: apitmp(:,:,:,:,:)
        integer :: iAptmp, iAps(nZ), j,iA,iE,iH,iM,iZ, iaplb, iapub !, ielb0, ieub0, ielb1, ieub1, ielb2, ieub2
        real(8) :: ctmp,vtmp,aplbsearch,apubsearch
!         real(8), allocatable :: gAP(:),mlo(:)
!         integer, allocatable :: ilo(:)
!         integer :: iAPeq0 !,flag
        integer :: iAeq0 !,flag
        type(workdef) :: work

        allocate(apitmp(nA,nE,nH,nM,nZ))
 !        allocate(gAP(nAP),mlo(nAP))
 !        allocate(ilo(nAP))

        ! Construct a finer gA grid by placing factor points between each interval
 !        do iA = 1,nA-1
 !            do j = 1,factor
 !                iAptmp = j + (iA-1)*factor
 !                gAP(iAptmp) = gA(iA) + (gA(iA+1)-gA(iA))*(j-1)/real(factor,8)
 !                mlo(iAptmp) = 1d0 - (j-1)/real(factor,8)
 !                ilo(iAptmp) = iA
 !            end do
 !        end do
 !        gAP(nAP) = gA(nA)
 !        mlo(nAP) = 0d0
 !        ilo(nAP) = nA-1
 !        iAPeq0 = 1 + (nAmm)*factor

        iAeq0 = nAmm + 1

!         if (any(diff(gAP)<0d0)) STOP 'gAP is not increasing'
!         do iAptmp = 1,nAP
!             if (gAP(iAptmp)<gA(ilo(iAptmp)) .or. gAP(iAptmp)>gA(ilo(iAptmp)+1)) STOP 'error in ilo construction'
!             if (abs(-gAP(iAptmp) + gA(ilo(iAptmp))*mlo(iAptmp) + gA(ilo(iAptmp)+1)*(1d0-mlo(iAptmp)))>2d0*epsilon(0d0)*abs(gAP(iAptmp))) then
!                 print*,gAP(iAptmp) , gA(ilo(iAptmp))*mlo(iAptmp) + gA(ilo(iAptmp)+1)*(1d0-mlo(iAptmp))
!                 STOP 'error in mlo construction'
!             end if
!         end do

        if (nZ>2) STOP 'can only handle nZ<=2'

        vR(:,:,:,1:nM) = death
        cR(:,:,:,1:nM) = 0d0
        apR(:,:,:,:,1:nZ) = 0d0
        apitmp = iAeq0

        if (nZ==1) then
            !$OMP PARALLEL DO DEFAULT(SHARED) PRIVATE(iM,iH,iE,iaplb,iapub,iA,iAptmp,ctmp,vtmp,aplbsearch,apubsearch,work)
            do iE = 1,nE
                do iM = 1,nM
                    do iH = 1,nH
                        if (iH==1) then
                            iaplb = 1
                            iapub = nA
                        else
                            iaplb = iAeq0
                            iapub = nA
                        end if
                        do iA = 1,nA

                            do iAptmp = iaplb,iapub
                                ctmp = gA(iA) + endow(iE,iH,iM) - qap(iAptmp,iE,iH,iM,1)
                                if (ctmp>0d0) then
                                    vtmp = -csnEquiv/ctmp + CV(iAptmp,iE,iH,iM,1)
                                    if (vtmp>=vR(iA,iE,iH,iM)) then
                                        vR(iA,iE,iH,iM) = vtmp
                                        cR(iA,iE,iH,iM) = ctmp
                                        apitmp(iA,iE,iH,iM,1) = iAptmp
                                    end if
                                elseif (iAptmp>iAeq0) then
                                    ! any higher a' will result in negative consumption
                                    exit
                                end if
                            end do
                            apR(iA,iE,iH,iM,1) = gA(apitmp(iA,iE,iH,iM,1)) 

                        end do
                    end do
                end do
                do iM = 1,nM
                    do iH = nH,1,-1
                        if (iH==1) then
                            aplbsearch = gA(1)
                            apubsearch = gA(nA)
                        else
                            aplbsearch = 0d0
                            apubsearch = gA(nA)
                        end if
                        do iA = 1,nA

                            work%gA => gA
                            work%qap => qap(:,:,:,:,1:nZ)
                            work%CV => CV(:,:,:,:,1:nZ)
                            work%endow => endow(:,:,1:nM)
                            work%aplbsearch = aplbsearch
                            work%apubsearch = apubsearch
                            work%csnEquiv = csnEquiv
                            work%iA = iA
                            work%iE = iE
                            work%iH = iH
                            work%iM = iM
                            work%nZ = nZ

                            ! subroutine set_work(iA,iE,iH,iM,nZ,gA,qap,CV,endow,aplbsearch,apubse)
                            ! Do a search starting from best point on the grid
                            ! If h=0, check h=1 to make sure v(h=0)>=v(h=1)
                            if (iH==1) then
                                if (-negv(apR(iA,iE,iH+1,iM,1:1),work)>-negv(apR(iA,iE,iH,iM,1:1),work)) then
                                    apR(iA,iE,iH,iM,1:1) = apR(iA,iE,iH+1,iM,1:1)
                                end if
                            end if
                            ! If iA>1, check iA-1 to make sure v(a) is increasing
                            if (iA>1) then
                                if (-negv(apR(iA-1,iE,iH,iM,1:1),work)>-negv(apR(iA,iE,iH,iM,1:1),work)) then
                                    apR(iA,iE,iH,iM,1:1) = apR(iA-1,iE,iH,iM,1:1)
                                end if
                            end if
                            ! call sub_simplex_core(apR(iA,iE,iH,iM,1:nZ),negv,work,e=[1d0],xtoler=1d-4,maxiter=10000,printmod=merge(1,-1,iA==iAeq0))
                            call sub_simplex_core(apR(iA,iE,iH,iM,1:nZ),negv,work,e=[1d0],xtoler=1d-4,maxiter=10000,printmod=-1)
                            call checkap(vR(iA,iE,iH,iM),cR(iA,iE,iH,iM),apR(iA,iE,iH,iM,1:nZ),work)
                        end do
                    end do
                end do
            end do
            !$OMP END PARALLEL DO

        else
            !$OMP PARALLEL DO DEFAULT(SHARED) PRIVATE(iM,iE,iH,iA,iaplb,iapub,iAps,ctmp,vtmp,aplbsearch,apubsearch,work)
            do iM = 1,nM
                do iE = 1,nE
                    do iH = 1,nH
                        if (iH==1) then
                            iaplb = 1
                            iapub = nA
                        else
                            iaplb = iAeq0
                            iapub = nA
                        end if
                        do iA = 1,nA

                            iAps = iaplb ! iAps is a vector
                            do while (iAps(2)<=iapub) 

                                ! Check value at iAps
                                ctmp = gA(iA) + endow(iE,iH,iM) - qap(iAps(1),iE,iH,iM,1) - qap(iAps(2),iE,iH,iM,2)
                                if (ctmp>0d0) then
                                    vtmp = -csnEquiv/ctmp + CV(iAps(1),iE,iH,iM,1) + CV(iAps(2),iE,iH,iM,2)
                                    if (vtmp>=vR(iA,iE,iH,iM)) then
                                        vR(iA,iE,iH,iM) = vtmp
                                        cR(iA,iE,iH,iM) = ctmp
                                        apitmp(iA,iE,iH,iM,1) = iAps(1)
                                        apitmp(iA,iE,iH,iM,2) = iAps(2)
                                    end if
                                end if
                                    
                                ! Get next iAps
                                if (option(iE)==0) then

                                    ! if positive and c<0, higher a' will result in negative consumption, so cycle
                                    if (iAps(1)>=iAeq0 .and. ctmp<0d0) then
                                        iAPs(1) = iaplb
                                        iAPs(2) = iAPs(2) + 1
                                        cycle
                                    end if

                                    ! unrestricted
                                    iAps(1) = iAps(1) + 1
                                    if (iAps(1)>iapub) then
                                        iAps(1) = iaplb
                                        iAps(2) = iAps(2) + 1
                                    end if

                                elseif (option(iE)==1) then
                                    ! bond only

                                    ! if positive and c<0, higher a' will result in negative consumption, so cycle
                                    if (iAps(1)>=iAeq0 .and. ctmp<0d0) then
                                        exit 
                                    end if

                                    iAps = iAps + 1

                                elseif (option(iE)==2) then

                                    ! if positive and c<0, higher a' will result in negative consumption, so cycle
                                    if (iAps(1)>=iAeq0 .and. ctmp<0d0) then
                                        iAPs(1) = iAeq0 
                                        iAPs(2) = iAPs(2) + 1
                                        cycle
                                    end if

                                    ! bond when negative
                                    if (iAps(1)<iAeq0) then
                                        iAps = iAps + 1
                                    else
                                    ! anything if positive
                                        iAps(1) = iAps(1) + 1
                                        if (iAps(1)>iapub) then
                                            iAps(1) = iAeq0 ! restrict to A+ here
                                            iAps(2) = iAps(2) + 1
                                        end if
                                    end if
                                end if
                            end do

                            apR(iA,iE,iH,iM,1) = gA(apitmp(iA,iE,iH,iM,1))
                            apR(iA,iE,iH,iM,2) = gA(apitmp(iA,iE,iH,iM,2))
                        end do
                    end do
                    ! end grid search, now begin simplex search
                    do iH = nH,1,-1
                        do iA = 1,nA

                            ! Set search bounds each time as overwritten below
                            if (iH==1) then
                                aplbsearch = gA(1)
                                apubsearch = gA(nA)
                            else
                                aplbsearch = 0d0
                                apubsearch = gA(nA)
                            end if

                            work%gA => gA
                            work%qap => qap(:,:,:,:,1:nZ)
                            work%CV => CV(:,:,:,:,1:nZ)
                            work%endow => endow(:,:,1:nM)
                            work%aplbsearch = aplbsearch
                            work%apubsearch = apubsearch
                            work%csnEquiv = csnEquiv
                            work%iA = iA
                            work%iE = iE
                            work%iH = iH
                            work%iM = iM
                            work%nZ = nZ

                            ! Do a search starting from best point on the grid
                            ! If h=0, check h=1 to make sure v(h=0)>=v(h=1)
                            if (iH==1) then
                                if (-negv(apR(iA,iE,iH+1,iM,1:nZ),work)>-negv(apR(iA,iE,iH,iM,1:nZ),work)) then
                                    apR(iA,iE,iH,iM,1:nZ) = apR(iA,iE,iH+1,iM,1:nZ)
                                end if
                            end if
                            ! If iA>1, check iA-1 to make sure v(a) is increasing
                            if (iA>1) then
                                if (-negv(apR(iA-1,iE,iH,iM,1:nZ),work)>-negv(apR(iA,iE,iH,iM,1:nZ),work)) then
                                    apR(iA,iE,iH,iM,1:nZ) = apR(iA-1,iE,iH,iM,1:nZ)
                                end if
                            end if

                            if (option(iE)==0) then
                                ! unrestricted
                                call sub_simplex_core(apR(iA,iE,iH,iM,1:nZ),negv,work,e=[1d0,1d0],xtoler=1d-4,maxiter=10000,printmod=-1)
                            elseif (option(iE)==1) then
                                ! bond
                                call sub_simplex_core(apR(iA,iE,iH,iM,1:1),negv,work,e=[1d0],xtoler=1d-4,maxiter=10000,printmod=-1)
                                apR(iA,iE,iH,iM,2:nZ) = apR(iA,iE,iH,iM,1)
                            elseif (option(iE)==2) then
                                ! bond if negative, unrestricted if positive
                                if (apR(iA,iE,iH,iM,1)<0d0) then
                                    work%apubsearch = 0d0
                                    call sub_simplex_core(apR(iA,iE,iH,iM,1:1),negv,work,e=[1d0],xtoler=1d-4,maxiter=10000,printmod=-1)
                                    apR(iA,iE,iH,iM,2:nZ) = apR(iA,iE,iH,iM,1)
                                else
                                    work%aplbsearch = 0d0
                                    call sub_simplex_core(apR(iA,iE,iH,iM,1:nZ),negv,work,e=[1d0,1d0],xtoler=1d-4,maxiter=10000,printmod=-1)
                                end if
                            else
                                STOP 'option not found'
                            end if
                            call checkap(vR(iA,iE,iH,iM),cR(iA,iE,iH,iM),apR(iA,iE,iH,iM,1:nZ),work)

                        end do
                    end do
                end do
            end do
            !$OMP END PARALLEL DO

        end if

    end subroutine 


    function negv(aptmp,work)
        implicit none
        real(8), intent(in) :: aptmp(:)
        real(8) :: negv
        type(workdef), intent(in) :: work
        !local 
        real(8) :: vtmp,ctmp
        call checkap(vtmp,ctmp,aptmp,work)
        negv = -vtmp
    end function

    subroutine checkap(vtmp,ctmp,aptmpin,w)
        use mod_param, only: death
        use mod_matlab
        implicit none
        real(8), intent(in) :: aptmpin(:)
        real(8), intent(out) :: vtmp,ctmp
        type(workdef), intent(in) :: w
        ! local
        real(8) :: mlo,aptmp(3)
        integer :: ilo,iZ,nZ

        nZ = w%nZ

        if (nZ>1 .and. size(aptmpin)==1) then
            aptmp(1:nZ) = aptmpin(1)
        else
            aptmp(1:nZ) = aptmpin
        endif 
        
        if (minval(aptmp(1:nZ))<w%aplbsearch .or. maxval(aptmp(1:nZ))>w%apubsearch) then
            vtmp = death
            ctmp = 0d0
            return 
        end if

        ctmp = w%gA(w%iA) + w%endow(w%iE,w%iH,w%iM)
        vtmp = 0d0
        do iZ = 1,nZ
            call interp1qlocweight(ilo,mlo,aptmp(iZ),w%gA)
            ctmp = ctmp - mlo*w%qap(ilo,w%iE,w%iH,w%iM,iZ) - (1d0-mlo)*w%qap(ilo+1,w%iE,w%iH,w%iM,iZ)
            vtmp = vtmp + mlo*w%CV(ilo,w%iE,w%iH,w%iM,iZ) + (1d0-mlo)*w%CV(ilo+1,w%iE,w%iH,w%iM,iZ)
        end do

        if (ctmp>0d0) then
            vtmp = vtmp - w%csnEquiv/ctmp 
        else
            vtmp = death
        end if

    end subroutine

    ! Solves the following bellman routines
    ! 
    ! If option==0, then solves a'_g a'_b unrestricted
    ! If option==1, then solves a'_g=a'_b (bond only)
    ! If option==2, then solves a'_g=a'_b whenever negative
    ! 
    subroutine compute_vR_core_new(vR,cR,apR,CV,qap,endow,gA,csnEquiv,nM,nZ,option)
        use mod_param, only: nA,nE,nH,death,nAmm
        use mod_valueopt
        implicit none
        integer, intent(in) :: nM,nZ
        real(8), intent(out) :: vR(nA,nE,nH,*), cR(nA,nE,nH,*) ! (a,e,h,m)
        real(8), intent(out) :: apR(nA,nE,nH,nM,*) ! (a,e,h,m,z)
        real(8), intent(in) :: CV(nA,nE,nH,nM,*), qap(nA,nE,nH,nM,*) ! (a,e,h,m,z)
        real(8), intent(in) :: endow(nE,nH,*) ! (e,h,m)
        real(8), intent(in) :: gA(nA)
        real(8), intent(in) :: csnEquiv
        integer, intent(in) :: option(nE)

        ! local
        real(8), allocatable :: CVbond(:,:,:,:), qapbond(:,:,:,:), endowrep(:,:,:,:), CVjoint(:,:,:,:,:), qapjoint(:,:,:,:,:)
        integer, allocatable :: apitmp(:,:,:,:,:)
        integer :: iA1,iA2, iaplb, iapub !, ielb0, ieub0, ielb1, ieub1, ielb2, ieub2

        allocate(CVbond(nA,nE,nH,nM),qapbond(nA,nE,nH,nM),endowrep(nA,nE,nH,nM))
        allocate(apitmp(nA,nE,nH,nM,nZ))

        CVbond = 0d0
        qapbond = 0d0
        do iZ = 1,nZ
            CVbond = CVbond + CV(:,:,:,:,iZ)
            qapbond = qapbond + qap(:,:,:,:,iZ)
        end do
        endowrep = spread(endow(:,:,1:nM),1,nA)

        if (nZ>2) STOP 'can only handle nZ<=2'

        if (nZ==2) then
            allocate(CVjoint(nA,nA,nE,nH,nM),qapjoint(nA,nA,nE,nH,nM))
            do iM = 1,nM
                do iH = 1,nH
                    do iE = 1,nE
                        do iA2 = 1,nA
                            do iA1 = 1,nA
                                CVjoint(iA1,iA2,iE,iH,iM) = CV(iA1,iE,iH,iM,1) + CV(iA2,iE,iH,iM,2)
                                qapjoint(iA1,iA2,iE,iH,iM) = qap(iA1,iE,iH,iM,1) + qap(iA2,iE,iH,iM,2)
                            end do
                        end do
                    end do
                end do
            end do
        end if

        csnEquivPrivate = csnEquiv
        call setPUcustom(PU)


        do iM = 1,nM
            do iH = 1,nH
                if (iH==1) then
                    iaplb = 1
                    iapub = nA
                else
                    iaplb = nAmm+1
                    iapub = nA
                end if

                do iE = 1,nE

                    if (option(iE)==1) then
                        ! bond only
                        call sub_bellman_discrete_core(vR(:,iE,iH,iM),cR(:,iE,iH,iM),apitmp(:,iE,iH,iM,1),CVbond(iaplb:iapub,iE,iH,iM),endowrep(:,iE,iH,iM),qapbond(iaplb:iapub,iE,iH,iM),nA,1,iapub-iaplb+1,-1d0,[1,1,0])
                        apR(:,iE,iH,iM,1) = gA(apitmp(:,iE,iH,iM,1))
                        do iZ = 2,nZ
                            apR(:,iE,iH,iM,iZ) = apR(:,iE,iH,iM,1)
                        end do
                    elseif (option(iE)==0 .or. option(iE)==2) then
                        ! unrestricted
                        call sub_bellman_discrete_choose2_core(vR(:,iE,iH,iM),cR(:,iE,iH,iM),apitmp(:,iE,iH,iM,1),apitmp(:,iE,iH,iM,2),&
                                    CVjoint(iaplb:iapub,iaplb:iapub,iE,iH,iM),endowrep(:,iE,iH,iM),qapjoint(iaplb:iapub,iaplb:iapub,iE,iH,iM),nA,1,iapub-iaplb+1,iapub-iaplb+1,-1d0,[1,1,3])
                    end if
                end do
            end do
        end do

    end subroutine 
    
    function PU(c) result(u)
        implicit none
        real(8), intent(in) :: c
        real(8) :: u

        u = -csnEquivPrivate/c

    end function 

    ! Solves the following bellman routines
    ! 
    ! If option==0, then solves a'_g a'_b unrestricted
    ! If option==1, then solves a'_g=a'_b (bond only)
    ! If option==2, then solves a'_g=a'_b whenever negative
    ! 
    subroutine compute_vR_core(vR,cR,apR,CV_,qap_,endow,gA_,csnEquiv_,nM,nZ_,option)
        use mod_param, only: nA,nE,nH,death,nAmm
        use mod_matlab, only: interp1q,linspace,bsearch,sort,disp,find1
        use mod_min1dim
        implicit none
        integer, intent(in) :: nM,nZ_
        real(8), intent(out) :: vR(nA,nE,nH,*), cR(nA,nE,nH,*) ! (a,e,h,m)
        real(8), intent(out) :: apR(nA,nE,nH,nM,*) ! (a,e,h,m,z)
        real(8), intent(in), target :: CV_(nA,nE,nH,nM,*), qap_(nA,nE,nH,nM,*) ! (a,e,h,m,z)
        real(8), intent(in) :: endow(nE,nH,*) ! (e,h,m)
        real(8), intent(in), target :: gA_(nA)
        real(8), intent(in) :: csnEquiv_
        integer, intent(in) :: option(nE)
        ! local
        real(8), allocatable :: qapL(:,:,:)
        real(8) :: vbest,vtmp,apbest,ytmp,vMtmp
        integer :: iA,iAp,flag,iaplb,iapub,iE_ 
        logical :: computeL,computeM
        integer :: nY


        nY = nA 

        ! Set module variables equal to small variables 
        nZ = nZ_
        csnEquiv = csnEquiv_
        
        if (any(isnan(qap_(:,:,:,:,1:nZ)))) STOP 'qap is nan'

        ! Assign pointers to large input arrays
        gA => gA_
        qap => qap_(:,:,:,:,1:nZ)
        CV => CV_(:,:,:,:,1:nZ)

        if (any(isnan(qap))) STOP 'qap is nan'
!         if (allocated(gA)) deallocate(gA); allocate(gA(nA)); gA = gA_
!         if (allocated(qap)) deallocate(qap); allocate(qap(nA,nE,nH,nM,nZ)); qap = qap_(:,:,:,:,1:nZ)
!         if (allocated(CV)) deallocate(CV); allocate(CV(nA,nE,nH,nM,nZ)); CV = CV_(:,:,:,:,1:nZ)

        ! Compute the cash at hand values
        !$OMP PARALLEL DO DEFAULT(SHARED) PRIVATE(vbest,vtmp,apbest,ytmp,vMtmp,iA,flag,iaplb,iapub,computeL,computeM,qapL,iAp)
        do iE_ = 1,nE

            iE = iE_

            allocate(gY(nY),vL(nY),vLnew(nY),apL(nY,nZ,nH),qapL(nY,nZ,nH),CVsumz(nA),qapsumz(nA),vM(nY),apM(nY,nH))

            do iM = 1,nM
                do iH = nH,1,-1 ! go in reverse order for ease in ensuring V(h=0)>V(h=1).

                    ! Custom grid so that the special case of cash at hand at a=0 is on the grid
                    !!! This makes things nice because then one choice in particular, a'=0, requires
                    !!! no interpolation. Consequently, vR(a>=0,h=1) is always greater than vD
                    !!! without having to jump through any hoops. This is a key theoretical result
                    !!! that is important to have true in the code also (especially since I allow
                    !!! default on positive values of a).
                    gY = gA + endow(iE,iH,iM)
                    gY(1) = gA(1)
                    gY(nY) = gA(nA)
                    gY = sort(gY)

!                     if (gY(nAmm+1)/=endow(iE,iH,iM)) STOP 'I assume gY(nAmm+1)=endow'

!                     ! Do not delete this initialization: if iH==2, vL(1:nAmm) is never written to, 
!                     ! but it needs to be negative infinite so the household never chooses in this region.
!                     vLnew = death
!                     vL = death
!                     vM = death

                    ! See whether to compute the vM value function and for what portfolio restrictions
                    select case (option(iE))
                        case (0)
                            computeM = .false.
                        case (1) 
                            iaplb = 1
                            iapub = nA
                            computeM = .true.
                        case (2) 
                            if (iH==1) then
                                iaplb = 1
                                iapub = nAmm+1
                                computeM = .true.
                            else
                                ! would be a'=0, but then computeL checks that anyway
                                computeM = .false.
                            end if
                    end select
                    if (iH==2) then
                        if (iaplb<nAmm+1) iaplb = nAmm+1 ! Force search only a'>=0
                    end if

                    if (computeM) then

                        aplb = gA(iaplb)
                        apub = gA(iapub)

                        CVsumz = sum(CV(:,iE,iH,iM,1:nZ),dim=2)
                        qapsumz = sum(qap(:,iE,iH,iM,1:nZ),dim=2)

                        do iY = 1,nY

                            ! Maximize choice of a'
                            apbest = 0d0
                            vbest = -negutil_m(apbest)
                            if (isnan(apbest)) STOP 'apbest is nan 1'

                            ! Do grid search to avoid local minima
                            do iAp = iaplb,iapub
                                ! vtmp = -negutil_m(gA(iAp))
                                vtmp = -negutil_m_ongrid(iAp)
                                if (vtmp>vbest) then
                                    vbest = vtmp
                                    apbest = gA(iAp)
                                end if
                            end do


                            ! If h=0, check this optimal policy against the best one found for h=1. Use the h=1 as an
                            ! initial guess to the simplex routine if better.
                            if (iH==1) then
                                vtmp = -negutil_m(apM(iY,2))
                                if (vtmp>vbest) then
                                    vbest = vtmp
                                    apbest = apM(iY,2)
                                end if
                            end if
                            if (isnan(apbest)) STOP 'apbest is nan 2'

                            ! If iY>1, then also compare against iY-1 so that the value function is increasing in y, 
                            ! as it should be.
                            if (iY>1) then
                                vtmp = -negutil_m(apM(iY-1,iH))
                                if (vtmp>vbest) then
                                    vbest = vtmp
                                    apbest = apM(iY-1,iH)
                                end if
                            end if
                            if (isnan(apbest)) STOP 'apbest is nan 3'

                            ! Use grid search as initial guess
                            call sub_simplex1dim(apbest,negutil_m,e=100d0,xtoler=1d-6,maxiter=1000,printmod=-1,flag=flag)
                            vbest = -negutil_m(apbest)
                            if (isnan(apbest)) STOP 'apbest is nan 4'

                            vM(iY) = vbest
                            apM(iY,iH) = apbest

                        end do

                        if (any(isnan(apM(:,iH)))) STOP 'apM is nan'

                    end if


                    ! See whether to compute the vL value function and for what portfolio restrictions
                    select case (option(iE))
                        case (0)
                            iaplb = 1
                            iapub = nA
                            computeL = .true.
                        case (1) 
                            computeL = .false.
                        case (2) 
                            iaplb = nAmm+1
                            iapub = nA
                            computeL = .true.
                        case default
                            STOP 'error: option not found'
                    end select

                    if (iH==2) then
                        if (iaplb<nAmm+1) iaplb = nAmm+1 ! Force search only a'>=0
                    end if

                    if (computeL) then

                            aplb = gA(iaplb)
                            apub = gA(iapub)

                            ! Initialize L to U(y,s)
                            where (gY>0d0) 
                                vL = -csnEquiv/gY
                            elsewhere
                                vL = death
                            end where

                            ! For each z', solve one problem
                            do iZ = nZ,1,-1

                                do iY = 1,nY

                                    ! Maximize choice of a' using a grid
                                    apbest = 0d0 ! must come first
                                    vbest = -negutil_l(apbest)

                                    ! Do grid search to avoid local min
                                    do iAp = iaplb,iapub
                                        ! vtmp = -negutil_l(gA(iAp))
                                        vtmp = -negutil_l_ongrid(iAp)
                                        if (vtmp>vbest) then
                                            vbest = vtmp
                                            apbest = gA(iAp)
                                        end if
                                    end do

                                    ! If h=0, check this optimal policy against the best one found for h=1. Use the h=1 as an
                                    ! initial guess to the simplex routine if better.
                                    if (iH==1) then
                                        vtmp = -negutil_l(apL(iY,iZ,2))
                                        if (vtmp>vbest) then
                                            vbest = vtmp
                                            apbest = apL(iY,iZ,2)
                                        end if
                                    end if

                                    ! Also, want to ensure L does at least as well as M. So, check the M policy against the L
                                    if (computeM) then
                                        vtmp = -negutil_l(apM(iY,iH))
                                        if (vtmp>vbest) then
                                            vbest = vtmp
                                            apbest = apM(iY,iH)
                                        end if
                                    end if

                                    ! If iY>1, then also compare against iY-1 so that the value function is increasing in y, 
                                    ! as it should be.
                                    if (iY>1) then
                                        vtmp = -negutil_l(apL(iY-1,iZ,iH))
                                        if (vtmp>vbest) then
                                            vbest = vtmp
                                            apbest = apL(iY-1,iZ,iH)
                                        end if
                                    end if

                                    ! Use grid search as initial guess
                                    call sub_simplex1dim(apbest,negutil_l,e=100d0,xtoler=1d-6,maxiter=1000,printmod=-1,flag=flag)
                                    vbest = -negutil_l(apbest)

                                    ! Save best
                                    vLnew(iY) = vbest
                                    apL(iY,iZ,iH) = apbest
                                    qapL(iY,iZ,iH) = interp_if_off_grid(gA,qap(:,iE,iH,iM,iZ),apbest)

                                end do
                                vL = vLnew
                            end do

!                             call disp('vL',vL)
!                             call disp('apL',apL)

                            ! Now, vL has the cash-at-hand value function that can be used to compute vR

                            if (any(isnan(apL(:,:,iH)))) STOP 'apL is nan'
                    end if




                    ! With vL and vM known (when necessary), recover the value and policy functions for vR
                    do iA = merge(1,nAmm+1,iH==1),nA
                        

                        ! Default to L function and override with M 
                        vR(iA,iE,iH,iM) = -huge(0d0)
                        apR(iA,iE,iH,iM,1:nZ) = 0d0
                        cR(iA,iE,iH,iM) = -1d0

                        if (computeL) then

                            ytmp = gA(iA) + endow(iE,iH,iM)

                            vR(iA,iE,iH,iM) = interp_if_off_grid(gY,vL,ytmp)
                            do iZ = 1,nZ
                                apR(iA,iE,iH,iM,iZ) = interp_if_off_grid(gY,apL(:,iZ,iH),ytmp)
!                                 ytmp = ytmp - interp_if_off_grid(gY,qapL(:,iZ,iH),ytmp)
                                ytmp = ytmp - interp_if_off_grid(gA,qap(:,iE,iH,iM,iZ),apR(iA,iE,iH,iM,iZ))
                                if (isnan(ytmp)) STOP 'ytmp is nan'
                            end do
                            cR(iA,iE,iH,iM) = ytmp

                            ! Compute the utility according to the policy function rather than vL
                            ! This is to get a more consistent comparison: vM only uses 1 interpolation, vY uses #Z
                            ! vR(iA,iE,iH,iM) = -csnEquiv/cR(iA,iE,iH,iM) 
                            ! do iZ = 1,nZ
                            !     vR(iA,iE,iH,iM) = vR(iA,iE,iH,iM) + interp_if_off_grid(gA,CV(:,iE,iH,iM,iZ),apR(iA,iE,iH,iM,iZ))
                            ! end do

                            if (isnan(apR(iA,iE,iH,iM,1))) STOP '1: apR(a,e,h,m,1) has nan'
                            if (isnan(apR(iA,iE,iH,iM,nZ))) STOP '1: apR(a,e,h,m,nZ) has nan'


                        end if

                        if (computeM .and. aplb/=apub) then

                            ytmp = gA(iA) + endow(iE,iH,iM)

                            vMtmp = interp_if_off_grid(gY,vM,ytmp)     

                            if (vMtmp>vR(iA,iE,iH,iM)) then

                                apR(iA,iE,iH,iM,1:nZ) = interp_if_off_grid(gY,apM(:,iH),ytmp)
                                vR(iA,iE,iH,iM) = interp_if_off_grid(gY,vM,ytmp)
                                cR(iA,iE,iH,iM) = ytmp - interp_if_off_grid(gA,qapsumz,apR(iA,iE,iH,iM,1)) ! here, all a' are the same

                                if (isnan(apR(iA,iE,iH,iM,1))) STOP '2: apR(a,e,h,m,1) has nan'
                                if (isnan(apR(iA,iE,iH,iM,nZ))) STOP '2: apR(a,e,h,m,nZ) has nan'


                            end if

                        end if

                    end do

                    ! To the off equilibrium states, write some value
                    if (iH==nH) then
                        do iA = 1,nAmm
                            vR(iA,iE,iH,iM) = 0d0
                            cR(iA,iE,iH,iM) = 0d0
                            apR(iA,iE,iH,iM,1:nZ) = 0d0
                        end do
                    end if
                    
                end do
            end do

            deallocate(gY,vL,vLnew,apL,qapL,CVsumz,qapsumz,vM,apM)

        end do
        !$OMP END PARALLEL DO

        ! Where v(h=0) and v(h=1) are very close together, ensure v(h=0)>=v(h=1)
        where (abs(vR(:,:,2,1:nM)-vR(:,:,1,1:nM))<1d-12) vR(:,:,1,1:nM) = max(vR(:,:,1,1:nM),vR(:,:,2,1:nM)) + 1d-14


    end subroutine 
    
    function negutil_l(ap) result(neg_vy)
        implicit none
        real(8), intent(in) :: ap
        real(8) :: neg_vy
        neg_vy = negutil_l_core(ap)
    end function
    function negutil_l_ongrid(iap) result(neg_vy)
        implicit none
        integer, intent(in) :: iap
        real(8) :: neg_vy
        neg_vy = negutil_l_core(iap=iap)
    end function

    function negutil_l_core(ap,iap) result(neg_vy)
        use mod_param, only: death
        implicit none
        real(8), intent(in), optional :: ap
        integer, intent(in), optional :: iap
        real(8) :: neg_vy
        ! local
        real(8) :: yptmp, ypptmp, apptmp, qaptmp, CVtmp, qapptmp, CVptmp

        logical :: offgrid 

        offgrid = present(ap)

        if (offgrid) then
            if (ap<aplb .or. ap>apub) then
                neg_vy = -death
                return
            end if

            ! Interpolate to get qap and CV at ap
            call interp_2_if_off_grid(qaptmp,CVtmp,gA,qap(:,iE,iH,iM,iZ),CV(:,iE,iH,iM,iZ),ap) 

        else
            
            qaptmp = qap(iap,iE,iH,iM,iZ)
            CVtmp = CV(iap,iE,iH,iM,iZ)

        end if


        yptmp = gY(iY) - qaptmp

        ! Rather than interpolating the nonlinear value function to obtain a value of vY, 
        ! instead interpolate the future nearly linear policy functions to recover it 
        ! more precisely

        if (iZ==nZ) then
            ! If iZ==nZ, then we know what vL is without interpolation. This really helps numerical accuracy.
            
            ! y' is csn
            if (yptmp>0d0) then
                ! neg_vy = -( interp_if_off_grid(gA,CV(:,iE,iH,iM,iZ),ap) - csnEquiv/yptmp)
                neg_vy = -( CVtmp - csnEquiv/yptmp)
            else
                neg_vy = -death
            end if
            
        elseif (iZ==nZ-1) then
            ! If iZ==nZ-1, recover a' and a'' which then gives c. This really helps numerical accuracy.
            

            ! the policy function for iZ=nZ at yptmp 
            apptmp = interp_if_off_grid(gY,apL(:,iZ+1,iH),yptmp)

            call interp_2_if_off_grid(qapptmp,CVptmp,gA,qap(:,iE,iH,iM,iZ+1),CV(:,iE,iH,iM,iZ+1),apptmp) 

            ! calculate what y'' would be
            ypptmp = yptmp - qapptmp

            ! y'' is csn
            if (ypptmp>0d0) then
                ! neg_vy = -( interp_if_off_grid(gA,CV(:,iE,iH,iM,iZ),ap) + interp_if_off_grid(gA,CV(:,iE,iH,iM,iZ+1),apptmp) - csnEquiv/ypptmp)
                neg_vy = -( CVtmp + CVptmp - csnEquiv/ypptmp)
            else
                neg_vy = -death
            end if

        else
            
            ! in the general case, which i don't need to use yet, just interp the vL function
            
            ! neg_vy = -( interp_if_off_grid(gA,CV(:,iE,iH,iM,iZ),ap) + interp_if_off_grid(gY,vL,yptmp))
            neg_vy = -( CVtmp + interp_if_off_grid(gY,vL,yptmp))
            
        end if

    end function 

    function negutil_m(ap) result(neg_vm)
        use mod_param, only: death
        implicit none
        real(8), intent(in) :: ap
        real(8) :: neg_vm
        ! local
        real(8) :: qaptmp, CVtmp, ctmp

        if (ap<aplb .or. ap>apub) then
            neg_vm = -death
            return
        end if

        ! Interpolate to get qap and CV at ap
        call interp_2_if_off_grid(qaptmp,CVtmp,gA,qapsumz,CVsumz,ap) 

        ctmp = gY(iY) - qaptmp
        if (ctmp>0d0) then
            neg_vm = csnEquiv/ctmp - CVtmp ! -(-csnEquiv/ctmp + CVtmp)
        else
            neg_vm = -death
        end if

    end function 
    function negutil_m_ongrid(iap) result(neg_vm)
        use mod_param, only: death
        implicit none
        integer, intent(in) :: iap
        real(8) :: neg_vm
        ! local
        real(8) :: ctmp

        ctmp = gY(iY) - qapsumz(iap)
        if (ctmp>0d0) then
            neg_vm = csnEquiv/ctmp - CVsumz(iap) ! -(-csnEquiv/ctmp + CVtmp)
        else
            neg_vm = -death
        end if

    end function 


    function interp_if_off_grid(x,y,xi) result(yi)
        use mod_matlab, only: bsearch
        implicit none
        real(8), intent(in) :: x(:), y(:), xi
        real(8) :: yi
        ! local
        real(8) :: mlo
        integer :: ilo

        call bsearch(ilo,xi,x)
        mlo = (x(ilo+1)-xi)/(x(ilo+1)-x(ilo))

        if (abs(mlo-1d0)<1d-10) then
            yi = y(ilo)
        elseif (abs(mlo)<1d-10) then
            yi = y(ilo+1)
        else
            yi = y(ilo+1) + (y(ilo)-y(ilo+1))*mlo
        end if

    end function 

    subroutine interp_2_if_off_grid(yi,yi2,x,y,y2,xi) 
        use mod_matlab, only: bsearch
        implicit none
        real(8), intent(in) :: x(:), y(:), y2(:), xi
        real(8), intent(out) :: yi, yi2
        ! local
        real(8) :: mlo
        integer :: ilo

        call bsearch(ilo,xi,x)
        mlo = (x(ilo+1)-xi)/(x(ilo+1)-x(ilo))

        if (abs(mlo-1d0)<1d-10) then
            yi = y(ilo)
            yi2 = y2(ilo)
        elseif (abs(mlo)<1d-10) then
            yi = y(ilo+1)
            yi2 = y2(ilo+1)
        else
            yi = y(ilo+1) + (y(ilo)-y(ilo+1))*mlo
            yi2 = y2(ilo+1) + (y2(ilo)-y2(ilo+1))*mlo
        end if

    end subroutine

    subroutine compute_vR_BC(vR,cR,apR,CV,qap,endow,gA,csnEquiv,option)
        use mod_param, only: nA,nE,nH,nM,nZ
        implicit none
        real(8), intent(out) :: vR(:,:,:,:), cR(:,:,:,:), apR(:,:,:,:,:)
        real(8), intent(in) :: CV(:,:,:,:,:), qap(:,:,:,:,:), endow(:,:,:)
        real(8), intent(in) :: gA(:), csnEquiv
        integer, intent(in) :: option(:)

        ! Check shapes
        if (any(shape(vR)/=[nA,nE,nH,nM])) STOP 'vR: bad shape'
        if (any(shape(cR)/=[nA,nE,nH,nM])) STOP 'cR: bad shape'
        if (any(shape(apR)/=[nA,nE,nH,nM,nZ])) STOP 'apR: bad shape'
        if (any(shape(CV)/=[nA,nE,nH,nM,nZ])) STOP 'CV: bad shape'
        if (any(shape(qap)/=[nA,nE,nH,nM,nZ])) STOP 'qap: bad shape'
        if (any(shape(endow)/=[nE,nH,nM])) STOP 'endow: bad shape'
        if (any(shape(gA)/=[nA])) STOP 'gA: bad shape'
        if (any(shape(option)/=[nE])) STOP 'option: bad shape'

        ! Call the core routine
        call compute_vR_core_newest(vR,cR,apR,CV,qap,endow,gA,csnEquiv,nM,nZ,option)
        
    end subroutine 
    subroutine compute_vR_SS(vR,cR,apR,CV,qap,endow,gA,csnEquiv,option)
        use mod_param, only: nA,nE,nH
        implicit none
        real(8), intent(out) :: vR(:,:,:), cR(:,:,:), apR(:,:,:)
        real(8), intent(in) :: CV(:,:,:), qap(:,:,:), endow(:,:)
        real(8), intent(in) :: gA(:), csnEquiv
        integer, intent(in) :: option(:)

        ! Check shapes
        if (any(shape(vR)/=[nA,nE,nH])) STOP 'vR: bad shape'
        if (any(shape(cR)/=[nA,nE,nH])) STOP 'cR: bad shape'
        if (any(shape(apR)/=[nA,nE,nH])) STOP 'apR: bad shape'
        if (any(shape(CV)/=[nA,nE,nH])) STOP 'CV: bad shape'
        if (any(shape(qap)/=[nA,nE,nH])) STOP 'qap: bad shape'
        if (any(shape(endow)/=[nE,nH])) STOP 'endow: bad shape'
        if (any(shape(gA)/=[nA])) STOP 'gA: bad shape'
        if (any(shape(option)/=[nE])) STOP 'option: bad shape'

        ! Call the core routine
        call compute_vR_core_newest(vR,cR,apR,CV,qap,endow,gA,csnEquiv,1,1,option)
        
    end subroutine 

end module 
