!This file contains modules of data.  
!As even Fortran 2003 doesn't allow the passing of internal functions, 
!these often need to be separate to accomodate nonlinear solvers.
module mod_data
    implicit none

    ! Grids and shocks
    real(8), allocatable, dimension(:) :: gA
    real(8), allocatable, dimension(:,:) :: gE
    real(8), allocatable, dimension(:,:,:,:,:) :: trans_e
    real(8), allocatable, dimension(:,:,:,:,:) :: trans_ez
    real(8), allocatable, dimension(:,:) :: init_dsn_e
    real(8), allocatable, dimension(:,:) :: invar_dsn_e_SS 
    real(8), allocatable, dimension(:) :: init_dsn_e_SS
    real(8), allocatable, dimension(:,:,:) :: trans_e_SS
    
    !Prices / laws of motion
    real(8), allocatable, dimension(:) :: w !wage
    real(8), allocatable, dimension(:,:) :: qbar !riskfree arrow prices (m,z)
    real(8), allocatable, dimension(:,:,:,:) :: q_SS
    real(8), allocatable, dimension(:,:,:,:,:,:) :: q !arrow price schedules (a',e,age,h,m,z)
    real(8), allocatable, dimension(:,:) :: trans_m !Pr(m'|m) joint probability of moving from (z,Moments) to (z,Moments')
    real(8), allocatable, dimension(:,:,:) :: trans_m_cdnal_zp !Pr(m'|m,z')

    !Values, policies, and distributions
    real(8), allocatable, dimension(:,:,:,:) :: v_SS,c_SS,ap_SS
    logical, allocatable, dimension(:,:,:,:) :: d_SS
    real(8), allocatable, dimension(:,:,:,:) :: invar_dsn_SS
    real(8), allocatable, dimension(:,:,:,:,:) :: v,c,apg,apb
    logical, allocatable, dimension(:,:,:,:,:) :: d
    real(8), allocatable, dimension(:,:,:) :: EEE_SS

contains

subroutine set_gA() 
    use mod_matlab
    use mod_param, only: nA,nAmm
    implicit none
    real(8) :: min_a, max_a, kappa_borrow, kappa_save
    !local

    min_a = -20.d0 
    max_a = 3000.d0 

    !Setup borrowing grid
    kappa_borrow = .20d0  ! Was .3
    gA(1:nAmm+1) = sort(-(exp(linspace(log(0.d0+kappa_borrow),log(-min_a + kappa_borrow),nAmm+1)) - kappa_borrow))

    ! Setup savings grid in a unified way
    kappa_save = .1d0
    gA(nAmm+1:nA) = exp(linspace(log(0.d0+kappa_save),log(max_a+kappa_save),nA-nAmm))-kappa_save
    
    ! Make sure nAmm+1 is exactly equal to 0
    gA(nAmm+1) = 0.d0    
   
end subroutine

subroutine print_data_SS()
    use mod_myprint
    implicit none

    call myprint('gA_SS',gA)        
    call myprint('gE_SS',gE)
    call myprint('init_dsn_e_SS',init_dsn_e_SS)
    call myprint('invar_dsn_e_SS',invar_dsn_e_SS)
    call myprint('trans_e_SS',trans_e_SS)        
    call myprint('v_SS',v_SS)
    call myprint('c_SS',c_SS)
    call myprint('d_SS',d_SS)
    call myprint('ap_SS',ap_SS)
    call myprint('invar_dsn_SS',invar_dsn_SS)
    call myprint('EEE_SS',EEE_SS)

end subroutine 

subroutine load_data_SS()
    use mod_myprint
    implicit none
    
    call myread('v_SS',v_SS)
    call myread('c_SS',c_SS)
    call myread('d_SS',d_SS)
    call myread('ap_SS',ap_SS)
    call myread('invar_dsn_SS',invar_dsn_SS)
    call myread('EEE_SS',EEE_SS)

end subroutine

subroutine allocate_main()
    use mod_param, only: nA,nAmm,nE,Tend,nH,nM,nZ
    implicit none
        
    if (allocated(gA)) deallocate(gA);                              allocate(gA(nA))
    if (allocated(invar_dsn_SS)) deallocate(invar_dsn_SS);          allocate(invar_dsn_SS(nA,nE,Tend,nH))
    if (allocated(gE)) deallocate(gE);                              allocate(gE(nE,Tend))
    if (allocated(trans_e)) deallocate(trans_e);                    allocate(trans_e(nE,nE,Tend,nZ,nZ))
    if (allocated(trans_ez)) deallocate(trans_ez);                  allocate(trans_ez(nE,nE,nZ,nZ,Tend))
    if (allocated(q)) deallocate(q);                                allocate(q(nA,nE,Tend,nH,nM,nZ))
    if (allocated(d)) deallocate(d);                                allocate(d(nA,nE,Tend,nH,nM))
    if (allocated(v)) deallocate(v);                                allocate(v(nA,nE,Tend,nH,nM))
    if (allocated(c)) deallocate(c);                                allocate(c(nA,nE,Tend,nH,nM))
    if (allocated(apg)) deallocate(apg);                            allocate(apg(nA,nE,Tend,nH,nM))
    if (allocated(apb)) deallocate(apb);                            allocate(apb(nA,nE,Tend,nH,nM))
    if (allocated(trans_m)) deallocate(trans_m);                    allocate(trans_m(nM,nM))
    if (allocated(trans_m_cdnal_zp)) deallocate(trans_m_cdnal_zp);  allocate(trans_m_cdnal_zp(nM,nM,nZ))
    if (allocated(init_dsn_e)) deallocate(init_dsn_e);              allocate(init_dsn_e(nE,nZ))
    if (allocated(EEE_SS)) deallocate(EEE_SS);                      allocate(EEE_SS(nA,nE,Tend))
    if (allocated(w)) deallocate(w);                                allocate(w(nM))
    if (allocated(qbar)) deallocate(qbar);                          allocate(qbar(nM,nZ))
    if (allocated(v_SS)) deallocate(v_SS);                          allocate(v_SS(nA,nE,Tend,nH))
    if (allocated(c_SS)) deallocate(c_SS);                          allocate(c_SS(nA,nE,Tend,nH))
    if (allocated(ap_SS)) deallocate(ap_SS);                        allocate(ap_SS(nA,nE,Tend,nH))
    if (allocated(init_dsn_e_SS)) deallocate(init_dsn_e_SS);        allocate(init_dsn_e_SS(nE))
    if (allocated(invar_dsn_e_SS)) deallocate(invar_dsn_e_SS);      allocate(invar_dsn_e_SS(nE,Tend))
    if (allocated(trans_e_SS)) deallocate(trans_e_SS);              allocate(trans_e_SS(nE,nE,Tend))
    if (allocated(q_SS)) deallocate(q_SS);                          allocate(q_SS(nA,nE,Tend,nH))
    if (allocated(d_SS)) deallocate(d_SS);                          allocate(d_SS(nA,nE,Tend,nH))

    print*,'Required data size GB: ',(size(v)+size(c)+size(apg)+size(apb)+size(d)+size(q)+size(trans_m)+size(trans_m_cdnal_zp)+size(trans_ez)+size(trans_e)+size(invar_dsn_SS))*8d0/1028d0/1028d0/1028d0
    
end subroutine allocate_main
   
subroutine deallocate_main
    implicit none

    if (allocated(gA)) deallocate(gA)
    if (allocated(invar_dsn_SS)) deallocate(invar_dsn_SS)
    if (allocated(gE)) deallocate(gE)
    if (allocated(trans_e)) deallocate(trans_e)
    if (allocated(trans_ez)) deallocate(trans_ez)
    if (allocated(q)) deallocate(q)
    if (allocated(d)) deallocate(d)
    if (allocated(v)) deallocate(v)
    if (allocated(c)) deallocate(c)
    if (allocated(apg)) deallocate(apg)
    if (allocated(apb)) deallocate(apb)
    if (allocated(trans_m)) deallocate(trans_m)
    if (allocated(trans_m_cdnal_zp)) deallocate(trans_m_cdnal_zp)
    if (allocated(init_dsn_e)) deallocate(init_dsn_e)
    if (allocated(EEE_SS)) deallocate(EEE_SS)
    if (allocated(w)) deallocate(w)
    if (allocated(qbar)) deallocate(qbar)
    if (allocated(v_SS)) deallocate(v_SS)
    if (allocated(c_SS)) deallocate(c_SS)
    if (allocated(ap_SS)) deallocate(ap_SS)
    if (allocated(init_dsn_e_SS)) deallocate(init_dsn_e_SS)
    if (allocated(invar_dsn_e_SS)) deallocate(invar_dsn_e_SS)
    if (allocated(trans_e_SS)) deallocate(trans_e_SS)
    if (allocated(q_SS)) deallocate(q_SS)
    if (allocated(d_SS)) deallocate(d_SS)

end subroutine deallocate_main

end module mod_data
