!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Law
! This module gives a self-contained treatment of the law.
module mod_law
    use mod_param, only: nZ, len_KN, nM, len_WW, useKNinLaw, mom
    implicit none

    !moments grid 
    type(mom), dimension(nM) :: grid_m !Contains lots of information
    
    !now from grid_m, one can ascertain all of the following:
    ! 1. map_from_m_to_X_ind = grid_m%X_ind
    ! 2. map_from_m_to_X  = grid_m%X

    !Moment mapping to and from data
    integer, dimension(nZ,len_KN,len_WW), private :: map_to_m

    real(8), dimension(2), private :: betaWW_gg, betaWW_gb, betaWW_bg, betaWW_bb, betaWW_gg_new, betaWW_gb_new, betaWW_bg_new, betaWW_bb_new
    real(8), dimension(3), private :: betaKN_gg, betaKN_gb, betaKN_bg, betaKN_bb, betaKN_gg_new, betaKN_gb_new, betaKN_bg_new, betaKN_bb_new

contains

subroutine sub_compute_law_update(mseq,diffLaw)
    use mod_myprint
    use mod_matlab
    use mod_param, only: nT,len_t0,g_ind,b_ind
    implicit none
    type(mom), dimension(1:nT), intent(in) :: mseq
    real(8), intent(out) :: diffLaw(8)
    !local
    type(mom), dimension(1:nT-(len_t0+1)+1-1) :: m_fcast1_local,m_real1_local
    real(8), dimension(:), allocatable :: WWp_cdnal_gg,WWp_cdnal_gb,WWp_cdnal_bg,WWp_cdnal_bb,&
                                          KNp_cdnal_gg,KNp_cdnal_gb,KNp_cdnal_bg,KNp_cdnal_bb
    real(8), dimension(:,:), allocatable :: X_gg,X_gb,X_bg,X_bb
    integer, dimension(1:nT-(len_t0+1)+1-1-2) :: fcast_subsample,curr_subsample
    integer, dimension(:), allocatable :: gg_subsample,gb_subsample,bg_subsample,bb_subsample
    integer :: len_fcast    
    real(8) :: tmpscal

    call sub_fcastJ(m_fcast1_local,m_real1_local,1,mseq)

    !Break into different subsamples zz'=(gg,gb,bg,bb)

    len_fcast = size(m_real1_local) !size(A1_real1_local)

    fcast_subsample = colon(len_t0+2,nT-2)
    curr_subsample = fcast_subsample-1

    allocate(gg_subsample(count(g_ind==mseq(curr_subsample)%z_ind .and. g_ind==mseq(fcast_subsample)%z_ind)))
    allocate(gb_subsample(count(g_ind==mseq(curr_subsample)%z_ind .and. b_ind==mseq(fcast_subsample)%z_ind)))
    allocate(bg_subsample(count(b_ind==mseq(curr_subsample)%z_ind .and. g_ind==mseq(fcast_subsample)%z_ind)))
    allocate(bb_subsample(count(b_ind==mseq(curr_subsample)%z_ind .and. b_ind==mseq(fcast_subsample)%z_ind)))

    gg_subsample = pack(curr_subsample,g_ind==mseq(curr_subsample)%z_ind .and. g_ind==mseq(fcast_subsample)%z_ind)
    gb_subsample = pack(curr_subsample,g_ind==mseq(curr_subsample)%z_ind .and. b_ind==mseq(fcast_subsample)%z_ind)
    bg_subsample = pack(curr_subsample,b_ind==mseq(curr_subsample)%z_ind .and. g_ind==mseq(fcast_subsample)%z_ind)
    bb_subsample = pack(curr_subsample,b_ind==mseq(curr_subsample)%z_ind .and. b_ind==mseq(fcast_subsample)%z_ind)

    allocate(WWp_cdnal_gg(size(gg_subsample)),KNp_cdnal_gg(size(gg_subsample)))    
    allocate(WWp_cdnal_gb(size(gb_subsample)),KNp_cdnal_gb(size(gb_subsample)))    
    allocate(WWp_cdnal_bg(size(bg_subsample)),KNp_cdnal_bg(size(bg_subsample)))    
    allocate(WWp_cdnal_bb(size(bb_subsample)),KNp_cdnal_bb(size(bb_subsample)))    

    KNp_cdnal_gg = m_real1_local(gg_subsample-(len_t0))%KN
    KNp_cdnal_gb = m_real1_local(gb_subsample-(len_t0))%KN
    KNp_cdnal_bg = m_real1_local(bg_subsample-(len_t0))%KN
    KNp_cdnal_bb = m_real1_local(bb_subsample-(len_t0))%KN

    WWp_cdnal_gg = m_real1_local(gg_subsample-(len_t0))%WW
    WWp_cdnal_gb = m_real1_local(gb_subsample-(len_t0))%WW
    WWp_cdnal_bg = m_real1_local(bg_subsample-(len_t0))%WW
    WWp_cdnal_bb = m_real1_local(bb_subsample-(len_t0))%WW

    ! If using a full sample including endogenous variables
    ! I regress WW on the fundamental part of the distribution AA rather than a previous price WW
    allocate(X_gg(size(gg_subsample),3))
    allocate(X_gb(size(gb_subsample),3))
    allocate(X_bg(size(bg_subsample),3))
    allocate(X_bb(size(bb_subsample),3))

    X_gg(:,1) = 1.d0
    X_gb(:,1) = 1.d0
    X_bg(:,1) = 1.d0
    X_bb(:,1) = 1.d0

    X_gg(:,2) = mseq(gg_subsample)%KN
    X_gb(:,2) = mseq(gb_subsample)%KN
    X_bg(:,2) = mseq(bg_subsample)%KN
    X_bb(:,2) = mseq(bb_subsample)%KN

    X_gg(:,3) = mseq(gg_subsample)%WW
    X_gb(:,3) = mseq(gb_subsample)%WW
    X_bg(:,3) = mseq(bg_subsample)%WW
    X_bb(:,3) = mseq(bb_subsample)%WW

    tmpscal = cond(matmul(transpose(X_gg(:,1:2)),X_gg(:,1:2)))
    print*,'Condition number regression 1 is :',tmpscal
    if (useKNinLaw .and. tmpscal<1d6) then 
        betaWW_gg_new(1:2) = regress(WWp_cdnal_gg,X_gg(:,1:2))
        betaWW_gb_new(1:2) = regress(WWp_cdnal_gb,X_gb(:,1:2))
        betaWW_bg_new(1:2) = regress(WWp_cdnal_bg,X_bg(:,1:2))
        betaWW_bb_new(1:2) = regress(WWp_cdnal_bb,X_bb(:,1:2))
    else
        betaWW_gg_new(1:1) = regress(WWp_cdnal_gg,X_gg(:,(/1/)))
        betaWW_gb_new(1:1) = regress(WWp_cdnal_gb,X_gb(:,(/1/)))
        betaWW_bg_new(1:1) = regress(WWp_cdnal_bg,X_bg(:,(/1/)))
        betaWW_bb_new(1:1) = regress(WWp_cdnal_bb,X_bb(:,(/1/)))
        betaWW_gg_new(2) = 0.d0
        betaWW_gb_new(2) = 0.d0
        betaWW_bg_new(2) = 0.d0
        betaWW_bb_new(2) = 0.d0        
    end if

    tmpscal = cond(matmul(transpose(X_gg(:,1:2)),X_gg(:,1:2)))
    print*,'Condition number regression 2 is :',tmpscal
    if (useKNinLaw) then 
        if (tmpscal<1d6) then
            betaKN_gg_new(1:2) = regress(KNp_cdnal_gg,X_gg(:,1:2))
            betaKN_gb_new(1:2) = regress(KNp_cdnal_gb,X_gb(:,1:2))
            betaKN_bg_new(1:2) = regress(KNp_cdnal_bg,X_bg(:,1:2))
            betaKN_bb_new(1:2) = regress(KNp_cdnal_bb,X_bb(:,1:2))
            betaKN_gg_new(3) = 0.d0  
            betaKN_gb_new(3) = 0.d0
            betaKN_bg_new(3) = 0.d0
            betaKN_bb_new(3) = 0.d0
        else
            betaKN_gg_new(1:1) = regress(KNp_cdnal_gg,X_gg(:,(/1/)))
            betaKN_gb_new(1:1) = regress(KNp_cdnal_gb,X_gb(:,(/1/)))
            betaKN_bg_new(1:1) = regress(KNp_cdnal_bg,X_bg(:,(/1/)))
            betaKN_bb_new(1:1) = regress(KNp_cdnal_bb,X_bb(:,(/1/)))
            betaKN_gg_new(2:3) = 0.d0  
            betaKN_gb_new(2:3) = 0.d0
            betaKN_bg_new(2:3) = 0.d0
            betaKN_bb_new(2:3) = 0.d0
        end if
    else
        betaKN_gg_new(1:1) = regress(KNp_cdnal_gg,X_gg(:,(/1/)))
        betaKN_gb_new(1:1) = regress(KNp_cdnal_gb,X_gb(:,(/1/)))
        betaKN_bg_new(1:1) = regress(KNp_cdnal_bg,X_bg(:,(/1/)))
        betaKN_bb_new(1:1) = regress(KNp_cdnal_bb,X_bb(:,(/1/)))
        betaKN_gg_new(2:3) = 0.d0  
        betaKN_gb_new(2:3) = 0.d0
        betaKN_bg_new(2:3) = 0.d0
        betaKN_bb_new(2:3) = 0.d0
    end if

    ! Report new law of motion
    write(*,'(A,3f11.6)') 'betaWW_gg_new',betaWW_gg_new
    write(*,'(A,3f11.6)') 'betaWW_bg_new',betaWW_bg_new
    write(*,'(A,3f11.6)') 'betaWW_gb_new',betaWW_gb_new
    write(*,'(A,3f11.6)') 'betaWW_bb_new',betaWW_bb_new

    write(*,'(A,3f11.6)') 'betaKN_gg_new',betaKN_gg_new
    write(*,'(A,3f11.6)') 'betaKN_bg_new',betaKN_bg_new
    write(*,'(A,3f11.6)') 'betaKN_gb_new',betaKN_gb_new
    write(*,'(A,3f11.6)') 'betaKN_bb_new',betaKN_bb_new

    ! Calculate the max change in the supnorm of these things
    diffLaw(1) = maxval(abs(matmul(X_gg,betaKN_gg_new-betaKN_gg)))
    diffLaw(2) = maxval(abs(matmul(X_bg,betaKN_bg_new-betaKN_bg)))
    diffLaw(3) = maxval(abs(matmul(X_gb,betaKN_gb_new-betaKN_gb)))
    diffLaw(4) = maxval(abs(matmul(X_bb,betaKN_bb_new-betaKN_bb)))
    diffLaw(5) = maxval(abs(matmul(X_gg(:,1:2),betaWW_gg_new-betaWW_gg)))
    diffLaw(6) = maxval(abs(matmul(X_bg(:,1:2),betaWW_bg_new-betaWW_bg)))
    diffLaw(7) = maxval(abs(matmul(X_gb(:,1:2),betaWW_gb_new-betaWW_gb)))
    diffLaw(8) = maxval(abs(matmul(X_bb(:,1:2),betaWW_bb_new-betaWW_bb)))

    write(*,'(A,8f14.7)') 'dLaw:',diffLaw

end subroutine sub_compute_law_update

subroutine sub_update_law(relax_param_KN,relax_param_WW)
    implicit none
    real(8), intent(in) :: relax_param_KN,relax_param_WW

    !Update with relaxation
    betaWW_gg = (1.d0-relax_param_WW)*betaWW_gg_new + relax_param_WW*betaWW_gg
    betaWW_gb = (1.d0-relax_param_WW)*betaWW_gb_new + relax_param_WW*betaWW_gb
    betaWW_bg = (1.d0-relax_param_WW)*betaWW_bg_new + relax_param_WW*betaWW_bg
    betaWW_bb = (1.d0-relax_param_WW)*betaWW_bb_new + relax_param_WW*betaWW_bb

    betaKN_gg = (1.d0-relax_param_KN)*betaKN_gg_new + relax_param_KN*betaKN_gg 
    betaKN_gb = (1.d0-relax_param_KN)*betaKN_gb_new + relax_param_KN*betaKN_gb
    betaKN_bg = (1.d0-relax_param_KN)*betaKN_bg_new + relax_param_KN*betaKN_bg
    betaKN_bb = (1.d0-relax_param_KN)*betaKN_bb_new + relax_param_KN*betaKN_bb
    
end subroutine sub_update_law

!Basic setup
!  Bijective map between iM and the levels of K/N,W,z 
!  One subroutine initializes m by assigning each (K/N,W,z) pair an index
!  Another subroutine gives the (K/N,W,z) associated with an m
!  The law of motion takes m to Pr(m') using linear interpolation of moments 
!  and probabilities for stochastic things.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! init_m
! Takes as input the lengths of all aggregate grids.  
! Assign a unique value to each combination.
! Construct a map to and from the m index to the aggregate variables
subroutine init_m(grid_z,grid_KN,grid_WW)
    use mod_param, only:nM, nZ, len_KN, len_WW
    implicit none
    real(8), dimension(nZ), intent(in) :: grid_z
    real(8), dimension(len_KN), intent(in) :: grid_KN
    real(8), dimension(len_WW), intent(in) :: grid_WW
    !local
    integer :: KN_ind, z_ind, iM, WW_ind

    iM = 1
    do WW_ind = 1,len_WW
        do KN_ind=1,len_KN
            do z_ind=1,nZ            
                map_to_m(z_ind,KN_ind,WW_ind) = iM
                grid_m(iM)%m = iM
                grid_m(iM)%z_ind = z_ind
                grid_m(iM)%KN_ind = KN_ind
                grid_m(iM)%WW_ind = WW_ind
                iM = iM+1
            end do
        end do
    end do

    grid_m%z = grid_z(grid_m%z_ind)
    grid_m%KN = grid_KN(grid_m%KN_ind)
    grid_m%WW = grid_WW(grid_m%WW_ind)

end subroutine init_m

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! get_m_interp
! Given a moment m, returns the 4 indices and weights necessary
! to compute the bilinear interpolant of any function.
subroutine get_m_interp(indices,weights,m)
    use mod_matlab, only: diff
    use mod_interp, only: search
    use mod_param, only: len_KN, nZ, grid_KN, len_WW, grid_WW
    implicit none
    type(mom), intent(in) :: m
    integer, intent(out) :: indices(4)
    real(8), intent(out) :: weights(4)
    !local
    real(8) :: theta_WWl,theta_KNl
    integer :: ind_WWl,ind_KNl

    ! Make sure the grids are increasing
    if (any(diff(grid_WW)<=0d0)) STOP 'get_m_interp: grid_WW: must be monotonically increasing'
    if (any(diff(grid_KN)<=0d0)) STOP 'get_m_interp: grid_WW: must be monotonically increasing'

    ! Find the lower index bracketing the moments
    call search(ind_WWl,m%WW,grid_WW) 
    call search(ind_KNl,m%KN,grid_KN) 

    ! Compute the weights
    theta_WWl = (grid_WW(ind_WWl+1)-m%WW)/(grid_WW(ind_WWl+1)-grid_WW(ind_WWl))
    theta_KNl = (grid_KN(ind_KNl+1)-m%KN)/(grid_KN(ind_KNl+1)-grid_KN(ind_KNl))

    ! Don't allow extrapolation
    theta_WWl = min(max(theta_WWl,0d0),1d0)
    theta_KNl = min(max(theta_KNl,0d0),1d0)

    ! Map to m
    indices(1) = map_to_m(m%z_ind,ind_KNL  ,ind_WWl  )
    indices(2) = map_to_m(m%z_ind,ind_KNL+1,ind_WWl  )
    indices(3) = map_to_m(m%z_ind,ind_KNL  ,ind_WWl+1)
    indices(4) = map_to_m(m%z_ind,ind_KNL+1,ind_WWl+1)
    weights(1) = (     theta_KNl)*(     theta_WWl)
    weights(2) = (1.d0-theta_KNl)*(     theta_WWl)
    weights(3) = (     theta_KNl)*(1.d0-theta_WWl)
    weights(4) = (1.d0-theta_KNl)*(1.d0-theta_WWl)    

    ! Check for errors
    if (abs(sum(weights)-1.d0)>1d-10) STOP 'get_m_interp: error in computing interpolation weights'
    if (any(indices<1 .or. indices>nM)) STOP 'get_m_interp: error in computing interpolation indices'
    
end subroutine get_m_interp

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Laws
subroutine init_law(K_SS,N_SS)
    use mod_param, only: nM, nZ, g_ind, b_ind, delta
    use mod_price_fcns
    use mod_myprint
    use mod_matlab
    use mod_convertAtoK, only: convertAtoK
    implicit none
    !local
    real(8), intent(in) :: K_SS,N_SS
    integer :: err

    ! First try reading in the law of motion guesses
    do 
        call myread('betaWW_gg',betaWW_gg,err); if (err/=0) exit
        call myread('betaWW_gb',betaWW_gb,err); if (err/=0) exit
        call myread('betaWW_bg',betaWW_bg,err); if (err/=0) exit
        call myread('betaWW_bb',betaWW_bb,err); if (err/=0) exit
        call myread('betaKN_gg',betaKN_gg,err); if (err/=0) exit
        call myread('betaKN_gb',betaKN_gb,err); if (err/=0) exit
        call myread('betaKN_bg',betaKN_bg,err); if (err/=0) exit
        call myread('betaKN_bb',betaKN_bb,err); if (err/=0) exit
        exit
    end do

    ! If that fails, then use coarse guesses
    if (err/=0) then

        write(*,*) 'Reading guesses failed, using coarse guesses'

        betaKN_gg = 0.d0
        betaKN_gg(1) = K_SS/N_SS
        betaKN_gb = betaKN_gg
        betaKN_bg = betaKN_gg
        betaKN_bb = betaKN_gg

        betaWW_gg = 0.d0
        betaWW_gg(1) = .51d0 
        betaWW_gb = betaWW_gg
        betaWW_bg = betaWW_gg
        betaWW_bb = betaWW_gg

    end if
   
end subroutine init_law

subroutine save_law()
    use mod_myprint
    implicit none

    call myprint('betaWW_gg',betaWW_gg)
    call myprint('betaWW_bg',betaWW_bg)
    call myprint('betaWW_gb',betaWW_gb)
    call myprint('betaWW_bb',betaWW_bb)
    call myprint('betaKN_gg',betaKN_gg)
    call myprint('betaKN_bg',betaKN_bg)
    call myprint('betaKN_gb',betaKN_gb)
    call myprint('betaKN_bb',betaKN_bb)

end subroutine save_law

subroutine mp_cdnal_zp_offpath(mp,zp_ind,m)
    use mod_param, only: nM, len_KN, grid_z, g_ind, b_ind
    implicit none
    type(mom), intent(out) :: mp
    type(mom), intent(in) :: m
    integer, intent(in) :: zp_ind
    
    !Output in terms of mp
    if (zp_ind==g_ind) then
        mp%WW = Wpg_offpath(m)
        mp%KN = KNpg_offpath(m)
        mp%z_ind = g_ind
        mp%z = grid_z(g_ind)
    elseif (zp_ind==b_ind) then
        mp%WW = Wpb_offpath(m)
        mp%KN = KNpb_offpath(m)
        mp%z_ind = b_ind
        mp%z = grid_z(b_ind)        
    else
        STOP 'mp_cdnal_zp_offpath: zp_ind not found'
    end if

end subroutine mp_cdnal_zp_offpath

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Transition from m to m'
!
! The combination of linear interpolation across moments
! together with probabilistic transitions of the aggregate 
! shock means that the law of one aggregate state to the 
! next looks like a pure probabilty, with weights across 
! states summing to 1.  
!
! Therefore, m_law returns a transition matrix using the laws.
!
subroutine m_law(trans_m,trans_m_cdnal_zp,trans_z)
    use mod_param, only: g_ind,b_ind,nM,nZ 
    use mod_myprint
    implicit none
    real(8), dimension(nZ,nZ), intent(in) :: trans_z
    real(8), dimension(nM,nM), intent(out) :: trans_m
    real(8), dimension(nM,nM,nZ), intent(out) :: trans_m_cdnal_zp !Pr(m'|m,z')
    !local
    integer :: iM, iMp, indices(4) ! , i
    real(8) :: weights(4)
    type(mom) :: m,mp
    integer :: iZp

    do iM = 1,nM    

        !Get the current moments for m
        m = grid_m(iM)
           
        do iZp = 1,nZ

            !Get the associated next period moments associated with zp
            call mp_cdnal_zp_offpath(mp,iZp,m)

            ! Get the interpolation weights associated with the bilinear interpolation
            call get_m_interp(indices,weights,mp)

            ! Setup the transition matrix conditional 
            trans_m_cdnal_zp(iM,:,iZp) = 0d0
            trans_m_cdnal_zp(iM,indices,iZp) = weights

!             do i = 1,size(indices)
!                 trans_m_cdnal_zp(iM,indices(i),iZp) = weights(i)
!             end do

        end do

        do iMp = 1,nM
            ! Now compute the unconditional transition (not conditioned on z')
!             trans_m(iM,iMp) = trans_z(m%z_ind,g_ind)*trans_m_cdnal_zp(iM,iMp,g_ind) &
!                                   + trans_z(m%z_ind,b_ind)*trans_m_cdnal_zp(iM,iMp,b_ind)
            trans_m(iM,iMp) = 0d0
            do iZp = 1,nZ
                ! Now compute the unconditional transition (not conditioned on z')
                trans_m(iM,iMp) = trans_m(iM,iMp) + trans_z(m%z_ind,iZp)*trans_m_cdnal_zp(iM,iMp,iZp)
            end do
        end do
    end do

!         ! Do the same thing for the b shock
!         call mp_cdnal_zp_offpath(mpb,b_ind,m)
!         call get_m_interp(indices,weights,mpb)
!         trans_m_cdnal_zpb(iM,:) = 0d0
!         do i = 1,size(indices)
!             trans_m_cdnal_zpb(iM,indices(i)) = weights(i)
!         end do
! 
    ! Ensure that the transition matrix sums to 1
    do iM = 1,nM
        if (abs(sum(trans_m(iM,:))-1.d0)>1d-10) write(*,*) 'WARNING: trans_m does not sum to 1, but is ',sum(trans_m(iM,:))-1.d0, 'far away'
        do iZp = 1,nZ
            if (abs(sum(trans_m_cdnal_zp(iM,:,iZp))-1.d0)>1d-10) write(*,*) 'WARNING: trans_m_cdnal_zp does not sum to 1 for some iZp, but is ',sum(trans_m_cdnal_zp(iM,:,iZp))-1.d0, 'far away'
        end do
    end do
    
    ! Check that there are no negative elements of trans_m
    if (any(trans_m<-1d10)) write(*,*) 'WARNING: trans_m has negative elements, ', minval(trans_m)
    if (any(trans_m_cdnal_zp<-1d10)) write(*,*) 'WARNING: trans_m_cdnal_zp has negative elements, ', minval(trans_m_cdnal_zp)

end subroutine m_law

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
function KNpg_offpath(m) result(KNp)
    use mod_param, only: g_ind, b_ind
    implicit none
    type(mom), intent(in) :: m
    real(8) :: KNp

    if (m%z_ind==g_ind) then
        KNp = dot_product(betaKN_gg,(/1.d0,m%KN,m%WW/))
    elseif (m%z_ind==b_ind) then
        KNp = dot_product(betaKN_bg,(/1.d0,m%KN,m%WW/))
    else
        STOP 'KNpg_offpath: z_ind not found'
    end if

end function KNpg_offpath

function KNpb_offpath(m) result(KNp)
    use mod_param, only: g_ind, b_ind
    implicit none
    type(mom), intent(in) :: m
    real(8) :: KNp

    if (m%z_ind==g_ind) then
        KNp = dot_product(betaKN_gb,(/1.d0,m%KN,m%WW/))
    elseif (m%z_ind==b_ind) then
        KNp = dot_product(betaKN_bb,(/1.d0,m%KN,m%WW/))
    else
        STOP 'KNpb_offpath: z_ind not found'
    end if
    
end function KNpb_offpath

!Compute the rg=g*F_K(K',N'g) implied by the law of motion
function rg_offpath(m) result(rg)
    use mod_param, only: g,alpha_K, g_ind, b_ind
    implicit none
    type(mom), intent(in) :: m
    real(8) :: rg
    !local
    real(8) :: KNpg
    KNpg = KNpg_offpath(m)
    rg = g*alpha_K*(KNpg)**(alpha_K-1.d0)
end function rg_offpath

!Compute the rb=b*F_K(K',N'b) implied by the law of motion
function rb_offpath(m) result(rb)
    use mod_param, only: b,alpha_K,g_ind, b_ind
    implicit none
    type(mom), intent(in) :: m
    real(8) :: rb
    !local
    real(8) :: KNpb
    KNpb = KNpb_offpath(m)
    rb = b*alpha_K*(KNpb)**(alpha_K-1.d0)
end function rb_offpath

function Wpg_offpath(m) result(Wp)
    use mod_param, only: g_ind, b_ind
    implicit none
    type(mom), intent(in) :: m
    real(8) :: Wp

    if (m%z_ind==g_ind) then
        Wp = dot_product(betaWW_gg,(/1.d0, m%KN/))
    elseif (m%z_ind==b_ind) then
        Wp = dot_product(betaWW_bg,(/1.d0, m%KN/))
    else
        STOP 'Wpg_offpath: z_ind not found'
    end if

end function Wpg_offpath

function Wpb_offpath(m) result(Wp)
    use mod_param, only: g_ind, b_ind
    implicit none
    type(mom), intent(in) :: m
    real(8) :: Wp

    if (m%z_ind==g_ind) then
        Wp = dot_product(betaWW_gb,(/1.d0, m%KN/))
    elseif (m%z_ind==b_ind) then
        Wp = dot_product(betaWW_bb,(/1.d0, m%KN/))
    else
        STOP 'Wpb_offpath: z_ind not found'
    end if    
end function Wpb_offpath


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Calculation of prices (qbarg,qbarb) today from moments 
!
! Using the rg and rb implied by the law of motion, together 
! with an equity premium and the no arbitrage conditions,
! compute qbarg and qbarb
!
subroutine qbargqbarb_law(qbar)
    use mod_param, only:nM, nZ, g_ind, b_ind, delta
    implicit none
    real(8), dimension(nM,nZ), intent(out) :: qbar
    !local
    integer :: iM

    do iM = 1,nM
        call qbargqbarb_offpath(qbar(iM,g_ind),qbar(iM,b_ind),grid_m(iM))
    end do

end subroutine qbargqbarb_law

! Same as above but evaluates off grid (used in simulation)
subroutine qbargqbarb_offpath(qbarg,qbarb,m)
    use mod_param, only: nZ, g_ind, b_ind, delta, trans_z
    implicit none
    real(8), intent(out) :: qbarg, qbarb
    type(mom), intent(in) :: m
    !local
    real(8) :: rg, rb, WW, qBond, Fg, Fb, qbargCheck, qbarbCheck
    
    Rg = 1.d0 - delta + rg_offpath(m)
    Rb = 1.d0 - delta + rb_offpath(m)
    WW = m%WW
    Fg = trans_z(m%z_ind,g_ind)
    Fb = trans_z(m%z_ind,b_ind)

    !qB(w) is s.t. 1/qB = w*(1+rb-delta) + (1-w)*(1+rg-delta)
    ! NEW: I've now redefined these so that if the aggregate shock is persistent w will still be around 1/2 in eq
    !qB(w) is s.t. 1/qB = (w*F(b|z)*(1+rb-delta) + (1-w)*F(g|z)*(1+rg-delta))/(w*F(b|z) + (1-w)*F(g|z))
    ! w = 0 still implies 1+rg-delta, w = 1 still implies 1+rb-delta.  Also, if serially uncorrelated 
    ! then collapses to the previous formula.
    qBond = (WW*Fb+(1.d0-WW)*Fg)/&
            (WW*Fb*Rb + (1.d0-WW)*Fg*Rg)

    if (1.d0/qBond < Rb .OR. &
        1.d0/qBond > Rg) then
        write(*,*) 'WARNING: qBond does not satisfy NA, truncating'
        write(*,*) 'WW is ', WW
        write(*,*) 'qBond is ', qBond
        write(*,*) 'Rg is ', Rg
        write(*,*) 'Rb is ', Rb
        write(*,*) '1/qBond is ', 1.d0/qBond
        STOP 'error'
    end if

    ! This calculation works so long as Rb/=Rg
    qbargCheck = (qBond*Rb-1.d0)/(Rb-Rg)
    qbarbCheck= (1.d0-qBond*Rg)/(Rb-Rg)

    ! Bypass solving for the bond because in steady state version, 1/qB and R are the same. 
    qbarg = 1.d0/(    Rg + WW/(1.d0-WW) * Fb/Fg * Rb     )
    qbarb = 1.d0/(    (1.d0-WW)/WW * Fg/Fb * Rg + Rb     )

    if (qbarg<1d-12 .OR. qbarb<1d-12) STOP 'qbarg or qbarb is not positive'
    if (abs(qbargCheck-qbarg)>1d-10 .or. abs(qbarbCheck-qbarb)>1d-10) then
        print*, 'qbarg',qbarg,'qbargCheck',qbargCheck
        print*, 'qbarb',qbarb,'qbarbCheck',qbarbCheck
        STOP 'two methods of obtaining qbarg,qbarb do not give same answer'
    end if

end subroutine qbargqbarb_offpath

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! sub_fcastJ
! Compute J step ahead forecasts given a realized sequence of moments {A1t,A2t,A3t}
!
subroutine sub_fcastJ(m_fcastN,m_realN,J,mseq)
    use mod_param, only: len_t0,nT
    implicit none
    integer, intent(in) :: J
    type(mom), dimension(1:nT), intent(in) :: mseq
    type(mom), dimension(1:nT-(len_t0+1)+1-J), intent(out) :: m_realN,m_fcastN
    !local     
    integer :: T_ind
    
    !realized N step ahead sequences
    m_realN = mseq(len_t0+1+J:nT)
    
    !forecasts implied by law of motion
    do T_ind = len_t0+1,nT-J        
        call J_step(m_fcastN(T_ind-len_t0),T_ind,mseq)
    end do
    
contains
    
    !Compute J step ahead forecasts starting from t
    subroutine J_step(mNp,T_ind,mseq)
        implicit none
        type(mom), intent(out) :: mNp
        type(mom), dimension(1:), intent(in) :: mseq
        integer, intent(in) :: T_ind
        !local
        type(mom) :: mp,m
        integer :: J_ind 
                      
        m = mseq(T_ind)

        !Compute the one step ahead forecast of moments J times
        do J_ind = 1,J
            call one_step(mp,mseq(T_ind+J_ind)%z_ind,m)
            m=mp
        end do        

        mNp = mp
        
    end subroutine J_step   
    
    subroutine one_step(mp,zp_ind,m)
        implicit none        
        type(mom), intent(out) :: mp
        type(mom), intent(in) :: m
        integer, intent(in) :: zp_ind
        call mp_cdnal_zp_offpath(mp,zp_ind,m)         
    end subroutine one_step
    
end subroutine sub_fcastJ

subroutine sub_print_law()
    use mod_myprint
    use mod_param, only: nM, g_ind, b_ind
    implicit none
    !local
    real(8), dimension(nM,nZ) :: qbar_law
    
    !Maps from values to moments
    call myprint('grid_m__z',grid_m%z)
    call myprint('grid_m__KN',grid_m%KN)
    call myprint('grid_m__WW',grid_m%WW)

    !Maps from values to moments
    call myprint('grid_m__z_ind',grid_m%z_ind)
    call myprint('grid_m__KN_ind',grid_m%KN_ind)
    call myprint('grid_m__WW_ind',grid_m%WW_ind)

    !Coefficients of laws
    call myprint('betaWW_gg_check',betaWW_gg)
    call myprint('betaWW_gb_check',betaWW_gb)
    call myprint('betaWW_bg_check',betaWW_bg)
    call myprint('betaWW_bb_check',betaWW_bb)
    call myprint('betaKN_gg_check',betaKN_gg)
    call myprint('betaKN_gb_check',betaKN_gb)
    call myprint('betaKN_bg_check',betaKN_bg)
    call myprint('betaKN_bb_check',betaKN_bb)

    !Laws for aggregate quantities and prices
    call qbargqbarb_law(qbar_law)

    call myprint('qbar_law',qbar_law)
    
end subroutine sub_print_law



end module mod_law

