module mod_param
    implicit none

    ! master directory where output files should be written.
    ! this is set in main.f90 to $HOME/Data/bankKS/
    character(len=100) :: rootdatadir

    ! Different experiment or modeling switches
    logical :: &
        defBnkAllowed = .true., & ! whether default is allowed or not
        isBAPCPA = .false., & ! whether BAPCPA rules in place --- requires defBnkAllowed == .true.
        isPE = .true., & ! whether PE experiment in which case prices kept constant at benchmark
        defZcontingent = .true. ! whether default is allowed in only some agg states
    integer :: defZcontingent_AllowedZind = 2 !expansions only is 1, contractions only is 2
    real(8), parameter :: chi13 = .75d0
    real(8) :: medianBAPCPA = huge(0d0) !0.628715d0 !Only allowed to default if e is below this (having normalized mean(e) to 1).
    real(8) :: qbarPE = -1d0 !0.9835543716629702d0
    real(8) :: usuryLimit = .5d0 ! usuryLimit is lowerbound on q discounted price.  .5 means 100% net interest limit
    logical :: doCalibrate = .false. ! whether or not to calibrate the model
    real(8) :: lambda = .9d0 ! Prob bad stays on record !1/(1-lamba) is average # periods spent in bankruptcy

    logical :: onlySaveLaw = .false. ! If false, all results are saved

    !!!!!!!!!!!!!!!!!!!!!!!
    ! Lengths
    !
    integer, parameter :: Tret = 64-20+1 !Households work from 1:Tret
    integer, parameter :: Tend = 85-20+1 !Households are retired from Tret+1:Tend 
    integer, parameter :: T1age = 20 ! What the real life age t=1 corresponds to (20 years) 

    integer, parameter :: npers = 15, ntran = 5
    integer, parameter :: nCCNRaugment = 10
    integer, parameter :: nA = 150 
    integer, parameter :: nAmm = 60

    ! These two are set early in the program
    integer :: nExpenditure = 0 ! 3 ! set to 1 for no expenditure shock
    integer :: nE = 0 ! = nExpenditure*(npers*ntran+nCCNRaugment) ! Total number of idiosyncratic states

    integer, parameter :: nH = 2 ! 2 = #{h=0,h=1}
    integer, parameter :: nZ = 2 ! 2 = #{z=g,z=b}
    integer, parameter :: len_KN = 5 ! # of points for the capital-output ratio
    integer, parameter :: len_WW = 3 !3 !3 !Number of points on equity premium grid
    integer, parameter :: nM = nZ*len_KN*len_WW
    integer, parameter :: nT = 5000 ! was 1000 in first submission and until 9/22/14 ! # of periods to simulate
    integer, parameter :: len_t0 = 250 ! # of period to throwout (only use len_t0+1...T)

    !!!!!!!!!!!!!!!!!!!!!!!
    ! Demographic parameters for life cycle
    real(8) :: probLive(Tend) = & !Conditional Survival Probabilities (from Hubbard et al 1994 excepting the first value which is linearly extrapolated)
        (/0.99945d0,0.99942d0,0.99939d0,0.99938d0,0.99936d0,0.99935d0,0.99933d0,0.99931d0,0.99930d0,0.99928d0,&
          0.99925d0,0.99922d0,0.99918d0,0.99914d0,0.99909d0,0.99902d0,0.99895d0,0.99885d0,0.99872d0,0.99856d0,&
          0.99839d0,0.99820d0,0.99800d0,0.99779d0,0.99758d0,0.99734d0,0.99708d0,0.99680d0,0.99651d0,0.99620d0,&
          0.99587d0,0.99550d0,0.99510d0,0.99467d0,0.99419d0,0.99368d0,0.99311d0,0.99251d0,0.99189d0,0.99122d0,&
          0.99048d0,0.98967d0,0.98876d0,0.98777d0,0.98668d0,0.98545d0,0.98410d0,0.98270d0,0.98126d0,0.97972d0,&
          0.97797d0,0.97596d0,0.97377d0,0.97137d0,0.96872d0,0.96568d0,0.96222d0,0.95834d0,0.95403d0,0.94922d0,&
          0.94385d0,0.93786d0,0.93115d0,0.92369d0,0.91545d0,0d0/)

    !Consumption equivalent scale (Bick and Choi provided the household size, the scale is from Krueger and Fernandez-Villaverde)
    real(8) :: csnEquiv(Tend) = & 
        (/1.1788d0,1.1836d0,1.1887d0,1.2538d0,1.3339d0,1.3976d0,1.4570d0,1.5085d0,1.5524d0,1.5891d0,&
          1.6192d0,1.6428d0,1.6609d0,1.6736d0,1.6810d0,1.6836d0,1.6817d0,1.6759d0,1.6664d0,1.6536d0,&
          1.6385d0,1.6210d0,1.6016d0,1.5804d0,1.5580d0,1.5345d0,1.5103d0,1.4856d0,1.4609d0,1.4362d0,&
          1.4119d0,1.3882d0,1.3652d0,1.3432d0,1.3207d0,1.2992d0,1.2792d0,1.2608d0,1.2442d0,1.2293d0,&
          1.2241d0,1.2206d0,1.2169d0,1.2128d0,1.2084d0,1.2037d0,1.1986d0,1.1933d0,1.1875d0,1.1814d0,&
          1.1750d0,1.1681d0,1.1609d0,1.1533d0,1.1455d0,1.1372d0,1.1278d0,1.1184d0,1.1085d0,1.0982d0,&
          1.0873d0,1.0760d0,1.0641d0,1.0516d0,1.0386d0,1.0250d0/)

    !!!!!!!!!!!!!!!!!!!!!!!!
    ! Estimated parameters
    !
    real(8) :: discount = -100d0 
    real(8) :: chi0 = 0d0, chi1=0d0

    !!!!!!!!!!!!!!!!!!!!!!!!
    ! Bankruptcy Parameters
    !
    real(8), parameter :: death = -sqrt(huge(0d0)) !Value of negative consumption
    real(8), parameter :: alpha_k = .36d0              !Capital share
    real(8), parameter :: delta = .10d0                !Depreciation rate
!     real(8), parameter :: iota = .04d0 ! Transaction cost parameter

    !!!!!!!!!!!!!!!!!!!!!!!!
    ! Shock parameters
    !
    real(8), parameter :: trans_gg = 2d0/3d0 !prob of trans from g to g !avg business cycle duration ~ 2yrs
    real(8), parameter :: trans_bb = 2d0/3d0 !prob of trans from b to b
    real(8) :: g = 1.d0+.0224d0 !1.02d0 !value of z in good state !cf KMS calibration
    real(8) :: b = 1.d0-.0224d0 !0.98d0 !value of z in bad state

    real(8), dimension(1:nZ) :: eshift = (/-1.d0,-1.d0/) !this is for the above calibration but with a large trans shock
    real(8) :: eshiftStdevTarget = .0174d0 ! target for standard deviation target

    ! integer, parameter :: g_ind = 2 ! index of z grid associated with the good agg state
    ! integer, parameter :: b_ind = 1 ! index of z grid associated with the bad agg state
    ! real(8), dimension(1:nZ) :: grid_z = (/1d0-.0224d0,1d0+.0224d0/)
    ! real(8), dimension(1:nZ,1:nZ), parameter :: trans_z = reshape((/trans_bb,1d0-trans_gg,1d0-trans_bb,trans_gg/),(/2,2/))
    integer, parameter :: g_ind = 1 ! index of z grid associated with the good agg state
    integer, parameter :: b_ind = 2 ! index of z grid associated with the bad agg state
    real(8), dimension(1:nZ) :: grid_z = (/1d0+.0224d0,1d0-.0224d0/)
    real(8), dimension(1:nZ,1:nZ), parameter :: trans_z = reshape((/trans_gg,1.d0-trans_bb,1.d0-trans_gg,trans_bb/),(/2,2/))


    !!!!!!!!!!!!!!!!!!!!!!!!
    ! Grids 
    !
    real(8), dimension(len_KN) :: grid_KN
    real(8), dimension(len_WW) :: grid_WW

    !!!!!!!!!!!!!!!!!!!!!!!
    ! Loop controls, tolerances
    logical, parameter :: assume_mono = .false.
    real(8) :: lawUpdateRelax = .75d0 ! default value

    !!!!!!!!!!!!!!!!!!!!!!
    ! Additional model assumptions
    logical            :: isResrictedParticipation = .true., & ! bmark is true, only super rich have a'_g/=a'_b
                          isRestrictedPortfolio = .true. !bmark is true, whether a'<0 implies a'_g=a'_b
    logical, parameter :: isUsuryLaw = .true., & ! whether usury law is in place
                          isGarnished = .false., & ! whether default cost is garnished 
                          useKNinLaw = .true. ! whether to use KN in the law of motion

    !!!!!!!!!!!!!!!!!!!!!!!
    ! Other
    character(len=250) :: PEdir
    !
    ! Retirement income
    real(8) :: fracLastPeriod = .35d0  !Benchmark value is .35d0
    real(8) :: fracGuaranteed = .15d0  !Benchmark value is .15d0
    !
    ! Guaranteed threshold below which efficiency will be replaced
    real(8) :: threshold = .000d0  
    !
    ! Parameters of CCNR augmented process
    real(8) :: phiCCNRaugment = -100.d0, eloCCNRaugment = -100.d0, ehiCCNRaugment = -100.d0,&
               pi11augment = -100.d0, pi21augment = -100.d0, pi1augment_init = -100d0
    !
    ! Coverages
    real(8), parameter :: stdevCoverPers = 6.25d0, stdevCoverPerm = 1.d0, stdevCoverTran = 5.d0  
    !
    ! Switches for different experiments
    logical :: useSTYnotKO = .false. ! use STY intead of KO
    logical :: useFlatEarn = .false. ! use flat earnings profile
    logical :: useNoCEV = .false. ! use no countercyclical earnings variance


    !!!!!!!!!!!!!!!!!!!!!!!
    ! Derived type definition for moments
    type mom 
        !(z,KN,WW) define a moment
        integer :: z_ind = -1
        real(8) :: z = -1.d0
        real(8) :: KN = -1.d0
        real(8) :: WW = -1.d0 !weight giving 1/qBond = w*(1+rb-delta) + (1-w)*(1+rg-delta)
        !for a moment (as above) on the grid, there is a unique m associated with it.
        integer :: m = -1
        integer :: KN_ind = -1
        integer :: WW_ind = -1
    end type mom

contains

    ! Given a parameter vector, set parameters (needed for calibration routine primarily)
    subroutine set_param(param)
        implicit none
        real(8), intent(in) :: param(1:)
        
        if (size(param)/=7) then
            STOP 'size of param not as expected'
        end if

        discount = param(1)
        eloCCNRaugment = param(2)
        phiCCNRaugment = param(3)
        ehiCCNRaugment = param(4)
        chi0 = param(6)
        chi1 = param(7)

        pi1augment_init = .2d0
        pi11augment = param(5)
        pi21augment = pi1augment_init/(1d0-pi1augment_init)*(1d0-pi11augment) ! this keeps working age super rich at pi1

!         pi1augment_init = .2d0 
! 
!         ! 5) x in pi11 = 1/(1+exp(-x)) <=> 1+exp(-x) = 1/pi11 <=> exp(-x) = 1/pi11 - 1 <=> x = -log(1/pi11)
!         pi11augment = 1d0/(1d0+exp(-param(5)))
! 
!         pi21augment = pi1augment_init/(1d0-pi1augment_init)*(1d0-pi11augment) ! this keeps working age super rich at pi1
!         eloCCNRaugment = exp(param(2))
! 
!         ! Choose elo, phi, and meanE. For ane meanE>elo and any phi, there will be a ehi that satisfies.
!         phiCCNRaugment = exp(param(3) - 1d0) 
!         ehiCCNRaugment = ((1d0+phiCCNRaugment)*exp(param(4)) - eloCCNRaugment)/phiCCNRaugment ! param(6) is meanE.

        ! Parameter values
        if (.not. doCalibrate) then
            print*,'discount',discount
            print*,'chi0',chi0
            print*,'chi1',chi1
            print*,'pi11augment',pi11augment
            print*,'pi21augment',pi21augment
            print*,'eloCCNRaugment',eloCCNRaugment
            print*,'ehiCCNRaugment',ehiCCNRaugment
            print*,'phiCCNRaugment',phiCCNRaugment
        end if


    end subroutine 

    ! Checks for invalid parameters
    function checkParam() result(flag)
        implicit none
        integer :: flag
        
        flag = 0

        if (discount<.7d0) call badparam('WARNING: discount is bad')
        if (pi11augment<0d0 .or. pi11augment>1.d0) call badparam('WARNING: pi11 is bad')
        if (pi21augment<0d0 .or. pi21augment>1d0) call badparam('WARNING: pi21 is bad')
        if (pi1augment_init<0d0 .or. pi1augment_init>1d0) call badparam('WARNING: pi1augment_init is bad')
        if (ehiCCNRaugment<1d-12 .or. ehiCCNRaugment>10000.d0) call badparam('WARNING: ehi is bad')
        if (phiCCNRaugment<=.000d0 .or. phiCCNRaugment>100.d0) call badparam('WARNING: phi is bad')
        if (ehiCCNRaugment<=eloCCNRaugment) call badparam('WARNING: ehi is less than elo, bad param')
        if (chi0>=1d0) call badparam('WARNING: chi0 is bad')
        if (chi1>1d-12) call badparam('WARNING: chi1 is bad')

    contains 
        subroutine badparam(msg)
            implicit none
            character(len=*), intent(in) :: msg
            flag = 1
            write(*,*) msg
        end subroutine
    end function


    ! Given a model option, set the parameters appropriately
    subroutine set_case(modelOption,doSetDir)
        use mod_myprint
        implicit none
        integer, intent(in) :: modelOption
        logical, intent(in) :: doSetDir ! if true, directory is set. o/w, not set

        ! Set the defaults
        defBnkAllowed = .true. ! whether default is allowed
        isBAPCPA = .false. ! whether BAPCPA is in place
        defZcontingent = .false. ! whether default is only allowed contingent on z
        isPE = .false. ! whether in PE (partial equilibrium)
        PEdir = trim(rootdatadir)//'/default/' ! where prices are to be taken from if PE experiment
        threshold = 0d0 ! lower bound on earnings threshold (0 is benchmark)
        fracLastPeriod = .35d0 ! amount of retirement earnings proportional to last earnings
        fracGuaranteed = .15d0 ! amount of guaranteed retirement earnings
        useSTYnotKO = .false. ! use Storesletten et al. estimates instead of Karahan et al.
        useFlatEarn = .false. ! use Flat Earnings profile
        useNoCEV = .false. ! use no countercyclical earnings variance
        g = 1.d0+.0224d0;  b = 1.d0-.0224d0 ! TFP shock values
        grid_z(g_ind) = g; grid_z(b_ind) = b ! setup TFP grid
        doCalibrate = .false. ! whether to calibrate or not
        lambda = .9d0 ! staying probability of bad credit record
        lawUpdateRelax = .75d0 ! default value

        ! Overwrite the defaults according to the model options
        select case (modelOption)
            case (1)
                ! Note: in this and every instance, setdir only does something if doSetDir is .true.
                call setdir('default/') 
            case (2) 
                defBnkAllowed = .false.; lambda = 1d0
                call setdir('nodefault/')
            case (3)
                defBnkAllowed = .false.; lambda = 1d0
                isPE = .true.
                call setdir('nodefaultPE/')
            case (4)
                isBAPCPA = .true.
                call setdir('BAPCPA/')
            case (5)
                isBAPCPA = .true.
                isPE = .true.
                call setdir('BAPCPAPE/')
            case (6)
                defZcontingent = .true.
                defZcontingent_AllowedZind = b_ind
                call setdir('zcontingent/CDR/')
            case (7)
                defZcontingent = .true.
                defZcontingent_AllowedZind = b_ind
                isPE = .true.
                call setdir('zcontingentPE/CDR/')
            case (8)
                defZcontingent = .true.
                defZcontingent_AllowedZind = g_ind
                call setdir('zcontingent/CDE/')
            case (9)
                defZcontingent = .true.
                defZcontingent_AllowedZind = g_ind
                isPE = .true.
                call setdir('zcontingentPE/CDE/')
            case (21)
                call setdir('default_expend/')
            case (22) 
                defBnkAllowed = .false.; lambda = 1d0
                lawUpdateRelax = .98d0 ! wasn't converging for lower values
                call setdir('nodefault_expend/')
            case (23)
                defBnkAllowed = .false.; lambda = 1d0
                isPE = .true.
                PEdir = trim(rootdatadir)//'/default_expend/' ! prices taken from for PE experiment
                call setdir('nodefaultPE_expend/')
            case (24)
                isBAPCPA = .true.
                call setdir('BAPCPA_expend/')
            case (25)
                isBAPCPA = .true.
                isPE = .true.
                PEdir = trim(rootdatadir)//'/default_expend/' ! prices taken from for PE experiment
                call setdir('BAPCPAPE_expend/')
            case (26)
                defZcontingent = .true.
                defZcontingent_AllowedZind = b_ind
                call setdir('zcontingent/CDR_expend/')
            case (27)
                defZcontingent = .true.
                defZcontingent_AllowedZind = b_ind
                isPE = .true.
                PEdir = trim(rootdatadir)//'/default_expend/' ! prices taken from for PE experiment
                call setdir('zcontingentPE/CDR_expend/')
            case (28)
                defZcontingent = .true.
                defZcontingent_AllowedZind = g_ind
                call setdir('zcontingent/CDE_expend/')
            case (29)
                defZcontingent = .true.
                defZcontingent_AllowedZind = g_ind
                isPE = .true.
                PEdir = trim(rootdatadir)//'/default_expend/' ! prices taken from for PE experiment
                call setdir('zcontingentPE/CDE_expend/')
            case (31)
                call setdir('default_expend_largedebt/')
            case (32) 
                defBnkAllowed = .false.; lambda = 1d0
                call setdir('nodefault_expend_largedebt/')

            !!! Experiments with different chi levels 
            case (1000); call setdir('robustchi/00/');    chi1 = 0d0; chi0 = 0.0D0
            case (1025); call setdir('robustchi/25/');    chi1 = 0d0; chi0 = 0.25D0
            case (1050); call setdir('robustchi/50/');    chi1 = 0d0; chi0 = 0.5D0
            case (1075); call setdir('robustchi/75/');    chi1 = 0d0; chi0 = 0.75D0
            case (1090); call setdir('robustchi/90/');    chi1 = 0d0; chi0 = 0.9D0
            case (1099); call setdir('robustchi/99/');    chi1 = 0d0; chi0 = 0.99D0
            case (1104); call setdir('robustchi/1md4/');  chi1 = 0d0; chi0 = 1.d0-1d-4
            case (1108); call setdir('robustchi/1md8/');  chi1 = 0d0; chi0 = 1.d0-1d-8
            case (1120); call setdir('robustchi/1md20/'); chi1 = 0d0; chi0 = 1.d0-1d-20    

            !!! Experiments with different efficiency thresholds
            case (211); call setdir('robusttau/CD_008/'); threshold = .008d0
            case (221); call setdir('robusttau/CD_058/'); threshold = .058d0
            case (231); call setdir('robusttau/CD_127/'); threshold = .127d0
            case (241); call setdir('robusttau/CD_233/'); threshold = .233d0
            case (210); call setdir('robusttau/ND_008/'); defBnkAllowed = .false.; lambda = 1d0; threshold = .008d0
            case (220); call setdir('robusttau/ND_058/'); defBnkAllowed = .false.; lambda = 1d0; threshold = .058d0
            case (230); call setdir('robusttau/ND_127/'); defBnkAllowed = .false.; lambda = 1d0; threshold = .127d0
            case (240); call setdir('robusttau/ND_233/'); defBnkAllowed = .false.; lambda = 1d0; threshold = .233d0

            !!! Experiments with different (kappaG,kappaF) mixes
            case (301); call setdir('robustkappa/CD_0050/'); fracGuaranteed = .00d0; fracLastPeriod = .50d0
            case (311); call setdir('robustkappa/CD_3020/'); fracGuaranteed = .30d0; fracLastPeriod = .20d0
            case (321); call setdir('robustkappa/CD_5000/'); fracGuaranteed = .50d0; fracLastPeriod = .00d0
            case (300); call setdir('robustkappa/ND_0050/'); defBnkAllowed = .false.; lambda = 1d0; fracGuaranteed = .00d0; fracLastPeriod = .50d0
            case (310); call setdir('robustkappa/ND_3020/'); defBnkAllowed = .false.; lambda = 1d0; fracGuaranteed = .30d0; fracLastPeriod = .20d0
            case (320); call setdir('robustkappa/ND_5000/'); defBnkAllowed = .false.; lambda = 1d0; fracGuaranteed = .50d0; fracLastPeriod = .00d0

            !!! Experiments with flat profiles 
            case (401); call setdir('robustprof/CD_live/'); probLive(1:Tend-1) = 1d0; probLive(Tend) = 0d0
            case (411); call setdir('robustprof/CD_cons/'); csnEquiv = 1d0
            case (421); call setdir('robustprof/CD_earn/'); useFlatEarn = .true.
            case (431); call setdir('robustprof/CD_STY/'); useSTYnotKO = .true.
            case (400); call setdir('robustprof/ND_live/'); defBnkAllowed = .false.; lambda = 1d0; probLive(1:Tend-1) = 1d0; probLive(Tend) = 0d0
            case (410); call setdir('robustprof/ND_cons/'); defBnkAllowed = .false.; lambda = 1d0; csnEquiv = 1d0
            case (420); call setdir('robustprof/ND_earn/'); defBnkAllowed = .false.; lambda = 1d0; useFlatEarn = .true.
            case (430); call setdir('robustprof/ND_STY/');  defBnkAllowed = .false.; lambda = 1d0; useSTYnotKO = .true.; lawUpdateRelax = .995d0 ! need tighter updating

            !!! Experiment with flexible portolfio
            case (501); call setdir('robustflex/CD/'); isResrictedParticipation = .false.; isRestrictedPortfolio = .false. 
            case (500); call setdir('robustflex/ND/'); defBnkAllowed = .false.; lambda = 1d0; isResrictedParticipation = .false.; isRestrictedPortfolio = .false. 

            !!! Experiments breaking down components of business cycles
            case (601); call setdir('robustcycles/CD_CEV/'); useNoCEV = .true.
            case (611); call setdir('robustcycles/CD_CEVLSS/'); useNoCEV = .true.; eshiftStdevTarget = 0d0
            case (621); call setdir('robustcycles/CD_CEVLSSTFP/'); useNoCEV = .true.; eshiftStdevTarget = 0d0; g = 1d0; b = 1d0; grid_z = 1d0
            case (600); call setdir('robustcycles/ND_CEV/'); defBnkAllowed = .false.; lambda = 1d0; useNoCEV = .true.
            case (610); call setdir('robustcycles/ND_CEVLSS/'); defBnkAllowed = .false.; lambda = 1d0; useNoCEV = .true.; eshiftStdevTarget = 0d0
            case (620); call setdir('robustcycles/ND_CEVLSSTFP/'); defBnkAllowed = .false.; lambda = 1d0; useNoCEV = .true.; eshiftStdevTarget = 0d0; g = 1d0; b = 1d0; grid_z = 1d0

            !!! Calibrate
            case (9999); call setdir('calibration/'); doCalibrate = .true.

            !!! Catch
            case default
                STOP 'model option not found!'
        end select
        
        if (doSetDir) call myprint('ifThisCrashesThenDirectoryDNE',1) ! if the directory does not exist, this causes a fatal error

        if (doSetDir) print*,'Using directory '//directory

    contains

        subroutine setdir(str)
            implicit none
            character(len=*), intent(in) :: str

            if (doSetDir) call usedir(trim(rootdatadir)//trim(str))

        end subroutine

    end subroutine

    ! Print all parameter values to a file and save it
    subroutine printConfig()
        use mod_myprint
        implicit none
        
        call myprint('isPE',merge(1,0,isPE))
        call myprint('defBnkAllowed',merge(1,0,defBnkAllowed))
        call myprint('isResrictedParticipation',merge(1,0,isResrictedParticipation))
        call myprint('isRestrictedPortfolio',merge(1,0,isRestrictedPortfolio))
        call myprint('isBAPCPA',merge(1,0,isBAPCPA))
        call myprint('isUsuryLaw',merge(1,0,isUsuryLaw))
        call myprint('useKNinLaw',merge(1,0,useKNinLaw))
        call myprint('defZcontingent',merge(1,0,defZcontingent))
        call myprint('defZcontingent_AllowedZind',defZcontingent_AllowedZind)

        call myprint('discount',discount)
        call myprint('chi0',chi0)
        call myprint('chi1',chi1)
        call myprint('chi13',chi13)
        call myprint('medianBAPCPA',medianBAPCPA)
        call myprint('qbarPE',qbarPE)
        call myprint('usuryLimit',usuryLimit)
        call myprint('lambda',lambda)

        call myprint('npers',npers)
        call myprint('ntran',ntran)
        call myprint('nCCNRaugment',nCCNRaugment)
        call myprint('fracGuaranteed',fracGuaranteed)
        call myprint('fracLastPeriod',fracLastPeriod)
        call myprint('stdevCoverPerm',stdevCoverPerm)
        call myprint('stdevCoverPers',stdevCoverPers)
        call myprint('stdevCoverTran',stdevCoverTran)

        call myprint('phiCCNRaugment',phiCCNRaugment)
        call myprint('eloCCNRaugment',eloCCNRaugment)
        call myprint('ehiCCNRaugment',ehiCCNRaugment)
        call myprint('pi1augment_init',pi1augment_init)
        call myprint('pi11augment',pi11augment)
        call myprint('pi21augment',pi21augment)

        call myprint('nA',nA)
        call myprint('nAmm',nAmm)
        call myprint('nE',nE)
        call myprint('nExpenditure',nExpenditure)
        call myprint('nH',nH)
        call myprint('Tret',Tret)
        call myprint('Tend',Tend)
        call myprint('len_KN',len_KN)
        call myprint('len_WW',len_WW)
        call myprint('nZ',nZ)
        call myprint('nM',nM)
        call myprint('nT',nT)
        call myprint('len_t0',len_t0)

        call myprint('eshift',eshift)
        call myprint('grid_KN',grid_KN)
        call myprint('grid_WW',grid_WW)
        
    end subroutine printConfig



    ! The default cost, assumed to be a function of e. 
    elemental function chi(e,eshift)
        implicit none
        real(8) :: chi
        real(8), intent(in) :: e,eshift

        ! Note how the median thresholds is shifted by eshift
        if (isBAPCPA .and. e>medianBAPCPA*eshift) then
            ! I use this format to tame the large possible e values
            ! chi = max(chi13,chi0) 
            chi = max(chi13,chiBnk(e,eshift))
        else
            ! chi = chi0
            chi = chiBnk(e,eshift)
        end if

    end function chi

    ! Default cost when in bankruptcy
    elemental function chiBnk(e,eshift)
        implicit none   
        real(8) :: chiBnk
        real(8), intent(in) :: e,eshift
        
!         chiBnk = min(max(chi0 - chi1/e,0d0),1d0) ! 1st submission
        chiBnk = min(max(chi0 + chi1/e,0d0),1d0) 

    end function chiBnk

    ! How much is garnished
    elemental function chiGarnish(e,eshift)
        implicit none
        real(8) :: chiGarnish
        real(8), intent(in) :: e,eshift


        ! Note how the median thresholds is shifted by eshift
        if (isBAPCPA .and. e>medianBAPCPA*eshift) then
            if (isGarnished) then
                ! chiGarnish = max(chi13,min(max(chi0 - chi1/e,0d0),1d0)) 
                chiGarnish = chi(e,eshift)
            else
                chiGarnish = chi13 
            end if
        else
            if (isGarnished) then
                ! chiGarnish = min(max(chi0 - chi1/e,0d0),1d0)
                chiGarnish = chi(e,eshift)
            else
                chiGarnish = 0d0
            end if
        end if

    end function chiGarnish

    subroutine print_common_var()
        use mod_myprint
        implicit none

        call myprint('discount',discount)
        call myprint('chi0',chi0)
        call myprint('chi1',chi1)
        call myprint('chi13',chi13)
        call myprint('medianBAPCPA',medianBAPCPA)
        call myprint('nZ',nZ)
        call myprint('nH',nH)
        call myprint('len_KN',len_KN)
        call myprint('nM',nM)
        call myprint('nT',nT)
        call myprint('len_t0',len_t0)
        call myprint('alpha_k',alpha_k)
        call myprint('delta',delta)    
        call myprint('lambda',lambda)
        call myprint('probLive',probLive)
        call myprint('csnEquiv',csnEquiv)
        call myprint('g',g)
        call myprint('b',b)
        call myprint('g_ind',g_ind)
        call myprint('b_ind',b_ind)
        call myprint('grid_z',grid_z)
        call myprint('trans_z',trans_z)
        call myprint('eshift',eshift)
      
        call myprint('nA',nA)
        call myprint('nAmm',nAmm)
        call myprint('nE',nE)
        call myprint('nExpenditure',nExpenditure)
        call myprint('nH',nH)
        call myprint('Tret',Tret)
        call myprint('Tend',Tend)
        call myprint('T1age',T1age)

    end subroutine print_common_var

end module mod_param

