module mod_stats
    implicit none

    !Type definitions
    type stat_type
        real(8) :: K,K_msr_1,K_msr_2,N,Y,A,w,z
        real(8) :: meanWealth,medianWealth,wealthGini,wealthTop5,wealthTop20,wealth8060,wealth6040,wealth4020
        real(8) :: meanEarn,medianEarn,earnGini,earnTop5,earnTop20,earn8060,earn6040,earn4020
        real(8) :: avgIntPaid,avgNonWeightIntPaid
        real(8) :: pop_in_debt, total_debt, debt_to_earn_def, earn_def, pop_def, debt_def, debt_discharge, a_debt_discharge
        real(8) :: frac_def_below_median, debt_def_below_median, earn_def_below_median
        real(8) :: frac_def_above_median, debt_def_above_median, earn_def_above_median
        real(8) :: cv_c, cv_a, cv_y, mean_c, mean_a, mean_y, tot_napx
        real(8) :: cv_c_80, cv_a_80, cv_y_80, mean_c_80, mean_a_80, mean_y_80, tot_napx_80
        real(8) :: cv_c_20, cv_a_20, cv_y_20, mean_c_20, mean_a_20, mean_y_20, tot_napx_20
        real(8) :: h0_pop_def,h0_pop_in_debt,h0_debt_discharge,h0_total_debt,h0_debt_to_earn_def,h0_frac_def_below_median
        real(8) :: h0_debt_def,h0_debt_def_below_median,h0_debt_def_above_median,h0_earn_def,h0_earn_def_below_median,h0_earn_def_above_median 
        real(8) :: h1_pop_def,h1_pop_in_debt,h1_debt_discharge,h1_total_debt,h1_debt_to_earn_def,h1_frac_def_below_median
        real(8) :: h1_debt_def,h1_debt_def_below_median,h1_debt_def_above_median,h1_earn_def,h1_earn_def_below_median,h1_earn_def_above_median 
    end type

contains

    function computeKp(dsn,q,ap) result(Kp)
        use mod_matlab, only: interp1q
        use mod_data, only: gA
        use mod_param, only: nA,nAmm,nE,Tend,nH
        implicit none
        real(8), dimension(:,:,:,:), intent(in) :: dsn,q,ap
        real(8) :: Kp
        !local
        real(8) :: accum
        integer :: iA,iE,age,iH 
       
        accum = 0.d0
        do iH = 1,nH
            do age = 1,Tend-1
                do iE = 1,nE
                    do iA = merge(1,nAmm+1,iH==1),nA
                        if (ap(iA,iE,age,iH)/=0d0 .and. dsn(iA,iE,age,iH)/=0d0) then 
                            accum = accum + interp1q(gA,q(:,iE,age,iH),ap(iA,iE,age,iH))*dsn(iA,iE,age,iH)*ap(iA,iE,age,iH)
                        end if
                        if (isNaN(accum)) then
                           print*,'NaN1', interp1q(gA,q(:,iE,age,iH),ap(iA,iE,age,iH))
                           print*,'NaN2',dsn(iA,iE,age,iH)
                           print*,'NaN3',ap(iA,iE,age,iH)
                           print*,'NaN encountered'
                        end if
                    end do
                end do
            end do
        end do
        Kp = accum

    end function computeKp

    function computeK(dsn,z,d,gE) result(Kout)
        use mod_convertAtoK
        use mod_root1dim, only: sub_brent
        use mod_param, only: chiGarnish, nA,nAmm,nE,Tend,nH,alpha_K
        use mod_data, only: gA
        implicit none
        real(8), intent(in) :: z
        real(8), dimension(nA,nE,Tend,nH), intent(in) :: dsn
        logical, dimension(nA,nE,Tend,nH), intent(in) :: d
        real(8), intent(in) :: gE(nE,Tend)
        real(8) :: Kout
        !local
        real(8) :: K,N
        real(8) :: accum, A, chiPaid, w
        integer :: iE, iA, age, iH

        N = sum(gE*sum(sum(dsn,4),1))

        call sub_brent(K,res,1d-8,1d3)

        Kout = K

    contains

        ! Given K, computes the wage and uses it to find the implied K
        function res(Kin)
            implicit none
            real(8), intent(in) :: Kin
            real(8) :: res
            
            w = z*(1d0-alpha_K)*(Kin/N)**alpha_K
       
            accum = 0d0
            do iH = 1,nH
                do age = 1,Tend
                    do iE = 1,nE
                        do iA = merge(1,nAmm+1,iH==1),nA
                            chiPaid = chiGarnish(gE(iE,age),1d0)
                            accum = accum + dsn(iA,iE,age,iH)*merge(gA(iA),chiPaid*w*gE(iE,age),.not. d(iA,iE,age,iH))
                        end do
                    end do
                end do
            end do
            A = accum

            res = Kin - convertAtoK(A,N,z) ! convertAtoK gives implied K

        end function

    end function computeK

    ! computeStats
    subroutine computeStats(statlist,s,Kmethod,dsn,z,eshiftval,v,c,ap,d,q,gEin)
        use mod_param
        use mod_matlab, only: interp1, interp1q, sort2, cumsum
        use mod_gini
        use mod_shocks, only: state
        use mod_data, only: gA
        implicit none
        real(8), dimension(100), intent(out) :: statlist
        type(stat_type), intent(out) :: s
        integer, intent(in) :: Kmethod
        real(8), dimension(:,:,:,:), intent(in) :: dsn,v,c,ap,q
        real(8), intent(in) :: gEin(:,:),z,eshiftval
        logical, intent(in) :: d(:,:,:,:)
        ! local
        real(8), allocatable, dimension(:,:,:,:) :: debtlevel, earnlevel, a
        real(8), allocatable, dimension(:,:) :: gE, marginalE
        real(8), allocatable, dimension(:) :: sortedE, tmpE,  tmpE2, LorenzGrid, Lorenz, marginalA
        integer, allocatable, dimension(:) :: isortE
        logical, allocatable, dimension(:,:,:,:) :: indebt, belowmedian, top20, bottom80, abovemedian, d_h0, d_h1
        real(8) :: accum,accum2,accum3,accum4,tmp
        integer :: iA,iE,iH,age

        allocate(isortE(nE*Tend), sortedE(nE*Tend), tmpE(nE*Tend),  tmpE2(nE*Tend), LorenzGrid(nLorenzPts), Lorenz(nLorenzPts))
        allocate(gE(nE,Tend))
        allocate(a(nA,nE,Tend,nH))
        allocate(marginalA(nA))
        allocate(marginalE(nE,Tend))
        allocate(debtlevel(nA,nE,Tend,nH))
        allocate(indebt(nA,nE,Tend,nH))
        allocate(earnlevel(nA,nE,Tend,nH))
        allocate(belowmedian(nA,nE,Tend,nH))
        allocate(top20(nA,nE,Tend,nH))
        allocate(bottom80(nA,nE,Tend,nH))
        allocate(abovemedian(nA,nE,Tend,nH))
        allocate(d_h0(nA,nE,Tend,nH))
        allocate(d_h1(nA,nE,Tend,nH))

        ! Get gE grid (taking into account that z changes the level of e for everyone)
        gE = eshiftval*gEin

        ! Compute mean and median wealth
        marginalA = 0d0
        marginalE = 0d0
        do iH = 1,nH
            do age = 1,Tend
                do iE = 1,nE
                    do iA = 1,nA
                        marginalA(iA) = marginalA(iA) + dsn(iA,iE,age,iH)
                        marginalE(iE,age) = marginalE(iE,age) + dsn(iA,iE,age,iH)
                        a(iA,iE,age,iH) = gA(iA)
                    end do
                end do
            end do
        end do

        if (Kmethod==0) then
            s%K_msr_1 = computeKp(dsn,q,ap)
            s%K_msr_1 = max(s%K_msr_1,1d-6)
        end if

        s%K_msr_2 = computeK(dsn,z,d,gE) !computeK(dsn,d,gA,gE,w,nA,nAmm,nE,Tend,nH)
        s%K_msr_2 = max(s%K_msr_2,1d-6)

        if (Kmethod==0) then
            s%K = .5d0*(s%K_msr_1 + s%K_msr_2)     
        elseif (Kmethod==2) then
            s%K = s%K_msr_2
        else
            STOP 'ERROR: did not think I would need different Kmethod'
        end if

        s%N = sum(gE*marginalE)
        s%Y = s%K**alpha_k*s%N**(1d0-alpha_k)
        s%A = s%K*(1d0+alpha_K*(s%K/s%N)**(alpha_K-1d0)-delta)
        s%z = z
        s%w = z*(1d0-alpha_K)*(s%K/s%N)**(alpha_K)

        s%meanWealth = sum(gA*marginalA) ! dot_product(gA,sum(sum(sum(dsn,4),3),2))
        s%medianWealth = interp1q(cumsum(marginalA),gA,.5d0) ! interp1(cumsum(sum(sum(sum(dsn,4),3),2)),gA,.5d0)
        
        ! call sub_gini(s%wealthGini,Lorenz,LorenzGrid,sum(sum(sum(dsn,4),3),2),gA)
        call sub_gini(s%wealthGini,Lorenz,LorenzGrid,marginalA,gA)
        
        s%wealthTop5 =  1.d0 - interp1(LorenzGrid,Lorenz,.95d0)
        s%wealthTop20 = 1.d0 - interp1(LorenzGrid,Lorenz,.8d0)
        s%wealth4020 = interp1(LorenzGrid,Lorenz,.8d0) - interp1(LorenzGrid,Lorenz,.6d0)
        s%wealth6040 = interp1(LorenzGrid,Lorenz,.6d0) - interp1(LorenzGrid,Lorenz,.4d0)
        s%wealth8060 = interp1(LorenzGrid,Lorenz,.4d0) - interp1(LorenzGrid,Lorenz,.2d0)
        
        ! Compute mean earnings to median earnings
        s%meanEarn = s%w * s%N
        call sort2(sortedE,isortE,reshape(gE,(/nE*Tend/)))
        tmpE = reshape(marginalE,(/nE*Tend/)) !Now the invariant dsn over e has been sorted according to e
        tmpE = cumsum(tmpE(isortE))
        s%medianEarn = s%w * interp1(tmpE,sortedE,.5d0)
        
        tmpE2 = reshape(marginalE,(/nE*Tend/))
        tmpE2 = tmpE2(isortE)
        call sub_gini(s%earnGini,Lorenz,LorenzGrid,tmpE2,sortedE)
        
        s%earnTop5 = 1d0 - interp1(LorenzGrid,Lorenz,.95d0)
        s%earnTop20 = 1d0 - interp1(LorenzGrid,Lorenz,.8d0)
        s%earn4020 = interp1(LorenzGrid,Lorenz,.8d0) - interp1(LorenzGrid,Lorenz,.6d0)
        s%earn6040 = interp1(LorenzGrid,Lorenz,.6d0) - interp1(LorenzGrid,Lorenz,.4d0)
        s%earn8060 = interp1(LorenzGrid,Lorenz,.4d0) - interp1(LorenzGrid,Lorenz,.2d0)

        ! Calculate the debt-weighted average interest rate paid on debt (for iH=1)
        accum = 0.d0
        accum2 = 0.d0
        accum3 = 0.d0
        accum4 = 0.d0
        do age = 1,Tend
            do iE = 1,nE
                do iA = 1,nA
                    if (ap(iA,iE,age,1)<-1d-12) then
                        tmp = interp1q(gA,q(:,iE,age,1),ap(iA,iE,age,1))
                        if (tmp>0.d0) then
                            accum = accum + ap(iA,iE,age,1)*dsn(iA,iE,age,1)
                            accum2 = accum2 + (1d0/tmp-1d0)*ap(iA,iE,age,1)*dsn(iA,iE,age,1)
                            accum3 = accum3 + dsn(iA,iE,age,1)
                            accum4 = accum4 + (1d0/tmp-1d0)*dsn(iA,iE,age,1)
                        end if
                    end if
                end do
            end do
        end do
        s%avgIntPaid = accum2/accum
        s%avgNonWeightIntPaid = accum4/accum3


        ! Create some large arrays to make computing stats easier
        do iH = 1,nH
            do age = 1,Tend
                do iE = 1,nE
                    do iA = 1,nA
                        ! Debt measured as -a + x
                        debtlevel(iA,iE,age,iH) = - gA(iA) + state(iE)%expend
                        ! Earnings 
                        earnlevel(iA,iE,age,iH) = s%w * gE(iE,age)
                        ! Whether super rich or not
                        top20(iA,iE,age,iH) = state(iE)%isSuperRich
                    end do
                end do
            end do
        end do
        indebt = debtlevel>0d0  ! in debt indicator
        belowmedian = earnlevel < s%medianEarn ! below median indicator
        abovemedian = .not. belowmedian
        bottom80 = .not. top20

        ! Debt and default statistics for d=1, regardless of h
        s%pop_def = sum(dsn,mask=d) ! Population of defaulters
        s%pop_in_debt = sum(dsn,mask=indebt) ! Population in debt
        s%debt_discharge = mean(debtlevel,dsn,size(dsn),d,opt=1)  ! Total debt, measured as -a+x, discharged 
        s%a_debt_discharge = -mean(a,dsn,size(dsn),d,opt=1)  ! Total debt, measured as -a, discharged 
        s%total_debt = mean(debtlevel,dsn,size(dsn),indebt,opt=1) ! Total msr in debt 
        s%debt_to_earn_def = s%debt_discharge/mean(earnlevel,dsn,size(dsn),d,opt=1) ! Total debt of defaulters / total earnings of defaulters 
        s%frac_def_below_median = sum(dsn,mask=d .and. belowmedian)/s%pop_def ! Compute the percentage of defaulters with below median earnings
        s%debt_def = mean(debtlevel,dsn,size(dsn),indebt,d) ! average debt level conditional on defaulting
        s%debt_def_below_median = mean(debtlevel,dsn,size(dsn),indebt,d,belowmedian) ! average debt level conditional on defaulting and having below median earnings
        s%debt_def_above_median = mean(debtlevel,dsn,size(dsn),indebt,d,abovemedian) ! average debt level conditional on defaulting and having above median earnings
        s%earn_def = mean(earnlevel,dsn,size(dsn),d) ! Average earnings conditional on d
        s%earn_def_below_median = mean(earnlevel,dsn,size(dsn),d,belowmedian) ! Average earnings conditional on d and below median
        s%earn_def_above_median = mean(earnlevel,dsn,size(dsn),d,abovemedian) ! Average earnings conditional on d and above median

        ! Debt and default statistics for d=1 and h=0
        d_h0 = d
        d_h0(:,:,:,2) = .false. ! condition on h=0
        s%h0_pop_def = sum(dsn,mask=d_h0) ! Population of defaulters
        s%h0_pop_in_debt = sum(dsn,mask=indebt) ! Population in debt, any h
        s%h0_debt_discharge = mean(debtlevel,dsn,size(dsn),d_h0,opt=1)  ! Total debt discharged 
        s%h0_total_debt = mean(debtlevel,dsn,size(dsn),indebt,opt=1) ! Total msr in debt 
        s%h0_debt_to_earn_def = s%h0_debt_discharge/mean(earnlevel,dsn,size(dsn),d_h0,opt=1) ! Total debt of defaulters / total earnings of defaulters 
        s%h0_frac_def_below_median = sum(dsn,mask=d_h0 .and. belowmedian)/s%h0_pop_def ! Compute the percentage of defaulters with below median earnings
        s%h0_debt_def = mean(debtlevel,dsn,size(dsn),indebt,d_h0) ! average debt level conditional on defaulting
        s%h0_debt_def_below_median = mean(debtlevel,dsn,size(dsn),indebt,d_h0,belowmedian) ! average debt level conditional on defaulting and having below median earnings
        s%h0_debt_def_above_median = mean(debtlevel,dsn,size(dsn),indebt,d_h0,abovemedian) ! average debt level conditional on defaulting and having above median earnings
        s%h0_earn_def = mean(earnlevel,dsn,size(dsn),d_h0) ! Average earnings conditional on d
        s%h0_earn_def_below_median = mean(earnlevel,dsn,size(dsn),d_h0,belowmedian) ! Average earnings conditional on d and below median
        s%h0_earn_def_above_median = mean(earnlevel,dsn,size(dsn),d_h0,abovemedian) ! Average earnings conditional on d and above median

        d_h1 = d
        d_h1(:,:,:,1) = .false. ! condition on h=1
        s%h1_pop_def = sum(dsn,mask=d_h1) ! Population of defaulters
        s%h1_pop_in_debt = sum(dsn,mask=indebt) ! Population in debt
        s%h1_debt_discharge = mean(debtlevel,dsn,size(dsn),d_h1,opt=1)  ! Total debt discharged 
        s%h1_total_debt = mean(debtlevel,dsn,size(dsn),indebt,opt=1) ! Total msr in debt 
        s%h1_debt_to_earn_def = s%h1_debt_discharge/mean(earnlevel,dsn,size(dsn),d_h1,opt=1) ! Total debt of defaulters / total earnings of defaulters 
        s%h1_frac_def_below_median = sum(dsn,mask=d_h1 .and. belowmedian)/s%h1_pop_def ! Compute the percentage of defaulters with below median earnings
        s%h1_debt_def = mean(debtlevel,dsn,size(dsn),indebt,d_h1) ! average debt level conditional on defaulting
        s%h1_debt_def_below_median = mean(debtlevel,dsn,size(dsn),indebt,d_h1,belowmedian) ! average debt level conditional on defaulting and having below median earnings
        s%h1_debt_def_above_median = mean(debtlevel,dsn,size(dsn),indebt,d_h1,abovemedian) ! average debt level conditional on defaulting and having above median earnings
        s%h1_earn_def = mean(earnlevel,dsn,size(dsn),d_h1) ! Average earnings conditional on d
        s%h1_earn_def_below_median = mean(earnlevel,dsn,size(dsn),d_h1,belowmedian) ! Average earnings conditional on d and below median
        s%h1_earn_def_above_median = mean(earnlevel,dsn,size(dsn),d_h1,abovemedian) ! Average earnings conditional on d and above median

        ! Top 20% versus bottom 80%
        !!! c.v. of consumption, mean of consumption, c.v. of wealth, mean of wealth, c.v. of earnings, mean of earnings
        s%mean_c = mean(c,dsn,size(dsn))
        s%mean_a = mean(a,dsn,size(dsn))
        s%mean_y = mean(earnlevel,dsn,size(dsn))
        s%mean_c_20 = mean(c,dsn,size(dsn),top20)
        s%mean_a_20 = mean(a,dsn,size(dsn),top20)
        s%mean_y_20 = mean(earnlevel,dsn,size(dsn),top20)
        s%mean_c_80 = mean(c,dsn,size(dsn),bottom80)
        s%mean_a_80 = mean(a,dsn,size(dsn),bottom80)
        s%mean_y_80 = mean(earnlevel,dsn,size(dsn),bottom80)

        s%cv_c = sqrt(var(c,dsn,size(dsn)))/s%mean_c
        s%cv_a = sqrt(var(a,dsn,size(dsn)))/s%mean_a
        s%cv_y = sqrt(var(earnlevel,dsn,size(dsn)))/s%mean_y
        s%cv_c_20 = sqrt(var(c,dsn,size(dsn),top20))/s%mean_c_20
        s%cv_a_20 = sqrt(var(a,dsn,size(dsn),top20))/s%mean_a_20
        s%cv_y_20 = sqrt(var(earnlevel,dsn,size(dsn),top20))/s%mean_y_20
        s%cv_c_80 = sqrt(var(c,dsn,size(dsn),bottom80))/s%mean_c_80
        s%cv_a_80 = sqrt(var(a,dsn,size(dsn),bottom80))/s%mean_a_80
        s%cv_y_80 = sqrt(var(earnlevel,dsn,size(dsn),bottom80))/s%mean_y_80

        s%tot_napx = mean(debtlevel,dsn,size(dsn),indebt,opt=1) ! total debt level of top 20 (not the average)
        s%tot_napx_20 = mean(debtlevel,dsn,size(dsn),top20,indebt,opt=1) ! debt level of top 20 households
        s%tot_napx_80 = mean(debtlevel,dsn,size(dsn),bottom80,indebt,opt=1) ! debt level of bottom 80
        
        ! Prep and display some statistics
        statlist = 0d0
        statlist(01)=s%K/s%Y                                                                            
        statlist(02)=s%pop_def                                                                          
        statlist(03)=sum(dsn(1:nAmm,:,:,1)) ! population with a<0                                                      
        statlist(04)=dot_product(gA(1:nAmm),sum(sum(dsn(1:nAmm,:,:,1),3),2))/s%Y ! total "a" debt of h=0 households / Y 
        statlist(05)=sum(dsn(:,1:nCCNRaugment,:,1),mask=d(:,1:nCCNRaugment,:,1)) ! measure of h=0,d=1,superrich households
        statlist(06)=sum(dsn(1:nAmm,1:nCCNRaugment,:,1)) ! measure of a<0,superrich households 
        statlist(07)=dot_product(gA(1:nAmm),sum(sum(dsn(1:nAmm,1:nCCNRaugment,:,1),3),2))/s%Y ! total "a" debt of superrich households / Y     
        statlist(08)=sum(dsn(:,nCCNRaugment+1:nE,:,1),mask=d(:,nCCNRaugment+1:nE,:,1)) ! measure of h=0,d=1,nonsuperrich households
        statlist(09)=sum(dsn(1:nAmm,nCCNRaugment+1:nE,:,1)) ! measure of a<0,nonsuperrich households
        statlist(10)=dot_product(gA(1:nAmm),sum(sum(dsn(1:nAmm,nCCNRaugment+1:nE,:,1),3),2))/s%Y ! total "a" debt of nonsuperrich households / Y
        statlist(11)=s%meanWealth/s%medianWealth                                                         
        statlist(12)=s%meanEarn/s%medianEarn
        statlist(13)=s%wealthTop5
        statlist(14)=s%wealthTop20
        statlist(15)=s%wealth4020
        statlist(16)=s%wealth6040
        statlist(17)=s%wealth8060
        statlist(18)=s%wealthGini
        statlist(19)=s%earnTop5
        statlist(20)=s%earnTop20
        statlist(21)=s%earn4020
        statlist(22)=s%earn6040
        statlist(23)=s%earn8060
        statlist(24)=s%earnGini
        statlist(25)=s%avgIntPaid
        statlist(26)=s%avgNonWeightIntPaid
        statlist(27)=s%frac_def_below_median
        statlist(28)=s%debt_discharge/s%Y
        statlist(29)=s%debt_to_earn_def
        statlist(30)=s%earn_def/s%Y
        statlist(31)=s%earn_def_below_median/s%Y
        statlist(32)=s%earn_def_above_median/s%Y
        statlist(33)=s%debt_def/s%Y
        statlist(34)=s%debt_def_below_median/s%Y
        statlist(35)=s%debt_def_above_median/s%Y
        statlist(36)=s%pop_in_debt
        statlist(37)=s%total_debt/s%Y
        statlist(38)=sum(dsn(:,:,:,1),mask = d(:,:,:,1)) ! h=0,d=1
        statlist(39)=s%mean_c
        statlist(40)=s%mean_a
        statlist(41)=s%mean_y
        statlist(42)=s%cv_c
        statlist(43)=s%cv_a
        statlist(44)=s%cv_y
        statlist(45)=s%mean_c_20
        statlist(46)=s%mean_a_20
        statlist(47)=s%mean_y_20
        statlist(48)=s%cv_c_20
        statlist(49)=s%cv_a_20
        statlist(50)=s%cv_y_20
        statlist(51)=s%mean_c_80
        statlist(52)=s%mean_a_80
        statlist(53)=s%mean_y_80
        statlist(54)=s%cv_c_80
        statlist(55)=s%cv_a_80
        statlist(56)=s%cv_y_80
        statlist(57)=sum(v(nAmm+1,:,1,1)*dsn(nAmm+1,:,1,1))/sum(dsn(nAmm+1,:,1,1)) ! newborn util
        statlist(58)=s%K
        statlist(59)=s%N
        statlist(60)=z

        ! Additional statistics
        statlist(61)=s%h0_pop_def
        statlist(62)=s%h0_pop_in_debt
        statlist(63)=s%h0_debt_discharge/s%Y
        statlist(64)=s%h0_total_debt/s%Y
        statlist(65)=s%h0_debt_to_earn_def
        statlist(66)=s%h0_frac_def_below_median
        statlist(67)=s%h0_debt_def/s%Y
        statlist(68)=s%h0_debt_def_below_median/s%Y
        statlist(69)=s%h0_debt_def_above_median/s%Y
        statlist(70)=s%h0_earn_def/s%Y
        statlist(71)=s%h0_earn_def_below_median/s%Y
        statlist(72)=s%h0_earn_def_above_median/s%Y

        statlist(73)=s%h1_pop_def
        statlist(74)=s%h1_pop_in_debt
        statlist(75)=s%h1_debt_discharge/s%Y
        statlist(76)=s%h1_total_debt/s%Y
        statlist(77)=s%h1_debt_to_earn_def
        statlist(78)=s%h1_frac_def_below_median
        statlist(79)=s%h1_debt_def/s%Y
        statlist(80)=s%h1_debt_def_below_median/s%Y
        statlist(81)=s%h1_debt_def_above_median/s%Y
        statlist(82)=s%h1_earn_def/s%Y
        statlist(83)=s%h1_earn_def_below_median/s%Y
        statlist(84)=s%h1_earn_def_above_median/s%Y

        ! Even more stats
        statlist(85)=s%a_debt_discharge/s%Y
        statlist(86)=s%tot_napx/s%Y
        statlist(87)=s%tot_napx_20/s%Y
        statlist(88)=s%tot_napx_80/s%Y

    end subroutine

    subroutine print_statlist(statlist)
        implicit none
        real(8), intent(in) :: statlist(:)

        write(*,*) 'K/Y:                                   ',statlist(01)
        write(*,*) 'msr(d=1) any h:                        ',statlist(02)
        write(*,*) 'msr(a<0):                              ',statlist(03)
        write(*,*) 'sum a<0 / Y:                           ',statlist(04)
        write(*,*) 'msr(h=0,d=1,superrich):                ',statlist(05)
        write(*,*) 'msr(a<0,superrich):                    ',statlist(06)
        write(*,*) 'sum a<0 / Y superrich:                 ',statlist(07)
        write(*,*) 'msr(h=0,d=1,normal):                   ',statlist(08)
        write(*,*) 'msr(a<0,normal):                       ',statlist(09)
        write(*,*) 'sum a<0 / Y normal:                    ',statlist(10)
        write(*,*) 'Mean/Median Wealth:                    ',statlist(11)
        write(*,*) 'Mean/Median Earnings:                  ',statlist(12)
        write(*,*) 'Wealth Top 5%:                         ',statlist(13)
        write(*,*) 'Wealth Top 20%:                        ',statlist(14)
        write(*,*) 'Wealth 20%-40%:                        ',statlist(15)
        write(*,*) 'Wealth 40%-60%:                        ',statlist(16)
        write(*,*) 'Wealth 60%-80%:                        ',statlist(17)
        write(*,*) 'Wealth Gini:                           ',statlist(18)
        write(*,*) 'Earn Top 5%:                           ',statlist(19)
        write(*,*) 'Earn Top 20%:                          ',statlist(20)
        write(*,*) 'Earn 20%-40%:                          ',statlist(21)
        write(*,*) 'Earn 40%-60%:                          ',statlist(22)
        write(*,*) 'Earn 60%-80%:                          ',statlist(23)
        write(*,*) 'Earn Gini:                             ',statlist(24)
        write(*,*) 'Average Debt-Weighted Interest Rate:   ',statlist(25)
        write(*,*) 'Average Non-Weighted Interest Rate:    ',statlist(26)
        write(*,*) 'Fraction of Def w/ below median earn:  ',statlist(27)
        write(*,*) 'Discharged Debt to Y:                  ',statlist(28)
        write(*,*) 'Total debt / earnings of defaulters:   ',statlist(29)
        write(*,*) 'Earnings of defaulters to Y:           ',statlist(30)
        write(*,*) 'Earnings of below median defaulters to Y:',statlist(31)
        write(*,*) 'Earnings of above median defaulters to Y:',statlist(32)
        write(*,*) 'Debt of defaulters to Y:               ',statlist(33)
        write(*,*) 'Debt of below median defaulters to Y:  ',statlist(34)
        write(*,*) 'Debt of above median defaulters to Y:  ',statlist(35)
        write(*,*) 'msr(a-x<0):                            ',statlist(36)
        write(*,*) 'sum a-x<0 / Y:                         ',statlist(37)
        write(*,*) 'msr(h=0,d=1):                          ',statlist(38)
        write(*,*) 'mean of consumption                    ',statlist(39)
        write(*,*) 'mean of assets                         ',statlist(40)
        write(*,*) 'mean of earnings                       ',statlist(41)
        write(*,*) 'c.v. of consumption                    ',statlist(42)
        write(*,*) 'c.v. of assets                         ',statlist(43)
        write(*,*) 'c.v. of earnings                       ',statlist(44)
        write(*,*) 'mean of consumption (top 20)           ',statlist(45)
        write(*,*) 'mean of assets (top 20)                ',statlist(46)
        write(*,*) 'mean of earnings (top 20)              ',statlist(47)
        write(*,*) 'c.v. of consumption (top 20)           ',statlist(48)
        write(*,*) 'c.v. of assets (top 20)                ',statlist(49)
        write(*,*) 'c.v. of earnings (top 20)              ',statlist(50)
        write(*,*) 'mean of consumption (bottom 80)        ',statlist(51)
        write(*,*) 'mean of assets (bottom 80)             ',statlist(52)
        write(*,*) 'mean of earnings (bottom 80)           ',statlist(53)
        write(*,*) 'c.v. of consumption (bottom 80)        ',statlist(54)
        write(*,*) 'c.v. of assets (bottom 80)             ',statlist(55)
        write(*,*) 'c.v. of earnings (bottom 80)           ',statlist(56)
        write(*,*) 'Utility of newborn households          ',statlist(57)
        write(*,*) 'K                                      ',statlist(58)
        write(*,*) 'N                                      ',statlist(59)
        write(*,*) 'z                                      ',statlist(60)

    end subroutine 

    ! Computes the mean sum_i x(i)w(i). In the case of a mask being present,
    ! the conditional mean sum_i x(i)w(i)mask(i) / sum_i w(i)mask(i) is computed if opt==0 or is not present
    ! or if opt==1 then sum_i x(i)w(i)mask(i) is returned
    function mean(x,w,n,mask,mask2,mask3,opt)
        implicit none
        real(8), intent(in) :: x(*),w(*)
        integer, intent(in) :: n
        logical, intent(in), optional :: mask(*),mask2(*),mask3(*)
        integer, intent(in), optional :: opt
        real(8) :: mean
        ! local
        integer :: i,nmask
        real(8) :: accum,accum2
        logical :: doscale

        nmask = merge(1,0,present(mask)) + merge(1,0,present(mask2)) + merge(1,0,present(mask3))


        doscale = present(mask) .and. present(opt)
        if (.not. present(opt)) then
            doscale = .true.
        else
            doscale = opt==0
        end if
        
        accum = 0d0
        accum2 = 0d0

        if (nmask==0) then
!             !$OMP PARALLEL DO SCHEDULE(STATIC) DEFAULT(SHARED) PRIVATE(i) REDUCTION(+:accum)
            do i = 1,n
                accum = accum + x(i)*w(i) 
            end do
!             !$OMP END PARALLEL DO
            mean = accum
        else

            if (nmask==1) then
!                 !$OMP PARALLEL DO SCHEDULE(STATIC) DEFAULT(SHARED) PRIVATE(i) REDUCTION(+:accum,accum2)
                do i = 1,n
                    if (mask(i)) then
                        accum = accum + x(i)*w(i) 
                        accum2 = accum2 + w(i)
                    end if
                end do
!                 !$OMP END PARALLEL DO
            elseif (nmask==2) then
!                 !$OMP PARALLEL DO SCHEDULE(STATIC) DEFAULT(SHARED) PRIVATE(i) REDUCTION(+:accum,accum2)
                do i = 1,n
                    if (mask(i) .and. mask2(i)) then
                        accum = accum + x(i)*w(i) 
                        accum2 = accum2 + w(i)
                    end if
                end do
!                 !$OMP END PARALLEL DO
            elseif (nmask==3) then
!                 !$OMP PARALLEL DO SCHEDULE(STATIC) DEFAULT(SHARED) PRIVATE(i) REDUCTION(+:accum,accum2)
                do i = 1,n
                    if (mask(i) .and. mask2(i) .and. mask3(i)) then
                        accum = accum + x(i)*w(i) 
                        accum2 = accum2 + w(i)
                    end if
                end do
!                 !$OMP END PARALLEL DO
            end if

            if (doscale) then
                if (accum2/=0d0) then
                    mean = accum/accum2
                else
                    mean = -1d0
                end if
            else
                mean = accum
            end if
        end if


    end function

    ! Computes the var(x|m) = E(x-E[x|m]|m)**2 = E[x**2|m] - (Ex|m)**2 where Ey|m = sum_i y(i)w(i)m(i)/sum_i w(i)m(i)
    function var(x,w,n,mask)
        implicit none
        real(8), intent(in) :: x(*),w(*)
        logical, intent(in), optional :: mask(*)
        integer, intent(in) :: n
        real(8) :: var
        ! local
        integer :: i
        real(8) :: accum,accum2,accum3,tmp
        
        accum = 0d0
        accum2 = 0d0
        accum3 = 0d0

        if (.not. present(mask)) then
            do i = 1,n
                tmp = x(i)*w(i)
                accum = accum + tmp
                accum2 = accum2 + x(i)*tmp
            end do
            var = accum2 - accum**2
        else
            do i = 1,n
                if (mask(i)) then
                    tmp = x(i)*w(i)
                    accum = accum + tmp
                    accum2 = accum2 + x(i)*tmp
                    accum3 = accum3 + w(i)
                end if
            end do
            if (accum3/=0d0) then
                var = accum2/accum3 - (accum/accum3)**2
            else
                var = -1d0
            end if
        end if

    end function



end module 
