% analyzeresults contains many switches for different analyses that can
% be run. Many of the graphs/results in the paper are constructed using it.
% 
function [] = analyzeresults()

    clf reset
    cla reset

    global trans_e_SS

%     dfile = [getmatdir(),'results_def.mat'];
%     dBAPCPAfile = [getmatdir(),'results_defBAPCPA.mat'];
%     ndfile = [getmatdir(),'results_nodef.mat'];    
%     cdrfile = [getmatdir(),'results_defCDR.mat'];
%     cdefile = [getmatdir(),'results_defCDE.mat'];
    
    dfile = [getmatdir(),'results_defexpend.mat'];
    dBAPCPAfile = [getmatdir(),'results_defBAPCPAexpend.mat'];
    ndfile = [getmatdir(),'results_nodefexpend.mat'];    
    cdrfile = [getmatdir(),'results_defCDRexpend.mat'];
    cdefile = [getmatdir(),'results_defCDEexpend.mat'];
    
    
    load(dfile,'*_ind','n*','len*','g*','Tend','invar_dsn_e_SS','trans_e_SS')
    D = load(dfile,'q_SS','Eqg','Eqb','*_fcast*','*_real*','invar_dsn_SS','seq_pdfAvg','Ev*','v*','qbar_SS','w_SS','delta','g*','trans_e_SS','Tend','invar_dsn_e_SS','K_SS','N_SS','alpha_k','n*','d_SS','invar_dsn_m','Tret','Tend','probLive');
    ND = load(ndfile,'q_SS','v_SS','Ev','Eqg','Eqb','*_fcast*','*_real*','invar_dsn_SS','seq_pdfAvg','trans_m_cdnal_zp','grid_m*','qbar','invar_dsn_m','qbar_SS','w_SS','g*','n*','trans_e_SS','Tend','invar_dsn_e_SS','Ev');
    
    doBC = 1;
    
    % Define some colors 
    blue = [0 0 1];
    red = [1 0 0];
    green = [0 .3 0];     
    
    doComparePriceSchedules = 1;
    doComparePriceSchedules_zContingent = 0;
    doComparePriceSchedules_BAPCPA = 0;
    doCompareLifecycleDebtDef_BAPCPA = 0;
    doCompareLogCsnDecomp = 0;
    doCheckRecessionPath = 0;
    doWelfareCyclical = 0;
    doBAPCPAfilingprotractedrecesssion = 0;
    
    if (doWelfareCyclical) 
        o = {'LineWidth',2};
        f = {'FontSize',12};
        
        [~,~,~,~,~,welfareseq_dnd]= welfareCompare(1,'results_defexpend','results_nodefexpend');
        [~,~,~,~,~,welfareseq_dbap]= welfareCompare(1,'results_defexpend','results_defBAPCPAexpend');
        
        if (any(welfareseq_dnd.Y~=welfareseq_dbap.Y))
            error('these should be equal');
        end
        
        close all
        set(gcf,'DefaultAxesColorOrder',[blue;red])
        plot(welfareseq_dnd.Y,welfareseq_dnd.gain,'o',...
             welfareseq_dnd.Y,welfareseq_dbap.gain,'+',...
             o{:});
        xlabel('Output per Household',f{:})
        ylabel('Welfare Gain (%)',f{:})
        h = legend('FS to NFS','FS to BAPCPA');
        set(h,f{:});
        title('Welfare Gains Over the Business Cycle',f{:})
        axis tight
        myprint('welfare_over_bc')
        
    end
    
    if (doCompareLifecycleDebtDef_BAPCPA) 
        
        D = lifeCycleAnalysis(dfile,0);
        B = lifeCycleAnalysis(dBAPCPAfile,0);

        clf reset
        cla reset
        set(gcf,'DefaultAxesColorOrder',[blue;green])
        
        subplot(3,1,1)
        plot(D.age,D.popDebt*100,'-',...
             B.age,B.popDebt*100,'--',...
             'LineWidth',2)
        title('Population in Debt')
        ylabel('Population (%)')       
        legend('CD','BAPCPA','Location','NorthEast')
        axis tight 
        subplot(3,1,2)
        plot(D.age,D.Edebt,'-',...
             B.age,B.Edebt,'--',...
             'LineWidth',2)
        axis tight 
        title('Debt conditional on being a debtor')
        ylabel('Debt')       
        subplot(3,1,3)
        plot(D.age,D.Ed*100,'-',...
             B.age,B.Ed*100,'--',...
             'LineWidth',2)
        title('Default Rate')
        ylabel('Defaults (%)')
        axis tight 
        xlabel('age')
        myprint('LifeCycleDebtDefBAPCPA')
        
        clf reset
        cla reset
        
        
    end 

    if (doComparePriceSchedules)

        if (doBC)
            [avgDiscNatSS avgDiscNatBC] = computeNatLimit(ND,doBC);
            [avgDiscEndogSS avgDiscEndogBC] = computeEndogLimit(D,doBC);
        else
            [avgDiscNatSS] = computeNatLimit(ND,doBC);
            [avgDiscEndogSS] = computeEndogLimit(D,doBC);
        end
        
        gAge = 20-1+(1:Tend)';
        
        clf
        plot(gAge,-avgDiscEndogSS,'b-',...
             gAge,-avgDiscNatSS,'r--',...
             'LineWidth',2)
        legend('Default','No Default','location','NorthWest')
        xlabel('Age')
        ylabel('Borrowing Limit')
        title('Borrowing Limits By Age')
        myprint('BorrowLimitByAgeSS')
        
        if (doBC)
               
            clf
            plot(gAge,-avgDiscEndogBC,'b-',...
                 gAge,-avgDiscEndogSS,'b-o',...
                 gAge,-avgDiscNatBC,'r--',...
                 gAge,-avgDiscNatSS,'r--o',...
                 'LineWidth',2)
            legend('Default BC','Default SS','No Default BC','No Default SS','location','NorthWest')
            xlabel('Age')
            ylabel('Debt Limit')
            title('Borrowing Limits By Age')
            myprint('BorrowLimitByAgeBC')
            return
        
        end
        
    end
    
    if (doComparePriceSchedules_BAPCPA)

        clear D ND

        D = load(dfile,'n*','g*','q_SS','Eqg','Eqb','*_fcast*','*_real*','invar_dsn_SS','seq_pdfAvg','Ev*','v*','qbar_SS','w_SS','delta','grid*','trans_e_SS','Tend','invar_dsn_e_SS','K_SS','N_SS','alpha_k','len_*');
        if (doBC) 
            [avgDiscEndogSS avgDiscEndogBC] = computeEndogLimit(D,1);
        else
            [avgDiscEndogSS] = computeEndogLimit(D,0);
        end
        clear D
        
        D_BAPCPA = load(dBAPCPAfile,'n*','g*','q_SS','Eqg','Eqb','*_fcast*','*_real*','invar_dsn_SS','seq_pdfAvg','Ev*','v*','qbar_SS','w_SS','delta','grid*','trans_e_SS','Tend','invar_dsn_e_SS','K_SS','N_SS','alpha_k','len_*');
        if (doBC)
            [avgDiscEndogSS_BAPCPA avgDiscEndogBC_BAPCPA] = computeEndogLimit(D_BAPCPA,1);        
        else
            [avgDiscEndogSS_BAPCPA] = computeEndogLimit(D_BAPCPA,0);        
        end
        clear D_BAPCPA
        
        ND = load(ndfile,'n*','g*','q_SS','v_SS','Ev','Eqg','Eqb','*_fcast*','*_real*','invar_dsn_SS','seq_pdfAvg','trans_m_cdnal_zpb','grid_m*','qbarg','qbarb','invar_dsn_m','grid_K_capital','qbar_SS','w_SS','grid_*','len_*','trans_e_SS','Tend','invar_dsn_e_SS','Ev','Eqg','Eqb');
        if (doBC)
            [avgDiscNatSS_ND avgDiscNatBC_ND] = computeNatLimit(ND,1);        
        else
            [avgDiscNatSS_ND] = computeNatLimit(ND,0);
        end
        clear ND

        gAge = 20-1+(1:Tend)';

        clf reset
        cla reset
        
        set(gcf,'DefaultAxesColorOrder',[green;green;blue;blue;red;red])
        plot(gAge,-avgDiscEndogBC_BAPCPA,'-',...
             gAge,-avgDiscEndogSS_BAPCPA,'-o',...
             gAge,-avgDiscEndogBC,'--',...
             gAge,-avgDiscEndogSS,'--+',...
             gAge,-avgDiscNatBC_ND,':',...
             gAge,-avgDiscNatSS_ND,':x',...             
             'LineWidth',2)
        legend('Default BAPCPA BC','Default BAPCPA SS','Default BC','Default SS','No Default BC','No Default SS','location','NorthWest')
        xlabel('Age')
        ylabel('Debt Limit')
        title('Borrowing Limits By Age')
        myprint('BorrowLimitByAge_BAPCPA_BC')
        
    end
    
    if (doComparePriceSchedules_zContingent)

        clear D ND

        D = load(dfile,'n*','g*','q_SS','Eqg','Eqb','invar_dsn_SS','seq_pdfAvg','Ev*','v*','qbar_SS','w_SS','delta','grid*','trans_e_SS','Tend','invar_dsn_e_SS','K_SS','N_SS','alpha_k','len_*');
        [~,avgDiscEndogBC] = computeEndogLimit(D,1);
        clear D

        D_Bonly = load(cdrfile,'n*','g*','q_SS','Eqg','Eqb','invar_dsn_SS','seq_pdfAvg','Ev*','v*','qbar_SS','w_SS','delta','grid*','Tend','invar_dsn_e_SS','K_SS','N_SS','alpha_k','len_*');
        [~,avgDiscNatBC_D_Bonly] = computeNatLimit(D_Bonly,1);        
        clear D_Bonly

        D_Gonly = load(cdefile,'n*','g*','q_SS','Eqg','Eqb','invar_dsn_SS','seq_pdfAvg','Ev*','v*','qbar_SS','w_SS','delta','grid*','Tend','invar_dsn_e_SS','K_SS','N_SS','alpha_k','len_*');
        [~,avgDiscNatBC_D_Gonly] = computeNatLimit(D_Gonly,1);        
        clear D_Gonly

        ND = load(ndfile,'n*','g*','q_SS','v_SS','Ev','Eqg','Eqb','invar_dsn_SS','seq_pdfAvg','grid_m*','qbar','invar_dsn_m','qbar_SS','w_SS','grid_*','len_*','Tend','invar_dsn_e_SS','Ev','Eqg','Eqb');
        [~,avgDiscNatBC_ND] = computeNatLimit(ND,1);        
        clear ND

        gAge = (20:85)';

        
        
        clf reset
        cla reset
        green = [0 .5 0];
        cyan = [0 .75 .75];
        purple = [.75 0 .75];
%         set(gcf,'DefaultAxesColorOrder',[purple;0 0 1;1 0 0])
        set(gcf,'DefaultAxesColorOrder',[purple;purple;blue;red])
        plot(gAge,-avgDiscNatBC_D_Bonly,'-x',...
             gAge,-avgDiscNatBC_D_Gonly,'-o',...
             gAge,-avgDiscEndogBC,'--',...
             gAge,-avgDiscNatBC_ND,'-.',...
             'LineWidth',2)
        legend('Default Recessions Only','Default Expansions Only','Default','No Default','location','NorthWest')
        xlabel('Age')
        ylabel('Debt Limit')
        axis([-Inf Inf 0 1.8])
        title('Borrowing Limits By Age')
        myprint('BorrowLimitByAge_DefBOnlyCompare')
         
    end
    
    if (doCompareLogCsnDecomp)
        [DSS DBC] = lifeCycleAnalysis(dfile,1);
        [NDSS NDBC] = lifeCycleAnalysis(ndfile,1);

        clf
        plot(DSS.age,DBC.Elogc,'b-',...
             DSS.age,DSS.Elogc,'b-o',...
             NDSS.age,NDBC.Elogc,'r--',...
             NDSS.age,NDSS.Elogc,'r--o',...
             'LineWidth',2)
        axis([20 30 -Inf -.05])
        title('Average of Log Consumption')        
        xlabel('Age')
        ylabel('E(log(c)|age)')
        legend('Default, Business Cycle','Default, Steady State','No Default, Business Cycle','No Default, Steady State','location','southeast')
        myprint('ELogCsnByAgeBC')

        clf
        plot(DSS.age,DBC.varlogc,'b-',...
             DSS.age,DSS.varlogc,'b-o',...
             NDSS.age,NDBC.varlogc,'r--',...
             NDSS.age,NDSS.varlogc,'r--o',...
             'LineWidth',2)
        axis([20 30 -Inf .22])
        legend('Default, Business Cycle','Default, Steady State','No Default, Business Cycle','No Default, Steady State','location','southeast')
        title('Variance of Log Consumption')        
        xlabel('Age')
        ylabel('Var(log(c)|age))')
        myprint('VarLogCsnByAgeBC')
        
        
    end
    
    if (doCheckRecessionPath)
        
        dsnNodes = NaN(nM,Tend);
        dsnNodes(:,1) = ND.invar_dsn_m;
        for t = 2:Tend
            dsnNodes(:,t) = ND.trans_m_cdnal_zp(:,:,b_ind)'*dsnNodes(:,t-1);
        end
        
        zPath = (ND.grid_m__z'*dsnNodes)';
        KNPath = (ND.grid_m__KN'*dsnNodes)';
        qbargPath = (ND.qbar(:,g_ind)'*dsnNodes)';
        qbarbPath = (ND.qbar(:,b_ind)'*dsnNodes)';
        qBondPath = qbargPath + qbarbPath;
        wPath = zPath.*(1-D.alpha_k).*(KNPath).^(D.alpha_k);
        rPath = zPath.*(D.alpha_k).*(KNPath).^(D.alpha_k-1);

        mortProfile = D.probLive();
        
        probnoexpend = 1 - .0242 - .0015;
        
        % adjust mortProfile to also have probnoexpend
        mortProfile = mortProfile*probnoexpend;
        
        wPathSS = repmat(ND.w_SS,size(qBondPath));
        wPathBC = wPath;
        cumqBondPathSS = cumprod(mortProfile.*repmat(ND.qbar_SS,size(qBondPath)));
        cumqBondPathBC = cumprod(mortProfile.*qBondPath);
        
        plot([sum(cumqBondPathSS.*wPathSS)-cumsum(cumqBondPathSS.*wPathSS),...
              sum(cumqBondPathBC.*wPathBC)-cumsum(cumqBondPathBC.*wPathBC)])
          
        % Also find lowest K and lowest N (typical deep recession)
        tmp = (ND.invar_dsn_m>0);
        
        [minKN locminKN] = min(ND.grid_m__KN(tmp));
        locminKN = locminKN(1);
        
        minw = min(ND.grid_m__z)*(1-D.alpha_k).*(minKN).^(D.alpha_k);
        minqBond = ND.qbar(locminKN,g_ind) + ND.qbar(locminKN,b_ind);

        disp('Lowest w and qB values found')
        fprintf(1,'w: %g \n',minw);
        fprintf(1,'qB: %g \n',minqBond);
        
        cumqBondMin = cumprod(mortProfile.*repmat(minqBond,size(qBondPath)));
        
        plot([sum(cumqBondPathSS.*wPathSS)-cumsum(cumqBondPathSS.*wPathSS),...
              sum(cumqBondMin.*minw)-cumsum(cumqBondMin.*minw)])
        
        % Check for income equal to kappa_G only in retirement and 0
        % elsewhere
        ePath = .15*((20:85)'>=66);
        qPath = mortProfile.*repmat(minqBond,size(qBondPath));
        wPath = repmat(minw,size(qBondPath));
        qPathSS = mortProfile.*repmat(ND.qbar_SS,size(qBondPath));
        
        plot(ePath)
        plot(qPath)
        
        accum = 0;
        accum2 = 0;
        for t = 2:Tend
            accum = accum + prod(qPath(2:t-1)).*wPath(t).*ePath(t);
            accum2 = accum2 + prod(qPathSS(2:t-1)).*ND.w_SS.*ePath(t);
        end
        
        disp('Worst NPV of guaranteed retirement income at age 1')
        fprintf(1,'BC:  %g \n', qPath(1)*accum);
        fprintf(1,'SS:  %g \n', qPathSS(1)*accum2);
        fprintf(1,'Ratio BC/SS:  %g \n', qPath(1)*accum/(qPathSS(1)*accum2));

    end

    
end
