% []=bankKSbcstats(matFile)
%
% Computes business cycles stats using the specified mat file
function [statsSS, statsBC, seqlist]=bankKSbcstats(matFile,debtMeasure,varargin)

    nargreq = 2;
    nargopt = nargin-nargreq;

    if nargopt==0
        fid = 1;
    elseif nargopt>=1
        fid = varargin{1};
    end

    load(matFile,'*seq*','len*','nT','grid*','alpha_k','delta','statlist',...
                 'medianBAPCPA','g*','n*','d_SS','invar_dsn_SS','Tend')

    grid_t = len_t0+2:nT-2;

    % Recover some macro aggregates
    Yseq = grid_z(zseq_ind).*Kseq.^(alpha_k).*Nseq.^(1-alpha_k);    
    Iseq(1:nT-1,1) = Kseq(2:nT)-Kseq(1:nT-1)*(1-delta); 
    Iseq(nT,1) = 0;
    Cseq = Yseq - Iseq; %This is not strictly speaking true unless pecuniary cost = 0, but roughly the case
    rSeq = alpha_k.*(Kseq./Nseq).^(alpha_k-1);
    wSeq = (1-alpha_k).*(Kseq./Nseq).^(alpha_k);
    
    % Recover debt statistics
    iKY = 1;
    iintrate = 26;
    
    
    %%% Default for any h
%     idebtY = 4;
%     idisdebtY = 28;
%     ipopdebt = 3;
%     ipop01 = 2;
%     ifracpop01belowmed = 27;
%     iintrate = 26;
    %%% Debt and default only for h = 1;
%     idebtY = 64;
%     idisdebtY = 63;
%     ipopdebt = 62;
%     ipop01 = 61; 
%     ifracpop01belowmed = 66;
    
    if (strcmp(debtMeasure,'revolving'))  
        % For these statistics, the following conventions are used
        ipop01 = 61; % Count households as defaulting if h=0 and d=1 (this is like Ch. 7 bankruptcy)
        ipopdebt = 3; % Count households as in debt based on revolving consumer credit measure -a, rather than model debt -a+x
        idisdebtY = 85; % Total discharged debt based on -a relative to output 
        idebtY = 4; % Total -a debt relative to output (implies h=0) 
        ifracpop01belowmed = 66; % Fraction of h=0,d=1 households with below median earnings
    elseif (strcmp(debtMeasure,'networth'))
        % For these statistics, the following conventions are used
        ipop01 = 61; % Count households as defaulting if h=0 and d=1 (this is like Ch. 7 bankruptcy)
        ipopdebt = 36; % Count households as in debt if -a+x>0
        idisdebtY = 63; % Total discharged debt based on -a+x but only for h=0,d=1 relative to output 
        idebtY = 37; % Total -a+x debt relative to output for any h
        ifracpop01belowmed = 66; % Fraction of h=0,d=1 households with below median earnings
    else
        error('Debt measure is inappropriately specified. Please choose either "revolving", i.e. -a, or "networth", i.e. -a+x')
    end
    
    seq_debt = seq_statlist(idebtY,:)'.*Yseq; % debt (measured as -a+x)/Y * Y
    seq_disdebt = seq_statlist(idisdebtY,:)'.*Yseq; % discharged debt/Y * Y
    seq_popDebt = seq_statlist(ipopdebt,:)'; % pop in debt (measured as a-x<0)
    seq_pop01 = seq_statlist(ipop01,:)'; % measure defaulting with any h (if no expenditure shocks, then defaults by h=1 are zero)
    seq_pop01belowMedian = seq_statlist(ifracpop01belowmed,:)'.*seq_statlist(ipop01,:)'; % defaulters any h who are below median
    seq_intrate = seq_statlist(iintrate,:)'; % 
    
    % Set some output variables
    seqlist.Y = Yseq(grid_t);
    seqlist.seq_debt = seq_debt(grid_t);
    seqlist.seq_disdebt = seq_disdebt(grid_t);
    seqlist.seq_popDebt = seq_popDebt(grid_t);
    seqlist.seq_pop01 = seq_pop01(grid_t);
    seqlist.seq_intrate = seq_intrate(grid_t);
    seqlist.seq_z = grid_z(zseq_ind(grid_t));
    
    if (any(seq_debt<0) || any(seq_disdebt<0))
        error('cannot proceed, debt measure is negative');
    end
    
    devY = log(Yseq(grid_t))-hpfilter(log(Yseq(grid_t)),100); 
    devI = log(Iseq(grid_t))-hpfilter(log(Iseq(grid_t)),100);
    devC = log(Cseq(grid_t))-hpfilter(log(Cseq(grid_t)),100);
    devN = log(Nseq(grid_t))-hpfilter(log(Nseq(grid_t)),100); 
    devmu01 = log(seq_pop01(grid_t))-hpfilter(log(seq_pop01(grid_t)),100); %mu01seq(grid_t);
    devDebt = log(seq_debt(grid_t))-hpfilter(log(seq_debt(grid_t)),100); %Debtseq(grid_t);
    devDisDebt = log(seq_disdebt(grid_t))-hpfilter(log(seq_disdebt(grid_t)),100); %Debtseq(grid_t);
    devPopDebt = log(seq_popDebt(grid_t))-hpfilter(log(seq_popDebt(grid_t)),100); %mu01seq(grid_t);
    devIntRate = log(seq_intrate(grid_t))-hpfilter(log(seq_intrate(grid_t)),100); %mu01seq(grid_t);
    
    r = (3:numel(devY)-2)'; %range
        
    Y = devY(r);
    Ym1 = devY(r-1);
    Ym2 = devY(r-2);
    Yp1 = devY(r+1);
    Yp2 = devY(r+2);

    C = devC(r);
    Cm1 = devC(r-1);
    Cm2 = devC(r-2);
    Cp1 = devC(r+1);
    Cp2 = devC(r+2);

    I = devI(r);
    Im1 = devI(r-1);
    Im2 = devI(r-2);
    Ip1 = devI(r+1);
    Ip2 = devI(r+2);

    N = devN(r);
    Nm1 = devN(r-1);
    Nm2 = devN(r-2);
    Np1 = devN(r+1);
    Np2 = devN(r+2);

    mu01 = devmu01(r);
    mu01m1 = devmu01(r-1);
    mu01m2 = devmu01(r-2);
    mu01p1 = devmu01(r+1);
    mu01p2 = devmu01(r+2);

    Debt = devDebt(r);
    Debtm1 = devDebt(r-1);
    Debtm2 = devDebt(r-2);
    Debtp1 = devDebt(r+1);
    Debtp2 = devDebt(r+2);

    DisDebt = devDisDebt(r);
    DisDebtm1 = devDisDebt(r-1);
    DisDebtm2 = devDisDebt(r-2);
    DisDebtp1 = devDisDebt(r+1);
    DisDebtp2 = devDisDebt(r+2);

    PopDebt = devPopDebt(r);
    PopDebtm1 = devPopDebt(r-1);
    PopDebtm2 = devPopDebt(r-2);
    PopDebtp1 = devPopDebt(r+1);
    PopDebtp2 = devPopDebt(r+2);

    IntRate = devIntRate(r);
    IntRatem1 = devIntRate(r-1);
    IntRatem2 = devIntRate(r-2);
    IntRatep1 = devIntRate(r+1);
    IntRatep2 = devIntRate(r+2);
    
    stdev1 = std([Y,C,I,N,mu01,Debt,PopDebt,DisDebt,IntRate])'*100;
    rho1 = corrcoef([Y,Ym2,Ym1,Y,Yp1,Yp2]);
    rho2 = corrcoef([Y,Cm2,Cm1,C,Cp1,Cp2]);
    rho3 = corrcoef([Y,Im2,Im1,I,Ip1,Ip2]);
    rho4 = corrcoef([Y,Nm2,Nm1,N,Np1,Np2]);
    rho5 = corrcoef([Y,mu01m2,mu01m1,mu01,mu01p1,mu01p2]);
    rho6 = corrcoef([Y,Debtm2,Debtm1,Debt,Debtp1,Debtp2]);
    rho7 = corrcoef([Y,PopDebtm2,PopDebtm1,PopDebt,PopDebtp1,PopDebtp2]);
    rho8 = corrcoef([Y,DisDebtm2,DisDebtm1,DisDebt,DisDebtp1,DisDebtp2]);
    rho9 = corrcoef([Y,IntRatem2,IntRatem1,IntRate,IntRatep1,IntRatep2]);
    
    fprintf(fid,'Output (y)             & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',stdev1(1),rho1(1,2:end));
    fprintf(fid,'Consumption            & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',stdev1(2),rho2(1,2:end));   
    fprintf(fid,'Investment             & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',stdev1(3),rho3(1,2:end));
    fprintf(fid,'Labor Supply           & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',stdev1(4),rho4(1,2:end));
    fprintf(fid,'Population Filing      & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',stdev1(5),rho5(1,2:end));
    fprintf(fid,'Debt                   & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',stdev1(6),rho6(1,2:end));
    fprintf(fid,'Discharged Debt        & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',stdev1(8),rho8(1,2:end));
    fprintf(fid,'Interest Rates         & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',stdev1(9),rho9(1,2:end));
    fprintf(fid,'Population in Debt     & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f \\\\ \n',stdev1(7),rho7(1,2:end));
    
    % Compute means of business cycle variables
    t1 = mean(Kseq(grid_t)./Yseq(grid_t));  %K/Y
    t2 = 100*(mean(rSeq(grid_t))-delta); %r-delta
    t3 = mean(wSeq(grid_t));
    t4 = 100*mean(seq_debt(grid_t)./Yseq(grid_t)); %Debt/Y
    t5 = 100*mean(seq_popDebt(grid_t)); % Pop Debt
    t6 = 100*mean(seq_pop01(grid_t)); % Pop Def
    t7 = 100*mean(seq_pop01belowMedian(grid_t)); % Below median filers
    
    % Compute means of steady state variables
    s1 = statlist(iKY); % k/y
    s4 = 100*abs(statlist(idebtY)); % debt/y
    s5 = 100*statlist(ipopdebt); % pop debt
    s6 = 100*statlist(ipop01); % pop defaulting
    s7 = 100*statlist(ifracpop01belowmed)*statlist(ipop01); % pop below median filers

    % Output some stats 
    statsSS = cell(5,2);
    statsSS(1,:) = {'Capital-Output Ratio',s1};
    statsSS(2,:) = {'Debt-Output Ratio $\times$ 100',s4};
    statsSS(3,:) = {'Percentage in Debt',s5};
    statsSS(4,:) = {'Percentage Filing',s6};
    statsSS(5,:) = {'Percentage Filing Below Median',s7};
    statsBC = cell(5,2);
    statsBC(1,:) = {'Capital-Output Ratio',t1};
    statsBC(2,:) = {'Debt-Output Ratio $\times$ 100',t4};
    statsBC(3,:) = {'Percentage in Debt',t5};
    statsBC(4,:) = {'Percentage Filing',t6};
    statsBC(5,:) = {'Percentage Filing Below Median',t7};

    % Write some stats
    fprintf(fid,'\n');
    fprintf(fid,'Means of Key Variables and Change from SS \n');
    fprintf(fid,'Capital-Output Ratio                  & %8.2f & %8.2f \\\\\n',t1,s1);
    fprintf(fid,'Debt-Output Ratio $\\times$ 100        & %8.2f & %8.2f \\\\\n',t4,s4);
    fprintf(fid,'Percentage in Debt                    & %8.2f & %8.2f \\\\\n',t5,s5);
    fprintf(fid,'Percentage Filing                     & %8.2f & %8.2f \\\\\n',t6,s6);    
    
    fprintf(fid,'\n');
    fprintf(fid,'Capital-Output Ratio in SS : %8.2f \n',s1);             
    fprintf(fid,'Debt-Output Ratio in SS    : %8.2f \n',s4);             
    fprintf(fid,'Percentage in Debt in SS   : %8.2f \n',s5);
    fprintf(fid,'Percentage Filing in SS    :  %0.2g \n',s6);
    fprintf(fid,'Capital-Output Ratio in BC : %8.2f \n',t1);             
    fprintf(fid,'Debt-Output Ratio in BC    : %8.2f \n',t4);             
    fprintf(fid,'Percentage in Debt in BC   : %8.2f \n',t5);
    fprintf(fid,'Percentage Filing in BC    :  %0.2g \n',t6);
    
end

