function [avgDiscEndogSS avgDiscEndogBC] = computeEndogLimit(S,doBC)

    avgDiscEndogSS = [];
    avgDiscEndogBC = [];

    invar_dsn_e_SS = S.invar_dsn_e_SS;
%     trans_e_SS = S.trans_e_SS;
    gA = S.gA_SS;
    gE = S.gE_SS;
    nA = S.nA;
    nAmm = S.nAmm;
    nE = S.nE;
    Tend = S.Tend;
    
    % Figure out the endogenous borrowing limit for each household in the default world
    tmpSS = repmat(gA(1:nAmm),[1,nE,Tend]).*S.q_SS(1:nAmm,:,:,1);
    endogDebtLimSS = NaN(nE,Tend);
    endogDiscDebtLimSS = NaN(nE,Tend);
    intRateLimSS = NaN(nE,Tend);
    for age = 1:Tend
        for eind = 1:nE
            [tmp2 I] = min(tmpSS(:,eind,age));
            endogDiscDebtLimSS(eind,age) = tmp2;
            if (tmp2==0)
                endogDebtLimSS(eind,age) = 0;
            else
                endogDebtLimSS(eind,age) = gA(I);
            end
            I = find(S.q_SS(:,eind,age,1)>1/2,1,'first'); % Less than 50
            if (age==Tend)
                intRateLimSS(eind) = 0; % Last hhs alive
            else
                intRateLimSS(eind) = gA(I);
            end
        end
    end
    
    avgEndogSS = NaN(Tend,1);
    avgDiscEndogSS = NaN(Tend,1);
    avgEarn = NaN(Tend,1);
    avgIntRateLimSS = NaN(Tend,1);
    for age = 1:Tend
        avgEndogSS(age,1) = endogDebtLimSS(:,age)'*invar_dsn_e_SS(:,age)/sum(invar_dsn_e_SS(:,age));
        avgDiscEndogSS(age,1) = endogDiscDebtLimSS(:,age)'*invar_dsn_e_SS(:,age)/sum(invar_dsn_e_SS(:,age));
        avgEarn(age,1) = gE(:,age)'*invar_dsn_e_SS(:,age)/sum(invar_dsn_e_SS(:,age));
        avgIntRateLimSS(age,1) = intRateLimSS(:,age)'*invar_dsn_e_SS(:,age)/sum(invar_dsn_e_SS(:,age));
    end

    if (doBC)
        tmpBC = repmat(gA(1:nAmm),[1,nE,Tend]).*(S.Eqg(1:nAmm,:,:,1)+S.Eqb(1:nAmm,:,:,1));
        endogDebtLimBC = NaN(nE,Tend);
        endogDiscDebtLimBC = NaN(nE,Tend);
        intRateLimBC = NaN(nE,Tend);
        for age = 1:Tend
            for eind = 1:nE
                [tmp2 I] = min(tmpBC(:,eind,age));
                endogDiscDebtLimBC(eind,age) = tmp2;
                if (tmp2==0)
                    endogDebtLimBC(eind,age) = 0;
                else
                    endogDebtLimBC(eind,age) = gA(I);
                end
                I = find(S.Eqg(:,eind,age)+S.Eqb(:,eind,age)>1/2,1,'first'); % Less than 50
                if (age==Tend)
                    intRateLimBC(eind) = 0;                
                else
                    intRateLimBC(eind) = gA(I);
                end            
            end       
        end

        avgEndogBC = NaN(Tend,1);
        avgDiscEndogBC = NaN(Tend,1);
        for age = 1:Tend
            avgEndogBC(age,1) = endogDebtLimBC(:,age)'*invar_dsn_e_SS(:,age)/sum(invar_dsn_e_SS(:,age));
            avgDiscEndogBC(age,1) = endogDiscDebtLimBC(:,age)'*invar_dsn_e_SS(:,age)/sum(invar_dsn_e_SS(:,age));
        end
    end

end