function [profSS profBC] = lifeCycleAnalysis(file,doBC)

    load(file,'grid_*','g*','invar_dsn_SS*','Tend','len_*','n*','c_SS','d_SS','seq_pdfAvg','invar_dsn_m','Ed','Ec')
    
    len_Amm = find(gA_SS==0,1,'first')-1;

    % Set a threshold to set the invariant dsn to 0 when below this
    thresh = 1d-10;
    invar_dsn_SS(invar_dsn_SS<thresh) = 0;
    invar_dsn_SS = invar_dsn_SS/sum(invar_dsn_SS(:));
    if (doBC)
        seq_pdfAvg(seq_pdfAvg<thresh) = 0;
        seq_pdfAvg = seq_pdfAvg/sum(seq_pdfAvg(:));
    end

    % 
    if (sum(vec(invar_dsn_SS(c_SS<=0)))>0) 
        error('there is mass where c_SS<0')
    else
        c_SS = max(1d-10,c_SS); % to avoid NaNs when computing log(c) stats
    end 
            
    % Compute the average life cycle profiles in steady state
    msrByAge = NaN(Tend,1);
    ageProf = NaN(Tend,1);
    cProf = NaN(Tend,1);
    aProf = NaN(Tend,1);
    eProf = NaN(Tend,1);
    dProf = NaN(Tend,1);
    popDebtProf = NaN(Tend,1);
    pop1Prof = NaN(Tend,1);
    debtProf = NaN(Tend,1);
    varloge = NaN(Tend,1);
    varlogc = NaN(Tend,1);
    Elogc = NaN(Tend,1);
    vara = NaN(Tend,1);    
    vDef = NaN(Tend,1);
    vRep1 = NaN(Tend,1);
    vRep2 = NaN(Tend,1);
        
    for age = 1:Tend

        ageProf(age) = 20-1+age;

        msrByAge(age) = sum(sum(sum(invar_dsn_SS(:,:,age,:))));
        condDsn = squeeze(invar_dsn_SS(:,:,age,:)/msrByAge(age));
        condDsn_e = sum(sum(condDsn,3),1)';
                
        cProf(age) = sum(sum(sum(squeeze(c_SS(:,:,age,:)).*condDsn)));
        aProf(age) = gA_SS'*sum(sum(condDsn,2),3);
        eProf(age) = sum(sum(sum(repmat(gE_SS(:,age)',[nA,1,2]).*condDsn)));
        dProf(age) = sum(sum(sum(squeeze(d_SS(:,:,age,:)).*condDsn))); % includes default from all sources if expenditure shocks
        popDebtProf(age) = sum(sum(sum(condDsn(1:len_Amm,:,:))));
        pop1Prof(age) = sum(sum(condDsn(:,:,2)));
        debtProf(age) = gA_SS(1:len_Amm)'*sum(sum(condDsn(1:len_Amm,:,:),3),2)/popDebtProf(age);
        tmp = discreteStats(condDsn,log(repmat(gE_SS(:,age)',[nA,1,2])));
        varloge(age) = tmp.var;
        tmp = discreteStats(condDsn,log(c_SS(:,:,age,:)));
        varlogc(age) = tmp.var;
        Elogc(age) = tmp.mean;
        tmp = discreteStats(condDsn,repmat(gA_SS,[1,nE,2]));
        vara(age) = tmp.var; % a can be equal to 0!!!
    end

    profSS.msrByAge = msrByAge;
    profSS.age = ageProf;
    profSS.Ec = cProf;
    profSS.Ea = aProf;
    profSS.Ee = eProf;
    profSS.Ed = dProf;
    profSS.varloge = varloge;
    profSS.varlogc = varlogc;
    profSS.Elogc = Elogc;
    profSS.vara = vara;
    profSS.EElogc = invar_dsn_SS(:)'*log(c_SS(:));
    profSS.EEc = invar_dsn_SS(:)'*c_SS(:);
    profSS.Evarlogc = profSS.msrByAge'*varlogc;
    profSS.varElogc = profSS.msrByAge'*(profSS.Elogc - profSS.msrByAge'*profSS.Elogc).^2;
    profSS.popDebt = popDebtProf;
    profSS.pop1 = pop1Prof;
    profSS.Edebt = -debtProf;
    profSS.vDef = vDef;
    profSS.vRep1 = vRep1;
    profSS.vRep2 = vRep2;
    
    if (doBC) 
        % Compute the average life cycle profiles in the BC
        msrByAge = NaN(Tend,1);
        cProf = NaN(Tend,1);
        aProf = NaN(Tend,1);
        eProf = NaN(Tend,1);
        dProf = NaN(Tend,1);
        varloge = NaN(Tend,1);
        varlogc = NaN(Tend,1);
        Elogc = NaN(Tend,1);
        vara = NaN(Tend,1);
        popDebtProf = NaN(Tend,1);
        pop1Prof = NaN(Tend,1);
        
        % NOW ED SHOULD BE SAVED IN THE MATFILE
        for age = 1:Tend
            
            msrByAge(age) = sum(sum(sum(seq_pdfAvg(:,:,age,:))));
            condDsn = squeeze(seq_pdfAvg(:,:,age,:)/msrByAge(age));
            condDsn_e = sum(sum(condDsn,3),1)';

            cProf(age) = sum(sum(sum(squeeze(Ec(:,:,age,:)).*condDsn)));
            aProf(age) = gA'*sum(sum(condDsn,2),3);
            eProf(age) = sum(sum(sum(repmat(gE(:,age)',[nA,1,2]).*condDsn)));
            dProf(age) = sum(sum(sum(squeeze(Ed(:,:,age,:)).*condDsn(:,:,:))));
            popDebtProf(age) = sum(sum(sum(condDsn(1:len_Amm,:,:))));
            pop1Prof(age) = sum(sum(condDsn(:,:,2)));
            debtProf(age) = gA(1:len_Amm)'*sum(sum(condDsn(1:len_Amm,:,:),3),2)/popDebtProf(age);
            tmp = discreteStats(condDsn,log(repmat(gE(:,age)',[nA,1,2])));
            varloge(age) = tmp.var;
            
            if (any(condDsn>0d0 & squeeze(Ec(:,:,age,:))<=0d0))  
                error('Ec is 0 or negative where mass is')
            else
                logEc = log(max(1d-10,Ec(:,:,age,:)));
            end
            tmp = discreteStats(condDsn,logEc);
            varlogc(age) = tmp.var;
            Elogc(age) = tmp.mean;
            tmp = discreteStats(condDsn,repmat(gA,[1,nE,2]));
            vara(age) = tmp.var; % a can be equal to 0!!!

        end

        profBC.msrByAge = msrByAge;
        profBC.Ec = cProf;
        profBC.Ea = aProf;
        profBC.Ee = eProf;
        profBC.Ed = dProf;
        profBC.varloge = varloge;
        profBC.varlogc = varlogc;
        profBC.Elogc = Elogc;
        profBC.vara = vara;
        tmp = max(1d-10,Ec(:));
        profBC.Evarlogc = seq_pdfAvg(:)'*(log(tmp)-seq_pdfAvg(:)'*log(tmp)).^2;
        profBC.popDebt = popDebtProf;
        profBC.pop1 = pop1Prof;
        
        t4 = profBC.msrByAge'*profBC.varlogc;
        t5 = profBC.msrByAge'*(profBC.Elogc - profBC.msrByAge'*profBC.Elogc).^2;
        t6 = profBC.Evarlogc;

    else
        t4 = -1; t5 = -1; t6 = -1;
    end

    t1 = profSS.msrByAge'*profSS.varlogc;
    t2 = profSS.msrByAge'*(profSS.Elogc - profSS.msrByAge'*profSS.Elogc).^2;
    t3 = profSS.Evarlogc;
   
%     fprintf(1,'SS  & %6.3f & %6.3f & %6.3f \\\\ \n',t2,t1,t3);
%     fprintf(1,'BC  & %6.3f & %6.3f & %6.3f \\\\ \n',t5,t4,t6);
    
    % Process the data of Bick and Choi on household size to equivalize
    % consumption
    [data] = csnEquivScales();
    profSS.EffHhs = data(:,5);
    profSS.Nhhs = data(:,4);
    profSS.cPerHhMember = profSS.Ec./profSS.Nhhs;    
    profSS.cPerEffHhMember = profSS.Ec./profSS.EffHhs;    
    
    % Process the (non-durable) consumption profiles of dirk and jesus
    % Note there consumption scale is deflated by family size
    data = dirkJesusCEX();
    ageCEX = data(:,1);
    csnCEX = data(:,2);
    % Average by year
    for age = 22:85
        tmp = ageCEX>=age & ageCEX<age+1;
        csnCEX(tmp) = mean(csnCEX(tmp));
    end    
    % Extract just the years of interest
    ageCEX = ageCEX(1:4:(85-22+1)*4);
    csnCEX = csnCEX(1:4:(85-22+1)*4);
    
    
    % Normalize so the two profiles line up at age 22
    csnCEX = csnCEX*profSS.cPerEffHhMember(3)/csnCEX(3);
  
    profSS.csnCEX = csnCEX;
    profSS.ageCEX = ageCEX;
    
    
    return
    
end

function [data] = csnEquivScales()

% The number of children and adults is from Bick and Choi

%Age Nadults Nchildren Nhh EquivalentHh
data = [
20	1.526	0	1.526	1.1788
21	1.54	0	1.54	1.1836
22	1.555	0	1.555	1.1887
23	1.569	0.1775	1.7465	1.2538
24	1.582	0.4	1.982	1.3339
25	1.5942	0.5915	2.1857	1.3976
26	1.6065	0.7709	2.3774	1.457
27	1.6181	0.9254	2.5435	1.5085
28	1.6291	1.056	2.6851	1.5524
29	1.6395	1.1642	2.8037	1.5891
30	1.6492	1.2513	2.9005	1.6192
31	1.6584	1.3185	2.9769	1.6428
32	1.6669	1.3673	3.0342	1.6609
33	1.6748	1.399	3.0738	1.6736
34	1.6821	1.4148	3.0969	1.681
35	1.6888	1.4161	3.1049	1.6836
36	1.695	1.4041	3.0991	1.6817
37	1.7006	1.3802	3.0808	1.6759
38	1.7056	1.3455	3.0511	1.6664
39	1.71	1.3014	3.0114	1.6536
40	1.7138	1.249	2.9628	1.6385
41	1.7171	1.1895	2.9066	1.621
42	1.7197	1.1241	2.8438	1.6016
43	1.7218	1.0538	2.7756	1.5804
44	1.7233	0.9799	2.7032	1.558
45	1.7242	0.9032	2.6274	1.5345
46	1.7245	0.8248	2.5493	1.5103
47	1.7242	0.7456	2.4698	1.4856
48	1.7233	0.6666	2.3899	1.4609
49	1.7218	0.5886	2.3104	1.4362
50	1.7196	0.5124	2.232	1.4119
51	1.7167	0.4387	2.1554	1.3882
52	1.7132	0.3681	2.0813	1.3652
53	1.709	0.3014	2.0104	1.3432
54	1.7041	0.239	1.9431	1.3207
55	1.6985	0.1814	1.8799	1.2992
56	1.6922	0.129	1.8212	1.2792
57	1.6851	0.0821	1.7672	1.2608
58	1.6773	0.041	1.7183	1.2442
59	1.6687	0.0057	1.6744	1.2293
60	1.6592	0	1.6592	1.2241
61	1.6489	0	1.6489	1.2206
62	1.6378	0	1.6378	1.2169
63	1.6258	0	1.6258	1.2128
64	1.6129	0	1.6129	1.2084
65	1.599	0	1.599	1.2037
66	1.5842	0	1.5842	1.1986
67	1.5684	0	1.5684	1.1933
68	1.5515	0	1.5515	1.1875
69	1.5336	0	1.5336	1.1814
70	1.5146	0	1.5146	1.175
71	1.4945	0	1.4945	1.1681
72	1.4732	0	1.4732	1.1609
73	1.4508	0	1.4508	1.1533
74	1.4271	0.0009	1.428	1.1455
75	1.4021	0.0015	1.4036	1.1372
76	1.3758	0	1.3758	1.1278
77	1.3482	0	1.3482	1.1184
78	1.3192	0	1.3192	1.1085
79	1.2888	0	1.2888	1.0982
80	1.2568	0	1.2568	1.0873
81	1.2234	0	1.2234	1.076
82	1.1884	0	1.1884	1.0641
83	1.1518	0	1.1518	1.0516
84	1.1135	0	1.1135	1.0386
85	1.0735	0	1.0735	1.025
];



end

function [data] = dirkJesusCEX()

    % Dirk and Jesus's CEX consumption estimates
    % These are deflated by family size
data = [    
   22.0000    1.0000
   22.2500    0.9983
   22.5000    0.9973
   22.7500    0.9967
   23.0000    0.9966
   23.2500    0.9968
   23.5000    0.9971
   23.7500    0.9977
   24.0000    0.9988
   24.2500    0.9998
   24.5000    1.0009
   24.7500    1.0019
   25.0000    1.0027
   25.2500    1.0036
   25.5000    1.0046
   25.7500    1.0055
   26.0000    1.0065
   26.2500    1.0086
   26.5000    1.0109
   26.7500    1.0136
   27.0000    1.0165
   27.2500    1.0201
   27.5000    1.0243
   27.7500    1.0288
   28.0000    1.0337
   28.2500    1.0384
   28.5000    1.0428
   28.7500    1.0469
   29.0000    1.0507
   29.2500    1.0543
   29.5000    1.0578
   29.7500    1.0613
   30.0000    1.0646
   30.2500    1.0679
   30.5000    1.0712
   30.7500    1.0744
   31.0000    1.0777
   31.2500    1.0809
   31.5000    1.0839
   31.7500    1.0869
   32.0000    1.0897
   32.2500    1.0926
   32.5000    1.0954
   32.7500    1.0981
   33.0000    1.1007
   33.2500    1.1032
   33.5000    1.1056
   33.7500    1.1078
   34.0000    1.1098
   34.2500    1.1117
   34.5000    1.1136
   34.7500    1.1153
   35.0000    1.1169
   35.2500    1.1187
   35.5000    1.1206
   35.7500    1.1227
   36.0000    1.1249
   36.2500    1.1272
   36.5000    1.1297
   36.7500    1.1323
   37.0000    1.1350
   37.2500    1.1379
   37.5000    1.1409
   37.7500    1.1440
   38.0000    1.1472
   38.2500    1.1504
   38.5000    1.1536
   38.7500    1.1567
   39.0000    1.1599
   39.2500    1.1630
   39.5000    1.1660
   39.7500    1.1690
   40.0000    1.1719
   40.2500    1.1748
   40.5000    1.1778
   40.7500    1.1807
   41.0000    1.1836
   41.2500    1.1865
   41.5000    1.1894
   41.7500    1.1924
   42.0000    1.1953
   42.2500    1.1983
   42.5000    1.2013
   42.7500    1.2044
   43.0000    1.2074
   43.2500    1.2103
   43.5000    1.2132
   43.7500    1.2161
   44.0000    1.2189
   44.2500    1.2216
   44.5000    1.2242
   44.7500    1.2266
   45.0000    1.2290
   45.2500    1.2313
   45.5000    1.2337
   45.7500    1.2360
   46.0000    1.2383
   46.2500    1.2406
   46.5000    1.2429
   46.7500    1.2452
   47.0000    1.2475
   47.2500    1.2497
   47.5000    1.2518
   47.7500    1.2538
   48.0000    1.2557
   48.2500    1.2573
   48.5000    1.2589
   48.7500    1.2602
   49.0000    1.2614
   49.2500    1.2625
   49.5000    1.2635
   49.7500    1.2644
   50.0000    1.2652
   50.2500    1.2660
   50.5000    1.2666
   50.7500    1.2673
   51.0000    1.2678
   51.2500    1.2682
   51.5000    1.2685
   51.7500    1.2687
   52.0000    1.2687
   52.2500    1.2686
   52.5000    1.2683
   52.7500    1.2678
   53.0000    1.2671
   53.2500    1.2663
   53.5000    1.2654
   53.7500    1.2644
   54.0000    1.2632
   54.2500    1.2620
   54.5000    1.2607
   54.7500    1.2594
   55.0000    1.2580
   55.2500    1.2565
   55.5000    1.2549
   55.7500    1.2532
   56.0000    1.2514
   56.2500    1.2495
   56.5000    1.2474
   56.7500    1.2453
   57.0000    1.2430
   57.2500    1.2405
   57.5000    1.2378
   57.7500    1.2349
   58.0000    1.2319
   58.2500    1.2288
   58.5000    1.2255
   58.7500    1.2222
   59.0000    1.2188
   59.2500    1.2154
   59.5000    1.2120
   59.7500    1.2086
   60.0000    1.2051
   60.2500    1.2016
   60.5000    1.1981
   60.7500    1.1945
   61.0000    1.1908
   61.2500    1.1870
   61.5000    1.1830
   61.7500    1.1790
   62.0000    1.1749
   62.2500    1.1706
   62.5000    1.1663
   62.7500    1.1619
   63.0000    1.1575
   63.2500    1.1531
   63.5000    1.1489
   63.7500    1.1447
   64.0000    1.1407
   64.2500    1.1367
   64.5000    1.1328
   64.7500    1.1290
   65.0000    1.1252
   65.2500    1.1214
   65.5000    1.1176
   65.7500    1.1138
   66.0000    1.1100
   66.2500    1.1062
   66.5000    1.1024
   66.7500    1.0986
   67.0000    1.0949
   67.2500    1.0911
   67.5000    1.0874
   67.7500    1.0837
   68.0000    1.0800
   68.2500    1.0765
   68.5000    1.0730
   68.7500    1.0696
   69.0000    1.0663
   69.2500    1.0630
   69.5000    1.0597
   69.7500    1.0564
   70.0000    1.0531
   70.2500    1.0497
   70.5000    1.0462
   70.7500    1.0427
   71.0000    1.0391
   71.2500    1.0355
   71.5000    1.0318
   71.7500    1.0281
   72.0000    1.0244
   72.2500    1.0208
   72.5000    1.0172
   72.7500    1.0136
   73.0000    1.0101
   73.2500    1.0067
   73.5000    1.0034
   73.7500    1.0002
   74.0000    0.9971
   74.2500    0.9940
   74.5000    0.9910
   74.7500    0.9879
   75.0000    0.9849
   75.2500    0.9817
   75.5000    0.9786
   75.7500    0.9754
   76.0000    0.9720
   76.2500    0.9687
   76.5000    0.9654
   76.7500    0.9621
   77.0000    0.9587
   77.2500    0.9553
   77.5000    0.9517
   77.7500    0.9481
   78.0000    0.9444
   78.2500    0.9407
   78.5000    0.9369
   78.7500    0.9332
   79.0000    0.9294
   79.2500    0.9257
   79.5000    0.9219
   79.7500    0.9182
   80.0000    0.9145
   80.2500    0.9107
   80.5000    0.9068
   80.7500    0.9029
   81.0000    0.8989
   81.2500    0.8948
   81.5000    0.8907
   81.7500    0.8865
   82.0000    0.8823
   82.2500    0.8780
   82.5000    0.8737
   82.7500    0.8694
   83.0000    0.8651
   83.2500    0.8610
   83.5000    0.8571
   83.7500    0.8536
   84.0000    0.8503
   84.2500    0.8471
   84.5000    0.8440
   84.7500    0.8410
   85.0000    0.8381
   85.2500    0.8353
   85.5000    0.8325
   85.7500    0.8298
   86.0000    0.8271
   86.2500    0.8245
   86.5000    0.8220
   86.7500    0.8195
   87.0000    0.8170
   87.2500    0.8146
   87.5000    0.8121
   87.7500    0.8097
];

end


