
% Evaluates the bicubicSpline at the points {(xhat1(i),xhat2(i))|i=1,...,n}
% where n is the length of xhat1 and xhat2.
function [yhat] = bicubicSplineEval(xhat1,xhat2,x1,x2,y,abcd_table,method)
    
% Check inputs
if (~isvector(x1) || ~isvector(x2))
    error('x1 and x2 must both be vectors');
end
if (any(diff(x1)<=0) || any(diff(x2)<=0)) 
    error('x1 and x2 must be monotonically increasing');
end
if (size(y)~=[numel(x1),numel(x2)])
    error('y must be n1 by n2');
end
if (~isvector(xhat1) || ~isvector(xhat2))
    error('xhat1 and xhat2 must both be vectors');
end
if (length(xhat1) ~= length(xhat2))
    error('xhat1 and xhat2 must have the same length');
end


% Ensure column vectors
x1 = x1(:);
x2 = x2(:);
xhat1 = xhat1(:);
xhat2 = xhat2(:);

% Get lengths
n1 = length(x1);
n2 = length(x2);
n = length(xhat1); % Same as length(xhat2)

% Switch for the type of spline desired
isNaturalSpline = 1;

% A spline has already been created for y(.,x2) for each x2.
% So evaluate the spline for each x2 at (xhat1,x2) to get
% yint(xhat1,x2)
yint1_on_grid = NaN(n,n2);
for i2 = 1:n2
    yint1_on_grid(:,i2) = cubicSplineEval(xhat1,x1,abcd_table(:,:,i2));
end

% Loop over each data point and use the interpolated grid values to
% construct a new spline and evaluation
yhat = NaN(n,1);
for i = 1:n
    yhat(i) = cubicSplineInt(xhat2(i),x2,yint1_on_grid(i,:),method);
end


end