% For a given value of x and knots t, finds the i such that x is in 
% [t(i),t(i+1)) and produces the i to i-k+1 B-splines of degree k evaluated 
% at x. These are the only non-zero basis functions for this x.
% 
% The routine, for convenience, returns ilo:=i-k+1 and ihi:=i which
% implicitly give the non-zero B-splines in the vector B(ilo:ihi). B itself
% may be much larger. 
function [B,ilo,ihi] = bspline2(x,t,k)

    i = find(x<t(2:end) & x>=t(1:end-1));
%     i
    
    if (isempty(i)) % || i==length(t)) 
        B = 0;
        ilo = 1;
        ihi = 1;
        return
    else
        % The non-zero B-splines are B_{i-k+1,k} to B_{i,k}.
        % For ease of indexing, all values from B_{1,k} to B_{i,k} are
        % returned.
        B = NaN(i,1);
        B(i) = 1;
        
        for p = 2:k
            Bnew = NaN(i,1);
            for j = i-p+1:i
                % Compute B_{j,p} using B_{j,p-1} and B_{j+1,p-1}
                if (j==i)
                    % Combine B_{j,p-1}~=0 with the zero term B_{j+1,p-1}
                    Bnew(j) = (x-t(j))/(t(j+p-1)-t(j))*B(j);
                elseif (j==i-p+1)
                    % Combine B_{j+1,p-1}~=0 with the zero term B_{j,p-1}
                    Bnew(j) = (1-(x-t(j+1))/(t(j+1+p-1)-t(j+1)))*B(j+1);
                else
                    % Combine B_{j,p-1}~=0 with B_{j+1,p-1}~=0
                    Bnew(j) = (x-t(j))/(t(j+p-1)-t(j))*B(j) + ...
                              (1-(x-t(j+1))/(t(j+1+p-1)-t(j+1)))*B(j+1);
                end
                
%                 % Compute B_{j,p} using B_{j,p-1} and B_{j+1,p-1}
%                 if (j==i)
%                     % Combine B_{j,p-1}~=0 with the zero term B_{j+1,p-1}
%                     Bnew(j) = (x-t(j))/(t(j+p)-t(j))*B(j);
%                 elseif (j==i-p+1)
%                     % Combine B_{j+1,p-1}~=0 with the zero term B_{j,p-1}
%                     Bnew(j) = (t(j+1+p)-x)/(t(j+1+p)-t(j+1))*B(j+1);
%                 else
%                     % Combine B_{j,p-1}~=0 with B_{j+1,p-1}~=0
%                     Bnew(j) = (x-t(j))/(t(j+p)-t(j))*B(j) + ...
%                               (t(j+1+p)-x)/(t(j+1+p)-t(j+1))*B(j+1);
%                 end
            end
            B = Bnew;
        end
        
        ilo = i-k+1;
        ihi = i;
        
        B = B(ilo:ihi);
        
%         B = B(i-k+1:i,1);
        return
        
    end
    
end