% Given an integer vector x>=1, returns all the different vectors
% y s.t. 1<=y<=x elt by elt. This gives the cartesian product of the 
% vectors (1:x(1)) cross (1:x(2)) cross ... (1:x(end)) 
% x will be treated as a row vector
% y is returned as a matrix with columns equal to the length of x and rows
% equal to (prod(x)) (which gives all the enumerations).
function [y] = cartProdInt(x)

    % The technique here is to convert the 1:prod(x) base 10 numbers into 
    % a custom base system given by x. 
    %
    % Given x = [x(1)...x(n)], 
    % we want for any i in 1:prod(x) a unique d = [d(1)...d(n)] s.t.
    % i = d(1)*(x(2)*x(3)...*x(n)) + d(2)*(x(3)*...*x(n)) + ... +
    %    d(n-1)*x(n) + d(n)
    % e.g., say
    % x = [3,1,2];
    % we convert i=0 to d=[0,0,0], 
    %            i=1 to d=[0,0,1], 
    %            i=2 to d=[1,0,0],
    %            i=3 to d=[1,0,1], ...
    % This will give all enumerates once we add 1 to each d.
    %
    % We see mod(i,x(n)) = d(n), mod(i,x(n-1)*x(n)) = d(n-1)*x(n) + d(n),
    % ... and so we can find all the d values recursively.

    nr = prod(x);
    nx = length(x);
    
    y = NaN(nr,nx);
    
    base = cumprod(x(nx:-1:1));        
    
    i = (0:nr-1)';
    
    % Convert from base 10 to the new base system
if (any(base==0))
    error('i thought base had to be positive')
end
    modvals(:,nx) = mod(i,base(1));    
    for ix = nx-1:-1:1
        modvals(:,ix) = mod(i,base(nx-ix+1));
    end
    
    y(:,nx) = modvals(:,nx);
    for ix = nx-1:-1:1
%         y(:,ix) = (mod(i,prod(x(ix:nx))) -
%         mod(i,prod(x(ix+1:nx))))/(prod(x(ix+1:nx)));
%         y(:,ix) = (modvals(:,ix) - modvals(:,ix+1))/(prod(x(ix+1:nx)));
        y(:,ix) = (modvals(:,ix) - modvals(:,ix+1))/base(nx-ix);
    end
    
    y = y+1;


end