% cubicSplineEval(X,x,abcd)
% 
% Evaluates a cubic spline at any values of x.
%
% The spline is stored in piecewise polynomial form
% y(x) = a + b(x-xi) + c(x-xi)^2 + d(x-xi)^3
function [Y] = cubicSplineEval(X,x,abcd)

% Ensure column vectors
if (~iscolumn(X)) 
    error('X must be a column vector')
end

% Use binary search to find the location of x in that grid
i = bsearch(X,x);

% 
del = X-x(i);

% Horner's method to evaluate polynomial
Y = abcd(i,1) + (abcd(i,2) + (abcd(i,3) + abcd(i,4).*del).*del).*del;

end

