% Takes a matrix A and constructs a string that can then be used to
% construct an identical matrix in fortran. For instance, if 
% A = [2 4;6 8], then the output is 'reshape((/2.d0,6d0,4d0,8d0/),(/2,2/))'
function str = formatMatFortran(A,isint)

    if (all(floor(A(:))==A(:)) && nargin==1), 
        warning('A appears to be an int but is not declared one'), 
    end

    if (nargin==1); isint=0; end
    
    if (isint)
        fmt = ' %i,';
        fmtend = ' %i';
        nline = 10;
    else
        fmt = ' %gd0,';
        fmtend = ' %gd0';
        nline = 5;
    end
    
    nA = numel(A);

    if (~isvector(A))
        str = 'reshape(&\n';
    else
        str = [];
    end
    
    if (nA>1)
        str = [str,'(/'];
        for i = 1:nA-1
            str = [str,sprintf(fmt,A(i))];
            if (mod(i,nline)==0) 
                str = [str,' &\n  '];
            end
        end
        str = [str,sprintf(fmtend,A(nA)),'/)'];
        str = sprintf(str); % Uses the new lines
    else
        str = ['(/',sprintf(fmtend,A(nA)),'/)'];
    end
    
    if (~isvector(A))
        % Get the representation for the array shape
        tmp = size(A);
        tmp2 = [sprintf('%i,',tmp(1:numel(tmp)-1)),sprintf('%i',tmp(end))];        
        str = [str,'&\n,(/',tmp2,'/))'];        
        str = sprintf(str);
    end
    
end

