% Given data, xlabels, ylabels, and format, this will produce a latex table.
% The data should be a real matrix, the others should be cell array vectors.
function [] = latextable(data,xlabels,ylabels,format,fid)

    m = size(data,1);
    n = size(data,2);
    
    if (length(ylabels)~=n+1) 
        error('ylabels and data do not conform');
    end        
    if (length(xlabels)~=m) 
        error('xlabels and data do not conform');
    end        
    
    if (~iscell(ylabels) || ~iscell(xlabels) || ~iscell(format)) 
        error('xlabels, ylabels, and format must all be cell arrays');
    end
    
    fmt = cell(m,n);
    if (numel(format)==1)
        for j = 1:n
            for i = 1:m
                fmt{i,j} = format{1};
            end
        end
    else
        if (any(size(format)~=[m n]))
            error('format and data do not conform');
        end
        fmt = format;
    end
        
    
    fprintf(fid,'\n');
    
    % Print the ylabels
    for j = 1:n+1
        fprintf(fid,'%s ',ylabels{j});
        if (j<n+1) 
            fprintf(fid,' & ');
        else
            fprintf(fid,'\\\\\n');
        end
    end    
    
    
    % Print out rows, xlabels first followed by the data
    for i = 1:m
        fprintf(fid,'%s & ',xlabels{i});
        for j = 1:n
            fprintf(fid,fmt{i,j},data(i,j));
            if (j<n) 
                fprintf(fid,' & ');
            else
                fprintf(fid,'\\\\\n');
            end
        end
    end


end