module mod_util_fcns

use mod_types
use mod_common_var, only: discount, sigma, death
implicit none

private
public :: PU_scal, dPU_scal, CRRA_scal, CRRA_inverse_scal, diffU_inv

contains

!Define period utility function 
elemental function PU_scal(c) result(PU)
    implicit none
    real(dp), intent(in) :: c
    real(dp) :: PU

    PU = (1.d0-discount)*c**(1.d0-sigma)/(1.d0-sigma) 

end function PU_scal

elemental function dPU_scal(c) result(dPU)
    implicit none
    real(dp), intent(in) :: c 
    real(dp) :: dPU

    dPU = (1.d0-discount)*c**(-sigma)

end function dPU_scal

elemental function CRRA_scal(x)
    implicit none   
    real(dp), intent(in) :: x
    real(dp) :: CRRA_scal

    CRRA_scal = x**(1.d0-sigma)/(1.d0-sigma)

end function CRRA_scal

elemental function CRRA_inverse_scal(x) 
    implicit none

    real(dp), intent(in) :: x
    real(dp) :: CRRA_inverse_scal

    CRRA_inverse_scal = ((1.d0-sigma)*x)**(1.d0/(1.d0-sigma))

end function CRRA_inverse_scal

elemental function diffU_inv(x)
    implicit none

    real(dp), intent(in) :: x
    real(dp) :: diffU_inv

    diffU_inv = (x/(1.d0-discount))**(-1.d0/sigma)

end function diffU_inv


end module mod_util_fcns