module mod_apres_private
    use mod_types
    use mod_pp
    !use omp_lib
    implicit none
        
    real(dp), dimension(:), allocatable :: breaks_xg, breaks_xb
    real(dp), dimension(:,:), allocatable :: coefs_xg, coefs_xb
    real(dp), dimension(:), allocatable :: grid_ap_local
    
    real(dp) :: xg_end_min,xg_end_max,xb_end_min,xb_end_max,Qbarg,Qbarb,a
    integer :: len_ap_local

    !$OMP THREADPRIVATE(breaks_xg,breaks_xb,coefs_xg,coefs_xb,grid_ap_local,xg_end_min,xg_end_max,&
    !$OMP xb_end_min,xb_end_max,Qbarg,Qbarb,a,len_ap_local)

contains

    subroutine init_common(grid_ap_in)
        implicit none
        real(dp), dimension(:), intent(in) :: grid_ap_in
        len_ap_local = size(grid_ap_in)
        if (allocated(grid_ap_local)) deallocate(grid_ap_local)        
        allocate(grid_ap_local(1:len_ap_local))
        grid_ap_local = grid_ap_in                
    end subroutine init_common

    !Return  apb - apb(apbguess) 
    pure function res_of_apb(apbguess) result(res)
        implicit none
        real(dp), intent(in) :: apbguess
        real(dp) :: res
        res = apbguess - apb_of_apb(apbguess)    
    end function res_of_apb

    !Return apb induced by apb
    pure function apb_of_apb(apb) result(apb_apb)
        implicit none
        real(dp), intent(in) :: apb
        real(dp) :: apb_apb
        real(dp) :: xg,xb,apg

        xg = a - Qbarb*apb
        apg = apg_of_xg(xg)
        xb = a - Qbarg*apg
        apb_apb = apb_of_xb(xb)

    end function apb_of_apb

    !For any xg, return the policy apg(xg)
    pure function apg_of_xg(xg) result(apg)
        implicit none
        real(dp), intent(in) :: xg    
        real(dp) :: apg
                
        if (xg<xg_end_min) then
            apg = 0.d0 
        elseif (xg>xg_end_max) then
            apg = grid_ap_local(len_ap_local)
        else
            !apg = interp_linear(grid_ap_local(ap_eq0_ind:len_ap_local),xg,xg_end,size(xg_end))
            apg = ppval_fn(breaks_xg,coefs_xg,xg)
        endif
        
    end function apg_of_xg

    !For any xb, return the policy apb(xb)
    pure function apb_of_xb(xb) result(apb)
        implicit none
        real(dp), intent(in) :: xb    
        real(dp) :: apb
        
        if (xb<xb_end_min) then
            apb = 0.d0 
        elseif (xb>xb_end_max) then
            apb = grid_ap_local(len_ap_local)
        else
            !apb = interp_linear(grid_ap_local(ap_eq0_ind:len_ap_local),xb,xb_end,size(xb_end))
            apb = ppval_fn(breaks_xb,coefs_xb,xb)
        endif

    end function apb_of_xb
    
end module mod_apres_private

module mod_value_fcns

implicit none

contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! sub_vGeneric_egm3
! 
! Computes V = max U + W using a combination of grid search and endogenous grid.
! Breaks maximization into several steps.  Fings the optimal (a'g,a'b) conditional
! on it being one of four regions: A--A--, A--A+, A+A--, A+A+.  EGM is used where
! possible (in the A+ regions).  Fixes apg, then solves for
! apb in A-- (using grid search) and apb in A+ (using EGM) separately.  
! Monotonicity is used on apg
subroutine sub_vGeneric_egm_fixede(&
                V_a, c_a, apg_a, apb_a, apg_ind_a, apb_ind_a, CVg_ap, CVb_ap, &
                MPL, qgap_ap, qbap_ap, grid_a, grid_ap, ap_eq0_ind, len_a, len_ap, dCVg_in, &
                dCVb_in, apg_feasible_in, apb_feasible_in)
use mod_types
use mod_common_var !, only: discount, sigma, death
use mod_util_fcns
!use mod_interp
use mod_schumaker
use mod_pp
!use mod_myprint
use mod_root1dim
use mod_apres_private
implicit none

integer, intent(in) :: len_a, len_ap

!value and policy functions
real(dp), dimension(1:len_a), intent(out):: V_a, c_a, apg_a, apb_a
integer, dimension(1:len_a), intent(out) :: apg_ind_a, apb_ind_a

real(dp), dimension(1:len_ap), intent(in) :: CVg_ap, CVb_ap !continuation value
real(dp), dimension(1:len_ap), intent(in) :: dCVg_in, dCVb_in

!prices
real(dp), intent(in) :: MPL !price of a labor unit (MPL=w*e or MPL=w*e*(1-chi))
real(dp), dimension(1:len_ap), intent(in) :: qgap_ap,qbap_ap !pricing fn times ap

!grids
real(dp), dimension(1:len_a), intent(in) :: grid_a 
real(dp), dimension(1:len_ap), intent(in) :: grid_ap 
integer, intent(in) :: ap_eq0_ind

!feasible ap apriori
logical, dimension(1:len_ap), optional, intent(in) :: apg_feasible_in, apb_feasible_in

!local
logical, dimension(1:len_ap) :: apg_feasible, apb_feasible
real(dp) :: apg_comp, apb_comp, c_comp, v_comp
real(dp), dimension(ap_eq0_ind:len_ap) :: xb_end,xg_end
real(dp), dimension(1:len_a) :: apg_MM, apb_MM, c_MM, v_MM, apg_PP, apb_PP, c_PP, v_PP, &
                                apg_MP, apb_MP, c_MP, v_MP, apg_PM, apb_PM, c_PM, v_PM
integer, dimension(1:len_a) :: apg_ind_MM, apb_ind_MM
integer, dimension(1:1) :: max_loc
integer :: a_ind, apb_ind, apg_ind
real(dp), dimension(:), allocatable :: breaksg, breaksb !, breaks_xg, breaks_xb
real(dp), dimension(:,:), allocatable :: coefsg, coefsb !, coefs_xg, coefs_xb

!Get risk free rates for households (this differs from risk free rate to bank b/c of mortality risk)
Qbarg = qgap_ap(len_ap)/grid_ap(len_ap) 
Qbarb = qbap_ap(len_ap)/grid_ap(len_ap) 

!Get those ap that are feasible apriori.
apg_feasible = .TRUE.
apb_feasible = .TRUE.
if (present(apg_feasible_in)) apg_feasible = apg_feasible_in
if (present(apb_feasible_in)) apb_feasible = apb_feasible_in

call init_common(grid_ap)

!Initialize values
v_MP = death; c_MP = -1.d0; apb_MP = 0.d0; apg_MP = 0.d0
v_PM = death; c_PM = -1.d0; apb_PM = 0.d0; apg_PM = 0.d0

!Initialize schumaker interp coefficients
!using clamped, not very appropriate(?) b/c of binding constraints ~approximation
call schumaker_ppmak(breaksg,coefsg,grid_ap,CVg_ap,dCVg_in)
call schumaker_ppmak(breaksb,coefsb,grid_ap,CVb_ap,dCVb_in)

! STEP 0: Construct policy functions of apg and apb, conditional 
! on being in A+, defined on endogenous resource grids, xg and xb.
! Functions apg_of_xg(xg) and apb_of_xb(xb) give the C0 policy
! functions (achieved through interpolation).
! Detail:
!   For each apb in A+, compute the resources xb := a - Qbarg*apg that would make apb optimal.  Then
!   we have the policy function apb(xb) defined on an endogenous resource grid {xb}.
!   For a given (a,apg), there is a unique xb.  If xb<min({xb}), then apb=0.  If 
!   xb>max({xb}), then apb=abar.  For xb interior, we can use linear interpolation 
!   to recover apb(xb).  

xg_end = -MPL + Qbarg*grid_ap(ap_eq0_ind:len_ap) + &
         (dCVg_in(ap_eq0_ind:len_ap)/Qbarg/(1.d0-discount))**(-1.d0/sigma) 
xg_end_min = minval(xg_end)
xg_end_max = maxval(xg_end)
call schumaker_ppmak(breaks_xg,coefs_xg,xg_end,grid_ap(ap_eq0_ind:len_ap))  !asset policy is ap of x
!call init_apgpol(breaks_xg,coefs_xg,xg_end_min,xg_end_max)

xb_end = -MPL + Qbarb*grid_ap(ap_eq0_ind:len_ap) + &
         (dCVb_in(ap_eq0_ind:len_ap)/Qbarb/(1.d0-discount))**(-1.d0/sigma)
xb_end_min = minval(xb_end)
xb_end_max = maxval(xb_end)
call schumaker_ppmak(breaks_xb,coefs_xb,xb_end,grid_ap(ap_eq0_ind:len_ap))
!call init_apbpol(breaks_xb,coefs_xb,xb_end_min,xb_end_max)

if (ap_eq0_ind>1) then

    !STEP 1: Find optimal solution conditional on apg,apb in A--,A-- using grid search
    call sub_vGeneric(v_MM, c_MM, apg_MM, apb_MM, apb_ind_MM, apb_ind_MM, CVg_ap(1:ap_eq0_ind-1), &
                      CVb_ap(1:ap_eq0_ind-1), (/MPL/), qgap_ap(1:ap_eq0_ind-1), &
                      qbap_ap(1:ap_eq0_ind-1), grid_a, grid_ap(1:ap_eq0_ind-1), ap_eq0_ind, &
                      len_a, ap_eq0_ind-1,1,apg_feasible,apb_feasible)
                      
    !STEP 2: Find optimal solution conditional on apg,apb in A--,A+ using mixed.
    do a_ind = 1,len_a
        a = grid_a(a_ind)
        do apg_ind = 1,ap_eq0_ind-1
            if (.NOT. apg_feasible(apg_ind)) cycle !this apriori info
            apg_comp = grid_ap(apg_ind)
            apb_comp = apb_of_xb(a - qgap_ap(apg_ind))
            c_comp = MPL + a - qgap_ap(apg_ind) - Qbarb*apb_comp
            if (c_comp<=0.d0) cycle
            v_comp = PU_scal(c_comp) + CVg_ap(apg_ind) + ppval_fn(breaksb,coefsb,apb_comp)
            if (v_comp>v_MP(a_ind)) then
                v_MP(a_ind) = v_comp
                c_MP(a_ind) = c_comp
                apg_MP(a_ind) = apg_comp
                apb_MP(a_ind) = apb_comp
            end if
        end do
    end do

    !STEP 3: Find optimal solution conditional on apg,apb in A+,A-- using mixed.
    do a_ind = 1,len_a
        a = grid_a(a_ind)
        do apb_ind = 1,ap_eq0_ind-1
            if (.NOT. apb_feasible(apb_ind)) cycle !this apriori info
            apb_comp = grid_ap(apb_ind)
            apg_comp = apg_of_xg(a - qbap_ap(apb_ind))
            c_comp = MPL + a - Qbarg*apg_comp - qbap_ap(apb_ind)
            if (c_comp<=0.d0) cycle
            v_comp = PU_scal(c_comp) + ppval_fn(breaksg,coefsg,apg_comp) + CVb_ap(apb_ind)
            if (v_comp>v_PM(a_ind)) then
                v_PM(a_ind) = v_comp
                c_PM(a_ind) = c_comp
                apg_PM(a_ind) = apg_comp
                apb_PM(a_ind) = apb_comp
            end if
        end do
    end do
end if

!STEP 4: Find optimal solution conditional on apg,apb in A+,A+.  For each apg,
!compute the resources xg := a - Qbarb*apb that would make apg optimal.  Then
!we have the policy function apg(xg) defined on an endogenous resource grid {xg}.
!For a given (a,apb), there is a unique xg.  If xg<min({xg}), then apg=0.  If 
!xg>max({xg}), then apg=abar.  For xg interior, we can use linear interpolation 
!to recover apg(xg).  
!Doing this as well for apb with xb:= a - Qbarg*apg, we have apg(xg) and apb(xb).
!To find apg(a) and apb(a), do the following
! 1. Fix an 'a'
! 2. Guess on apb
! 3. Compute xg = a - Qbarb*apb
! 4. Find apg(xg) using linear interpolation interior and known value elsewhere.
! 5. Compute xb = a - Qbarg*apg
! 6. Check whether |apb - apb(xb)|<eps.  If so, stop.  If not, go to 2.
!In particular to solve this, we'll use bisection method.  Since apb(xb)>=0 and
!apb(xb)<=abar, this means apb-apb(xb) is nonpositive for guess apb=0 and 
!nonnegative for apb=abar.  Since apb() is continuous, bisection will converge. 
do a_ind = 1,len_a
    a = grid_a(a_ind)
    
    !UPDATE: This code does work, but use the compiler option /fp:precise   The rounding errors
    !were causing the old method to give different results.
    call sub_bisect(apb_PP(a_ind),res_of_apb,a_in=0.d0,b_in=grid_ap(len_ap),&
                    toler_in=1d-6,maxiter_in=1000,xtoler_in=1d-8,printmod_in=-1)        
    
    apg_PP(a_ind) = apg_of_xg(a - Qbarb*apb_PP(a_ind))
    c_PP(a_ind) = MPL + a - Qbarb*apb_PP(a_ind) - Qbarg*apg_PP(a_ind)
    if (c_PP(a_ind)>0.d0) then
        v_PP(a_ind) =  PU_scal(c_PP(a_ind)) + ppval_fn(breaksg,coefsg,apg_PP(a_ind)) &
                                 + ppval_fn(breaksb,coefsb,apb_PP(a_ind))
    else
        v_PP(a_ind) = death
        c_PP(a_ind) = -1.d0
    end if
end do


!STEP 5: Find the maximum among the 4 regions.
if (ap_eq0_ind>1) then

    where (v_PP>=v_MM .AND. v_PP>=v_MP .AND. v_PP>=v_PM)
        V_a = v_PP
        c_a = c_PP
        apg_a = apg_PP
        apb_a = apb_PP
    elsewhere (v_PM>=v_MM .AND. v_PM>=v_MP .AND. v_PM>=v_PP)
        V_a = v_PM
        c_a = c_PM
        apg_a = apg_PM
        apb_a = apb_PM
    elsewhere (v_MP>=v_MM .AND. v_MP>=v_PM .AND. v_MP>=v_PP)
        V_a = v_MP
        c_a = c_MP
        apg_a = apg_MP
        apb_a = apb_MP
    elsewhere (v_MM>=v_MP .AND. v_MM>=v_PM .AND. v_MM>=v_PP)
        V_a = v_MM
        c_a = c_MM
        apg_a = apg_MM
        apb_a = apb_MM
    endwhere
else
    V_a = v_PP
    c_a = c_PP
    apg_a = apg_PP
    apb_a = apb_PP
end if

do a_ind = 1,len_a
    apg_ind_a(a_ind:a_ind) = minloc(abs(grid_ap-apg_a(a_ind)))
    apb_ind_a(a_ind:a_ind) = minloc(abs(grid_ap-apb_a(a_ind)))
end do

deallocate(breaksg,breaksb,coefsg,coefsb)
!deallocate(breaks_xg,breaks_xb,coefs_xg,coefs_xb)

end subroutine sub_vGeneric_egm_fixede

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! sub_vGeneric
! 
! Computes V = max U + W not assuming anything
! and the associated policy functions
subroutine sub_vGeneric(V_a_e, c_a_e, apg_a_e, apb_a_e, apg_ind_a_e, apb_ind_a_e, CVg_ap_e, CVb_ap_e, &
                        MPL, qgap_ap_e, qbap_ap_e, grid_a, grid_ap, ap_eq0_ind, len_a, len_ap, len_e, &
                        apg_feasible_in, apb_feasible_in)
use mod_common_var, only: death
use mod_util_fcns
!use omp_lib
use mod_types
implicit none

integer, intent(in) :: len_a, len_ap, len_e

!value and policy functions
real(dp), dimension(1:len_a,1:len_e), intent(out):: V_a_e, c_a_e, apg_a_e, apb_a_e
integer, dimension(1:len_a,1:len_e), intent(out) :: apg_ind_a_e, apb_ind_a_e

real(dp), dimension(1:len_ap,1:len_e), intent(in) :: CVg_ap_e, CVb_ap_e !continuation value

!prices
real(dp), dimension(1:len_e), intent(in) :: MPL !price of a labor unit (so MPL=w*grid_e or MPL=w*grid_e*(1-chi))
real(dp), dimension(1:len_ap,1:len_e), intent(in) :: qgap_ap_e,qbap_ap_e !pricing fn times ap

!grids
real(dp), dimension(1:len_a), intent(in) :: grid_a 
real(dp), dimension(1:len_ap), intent(in) :: grid_ap 
integer, intent(in) :: ap_eq0_ind

!feasible a priori
logical, dimension(1:len_ap,1:len_e), optional, intent(in) :: apg_feasible_in, apb_feasible_in

!local variables
!real(dp) :: Qbarg, Qbarb
real(dp), dimension(1:len_ap,1:len_ap) :: c, utils
logical, dimension(1:len_ap,1:len_ap) :: ap_feasible
logical, dimension(1:len_ap,1:len_e) :: apg_feasible, apb_feasible
logical, dimension(1:len_ap,1:len_ap,1:len_e) :: ap_feasible_init
real(dp) :: vstar, cstar, apgstar, apbstar
integer :: apgstar_ind, apbstar_ind
real(dp) :: a
integer, dimension(1:2) :: max_loc
integer :: e_ind, a_ind, apg_ind, apb_ind

!Get those ap that are feasible apriori.
apg_feasible = .TRUE.
apb_feasible = .TRUE.
if (present(apg_feasible_in)) apg_feasible = apg_feasible_in
if (present(apb_feasible_in)) apb_feasible = apb_feasible_in
do e_ind = 1,len_e
    do apb_ind = 1,len_ap
        do apg_ind = 1,len_ap
            ap_feasible_init(apg_ind,apb_ind,e_ind) = apg_feasible(apg_ind,e_ind) .AND. apb_feasible(apb_ind,e_ind)
        end do
    end do
end do

do e_ind = 1,len_e
    apgstar_ind=1
    apbstar_ind=1
    do a_ind = 1,len_a
        
        !ap_feasible=.TRUE.
        ap_feasible = ap_feasible_init(:,:,e_ind)
         
        a = grid_a(a_ind)

        !Compute c for all feasible apg,apb
        c = -1.d0
        do apb_ind=1,len_ap            
            where (ap_feasible(:,apb_ind))
                c(:,apb_ind) = MPL(e_ind) + a &
                    - qgap_ap_e(:,e_ind) - qbap_ap_e(apb_ind,e_ind)
            endwhere
        end do

        !Update the feasibility of apg,apb        
        ap_feasible = c > 0.d0 .AND. ap_feasible

        !Find maximum
        do apb_ind=1,len_ap
            where (ap_feasible(:,apb_ind)) utils(:,apb_ind) = PU_scal(c(:,apb_ind)) + CVg_ap_e(:,e_ind) + CVb_ap_e(apb_ind,e_ind)
        end do
        if (any(ap_feasible)) then
            max_loc = maxloc(utils,ap_feasible)
            vstar = utils(max_loc(1),max_loc(2))
            cstar = c(max_loc(1),max_loc(2))
            apgstar_ind = max_loc(1)
            apbstar_ind = max_loc(2)
            apgstar = grid_ap(max_loc(1))
            apbstar = grid_ap(max_loc(2))
        else
            vstar = death
            cstar = -1.d0
            apgstar_ind = -1 !ap_eq0_ind
            apbstar_ind = -1 !ap_eq0_ind technicall should be ap_eq0_ind, but that prevents montonicity?
            apgstar = 0.d0
            apbstar = 0.d0
        end if

        V_a_e(a_ind,e_ind) = vstar
        c_a_e(a_ind,e_ind) = cstar
        apg_ind_a_e(a_ind,e_ind) = apgstar_ind
        apb_ind_a_e(a_ind,e_ind) = apbstar_ind
        apg_a_e(a_ind,e_ind) = apgstar
        apb_a_e(a_ind,e_ind) = apbstar
        
    end do
end do

end subroutine sub_vGeneric

!Given a price schedule X and a continuation value V determines whether each a'* could 
!possible be optimal.  It tests whether there exists an a' with more resources today
!X(a')<X(a'*) and with higher continuation value CV(a')>CV(a'*).  In this case, a'* can't
!be optimal because it results in less resources today and less continuation value.
pure function get_ap_feasible(X,CV) result(ap_feasible)
    use mod_types
    implicit none
    real(dp), dimension(1:), intent(in) :: X,CV
    logical, dimension(1:size(X,1)) :: ap_feasible 
    !local
    integer :: ap_ind
    
    do ap_ind = 1,size(X,1)
         ap_feasible(ap_ind) = .NOT. any( X < X(ap_ind) .AND. CV > CV(ap_ind) )
    end do
    
end function get_ap_feasible

end module mod_value_fcns
