classdef shocks_classdef
    %SHOCKS_CLASSDEF Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        
        %grid of shocks
        grid_e
        
        %transition matrices
        trans_z %Pr(z->z')
        trans_e %Pr(e,z,z' -> e'), not a transition matrix
        trans_ez %Pr(e,z -> e',z')
        trans_casi %The 3-state transition matrix income shocks e~F(casi) iid
        
        %invariant dsns
        invar_dsn_casi %Pr(casi)
        invar_dsn_z %Pr(z)
        invar_dsn_e_cdnal_z %Pr(e|z)

    end
    
    methods
        %constructor
        function out = shocks_classdef(p)
            if nargin==0 
                return
            end
            %known and easily calculated values
            out.trans_z = p.trans_z;
            out.invar_dsn_z = null(eye(2)-out.trans_z');
            out.invar_dsn_z = out.invar_dsn_z/sum(out.invar_dsn_z);
            out.trans_casi = [p.trans_11, p.trans_12, p.trans_13;...
                              p.trans_21, p.trans_22, p.trans_23;...
                              p.trans_31, p.trans_32, p.trans_33 ];
            out.invar_dsn_casi = null(eye(3)-out.trans_casi');
            out.invar_dsn_casi = out.invar_dsn_casi/sum(out.invar_dsn_casi);
            
            %values to be calculated later
            [out.grid_e, out.trans_e] = shocks_classdef.sub_idshocks(p);
            [out.invar_dsn_e_cdnal_z, out.trans_ez] = shocks_classdef.sub_invarshocks(out.trans_e, out.invar_dsn_z, out.trans_z, p);
        end
        
        %ConsistencyCheck
        function bool = consistencyCheck(in,p)
            bool = true(1);
            
            %Ensure all have the correct dimension
            if (any(size(in.trans_z)~=[p.len_z,p.len_z])  || ...
                any(size(in.trans_e)~=[p.len_e,p.len_z,p.len_z,p.len_e]) || ...
                any(size(in.trans_ez)~=[p.len_e,p.len_z,p.len_e,p.len_z]) || ...
                any(size(in.trans_casi)~=[3,3])) 
                
                bool = false(1);
                error('consistency check failure: dims wrong')                
            end 
            
            %Ensure transition rows sum to 1
            cnt = 0;
            cnt = cnt + sum(abs(sum(in.trans_z,2)-1.d0)>1d-10);
            cnt = cnt + sum(abs(sum(in.trans_casi,2)-1.d0)>1d-10);
            %Note, trans_e is not a transition matrix, but gives pr(e,z,z'->e') (by that I mean pr(e'|e,z,z'))           
            for e_ind = 1:p.len_e
                for z_ind = 1:p.len_z
                    for zp_ind = 1:p.len_z
                        cnt = cnt + sum(abs(sum(sum(in.trans_e(e_ind,z_ind,zp_ind,:)))-1.d0)>1d-10);
                    end
                end
            end
            for e_ind = 1:p.len_e
                for z_ind = 1:p.len_z
                    cnt = cnt + sum(abs(sum(sum(in.trans_ez(e_ind,z_ind,:,:)))-1.d0)>1d-10);
                end
            end
            if (cnt>0)
                disp(cnt)
                bool = false(1);
                error('consistency check failure: trans mat doesn''t sum to 1')
            end
            
            %Ensure invariant dsns sum to 1
            cnt = 0;
            cnt = cnt + sum(abs(sum(in.invar_dsn_casi)-1.d0)>1d-10);
            cnt = cnt + sum(abs(sum(in.invar_dsn_z)-1.d0)>1d-10);
            cnt = cnt + sum(abs(sum(in.invar_dsn_e_cdnal_z,1)-1.d0)>1d-10);
            if (cnt>0) 
                disp(cnt)
                bool = false(1);
                error('consistency check failure: invariant dsns don''t sum to 1')
            end            
            
        end
    end
    methods (Static)
        function [grid_e, trans_e] = sub_idshocks(p, varargin)
            if length(varargin) == 3
                grid_e_xi1 = varargin(1);
                grid_e_xi2 = varargin(2);
                grid_e_xi3 = varargin(3);                
            else
                grid_e_xi1 = [];
                grid_e_xi2 = [];
                grid_e_xi3 = [];
            end
            %Check inputs
            if ~isempty(grid_e_xi1) || ~isempty(grid_e_xi2) || ~isempty(grid_e_xi3) 
                if length(grid_e_xi1)~=abs(p.xi1_end-p.xi1_beg)+1 || ...
                   length(grid_e_xi2)~=abs(p.xi2_end-p.xi2_beg)+1 || ...
                   length(grid_e_xi3)~=abs(p.xi3_end-p.xi3_beg)+1 
                    error('a grid has the wrong size')
                end
            end
            if p.n_1<=0 || p.n_2<=0 || p.n_3<=0 
                error('sub_idshocks: modify me')
            end

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %   Use CCNR conditional distributions
            % 
            grid_e = zeros(p.len_e,1);
            
            %Setup grid
            grid_e(p.xi1_beg:p.xi1_end) = shocks_classdef.CCNR_quad(p.n_1,mean(p.phi(:)),mean(p.emin_1(:)),mean(p.emax_1(:)));
            grid_e(p.xi2_beg:p.xi2_end) = shocks_classdef.CCNR_quad(p.n_2,mean(p.phi(:)),mean(p.emin_2(:)),mean(p.emax_2(:)));
            grid_e(p.xi3_beg:p.xi3_end) = shocks_classdef.CCNR_quad(p.n_3,mean(p.phi(:)),mean(p.emin_3(:)),mean(p.emax_3(:)));

            if (~isempty(grid_e_xi1) || ~isempty(grid_e_xi2) || ~isempty(grid_e_xi3))
                grid_e(p.xi1_beg:p.xi1_end) = grid_e_xi1;
                grid_e(p.xi2_beg:p.xi2_end) = grid_e_xi2;
                grid_e(p.xi3_beg:p.xi3_end) = grid_e_xi3;
            end

            for z_ind = 1:p.len_z
                for zp_ind = 1:p.len_z   
                    %Conditional prob
                    pr_ep_cdnal_xi1 = shocks_classdef.CCNR_cdnalmass(grid_e(p.xi1_beg:p.xi1_end),p.phi(z_ind,zp_ind),p.emin_1(z_ind,zp_ind),p.emax_1(z_ind,zp_ind));
                    pr_ep_cdnal_xi2 = shocks_classdef.CCNR_cdnalmass(grid_e(p.xi2_beg:p.xi2_end),p.phi(z_ind,zp_ind),p.emin_2(z_ind,zp_ind),p.emax_2(z_ind,zp_ind)); 
                    pr_ep_cdnal_xi3 = shocks_classdef.CCNR_cdnalmass(grid_e(p.xi3_beg:p.xi3_end),p.phi(z_ind,zp_ind),p.emin_3(z_ind,zp_ind),p.emax_3(z_ind,zp_ind));

                    %hi -> tomorrow
                    for e_ind = p.xi1_beg:p.xi1_end
                        trans_e(e_ind,z_ind,zp_ind,p.xi1_beg:p.xi1_end) = pr_ep_cdnal_xi1*p.trans_11;
                        trans_e(e_ind,z_ind,zp_ind,p.xi2_beg:p.xi2_end) = pr_ep_cdnal_xi2*p.trans_12;
                        trans_e(e_ind,z_ind,zp_ind,p.xi3_beg:p.xi3_end) = pr_ep_cdnal_xi3*p.trans_13;
                    end

                    %med -> tomorrow
                    for e_ind = p.xi2_beg:p.xi2_end
                        trans_e(e_ind,z_ind,zp_ind,p.xi1_beg:p.xi1_end) = pr_ep_cdnal_xi1*p.trans_21;
                        trans_e(e_ind,z_ind,zp_ind,p.xi2_beg:p.xi2_end) = pr_ep_cdnal_xi2*p.trans_22;
                        trans_e(e_ind,z_ind,zp_ind,p.xi3_beg:p.xi3_end) = pr_ep_cdnal_xi3*p.trans_23;
                    end

                    %lo -> tomorrow
                    for e_ind = p.xi3_beg:p.xi3_end
                        trans_e(e_ind,z_ind,zp_ind,p.xi1_beg:p.xi1_end) = pr_ep_cdnal_xi1*p.trans_31;
                        trans_e(e_ind,z_ind,zp_ind,p.xi2_beg:p.xi2_end) = pr_ep_cdnal_xi2*p.trans_32;
                        trans_e(e_ind,z_ind,zp_ind,p.xi3_beg:p.xi3_end) = pr_ep_cdnal_xi3*p.trans_33;
                    end
                end
            end
        end
        
        
        function [grid_e,trans_CCNR] = sub_idshocks_SS(p,varargin)
            if length(varargin) == 3
                grid_e_xi1 = varargin(1);
                grid_e_xi2 = varargin(2);
                grid_e_xi3 = varargin(3);
            else
                grid_e_xi1 = [];
                grid_e_xi2 = [];
                grid_e_xi3 = [];
            end
                    
            %Check inputs
            if ~isempty(grid_e_xi1) || ~isempty(grid_e_xi2) || ~isempty(grid_e_xi3) 
                if length(grid_e_xi1)~=abs(p.xi1_end-p.xi1_beg)+1 || ...
                   length(grid_e_xi2)~=abs(p.xi2_end-p.xi2_beg)+1 || ...
                   length(grid_e_xi3)~=abs(p.xi3_end-p.xi3_beg)+1 
                    error('a grid has the wrong size')
                end
            end
            if p.n_1<=0 || p.n_2<=0 || p.n_3<=0 
                error('sub_idshocks: modify me')
            end

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %   Use CCNR conditional distributions

            %Setup grid
            grid_e(p.xi1_beg:p.xi1_end) = shocks_classdef.CCNR_quad(p.n_1,p.phi_SS,p.emin_1_SS,p.emax_1_SS);
            grid_e(p.xi2_beg:p.xi2_end) = shocks_classdef.CCNR_quad(p.n_2,p.phi_SS,p.emin_2_SS,p.emax_2_SS);
            grid_e(p.xi3_beg:p.xi3_end) = shocks_classdef.CCNR_quad(p.n_3,p.phi_SS,p.emin_3_SS,p.emax_3_SS);

            if ~isempty(grid_e_xi1) || ~isempty(grid_e_xi2) || ~isempty(grid_e_xi3) 
                grid_e(p.xi1_beg:p.xi1_end) = grid_e_xi1;
                grid_e(p.xi2_beg:p.xi2_end) = grid_e_xi2;
                grid_e(p.xi3_beg:p.xi3_end) = grid_e_xi3;
            end

            %Conditional prob
            pr_ep_cdnal_xi1 = shocks_classdef.CCNR_cdnalmass(grid_e(p.xi1_beg:p.xi1_end),p.phi_SS,p.emin_1_SS,p.emax_1_SS);
            pr_ep_cdnal_xi2 = shocks_classdef.CCNR_cdnalmass(grid_e(p.xi2_beg:p.xi2_end),p.phi_SS,p.emin_2_SS,p.emax_2_SS); 
            pr_ep_cdnal_xi3 = shocks_classdef.CCNR_cdnalmass(grid_e(p.xi3_beg:p.xi3_end),p.phi_SS,p.emin_3_SS,p.emax_3_SS);
            
            %hi -> tomorrow
            for e_ind = p.xi1_beg:p.xi1_end
                trans_CCNR(e_ind,p.xi1_beg:p.xi1_end) = pr_ep_cdnal_xi1*p.trans_11;
                trans_CCNR(e_ind,p.xi2_beg:p.xi2_end) = pr_ep_cdnal_xi2*p.trans_12;
                trans_CCNR(e_ind,p.xi3_beg:p.xi3_end) = pr_ep_cdnal_xi3*p.trans_13;
            end

            %med -> tomorrow
            for e_ind = p.xi2_beg:p.xi2_end
                trans_CCNR(e_ind,p.xi1_beg:p.xi1_end) = pr_ep_cdnal_xi1*p.trans_21;
                trans_CCNR(e_ind,p.xi2_beg:p.xi2_end) = pr_ep_cdnal_xi2*p.trans_22;
                trans_CCNR(e_ind,p.xi3_beg:p.xi3_end) = pr_ep_cdnal_xi3*p.trans_23;
            end

            %lo -> tomorrow
            for e_ind = p.xi3_beg:p.xi3_end
                trans_CCNR(e_ind,p.xi1_beg:p.xi1_end) = pr_ep_cdnal_xi1*p.trans_31;
                trans_CCNR(e_ind,p.xi2_beg:p.xi2_end) = pr_ep_cdnal_xi2*p.trans_32;
                trans_CCNR(e_ind,p.xi3_beg:p.xi3_end) = pr_ep_cdnal_xi3*p.trans_33;
            end

            %Check that the probabilities sum to 1
            if any(abs(sum(trans_CCNR,2)-1.d0))>1d-15
                disp('WARNING: Transition matrix trans_CCNR rows do not sum to 1. Max diff is %f',maxval(abs(sum(trans_CCNR,2)-1.d0))) 
                error('aborted')
            end
            
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % CCNR_cdnalmass
        % Given a grid of e, an emin, emax, and phi, computes appropriate
        % mass levels.  For instance, if grid_e = (e_min,e_max) and phi = 1,
        % places prob (1/2,1/2) on grid
        % e should be sorted
        function pr = CCNR_cdnalmass(e,phi,emin,emax)

%             real(dp), dimension(1:), intent(in) :: e_in
%             real(dp), intent(in) :: emin,emax,phi
%             real(dp), dimension(1:size(e_in)) :: pr_out
%             %local
%             real(dp), dimension(1:size(e_in)) :: e, pr
%             integer, dimension(1:size(e_in)) :: index
%             real(dp), dimension(1:size(e_in)-1) :: midpt
%             integer :: e_ind
%             logical :: is_Asc
            pr = zeros(size(e));
            
            %check for sorted
            if ~(all(e==sort(e,'ascend')) || all(e==sort(e,'descend')))
                disp(e);
                error('CCNR_cdnalmass: e is not sorted%')
            end

            %check for ascending sort                
            is_Asc = all(e==sort(e,'ascend')); %issorted(e,'ascend');

            %sort into ascending if not
            if ~is_Asc
                e = sort(e,1,'ascend');
            end
            
            if (size(e)==1)
                pr = 1.d0;
                return
            end

            midpt = e(1:end-1) + diff(e)/2.d0;
            pr(1) = shocks_classdef.P(midpt(1),phi,emin,emax);            
            pr(length(e)) = 1.d0-shocks_classdef.P(midpt(length(e)-1),phi,emin,emax);
            pr(2:length(e)-1) = shocks_classdef.P(midpt(2:length(e)-1),phi,emin,emax)-...
                                shocks_classdef.P(midpt(1:length(e)-2),phi,emin,emax);

            %sort back into descending if was like that
            if (~is_Asc)
                [e,index] = sort(e,1,'descend');
                pr = pr(index);
            end

        end

        %Cdf of a point
        function cdf = P(pt,phi,emin,emax)
            cdf = zeros(size(pt));
            cdf(pt<=emin) = 0.d0;
            cdf(pt>=emax) = 1.d0;
            int = ~(pt<=emin | pt>=emax);
            cdf(int) = ((pt(int)-emin)./(emax-emin)).^phi;
        end

        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % CCNR_quad
        % Given a value for emin, emax, phi, and n, will approximate
        % [[y-emin]/[emax-emin]]^psi with n quadrature points placing endpoints
        % on grid
        function grid_e = CCNR_quad(n,phi,emin,emax)
            
            y = zeros(n,1);
            
            %Setup the [emax = 1 emin = 0], then rescale and shift       
            if n==1
                y = .5d0.^(1.d0/phi);
            else
                y(1) = 0.d0; %1/n mass assigned
                y(n) = 1.d0; %1/n mass assigned
                if (n>2) 
                    for i = 2:n-1
                        y(i) = (y(i-1).^phi + 1.d0/(n-1)).^(1.d0/phi);
                    end
                end
            end

            %Expand
            y = y*(emax-emin) + emin;

            %Sorts so in descending order
            for i=1:n
                yrev(i) = y(n+1-i);
            end

            grid_e = yrev;

        end
        
        function [invar_dsn_e_cdnal_z, trans_ez] = sub_invarshocks(trans_e, invar_dsn_z, trans_z, p)

            %Write joint transition matrix for (e,z)
            for e_ind = 1:p.len_e
                for z_ind = 1:p.len_z
                    for ep_ind = 1:p.len_e
                        for zp_ind = 1:p.len_z
                            trans_ez(e_ind,z_ind,ep_ind,zp_ind) = trans_e(e_ind,z_ind,zp_ind,ep_ind)*trans_z(z_ind,zp_ind);
                        end 
                    end 
                end
            end 

            %Compute joint invariant distribution of (e,z)
            dsn_old = ones(p.len_e,p.len_z);
            dsn_old = 1.d0/numel(dsn_old)*ones(size(dsn_old));
            eps = 100.d0;
            while (eps>1d-12)
                dsn = zeros(p.len_e,p.len_z);
                for ep_ind = 1:p.len_e
                    for zp_ind = 1:p.len_z
                        for e_ind = 1:p.len_e
                            for z_ind = 1:p.len_z
                                dsn(ep_ind,zp_ind) = dsn(ep_ind,zp_ind) + trans_ez(e_ind,z_ind,ep_ind,zp_ind)*dsn_old(e_ind,z_ind);
                            end 
                        end 
                    end 
                end 
                eps = max(abs(dsn-dsn_old));
                dsn_old = dsn;
            end
            invar_dsn_ez = dsn;

            %Given Pr(e,z) and Pr(z), compute Pr(e|z) = Pr(e,z)/Pr(z)
            for z_ind = 1:p.len_z
                invar_dsn_e_cdnal_z(:,z_ind) = invar_dsn_ez(:,z_ind)/invar_dsn_z(z_ind);
            end 
        
        end
    end
    
end

