% Estimates a probit model using likelihood.  This is guaranteed to
% converge.  Pr(y=1|x) = normcdf(x'*b)
% [b] = myprobit(y,x)
% 
function [b] = myprobit(y,x)

    % Guess on b
    b0 = zeros(size(x,2),1);

    % Define the negative log-likelihood equation
    neglogl = @(b) -loglike(b,y,x);
    
    % Minimize the negative log-likelihood
    o = optimset('Display','final','MaxFunEvals',10000,'MaxIter',10000);
    b = fminsearch(neglogl,b0,o);
    
end

function [l]=loglike(b,y,x)
    phi = normcdf(x*b,0,1);
    l = sum(y.*log(phi) + (1-y).*log(1-phi));
end