% Ridders' method
% Note: this could be easily vectorized, but is not currently.
% Algorithm is from wiki/NR.
% pmod of 0 means no print
function [xstar,fevals] = ridders(f,a,b,pmod)

toler = 1d-6;
xtoler = 1d-6;
maxiter = 100;

ya = f(a);
yb = f(b);
fevals = 2;

% Check whether should stop
if (abs(ya)<toler)
    xstar = a;
    return
elseif (abs(yb)<toler)
    xstar = b;
    return
elseif (sign(ya)==sign(yb))
    fprintf(1,'ya %f yb %f \n',ya,yb);
    error('Root is not bracketed')
end

% Main loop
step = 'Init';
for iter = 1:maxiter
    
    % Report progress
    if (mod(iter,pmod)==0) 
        fprintf(1,'iter %i a %f b %f ya %f yb %f fevals %i step %s\n ',iter,a,b,ya,yb,fevals,step);
    end
    
    % Check the stopping criterion
    if abs(b-a)<xtoler
        xstar = (b+a)/2;
        if (pmod>0)
            fprintf(1,'Converged: |b-a|<xtoler \n');
        end
        return
    end
    
    % Midpoint
    m = (b+a)/2;
    ym = f(m);        
    fevals = fevals + 1;
    if abs(ym)<toler % Check if got lucky
        xstar = m;
        if (pmod>0) 
            fprintf(1,'Converged at bisect pt: xstar %f f(xstar) %f toler %f \n',m,ym,toler);
        end
        return
    end

    % Update x
%     x = m + (m-a)*sign(ya-yb)*ym/sqrt(ym.^2-ya*yb);
    r = m + (m-a)*sign(ya)*ym/sqrt(ym.^2-ya*yb); % Equivalent to above (see wiki talk)
    yr = f(r);
    fevals = fevals + 1;

    % Check other stopping criterion
    if abs(yr)<toler
        xstar = r;
        if (pmod>0)
            fprintf(1,'Converged at ridder pt: xstar %f f(xstar) %f toler %f \n',r,yr,toler);
        end
        return
    end
    
    
    % To take best advantage of the information at hand, we want
    % the smallest interval possible. 
    rm_brack = sign(yr)~=sign(ym);
    ar_brack = sign(ya)~=sign(yr);
    rb_brack = sign(yr)~=sign(yb);
    am_brack = sign(ya)~=sign(ym); % This case may not be necessary
    mb_brack = sign(ym)~=sign(yb); % This case may not be necessary
    brack = [rm_brack,ar_brack,rb_brack,am_brack,mb_brack];
    
    % Among all possible brackets, find the one with the smallest length
    dist = [abs(r-m),abs(a-r),abs(r-b),abs(a-m),abs(m-b)];
    dist(~brack) = realmax;
    [tmp loc] = min(dist);
    
    switch loc
        case 1,
            % r,m interval best
            if (r<m)
                a = r;
                ya = yr;
                b = m;
                yb = ym;
            else
                a = m;
                ya = ym;
                b = r;
                yb = yr;
            end
            step = 'Best';
        case 2,
            % a,r interval best
            b = r;
            yb = yr;     
            step = 'Good';
        case 3,
            % r,b interval best
            a = r;
            ya = yr;
            step = 'Good';
        case 4,
            % a,m interval best
            % This case may not be necessary
            b = m;
            yb = ym;
            step = 'Bisect';
        case 5,
            % m,b interval best
            % This case may not be necessary
            a = m;
            ya = ym;
            step = 'Bisect';
    end
        
end


end