% [Yp Ypp] = schumakerSplineDeriv(X,knots,abcd)
% Compute the first and second derivatives at X of the spline given implicitly
% by knots and abc.
% 
% Note: if a second derivative is requested at a knot, either the left or
% right derivative may be returned
%
% The spline is stored in piecewise polynomial form
% y(x) = a + b(x-xi) + c(x-xi)^2 (where xi=knots(i))
% so
% y'(x) = b + 2c(x-xi)
% y''(x) = 2c
function [Yp Ypp] = schumakerSplineDeriv(X,knots,abc)

% Ensure column vectors
if (~iscolumn(X)) 
    error('X must be a column vector')
end

% Use binary search to find the index the lower brackets X in grid x.
i = bsearch(X,knots);

% Get the difference
del = X-knots(i);

% Compute the derivatives
Ypp = 2*abc(i,3);
Yp = abc(i,2) + Ypp.*del;

end