% sub_gini
% Given a dsn (pdf) defined on grid, 
% returns three things: gini coefficient, the lorenz curve, and a line of
% equality.  
% [gini Lorenz Equality Lorenz_grid] = sub_gini(dsn, grid)
function [gini Lorenz Equality Lorenz_grid] = sub_gini(dsn, grid)

%Check inputs
if (numel(dsn)~=numel(grid))
    error('sub_gini: dsn and grid must have same size')
end
if size(dsn,2)>1 || size(grid,2)>1
    error('sub_gini: only vectors are allowed')
end
if size(grid,1)~=size(dsn,1) || size(grid,2)~=size(dsn,2) || size(grid,3)~=size(dsn,3)
    disp({'Size of Dsn',size(dsn)})
    disp({'Size of Grid',size(grid)})
    error('sub_gini: can only handle vectors')
end
if any(diff(grid)<0) 
    error('sub_gini: grid must be increasing')
end 

Lorenz_grid = (0:1e-4:1)';
Lorenz = zeros(size(Lorenz_grid));
Equality = Lorenz_grid;

pct_of_stuff = cumsum(grid.*dsn)/(grid'*dsn);
pct_of_pop = cumsum(dsn)/sum(dsn);

pct_of_stuff = [0;pct_of_stuff;1];
pct_of_pop = [0;pct_of_pop;1];

%make values unique
[dum I] = unique(pct_of_pop);
if (numel(I)~=numel(pct_of_pop)) 
    disp('sub_gini: population cdf not unique, making unique')
end
pct_of_pop=pct_of_pop(I);
pct_of_stuff=pct_of_stuff(I);

Lorenz = interp1(pct_of_pop,pct_of_stuff,Lorenz_grid);
%Lorenz(1) = 0;
%Lorenz(end) = 1;

gini = sum(Equality-Lorenz)/(sum(Equality-Lorenz)+sum(Lorenz)); %Namibia has income(?) gini of .707, highest in the world

end 
