% Many examples from spinterp doc
function [] = testSpinterp()

%% EXAMPLE 1
clear
d = 100;
range = repmat([-d^2 d^2],d,1);
options = spset('DimensionAdaptive', 'on', ...
                'DimadaptDegree', 1, ...
                'FunctionArgType', 'vector', ...
                'RelTol', 1e-3, ...
                'MaxPoints', 40000);
z1 = spvals(@trid, d, range, options);
z1

% Compute 100 randomly sampled points
p = 100;
rand('state', 0);
x = -d^2 + 2*d^2*rand(p,d);
% Compute exact function values
y = zeros(p,1);
for k = 1:p
  y(k) = trid(x(k,:));
end
% Compute interpolated function values
xcell = num2cell(x,1);
ip1 = spinterp(z1, xcell{:});
% Compute relative errors
err_CC = max(abs(y-ip1))/(z1.fevalRange(2)-z1.fevalRange(1))

%% EXAMPLE 2
clear

fun = inline(['(5/pi*x-5.1/(4*pi^2)*x.^2+y-6).^2 + ' ...
'10*(1-1/(8*pi))*cos(x)+10']);
d = 2;
range = [-5, 10; 0, 15];

X = linspace(range(1,1),range(1,2));
Y = linspace(range(1,1),range(1,2));
[X,Y] = ndgrid(X,Y);
Z = fun(X,Y);
figure(1)
surf(X,Y,Z)

% options = spset('GridType', 'Chebyshev','DimensionAdaptive', 'on', ...
%   'MinPoints', 65, 'MaxPoints', 65, 'Vectorized', 'on', 'KeepGrid', 'on');
options = spset('DimensionAdaptive', 'on', ...
  'MinPoints', 65, 'MaxPoints', 65, 'Vectorized', 'on', 'KeepGrid', 'on');
z1 = spvals(fun, d, range, spset(options, 'DimadaptDegree', 0));
z2 = spvals(fun, d, range, spset(options, 'DimadaptDegree', 0.5));
z3 = spvals(fun, d, range, spset(options, 'DimadaptDegree', 1));

disp(sprintf(...
  'Degree of dimensional adaptivity: z1: %.2f, z2: %.2f, z3: %.2f', ...
  z1.dimadaptDegree, z2.dimadaptDegree, z3.dimadaptDegree));
    
figure(2)
z = {z1, z2, z3};
for k = 1:3
    subplot(2,3,k);
    plot(z{k}.grid{1}(:,1), z{k}.grid{1}(:,2), 'k.');
    axis([range(1,:), range(2,:)]);
    axis square;
    subplot(2,3,3+k);
    plotindices(z{k});
    title(sprintf('z%d, degree: %2.0f%%', k, 100*z{k}.dimadaptDegree))
    axis([0,8,0,6]);
end    

end

% From spinterp:
function y = trid(x)
    % TRID   Quadratic function with a tridiagonal Hessian.
    %   Y = TRID(X)   returns the function value Y for a D-
    %   dimensional input vector X.
    %
    % f(x)=[sum_{i=1}^d (x_i-1)^2] - [sum_{i=2}^d x_ix_{i-1}]
    %
    % The test function is due to Arnold Neumaier, listed
    % on the global optimization Web page at
    %   http://www.mat.univie.ac.at/~neum/glopt/
    d = length(x);
    y = sum((x-1).^2) - sum(x(2:d).*x(1:d-1));
end