% [welfareGainSS popGainSS welfareGainBC popGainBC UtilityLevels]=welfareCompare(varargin)
% Computes welfare of moving from 'dfile' to 'ndfile' and call this
% eliminating default. However, dfile and ndfile can be any result files,
% and so this is used to compute all the welfare gains reported in the
% paper. There are also various switches such as whether PE results should
% be computed using 'ndpefile', and whether the BC should be done or not.
%
function [welfareGainSS popGainSS welfareGainBC popGainBC UtilityLevels welfareseq]=welfareCompare(varargin)

global fid

% Switch on whether to transit hhs so no unanticipated losses
zeroProfits = 1; % Whether hhs who default should be migrated to a=0,h=0
doBC = 1;
doPE = 0;
conditionOnHeq0 = 0; % Whether to condition gains on h==0

% Object load list
commonLoad = {'nA','nE','nAmm','nM','nH','g_ind','b_ind','init_dsn*',...
           'len_*','nT','Tend','g*','trans_z','invar_dsn_e_SS','state_*',...
           'medianBAPCPA','zseq_ind','alpha_k','Nseq'};
dfileLoad = {'v_SS','d_SS','w_SS','qbar_SS','Ev',...
               'invar_dsn_SS','seq_pdfAvg','Ed','invar_dsn_m',...
               'seq_statlist','zseq_ind','Ev_cdnal_*','Kseq'};
ndfileLoad = {'v_SS','Ev','seq_pdfAvg','Ev_cdnal_*','seq_statlist'};

%
if nargin==0
    fid = 1;
%     dfile = [getmatdir(),'results_defexpend.mat'];
%     ndfile = [getmatdir(),'results_nodefexpend.mat'];
%     ndpefile = 'results_nodefPEexpend.mat';    
    dfile = [getmatdir(),'results_defexpend.mat'];
    ndfile = [getmatdir(),'results_nodefexpend.mat'];
    ndpefile = [getmatdir(),'results_nodefPEexpend.mat'];
    doPE = 1;
  
elseif nargin==3
    fid = varargin{1};
    dfile = [getmatdir(),varargin{2}];
    ndfile = [getmatdir(),varargin{3}];
    doPE = 0;

elseif nargin==4
    fid = varargin{1};
    dfile = [getmatdir(),varargin{2}];
    ndfile = [getmatdir(),varargin{3}];
    ndpefile = [getmatdir(),varargin{4}];
    doPE = 1;
    
else
    error('nargin is incorrect')
end


load(dfile,commonLoad{:});
D = load(dfile,dfileLoad{:});
ND = load(ndfile,ndfileLoad{:});
if doPE
    NDPE = load(ndpefile,ndfileLoad{:});
end

D.seq_EvNewborn = D.seq_statlist(57,:)';
ND.seq_EvNewborn = ND.seq_statlist(57,:)';
if (doPE)
    NDPE.seq_EvNewborn = NDPE.seq_statlist(57,:)';
end

invar_dsn_z = null(eye(2)-trans_z'); invar_dsn_z = invar_dsn_z/sum(invar_dsn_z);
gAge = (20:85)';
sigma = 2; % CRRA

%% Samples
AAA = repmat(gA,[1,nE,Tend,nH]);
EEE = repmat(reshape(gE,[1,nE,Tend,1]),[nA,1,1,nH]);
HHH(:,:,:,1) = zeros(nA,nE,Tend);
HHH(:,:,:,2) = ones(nA,nE,Tend);
GGG = repmat(reshape(gAge,[1 1 Tend]),[nA,nE,1,nH]);
TTT = repmat(state_tran',[nA,1,Tend,nH]);
PPP = repmat(state_pers',[nA,1,Tend,nH]);

% If one wants welfare gains broken out by various subgroups, can use this
col{1} = ones(nA,nE,Tend,nH); %Everyone
col{2} = col{1} & GGG==20; % Newborns
col{3} = col{1} & GGG>=20 & GGG<=64; % Workers
col{4} = col{1} & GGG>=65;
row{1} = col{1}; 
row{2} = row{1} & AAA<0;
row{3} = row{1} & TTT==1; % Low tran shock
row{4} = row{1} & EEE<medianBAPCPA; % Below median efficiency hhs
row{5} = row{1} & EEE>=medianBAPCPA; % Below median efficiency hhs

fprintf(fid,'\nEX ANTE WELFARE MEASURES\n\n');

U_D_SS = D.v_SS(nAmm+1,:,1,1)*init_dsn_e_SS;
U_ND_SS = ND.v_SS(nAmm+1,:,1,1)*init_dsn_e_SS;
fprintf(fid,'Utils Newborn  D SS: %g \n',U_D_SS);
fprintf(fid,'Utils Newborn ND SS: %g \n',U_ND_SS);

UtilityLevels.U_D_SS = U_D_SS;
UtilityLevels.U_ND_SS = U_ND_SS;

if (doBC)
    U_D_BC = invar_dsn_z(g_ind)*D.Ev_cdnal_g(nAmm+1,:,1,1)*init_dsn_e(:,g_ind) ...
           + invar_dsn_z(b_ind)*D.Ev_cdnal_b(nAmm+1,:,1,1)*init_dsn_e(:,b_ind);
    U_ND_BC = invar_dsn_z(g_ind)*ND.Ev_cdnal_g(nAmm+1,:,1,1)*init_dsn_e(:,g_ind) ...
            + invar_dsn_z(b_ind)*ND.Ev_cdnal_b(nAmm+1,:,1,1)*init_dsn_e(:,b_ind);
        
    fprintf(fid,'Utils Newborn  D BC: %g \n',U_D_BC);
    fprintf(fid,'Utils Newborn ND BC: %g \n',U_ND_BC);
    
    UtilityLevels.U_D_BC = U_D_BC;
    UtilityLevels.U_D_BC = U_ND_BC;

end

welfareGainSS = 100*csnequiv(U_D_SS,U_ND_SS,sigma);
welfareGainBC = 100*csnequiv(U_D_BC,U_ND_BC,sigma);
fprintf(fid,'Value of eliminating default in SS: %g\n',welfareGainSS);
if (doBC)
    fprintf(fid,'Value of eliminating default in BC: %g\n',welfareGainBC);
end

fprintf(fid,'Note: shouldn''t agree with value of elim def at age 1 \n');
fprintf(fid,'      in the figures because one is consumption gain of avg.\n');
fprintf(fid,'      and other is average of consumption gains. In other \n');
fprintf(fid,'      one is ex ante and one is ex post.\n\n');

%% Welfare gains over the business cycle
tmp = 100*csnequiv(D.seq_EvNewborn(1:end-1),ND.seq_EvNewborn(1:end-1),sigma);
plot(tmp)

trange = (len_t0+1:nT-1)';
tmp = 100*csnequiv(D.seq_EvNewborn(trange),ND.seq_EvNewborn(trange),sigma);

figure(1)
subplot(2,1,1)
plot(tmp)
xlabel('Time')
ylabel('Ex Ante Welfare Gain')

subplot(2,1,2)
Yseq = grid_z(zseq_ind).*D.Kseq.^(alpha_k).*Nseq.^(1-alpha_k);
plot(Yseq(trange),tmp,'.')
xlabel('Output')
ylabel('Ex Ante Welfare Gain')

welfareseq.Y = Yseq(trange);
welfareseq.gain = tmp; 

figure(2)

fprintf(fid,'Newborn welfare over the business cycle\n')
fprintf(fid,'Average gain:               %g \n',mean(tmp));
fprintf(fid,'Standard deviation of gain: %g \n\n',std(tmp));

clf
hold on
[edges,dsn] = myhist(tmp(D.zseq_ind(trange)==1),30);
plot(edges,dsn,'b')
[edges,dsn] = myhist(tmp(D.zseq_ind(trange)==2),30);
plot(edges,dsn,'r')
hold off
legend('Good shock','Bad shock')

figure(3)
%%

fprintf(fid,'EX POST WELFARE MEASURES\n\n')

clear tmp tmp2
tmp(:,:,:,1) = csnequiv(D.v_SS(:,:,:,1),ND.v_SS(:,:,:,1),sigma);
tmp(:,:,:,2) = csnequiv(D.v_SS(:,:,:,2),ND.v_SS(:,:,:,1),sigma); % NOTE: compare with transitioning to good standing
if (~zeroProfits)
    tmp2 = D.invar_dsn_SS;
else
    tmp2 = D.invar_dsn_SS.*(1-D.d_SS);
    tmp2(nAmm+1,:,:,1) = tmp2(nAmm+1,:,:,1) + sum(sum(D.invar_dsn_SS.*D.d_SS,4),1);
end
if (conditionOnHeq0) 
    tmp2(:,:,:,1) = tmp2(:,:,:,1)/sum(vec(tmp2(:,:,:,2)));
    tmp2(:,:,:,2) = 0;
end 

popGainElimDefByAgeSS = gainsByAge(100*(tmp>=0),tmp2);
csnGainElimDefByAgeSS = gainsByAge(100*tmp,tmp2);

plot(csnGainElimDefByAgeSS)

if doPE

    tmpPE(:,:,:,1) = csnequiv(D.v_SS(:,:,:,1),NDPE.v_SS(:,:,:,1),sigma);
    tmpPE(:,:,:,2) = csnequiv(D.v_SS(:,:,:,2),NDPE.v_SS(:,:,:,1),sigma);

    popGainElimDefByAgeSSPE = gainsByAge(100*(tmpPE>=0),tmp2);
    csnGainElimDefByAgeSSPE = gainsByAge(100*tmpPE,tmp2);

else
    csnGainElimDefByAgeSSPE = 0*csnGainElimDefByAgeSS;
end

plot(gAge,csnGainElimDefByAgeSS,'b-',...
     gAge,csnGainElimDefByAgeSSPE,'r--',...
     'LineWidth',2)
xlabel('Age')
ylabel('Consumption Gain (%)')
title('Consumption Gain of Eliminating Default by Age')
legend('General Eq','Partial Eq','location','northeast')

fprintf(fid,'Welfare gain of elim def in SS: \n');
basic_table(100*tmp,tmp2,row,col,2);
fprintf(fid,'\n');
fprintf(fid,'Frac preferring elim def in SS:\n');
table = basic_table(100*(tmp>=-1d-15),tmp2,row,col,1);

popGainSS = table(1,1);
fprintf(fid,'\nTotal pop preferring elim def in SS: %6.1f \n',popGainSS);

% Ed = zeros(nA,nE,Tend,nH);
% for mind = 1:nM
%     Ed(1:nAmm,:,:,1) = Ed(1:nAmm,:,:,1) + D.d(:,:,:,mind).*D.invar_dsn_m(mind);
% end    

tmp(:,:,:,1) = csnequiv(D.Ev(:,:,:,1),ND.Ev(:,:,:,1),sigma);
tmp(:,:,:,2) = csnequiv(D.Ev(:,:,:,2),ND.Ev(:,:,:,1),sigma);
if (~zeroProfits)
    tmp2(:,:,:,1) = D.seq_pdfAvg(:,:,:,1);
    tmp2(:,:,:,2) = D.seq_pdfAvg(:,:,:,2);
else
    tmp2 = D.invar_dsn_SS.*(1-D.d_SS);
    tmp2(nAmm+1,:,:,1) = tmp2(nAmm+1,:,:,1) + sum(sum(D.invar_dsn_SS.*D.d_SS,4),1);

    tmp2 = D.seq_pdfAvg.*(1-D.Ed);
    tmp2(nAmm+1,:,:,1) = tmp2(nAmm+1,:,:,1) + sum(sum(D.seq_pdfAvg.*D.Ed,4),1);

%     tmp2(:,:,:,1) = NaN(size(D.seq_pdfAvg(:,:,:,1)));
%     tmp2(nAmm+1:nA,:,:,1) = D.seq_pdfAvg(nAmm+1:nA,:,:,1);
%     tmp2(1:nAmm,:,:,1) = D.seq_pdfAvg(1:nAmm,:,:,1).*(D.Ed(1:nAmm,:,:,1)<1d-16);
%     tmp2(nAmm+1,:,:,1) = tmp2(nAmm+1,:,:,1) + sum(D.seq_pdfAvg(1:nAmm,:,:,1).*(D.Ed(1:nAmm,:,:,1)>=1d-16));
%     tmp2(:,:,:,2) = D.seq_pdfAvg(:,:,:,2);
end
if (conditionOnHeq0) 
    tmp2(:,:,:,1) = tmp2(:,:,:,1)/sum(vec(tmp2(:,:,:,2)));
    tmp2(:,:,:,2) = 0;
end 

popGainElimDefByAgeBC = gainsByAge(100*(tmp>=0),tmp2);
csnGainElimDefByAgeBC = gainsByAge(100*tmp,tmp2);

if (doPE) 
    tmpPE(:,:,:,1) = csnequiv(D.Ev(:,:,:,1),NDPE.Ev(:,:,:,1),sigma);
    tmpPE(:,:,:,2) = csnequiv(D.Ev(:,:,:,2),NDPE.Ev(:,:,:,1),sigma);

    popGainElimDefByAgeBCPE = gainsByAge(100*(tmpPE>=0),tmp2);
    csnGainElimDefByAgeBCPE = gainsByAge(100*tmpPE,tmp2);
else
    csnGainElimDefByAgeBCPE = 0*csnGainElimDefByAgeBC;
end

plot(gAge,csnGainElimDefByAgeBC,'b-',...
     gAge,csnGainElimDefByAgeSS,'b-o',...
     gAge,csnGainElimDefByAgeBCPE,'r--',...
     gAge,csnGainElimDefByAgeSSPE,'r--o',...
     'LineWidth',2)
xlabel('Age')
ylabel('Consumption Gain (%)')
title('Consumption Gain of Eliminating Default by Age')
legend('Business Cycle, General Eq','Steady State, General Eq','Business Cycle Partial Eq','Steady State, Partial Eq','location','northeast')

fprintf(fid,'\nWelfare gain of elim def in BC: \n');

basic_table(100*tmp,tmp2,row,col,2);

fprintf(fid,'\nFrac preferring elim def in BC:\n');
table = basic_table(100*(tmp>=-1d-15),tmp2,row,col,1);

popGainBC = table(1,1);
fprintf(fid,'\nTotal pop preferring elim def in BC: %6.1f \n',popGainBC);

return
% These are additional plots / analysis from previous versions of the paper
% that I may recincorporate.

% Various subsamples
sample{1} = csnGainElimDefSS(:)<0 & csnGainElimDefBC(:)>0 & D.invar_dsn_SS(:)>0 & D.seq_pdfAvg(:)>0; % & AAA(:)<5 & EEE(:)<2;
sample{2} = csnGainElimDefSS(:)>0 & csnGainElimDefBC(:)<0 & D.invar_dsn_SS(:)>0 & D.seq_pdfAvg(:)>0; % & AAA(:)<5 & EEE(:)<2;
sample{3} = D.invar_dsn_SS(:)>0 & D.seq_pdfAvg(:)>0;
sample{4} = [];
sample{5} = 100*(csnGainElimDefSS(:)-csnGainElimDefBC(:))>5 & D.invar_dsn_SS(:)>0 & D.seq_pdfAvg(:)>0 & D.invar_dsn_SS(:)>0 & Ed(:)==0 & d(:)==0;
sample{6} = 100*(csnGainElimDefSS(:)-csnGainElimDefBC(:))<-5 & D.invar_dsn_SS(:)>0 & D.seq_pdfAvg(:)>0 & D.invar_dsn_SS(:)>0 & Ed(:)==0 & d(:)==0; 
sample{7} = 100*(csnGainElimDefSS(:)-csnGainElimDefBC(:))>10 & D.invar_dsn_SS(:)>0 & D.seq_pdfAvg(:)>0 & D.invar_dsn_SS(:)>0 & Ed(:)==0 & d(:)==0; 
sample{8} = 100*(csnGainElimDefSS(:)-csnGainElimDefBC(:))<-2 & D.invar_dsn_SS(:)>0 & D.seq_pdfAvg(:)>0 & D.invar_dsn_SS(:)>0 & Ed(:)==0 & d(:)==0; 
sample{9} = 100*csnGainElimDefSS(:)>1 & D.invar_dsn_SS(:)>0 & Ed(:)==0; % & AAA(:)<2 & EEE(:) < 1;
sample{10} = 100*csnGainElimDefSS(:)<-1 & D.invar_dsn_SS(:)>0 & Ed(:)==0;


key{4} = [];
key{5} = 'More than 5% better';
key{6} = 'More than 5% worse';
key{7} = 'More than 10% better';
key{8} = 'More than 2% worse';
key{9} = 'Gain larger than 2%';
key{9} = 'Gain smaller than -2%';

% clf
% plot3(AAA(sample{9}),EEE(sample{9}),GGG(sample{9}),'bo',...
%       AAA(sample{10}),EEE(sample{10}),GGG(sample{10}),'r.')
% view([153.5 34]) % GE
% xlabel('Assets')
% ylabel('Efficiency')
% zlabel('Age')
% legend(key{9},'location','best')
% axis([-.4,.4,.0,1,20,74]);

% alo = find(gA>-.5,1,'first');
% ahi = find(gA>5,1,'first');
% einds = find(gE<2);
% clf
% contour(EEE(alo:ahi,einds,1),GGG(alo:ahi,einds,1),csnGainElimDefSS(alo:ahi,einds,1),linspace(-10,10,201))

%From the distribution conditional on gaining more than 1%, draw random
%samples
% draws = drawPdf(D.invar_dsn_SS(100*csnGainElimDefSS(:)>5)/sum(D.invar_dsn_SS(100*csnGainElimDefSS(:)>5)),10000);
% tmp = AAA(sample{9});
% hist(tmp(draws),100);


% clf
% plot3(AAA(sample{4}),EEE(sample{4}),GGG(sample{4}),'bo',...
%       AAA(sample{6}),EEE(sample{6}),GGG(sample{6}),'r.')
% xlabel('Assets')
% ylabel('Efficiency')
% zlabel('Age')
% view([153.5 34]) % GE
% legend(key{4},key{6},'location','best')
% title('How Default Looks After Including the Business Cycle')
% grid on

clf
plot3(AAA(sample{5}),EEE(sample{5}),GGG(sample{5}),'bo',...
      AAA(sample{6}),EEE(sample{6}),GGG(sample{6}),'r.')
xlabel('Assets')
ylabel('Efficiency')
zlabel('Age')
% view([153.5 34]) % GE
% view([-188.5,-344])
view([161.5,20])
legend(key{5},key{6},'location','northwest')
title('How Default Looks After Including the Business Cycle')
grid on
myprint('DefaultWelfareChangeBC')

clf
sample{7} = sample{7} & PPP(:)~=-1;
sample{8} = sample{8} & PPP(:)~=-1;
plot3(AAA(sample{7}),log(EEE(sample{7})),GGG(sample{7}),'b.',...
      AAA(sample{8}),log(EEE(sample{8})),GGG(sample{8}),'ro')
xlabel('Assets')
ylabel('Log Efficiency)')
zlabel('Age')
view([153.5 34]) % GE
legend(key{7},key{8},'location','best')
title('How Default Looks After Including the Business Cycle')
grid on

% look at just assets and efficiency direction
% 
% this excludes households who would default. so the highly indebted
% high efficiency households are ones that are just on the cusp of 
% default. if they were to default, they would face very high default 
% costs. so they are something near the optimal default policy: they 
% have good credit prices, but they are insured a bit. probably for 
% them, they are quite close to the natural borrowing limit and have 
% been borrowing for a long time to get there --- so to experiencing 
% the contraction caused by BC puts them very close to the natural 
% borrowing limit. the \CD\ economy's borrowing limit may be greater 
% for these guys than the \ND\ economy's.



end

function [avgGain] = gainsByAge(gain,dsn)

    T = size(gain,3);
    
    avgGain = NaN(T,1);
    for age = 1:T
        avgGain(age) = sum(sum(sum(gain(:,:,age,:).*dsn(:,:,age,:))))./sum(sum(sum(dsn(:,:,age,:))));
    end
    
    
end


function [avgGain list] = gainsByState(gain,dsn,state)

    list = unique(state(state>0));
    N = numel(list);
    
    avgGain = NaN(N,1);
    for i = 1:N
        avgGain(i) = sum(sum(gain(:,state==list(i)).*dsn(:,state==list(i))))./sum(sum(dsn(:,state==list(i))));
    end
    
    
end

function [table samplesizes]= basic_table(tau,dsn,row,col,sigdig)

    global fid

    nrow = numel(row);
    ncol = numel(col);
    table = NaN(nrow,ncol);
    samplesizes = NaN(nrow,ncol);
    
    for ir = 1:nrow
        for ic = 1:ncol
            sample = row{ir} & col{ic};
            samplesizes(ir,ic) = sum(dsn(sample));
            table(ir,ic) = tau(sample)'*dsn(sample)/samplesizes(ir,ic);
        end
    end   

    if (sigdig==2)
        for ir = 1:numel(row)
            for ic = 1:numel(col)
                fprintf(fid,' & %6.2f  ',table(ir,ic));
            end
            fprintf(fid,' \\\\ \n');
        end
    elseif (sigdig==1)
    for ir = 1:numel(row)
        for ic = 1:numel(col)
            fprintf(fid,' & %6.1f  ',table(ir,ic));
        end
        fprintf(fid,' \\\\ \n');
    end
        
    end
    
end
